/* Generated by Cython 3.0.5 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "linux/ftd2xx.h",
            "linux/libft4222.h"
        ],
        "include_dirs": [
            "linux"
        ],
        "libraries": [
            "ft4222"
        ],
        "library_dirs": [
            "linux/build-x86_64"
        ],
        "name": "ft4222.ft4222",
        "runtime_library_dirs": [
            "$ORIGIN/."
        ],
        "sources": [
            "ft4222/ft4222.pyx"
        ]
    },
    "module_name": "ft4222.ft4222"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#if defined(CYTHON_LIMITED_API) && 0
  #ifndef Py_LIMITED_API
    #if CYTHON_LIMITED_API+0 > 0x03030000
      #define Py_LIMITED_API CYTHON_LIMITED_API
    #else
      #define Py_LIMITED_API 0x03030000
    #endif
  #endif
#endif

#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02070000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.7+ or Python 3.3+.
#else
#if defined(CYTHON_LIMITED_API) && CYTHON_LIMITED_API
#define __PYX_EXTRA_ABI_MODULE_NAME "limited"
#else
#define __PYX_EXTRA_ABI_MODULE_NAME ""
#endif
#define CYTHON_ABI "3_0_5" __PYX_EXTRA_ABI_MODULE_NAME
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."
#define CYTHON_HEX_VERSION 0x030005F0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #define HAVE_LONG_LONG
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#define __PYX_LIMITED_VERSION_HEX PY_VERSION_HEX
#if defined(GRAALVM_PYTHON)
  /* For very preliminary testing purposes. Most variables are set the same as PyPy.
     The existence of this section does not imply that anything works or is even tested */
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 1
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS (PY_MAJOR_VERSION >= 3)
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(PYPY_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS (PY_MAJOR_VERSION >= 3)
  #endif
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1 && PYPY_VERSION_NUM >= 0x07030C00)
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(CYTHON_LIMITED_API)
  #ifdef Py_LIMITED_API
    #undef __PYX_LIMITED_VERSION_HEX
    #define __PYX_LIMITED_VERSION_HEX Py_LIMITED_API
  #endif
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_CLINE_IN_TRACEBACK
  #define CYTHON_CLINE_IN_TRACEBACK 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 1
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(PY_NOGIL)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #ifndef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL (PY_MAJOR_VERSION < 3 || PY_VERSION_HEX >= 0x03060000 && PY_VERSION_HEX < 0x030C00A6)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL (PY_VERSION_HEX >= 0x030700A1)
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #if PY_VERSION_HEX < 0x030400a1
    #undef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #elif !defined(CYTHON_USE_TP_FINALIZE)
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #if PY_VERSION_HEX < 0x030600B1
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS  (PY_VERSION_HEX < 0x030C00A5)
  #endif
  #if PY_VERSION_HEX < 0x030700A3
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if !defined(CYTHON_VECTORCALL)
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL && PY_VERSION_HEX >= 0x030800B1)
#endif
#define CYTHON_BACKPORT_VECTORCALL (CYTHON_METH_FASTCALL && PY_VERSION_HEX < 0x030800B1)
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(maybe_unused) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(maybe_unused)
        #define CYTHON_UNUSED [[maybe_unused]]
      #endif
    #endif
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_USE_CPP_STD_MOVE
  #if defined(__cplusplus) && (\
    __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600))
    #define CYTHON_USE_CPP_STD_MOVE 1
  #else
    #define CYTHON_USE_CPP_STD_MOVE 0
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
            typedef unsigned char     uint8_t;
            typedef unsigned short    uint16_t;
            typedef unsigned int      uint32_t;
        #else
            typedef unsigned __int8   uint8_t;
            typedef unsigned __int16  uint16_t;
            typedef unsigned __int32  uint32_t;
        #endif
    #endif
    #if _MSC_VER < 1300
        #ifdef _WIN64
            typedef unsigned long long  __pyx_uintptr_t;
        #else
            typedef unsigned int        __pyx_uintptr_t;
        #endif
    #else
        #ifdef _WIN64
            typedef unsigned __int64    __pyx_uintptr_t;
        #else
            typedef unsigned __int32    __pyx_uintptr_t;
        #endif
    #endif
#else
    #include <stdint.h>
    typedef uintptr_t  __pyx_uintptr_t;
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(fallthrough) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(fallthrough)
        #define CYTHON_FALLTHROUGH [[fallthrough]]
      #endif
    #endif
    #ifndef CYTHON_FALLTHROUGH
      #if __has_cpp_attribute(clang::fallthrough)
        #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
      #elif __has_cpp_attribute(gnu::fallthrough)
        #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
      #endif
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif
#ifdef __cplusplus
  template <typename T>
  struct __PYX_IS_UNSIGNED_IMPL {static const bool value = T(0) < T(-1);};
  #define __PYX_IS_UNSIGNED(type) (__PYX_IS_UNSIGNED_IMPL<type>::value)
#else
  #define __PYX_IS_UNSIGNED(type) (((type)-1) > 0)
#endif
#if CYTHON_COMPILING_IN_PYPY == 1
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x030A0000)
#else
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000)
#endif
#define __PYX_REINTERPRET_FUNCION(func_pointer, other_pointer) ((func_pointer)(void(*)(void))(other_pointer))

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_DefaultClassType PyClass_Type
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if CYTHON_COMPILING_IN_LIMITED_API
    static CYTHON_INLINE PyObject* __Pyx_PyCode_New(int a, int p, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *exception_table = NULL;
        PyObject *types_module=NULL, *code_type=NULL, *result=NULL;
        #if __PYX_LIMITED_VERSION_HEX < 0x030B0000
        PyObject *version_info; // borrowed
        #endif
        PyObject *py_minor_version = NULL;
        long minor_version = 0;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        #if __PYX_LIMITED_VERSION_HEX >= 0x030B0000
        minor_version = 11; // we don't yet need to distinguish between versions > 11
        #else
        if (!(version_info = PySys_GetObject("version_info"))) goto end;
        if (!(py_minor_version = PySequence_GetItem(version_info, 1))) goto end;
        minor_version = PyLong_AsLong(py_minor_version);
        if (minor_version == -1 && PyErr_Occurred()) goto end;
        #endif
        if (!(types_module = PyImport_ImportModule("types"))) goto end;
        if (!(code_type = PyObject_GetAttrString(types_module, "CodeType"))) goto end;
        if (minor_version <= 7) {
            (void)p;
            result = PyObject_CallFunction(code_type, "iiiiiOOOOOOiOO", a, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else if (minor_version <= 10) {
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOiOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else {
            if (!(exception_table = PyBytes_FromStringAndSize(NULL, 0))) goto end;
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOOiOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, name, fline, lnos, exception_table, fv, cell);
        }
    end:
        Py_XDECREF(code_type);
        Py_XDECREF(exception_table);
        Py_XDECREF(types_module);
        Py_XDECREF(py_minor_version);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return result;
    }
    #ifndef CO_OPTIMIZED
    #define CO_OPTIMIZED 0x0001
    #endif
    #ifndef CO_NEWLOCALS
    #define CO_NEWLOCALS 0x0002
    #endif
    #ifndef CO_VARARGS
    #define CO_VARARGS 0x0004
    #endif
    #ifndef CO_VARKEYWORDS
    #define CO_VARKEYWORDS 0x0008
    #endif
    #ifndef CO_ASYNC_GENERATOR
    #define CO_ASYNC_GENERATOR 0x0200
    #endif
    #ifndef CO_GENERATOR
    #define CO_GENERATOR 0x0020
    #endif
    #ifndef CO_COROUTINE
    #define CO_COROUTINE 0x0080
    #endif
#elif PY_VERSION_HEX >= 0x030B0000
  static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int p, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
    PyCodeObject *result;
    PyObject *empty_bytes = PyBytes_FromStringAndSize("", 0);  // we don't have access to __pyx_empty_bytes here
    if (!empty_bytes) return NULL;
    result =
      #if PY_VERSION_HEX >= 0x030C0000
        PyUnstable_Code_NewWithPosOnlyArgs
      #else
        PyCode_NewWithPosOnlyArgs
      #endif
        (a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, name, fline, lnos, empty_bytes);
    Py_DECREF(empty_bytes);
    return result;
  }
#elif PY_VERSION_HEX >= 0x030800B2 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_NewWithPosOnlyArgs(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
#endif
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_Is)
  #define __Pyx_Py_Is(x, y)  Py_Is(x, y)
#else
  #define __Pyx_Py_Is(x, y) ((x) == (y))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsNone)
  #define __Pyx_Py_IsNone(ob) Py_IsNone(ob)
#else
  #define __Pyx_Py_IsNone(ob) __Pyx_Py_Is((ob), Py_None)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsTrue)
  #define __Pyx_Py_IsTrue(ob) Py_IsTrue(ob)
#else
  #define __Pyx_Py_IsTrue(ob) __Pyx_Py_Is((ob), Py_True)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsFalse)
  #define __Pyx_Py_IsFalse(ob) Py_IsFalse(ob)
#else
  #define __Pyx_Py_IsFalse(ob) __Pyx_Py_Is((ob), Py_False)
#endif
#define __Pyx_NoneAsNull(obj)  (__Pyx_Py_IsNone(obj) ? NULL : (obj))
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef CO_COROUTINE
  #define CO_COROUTINE 0x80
#endif
#ifndef CO_ASYNC_GENERATOR
  #define CO_ASYNC_GENERATOR 0x200
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef Py_TPFLAGS_SEQUENCE
  #define Py_TPFLAGS_SEQUENCE 0
#endif
#ifndef Py_TPFLAGS_MAPPING
  #define Py_TPFLAGS_MAPPING 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#elif CYTHON_BACKPORT_VECTORCALL
  typedef PyObject *(*__pyx_vectorcallfunc)(PyObject *callable, PyObject *const *args,
                                            size_t nargsf, PyObject *kwnames);
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  ((size_t)1 << (8 * sizeof(size_t) - 1))
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(((size_t)(n)) & ~__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_MAJOR_VERSION >= 0x030900B1
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_CheckExact(func)
#else
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_Check(func)
#endif
#define __Pyx_CyOrPyCFunction_Check(func)  PyCFunction_Check(func)
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  (((PyCFunctionObject*)(func))->m_ml->ml_meth)
#elif !CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  PyCFunction_GET_FUNCTION(func)
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FLAGS(func)  (((PyCFunctionObject*)(func))->m_ml->ml_flags)
static CYTHON_INLINE PyObject* __Pyx_CyOrPyCFunction_GET_SELF(PyObject *func) {
    return (__Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_STATIC) ? NULL : ((PyCFunctionObject*)func)->m_self;
}
#endif
static CYTHON_INLINE int __Pyx__IsSameCFunction(PyObject *func, void *cfunc) {
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyCFunction_Check(func) && PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
#else
    return PyCFunction_Check(func) && PyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
#endif
}
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCFunction(func, cfunc)
#if __PYX_LIMITED_VERSION_HEX < 0x030900B1
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyThreadState_Current PyThreadState_GetUnchecked()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE void *__Pyx_PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#endif
#define __Pyx_PyObject_GetSlot(obj, name, func_ctype)  __Pyx_PyType_GetSlot(Py_TYPE(obj), name, func_ctype)
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((func_ctype) PyType_GetSlot((type), Py_##name))
#else
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((type)->name)
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if PY_MAJOR_VERSION < 3
    #if CYTHON_COMPILING_IN_PYPY
        #if PYPY_VERSION_NUM < 0x07030600
            #if defined(__cplusplus) && __cplusplus >= 201402L
                [[deprecated("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6")]]
            #elif defined(__GNUC__) || defined(__clang__)
                __attribute__ ((__deprecated__("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6")))
            #elif defined(_MSC_VER)
                __declspec(deprecated("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6"))
            #endif
            static CYTHON_INLINE int PyGILState_Check(void) {
                return 0;
            }
        #else  // PYPY_VERSION_NUM < 0x07030600
        #endif  // PYPY_VERSION_NUM < 0x07030600
    #else
        static CYTHON_INLINE int PyGILState_Check(void) {
            PyThreadState * tstate = _PyThreadState_Current;
            return tstate && (tstate == PyGILState_GetThisThreadState());
        }
    #endif
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000 || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030600B4 && PY_VERSION_HEX < 0x030d0000 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif PY_MAJOR_VERSION >= 3 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000)
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
  #define __Pyx_PyObject_GetIterNextFunc(obj)  (Py_TYPE(obj)->tp_iternext)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
  #define __Pyx_PyObject_GetIterNextFunc(obj)  PyIter_Next
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_SetItemOnTypeDict(tp, k, v) PyObject_GenericSetAttr((PyObject*)tp, k, v)
#else
  #define __Pyx_SetItemOnTypeDict(tp, k, v) PyDict_SetItem(tp->tp_dict, k, v)
#endif
#if CYTHON_USE_TYPE_SPECS && PY_VERSION_HEX >= 0x03080000
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE(obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GetLength(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#elif PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, (Py_UCS4) ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535U : 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((int)sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = (Py_UNICODE) ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains) || (PY_MAJOR_VERSION == 2 && PYPY_VERSION_NUM < 0x07030500)
    #undef PyUnicode_Contains
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#if CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_ITEM(o, i) PySequence_ITEM(o, i)
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) (PyTuple_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyList_SET_ITEM(o, i, v) (PyList_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_GET_SIZE(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_GET_SIZE(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_GET_SIZE(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_GET_SIZE(o)
#else
  #define __Pyx_PySequence_ITEM(o, i) PySequence_GetItem(o, i)
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) PyTuple_SetItem(o, i, v)
  #define __Pyx_PyList_SET_ITEM(o, i, v) PyList_SetItem(o, i, v)
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_Size(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_Size(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_Size(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_Size(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_Size(o)
#endif
#if PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#else
  static CYTHON_INLINE PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *module = PyImport_AddModule(name);
      Py_XINCREF(module);
      return module;
  }
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define __Pyx_Py3Int_Check(op)       PyLong_Check(op)
  #define __Pyx_Py3Int_CheckExact(op)  PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#else
  #define __Pyx_Py3Int_Check(op)       (PyLong_Check(op) || PyInt_Check(op))
  #define __Pyx_Py3Int_CheckExact(op)  (PyLong_CheckExact(op) || PyInt_CheckExact(op))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #if !defined(_USE_MATH_DEFINES)
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifdef CYTHON_EXTERN_C
    #undef __PYX_EXTERN_C
    #define __PYX_EXTERN_C CYTHON_EXTERN_C
#elif defined(__PYX_EXTERN_C)
    #ifdef _MSC_VER
    #pragma message ("Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.")
    #else
    #warning Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.
    #endif
#else
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__ft4222__ft4222
#define __PYX_HAVE_API__ft4222__ft4222
/* Early includes */
#include "ftd2xx.h"
#include <stdint.h>
#include "libft4222.h"

    #if CYTHON_COMPILING_IN_PYPY
    #ifdef _MSC_VER
    #pragma message ("This module uses CPython specific internals of 'array.array', which are not available in PyPy.")
    #else
    #warning This module uses CPython specific internals of 'array.array', which are not available in PyPy.
    #endif
    #endif
    
#include <string.h>
#include <stdio.h>
#include <stddef.h>
#include <time.h>

    #if defined(_WIN32) || defined(MS_WINDOWS) || defined(_MSC_VER)
      #include <malloc.h>
      #include <stdlib.h>
      #define msleep(m)  Sleep(m)
    #else
      #include <alloca.h>
      #include <unistd.h>
      #define msleep(m)  ((void) usleep((m) * 1000))
    #endif
    #define MAX_DESCRIPTION_SIZE 256
    
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s);
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char*);
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const wchar_t *u)
{
    const wchar_t *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#endif
#define __Pyx_PyUnicode_FromOrdinal(o)       PyUnicode_FromOrdinal((int)o)
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_VERSION_HEX >= 0x030C00A7
  #ifndef _PyLong_SIGN_MASK
    #define _PyLong_SIGN_MASK 3
  #endif
  #ifndef _PyLong_NON_SIZE_BITS
    #define _PyLong_NON_SIZE_BITS 3
  #endif
  #define __Pyx_PyLong_Sign(x)  (((PyLongObject*)x)->long_value.lv_tag & _PyLong_SIGN_MASK)
  #define __Pyx_PyLong_IsNeg(x)  ((__Pyx_PyLong_Sign(x) & 2) != 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (!__Pyx_PyLong_IsNeg(x))
  #define __Pyx_PyLong_IsZero(x)  (__Pyx_PyLong_Sign(x) & 1)
  #define __Pyx_PyLong_IsPos(x)  (__Pyx_PyLong_Sign(x) == 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  (__Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  ((Py_ssize_t) (((PyLongObject*)x)->long_value.lv_tag >> _PyLong_NON_SIZE_BITS))
  #define __Pyx_PyLong_SignedDigitCount(x)\
        ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * __Pyx_PyLong_DigitCount(x))
  #if defined(PyUnstable_Long_IsCompact) && defined(PyUnstable_Long_CompactValue)
    #define __Pyx_PyLong_IsCompact(x)     PyUnstable_Long_IsCompact((PyLongObject*) x)
    #define __Pyx_PyLong_CompactValue(x)  PyUnstable_Long_CompactValue((PyLongObject*) x)
  #else
    #define __Pyx_PyLong_IsCompact(x)     (((PyLongObject*)x)->long_value.lv_tag < (2 << _PyLong_NON_SIZE_BITS))
    #define __Pyx_PyLong_CompactValue(x)  ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * (Py_ssize_t) __Pyx_PyLong_Digits(x)[0])
  #endif
  typedef Py_ssize_t  __Pyx_compact_pylong;
  typedef size_t  __Pyx_compact_upylong;
  #else  // Py < 3.12
  #define __Pyx_PyLong_IsNeg(x)  (Py_SIZE(x) < 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (Py_SIZE(x) >= 0)
  #define __Pyx_PyLong_IsZero(x)  (Py_SIZE(x) == 0)
  #define __Pyx_PyLong_IsPos(x)  (Py_SIZE(x) > 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  ((Py_SIZE(x) == 0) ? 0 : __Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  __Pyx_sst_abs(Py_SIZE(x))
  #define __Pyx_PyLong_SignedDigitCount(x)  Py_SIZE(x)
  #define __Pyx_PyLong_IsCompact(x)  (Py_SIZE(x) == 0 || Py_SIZE(x) == 1 || Py_SIZE(x) == -1)
  #define __Pyx_PyLong_CompactValue(x)\
        ((Py_SIZE(x) == 0) ? (sdigit) 0 : ((Py_SIZE(x) < 0) ? -(sdigit)__Pyx_PyLong_Digits(x)[0] : (sdigit)__Pyx_PyLong_Digits(x)[0]))
  typedef sdigit  __Pyx_compact_pylong;
  typedef digit  __Pyx_compact_upylong;
  #endif
  #if PY_VERSION_HEX >= 0x030C00A5
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->long_value.ob_digit)
  #else
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->ob_digit)
  #endif
#endif
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
#include <string.h>
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = (char) c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#include <string.h>
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* #### Code section: filename_table ### */

static const char *__pyx_f[] = {
  "ft4222/ft4222.pyx",
  "<stringsource>",
  "array.pxd",
  "type.pxd",
};
/* #### Code section: utility_code_proto_before_types ### */
/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* BufferFormatStructs.proto */
struct __Pyx_StructField_;
#define __PYX_BUF_FLAGS_PACKED_STRUCT (1 << 0)
typedef struct {
  const char* name;
  struct __Pyx_StructField_* fields;
  size_t size;
  size_t arraysize[8];
  int ndim;
  char typegroup;
  char is_unsigned;
  int flags;
} __Pyx_TypeInfo;
typedef struct __Pyx_StructField_ {
  __Pyx_TypeInfo* type;
  const char* name;
  size_t offset;
} __Pyx_StructField;
typedef struct {
  __Pyx_StructField* field;
  size_t parent_offset;
} __Pyx_BufFmt_StackElem;
typedef struct {
  __Pyx_StructField root;
  __Pyx_BufFmt_StackElem* head;
  size_t fmt_offset;
  size_t new_count, enc_count;
  size_t struct_alignment;
  int is_complex;
  char enc_type;
  char new_packmode;
  char enc_packmode;
  char is_valid_array;
} __Pyx_BufFmt_Context;

/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()

/* #### Code section: numeric_typedefs ### */
/* #### Code section: complex_type_declarations ### */
/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
#ifndef _ARRAYARRAY_H
struct arrayobject;
typedef struct arrayobject arrayobject;
#endif
struct __pyx_obj_6ft4222_6ft4222__closer;
struct __pyx_obj_6ft4222_6ft4222_FT4222;

/* "ft4222/ft4222.pyx":183
 * 
 * 
 * cdef class _closer:             # <<<<<<<<<<<<<<
 *     """Wrap a hid_device *ptr and a provide a way to call hid_close() on it.
 *     Used internally for weakref.finalize, which only accepts Python objects.
 */
struct __pyx_obj_6ft4222_6ft4222__closer {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ft4222_6ft4222__closer *__pyx_vtab;
  FT_HANDLE _handle;
};


/* "ft4222/ft4222.pyx":201
 * 
 * 
 * cdef class FT4222:             # <<<<<<<<<<<<<<
 *     # Revision A chips report chipVersion as 0x42220100; revision B chips report
 *     # 0x42220200; revision C chips report 0x42220300.
 */
struct __pyx_obj_6ft4222_6ft4222_FT4222 {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ft4222_6ft4222_FT4222 *__pyx_vtab;
  FT_HANDLE _handle;
  DWORD _chip_version;
  DWORD _dll_version;
  PyObject *__weakref__;
  PyObject *_close;
};



/* "ft4222/ft4222.pyx":183
 * 
 * 
 * cdef class _closer:             # <<<<<<<<<<<<<<
 *     """Wrap a hid_device *ptr and a provide a way to call hid_close() on it.
 *     Used internally for weakref.finalize, which only accepts Python objects.
 */

struct __pyx_vtabstruct_6ft4222_6ft4222__closer {
  PyObject *(*wrap)(FT_HANDLE);
};
static struct __pyx_vtabstruct_6ft4222_6ft4222__closer *__pyx_vtabptr_6ft4222_6ft4222__closer;


/* "ft4222/ft4222.pyx":201
 * 
 * 
 * cdef class FT4222:             # <<<<<<<<<<<<<<
 *     # Revision A chips report chipVersion as 0x42220100; revision B chips report
 *     # 0x42220200; revision C chips report 0x42220300.
 */

struct __pyx_vtabstruct_6ft4222_6ft4222_FT4222 {
  PyObject *(*_get_version)(struct __pyx_obj_6ft4222_6ft4222_FT4222 *);
};
static struct __pyx_vtabstruct_6ft4222_6ft4222_FT4222 *__pyx_vtabptr_6ft4222_6ft4222_FT4222;
/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__))
  #define __Pyx_RefNannyFinishContextNogil() __Pyx_RefNannyFinishContext()
#endif
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#if PY_VERSION_HEX >= 0x030C00A6
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->current_exception != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->current_exception ? (PyObject*) Py_TYPE(__pyx_tstate->current_exception) : (PyObject*) NULL)
#else
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->curexc_type != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->curexc_type)
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  (PyErr_Occurred() != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A6
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* TupleAndListFromArray.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* fastcall.proto */
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_Arg_VARARGS(args, i) PySequence_GetItem(args, i)
#elif CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_Arg_VARARGS(args, i) PyTuple_GET_ITEM(args, i)
#else
    #define __Pyx_Arg_VARARGS(args, i) PyTuple_GetItem(args, i)
#endif
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_Arg_NewRef_VARARGS(arg) __Pyx_NewRef(arg)
    #define __Pyx_Arg_XDECREF_VARARGS(arg) Py_XDECREF(arg)
#else
    #define __Pyx_Arg_NewRef_VARARGS(arg) arg // no-op
    #define __Pyx_Arg_XDECREF_VARARGS(arg) // no-op - arg is borrowed
#endif
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_Arg_FASTCALL(args, i) args[i]
    #define __Pyx_NumKwargs_FASTCALL(kwds) PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) ((args) + (nargs))
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
    static CYTHON_UNUSED PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues);
  #else
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
  #endif
    #define __Pyx_Arg_NewRef_FASTCALL(arg) arg // no-op, __Pyx_Arg_FASTCALL is direct and this needs
    #define __Pyx_Arg_XDECREF_FASTCALL(arg)  // no-op - arg was returned from array
#else
    #define __Pyx_Arg_FASTCALL __Pyx_Arg_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
    #define __Pyx_Arg_NewRef_FASTCALL(arg) __Pyx_Arg_NewRef_VARARGS(arg)
    #define __Pyx_Arg_XDECREF_FASTCALL(arg) __Pyx_Arg_XDECREF_VARARGS(arg)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) __Pyx_PyTuple_FromArray(&__Pyx_Arg_VARARGS(args, start), stop - start)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(&__Pyx_Arg_FASTCALL(args, start), stop - start)
#else
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) PyTuple_GetSlice(args, start, stop)
#endif

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject *const *kwvalues,
    PyObject **argnames[],
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,
    const char* function_name);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#if !CYTHON_VECTORCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if !CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03080000
  #include "frameobject.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
  #define __Pxy_PyFrame_Initialize_Offsets()
  #define __Pyx_PyFrame_GetLocalsplus(frame)  ((frame)->f_localsplus)
#else
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif
#endif
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject **args, size_t nargs, PyObject *kwargs);

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyObjectCallNoArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kw, const char* function_name, int kw_allowed);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* IsLittleEndian.proto */
static CYTHON_INLINE int __Pyx_Is_Little_Endian(void);

/* BufferFormatCheck.proto */
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts);
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type);

/* BufferGetAndValidate.proto */
#define __Pyx_GetBufferAndValidate(buf, obj, dtype, flags, nd, cast, stack)\
    ((obj == Py_None || obj == NULL) ?\
    (__Pyx_ZeroBuffer(buf), 0) :\
    __Pyx__GetBufferAndValidate(buf, obj, dtype, flags, nd, cast, stack))
static int  __Pyx__GetBufferAndValidate(Py_buffer* buf, PyObject* obj,
    __Pyx_TypeInfo* dtype, int flags, int nd, int cast, __Pyx_BufFmt_StackElem* stack);
static void __Pyx_ZeroBuffer(Py_buffer* buf);
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info);
static Py_ssize_t __Pyx_minusones[] = { -1, -1, -1, -1, -1, -1, -1, -1 };
static Py_ssize_t __Pyx_zeros[] = { 0, 0, 0, 0, 0, 0, 0, 0 };

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_SubtractObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceSubtract(op1, op2) : PyNumber_Subtract(op1, op2))
#endif

/* PyFloatBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_TrueDivideCObj(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyFloat_TrueDivideCObj(op1, op2, floatval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceTrueDivide(op1, op2) : PyNumber_TrueDivide(op1, op2))
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_MultiplyCObj(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_MultiplyCObj(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceMultiply(op1, op2) : PyNumber_Multiply(op1, op2))
#endif

/* IncludeStructmemberH.proto */
#include <structmember.h>

/* FixUpExtensionType.proto */
#if CYTHON_USE_TYPE_SPECS
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type);
#endif

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* ValidateBasesTuple.proto */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases);
#endif

/* PyType_Ready.proto */
CYTHON_UNUSED static int __Pyx_PyType_Ready(PyTypeObject *t);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyTypeObject* typeptr , void* vtable);

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyTypeObject *type);

/* MergeVTables.proto */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_MergeVtables(PyTypeObject *type);
#endif

/* SetupReduce.proto */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_setup_reduce(PyObject* type_obj);
#endif

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_3_0_5
#define __PYX_HAVE_RT_ImportType_proto_3_0_5
#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if (defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_3_0_5(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_3_0_5(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_3_0_5 {
   __Pyx_ImportType_CheckSize_Error_3_0_5 = 0,
   __Pyx_ImportType_CheckSize_Warn_3_0_5 = 1,
   __Pyx_ImportType_CheckSize_Ignore_3_0_5 = 2
};
static PyTypeObject *__Pyx_ImportType_3_0_5(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_0_5 check_size);
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* Py3UpdateBases.proto */
static PyObject* __Pyx_PEP560_update_bases(PyObject *bases);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* SetNameInClass.proto */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && PY_VERSION_HEX < 0x030d0000
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? _PyDict_SetItem_KnownHash(ns, name, value, ((PyASCIIObject *) name)->hash) : PyObject_SetItem(ns, name, value))
#elif CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? PyDict_SetItem(ns, name, value) : PyObject_SetItem(ns, name, value))
#else
#define __Pyx_SetNameInClass(ns, name, value)  PyObject_SetItem(ns, name, value)
#endif

/* FetchSharedCythonModule.proto */
static PyObject *__Pyx_FetchSharedCythonABIModule(void);

/* FetchCommonType.proto */
#if !CYTHON_USE_TYPE_SPECS
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);
#else
static PyTypeObject* __Pyx_FetchCommonTypeFromSpec(PyObject *module, PyType_Spec *spec, PyObject *bases);
#endif

/* PyMethodNew.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    PyObject *typesModule=NULL, *methodType=NULL, *result=NULL;
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    typesModule = PyImport_ImportModule("types");
    if (!typesModule) return NULL;
    methodType = PyObject_GetAttrString(typesModule, "MethodType");
    Py_DECREF(typesModule);
    if (!methodType) return NULL;
    result = PyObject_CallFunctionObjArgs(methodType, func, self, NULL);
    Py_DECREF(methodType);
    return result;
}
#elif PY_MAJOR_VERSION >= 3
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    return PyMethod_New(func, self);
}
#else
    #define __Pyx_PyMethod_New PyMethod_New
#endif

/* PyVectorcallFastCallDict.proto */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw);
#endif

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CYFUNCTION_COROUTINE     0x08
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_CyFunction_GetClassObj(f)\
      (((__pyx_CyFunctionObject *) (f))->func_classobj)
#else
  #define __Pyx_CyFunction_GetClassObj(f)\
      ((PyObject*) ((PyCMethodObject *) (f))->mm_class)
#endif
#define __Pyx_CyFunction_SetClassObj(f, classobj)\
    __Pyx__CyFunction_SetClassObj((__pyx_CyFunctionObject *) (f), (classobj))
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject_HEAD
    PyObject *func;
#elif PY_VERSION_HEX < 0x030900B1
    PyCFunctionObject func;
#else
    PyCMethodObject func;
#endif
#if CYTHON_BACKPORT_VECTORCALL
    __pyx_vectorcallfunc func_vectorcall;
#endif
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_classobj;
#endif
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;  // used by FusedFunction for copying defaults
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
    PyObject *func_is_coroutine;
} __pyx_CyFunctionObject;
#undef __Pyx_CyOrPyCFunction_Check
#define __Pyx_CyFunction_Check(obj)  __Pyx_TypeCheck(obj, __pyx_CyFunctionType)
#define __Pyx_CyOrPyCFunction_Check(obj)  __Pyx_TypeCheck2(obj, __pyx_CyFunctionType, &PyCFunction_Type)
#define __Pyx_CyFunction_CheckExact(obj)  __Pyx_IS_TYPE(obj, __pyx_CyFunctionType)
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc);
#undef __Pyx_IsSameCFunction
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCyOrCFunction(func, cfunc)
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(PyObject *module);
#if CYTHON_METH_FASTCALL
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#if CYTHON_BACKPORT_VECTORCALL
#define __Pyx_CyFunction_func_vectorcall(f) (((__pyx_CyFunctionObject*)f)->func_vectorcall)
#else
#define __Pyx_CyFunction_func_vectorcall(f) (((PyCFunctionObject*)f)->vectorcall)
#endif
#endif

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* PyObjectCall2Args.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectLookupSpecial.proto */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_LookupSpecialNoError(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 0)
#define __Pyx_PyObject_LookupSpecial(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 1)
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error);
#else
#define __Pyx_PyObject_LookupSpecialNoError(o,n) __Pyx_PyObject_GetAttrStrNoError(o,n)
#define __Pyx_PyObject_LookupSpecial(o,n) __Pyx_PyObject_GetAttrStr(o,n)
#endif

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
#if !CYTHON_COMPILING_IN_LIMITED_API
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);
#endif

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* ArrayAPI.proto */
#ifndef _ARRAYARRAY_H
#define _ARRAYARRAY_H
typedef struct arraydescr {
    int typecode;
    int itemsize;
    PyObject * (*getitem)(struct arrayobject *, Py_ssize_t);
    int (*setitem)(struct arrayobject *, Py_ssize_t, PyObject *);
#if PY_MAJOR_VERSION >= 3
    char *formats;
#endif
} arraydescr;
struct arrayobject {
    PyObject_HEAD
    Py_ssize_t ob_size;
    union {
        char *ob_item;
        float *as_floats;
        double *as_doubles;
        int *as_ints;
        unsigned int *as_uints;
        unsigned char *as_uchars;
        signed char *as_schars;
        char *as_chars;
        unsigned long *as_ulongs;
        long *as_longs;
#if PY_MAJOR_VERSION >= 3
        unsigned long long *as_ulonglongs;
        long long *as_longlongs;
#endif
        short *as_shorts;
        unsigned short *as_ushorts;
        Py_UNICODE *as_pyunicodes;
        void *as_voidptr;
    } data;
    Py_ssize_t allocated;
    struct arraydescr *ob_descr;
    PyObject *weakreflist;
#if PY_MAJOR_VERSION >= 3
        int ob_exports;
#endif
};
#ifndef NO_NEWARRAY_INLINE
static CYTHON_INLINE PyObject * newarrayobject(PyTypeObject *type, Py_ssize_t size,
    struct arraydescr *descr) {
    arrayobject *op;
    size_t nbytes;
    if (size < 0) {
        PyErr_BadInternalCall();
        return NULL;
    }
    nbytes = size * descr->itemsize;
    if (nbytes / descr->itemsize != (size_t)size) {
        return PyErr_NoMemory();
    }
    op = (arrayobject *) type->tp_alloc(type, 0);
    if (op == NULL) {
        return NULL;
    }
    op->ob_descr = descr;
    op->allocated = size;
    op->weakreflist = NULL;
    __Pyx_SET_SIZE(op, size);
    if (size <= 0) {
        op->data.ob_item = NULL;
    }
    else {
        op->data.ob_item = PyMem_NEW(char, nbytes);
        if (op->data.ob_item == NULL) {
            Py_DECREF(op);
            return PyErr_NoMemory();
        }
    }
    return (PyObject *) op;
}
#else
PyObject* newarrayobject(PyTypeObject *type, Py_ssize_t size,
    struct arraydescr *descr);
#endif
static CYTHON_INLINE int resize(arrayobject *self, Py_ssize_t n) {
    void *items = (void*) self->data.ob_item;
    PyMem_Resize(items, char, (size_t)(n * self->ob_descr->itemsize));
    if (items == NULL) {
        PyErr_NoMemory();
        return -1;
    }
    self->data.ob_item = (char*) items;
    __Pyx_SET_SIZE(self, n);
    self->allocated = n;
    return 0;
}
static CYTHON_INLINE int resize_smart(arrayobject *self, Py_ssize_t n) {
    void *items = (void*) self->data.ob_item;
    Py_ssize_t newsize;
    if (n < self->allocated && n*4 > self->allocated) {
        __Pyx_SET_SIZE(self, n);
        return 0;
    }
    newsize = n + (n / 2) + 1;
    if (newsize <= n) {
        PyErr_NoMemory();
        return -1;
    }
    PyMem_Resize(items, char, (size_t)(newsize * self->ob_descr->itemsize));
    if (items == NULL) {
        PyErr_NoMemory();
        return -1;
    }
    self->data.ob_item = (char*) items;
    __Pyx_SET_SIZE(self, n);
    self->allocated = newsize;
    return 0;
}
#endif

/* BufferStructDeclare.proto */
typedef struct {
  Py_ssize_t shape, strides, suboffsets;
} __Pyx_Buf_DimInfo;
typedef struct {
  size_t refcount;
  Py_buffer pybuffer;
} __Pyx_Buffer;
typedef struct {
  __Pyx_Buffer *rcbuffer;
  char *data;
  __Pyx_Buf_DimInfo diminfo[8];
} __Pyx_LocalBuf_ND;

#if PY_MAJOR_VERSION < 3
    static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags);
    static void __Pyx_ReleaseBuffer(Py_buffer *view);
#else
    #define __Pyx_GetBuffer PyObject_GetBuffer
    #define __Pyx_ReleaseBuffer PyBuffer_Release
#endif


/* GCCDiagnostics.proto */
#if !defined(__INTEL_COMPILER) && defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_FT4222_STATUS(FT4222_STATUS value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_DWORD(DWORD value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_ULONG(ULONG value);

/* CIntFromPy.proto */
static CYTHON_INLINE DWORD __Pyx_PyInt_As_DWORD(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE ULONG __Pyx_PyInt_As_ULONG(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE FT4222_ClockRate __Pyx_PyInt_As_FT4222_ClockRate(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_FT4222_ClockRate(FT4222_ClockRate value);

/* CIntFromPy.proto */
static CYTHON_INLINE BOOL __Pyx_PyInt_As_BOOL(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint16_t(uint16_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE UCHAR __Pyx_PyInt_As_UCHAR(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE USHORT __Pyx_PyInt_As_USHORT(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_UCHAR(UCHAR value);

/* CIntFromPy.proto */
static CYTHON_INLINE GPIO_Dir __Pyx_PyInt_As_GPIO_Dir(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE GPIO_Port __Pyx_PyInt_As_GPIO_Port(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_BOOL(BOOL value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE GPIO_Trigger __Pyx_PyInt_As_GPIO_Trigger(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE uint16_t __Pyx_PyInt_As_uint16_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GPIO_Trigger(GPIO_Trigger value);

/* CIntFromPy.proto */
static CYTHON_INLINE uint32_t __Pyx_PyInt_As_uint32_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE uint8_t __Pyx_PyInt_As_uint8_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint8_t(uint8_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE SPI_DrivingStrength __Pyx_PyInt_As_SPI_DrivingStrength(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE FT4222_SPIMode __Pyx_PyInt_As_FT4222_SPIMode(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE FT4222_SPIClock __Pyx_PyInt_As_FT4222_SPIClock(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE FT4222_SPICPOL __Pyx_PyInt_As_FT4222_SPICPOL(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE FT4222_SPICPHA __Pyx_PyInt_As_FT4222_SPICPHA(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE SPI_SlaveProtocol __Pyx_PyInt_As_SPI_SlaveProtocol(PyObject *);

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
static __Pyx_TypeName __Pyx_PyType_GetName(PyTypeObject* tp);
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#else
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_CurrentExceptionType(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static unsigned long __Pyx_get_runtime_version(void);
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

/* #### Code section: module_declarations ### */
static PyObject *__pyx_f_6ft4222_6ft4222_7_closer_wrap(FT_HANDLE __pyx_v_h); /* proto*/
static PyObject *__pyx_f_6ft4222_6ft4222_6FT4222__get_version(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto*/

/* Module declarations from "ft4222.cftd2xx" */

/* Module declarations from "libc.stdint" */

/* Module declarations from "ft4222.clibft4222" */

/* Module declarations from "libc.string" */

/* Module declarations from "libc.stdio" */

/* Module declarations from "__builtin__" */

/* Module declarations from "cpython.type" */

/* Module declarations from "cpython" */

/* Module declarations from "cpython.object" */

/* Module declarations from "cpython.ref" */

/* Module declarations from "cpython.exc" */

/* Module declarations from "cpython.mem" */

/* Module declarations from "array" */

/* Module declarations from "cpython.array" */
static CYTHON_INLINE int __pyx_f_7cpython_5array_extend_buffer(arrayobject *, char *, Py_ssize_t); /*proto*/

/* Module declarations from "libc.stddef" */

/* Module declarations from "libc.time" */

/* Module declarations from "ft4222.ft4222" */
/* #### Code section: typeinfo ### */
static __Pyx_TypeInfo __Pyx_TypeInfo_nn_uint8 = { "uint8", NULL, sizeof(uint8), { 0 }, 0, __PYX_IS_UNSIGNED(uint8) ? 'U' : 'I', __PYX_IS_UNSIGNED(uint8), 0 };
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "ft4222.ft4222"
extern int __pyx_module_is_main_ft4222__ft4222;
int __pyx_module_is_main_ft4222__ft4222 = 0;

/* Implementation of "ft4222.ft4222" */
/* #### Code section: global_var ### */
static PyObject *__pyx_builtin_super;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_KeyError;
static PyObject *__pyx_builtin_xrange;
static PyObject *__pyx_builtin_round;
static PyObject *__pyx_builtin_MemoryError;
/* #### Code section: string_decls ### */
static const char __pyx_k_B[] = "B";
static const char __pyx_k_d[] = "d";
static const char __pyx_k_f[] = "f";
static const char __pyx_k_h[] = "h";
static const char __pyx_k_i[] = "i";
static const char __pyx_k_l[] = "l";
static const char __pyx_k_n[] = "n";
static const char __pyx_k_t[] = "t";
static const char __pyx_k_OK[] = "OK";
static const char __pyx_k__8[] = ".";
static const char __pyx_k_cs[] = "cs";
static const char __pyx_k_gc[] = "gc";
static const char __pyx_k_id[] = "id";
static const char __pyx_k_nb[] = "nb";
static const char __pyx_k_Dir[] = "Dir";
static const char __pyx_k_buf[] = "buf";
static const char __pyx_k_clk[] = "clk";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_req[] = "req";
static const char __pyx_k_res[] = "res";
static const char __pyx_k_str[] = "__str__";
static const char __pyx_k_GPIO[] = "GPIO";
static const char __pyx_k__111[] = "?";
static const char __pyx_k_addr[] = "addr";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_cpha[] = "cpha";
static const char __pyx_k_cpol[] = "cpol";
static const char __pyx_k_data[] = "data";
static const char __pyx_k_desc[] = "desc";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_enum[] = "enum";
static const char __pyx_k_flag[] = "flag";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_kbps[] = "kbps";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_mode[] = "mode";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_type[] = "type";
static const char __pyx_k_INPUT[] = "INPUT";
static const char __pyx_k_Rev_A[] = "Rev. A";
static const char __pyx_k_Rev_B[] = "Rev. B";
static const char __pyx_k_Rev_C[] = "Rev. C";
static const char __pyx_k_Rev_D[] = "Rev. D";
static const char __pyx_k_cdata[] = "cdata";
static const char __pyx_k_cdesc[] = "cdesc";
static const char __pyx_k_clock[] = "clock";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_flags[] = "flags";
static const char __pyx_k_gpio0[] = "gpio0";
static const char __pyx_k_gpio1[] = "gpio1";
static const char __pyx_k_gpio2[] = "gpio2";
static const char __pyx_k_gpio3[] = "gpio3";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_ioDir[] = "ioDir";
static const char __pyx_k_locId[] = "locId";
static const char __pyx_k_proto[] = "proto";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_round[] = "round";
static const char __pyx_k_sleep[] = "sleep";
static const char __pyx_k_start[] = "start";
static const char __pyx_k_super[] = "super";
static const char __pyx_k_utf_8[] = "utf-8";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_write[] = "write";
static const char __pyx_k_CLK_24[] = "CLK_24";
static const char __pyx_k_CLK_48[] = "CLK_48";
static const char __pyx_k_CLK_60[] = "CLK_60";
static const char __pyx_k_CLK_80[] = "CLK_80";
static const char __pyx_k_FT4222[] = "FT4222";
static const char __pyx_k_closer[] = "_closer";
static const char __pyx_k_devnum[] = "devnum";
static const char __pyx_k_enable[] = "enable";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_events[] = "events";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_handle[] = "handle";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_msgnum[] = "msgnum";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_serial[] = "serial";
static const char __pyx_k_spiIdx[] = "spiIdx";
static const char __pyx_k_ssoMap[] = "ssoMap";
static const char __pyx_k_status[] = "status";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_xrange[] = "xrange";
static const char __pyx_k_IntEnum[] = "IntEnum";
static const char __pyx_k_Trigger[] = "Trigger";
static const char __pyx_k_cserial[] = "cserial";
static const char __pyx_k_disable[] = "disable";
static const char __pyx_k_latency[] = "latency";
static const char __pyx_k_message[] = "message";
static const char __pyx_k_pRxSize[] = "pRxSize";
static const char __pyx_k_portNum[] = "portNum";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_sleep_c[] = "sleep_c";
static const char __pyx_k_timeout[] = "timeout";
static const char __pyx_k_trigger[] = "trigger";
static const char __pyx_k_weakref[] = "weakref";
static const char __pyx_k_IO_ERROR[] = "IO_ERROR";
static const char __pyx_k_KeyError[] = "KeyError";
static const char __pyx_k_SysClock[] = "SysClock";
static const char __pyx_k_finalize[] = "finalize";
static const char __pyx_k_getClock[] = "getClock";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_location[] = "location";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_readSize[] = "readSize";
static const char __pyx_k_setClock[] = "setClock";
static const char __pyx_k_set_name[] = "__set_name__";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_sizeRead[] = "sizeRead";
static const char __pyx_k_I2CMaster[] = "I2CMaster";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_bytesRead[] = "bytesRead";
static const char __pyx_k_bytesSent[] = "bytesSent";
static const char __pyx_k_chipReset[] = "chipReset";
static const char __pyx_k_gpio_Init[] = "gpio_Init";
static const char __pyx_k_gpio_Read[] = "gpio_Read";
static const char __pyx_k_gpio_Wait[] = "gpio_Wait";
static const char __pyx_k_isenabled[] = "isenabled";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_portNum_c[] = "portNum_c";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_queueSize[] = "queueSize";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_spi_Reset[] = "spi_Reset";
static const char __pyx_k_timeout_c[] = "timeout_c";
static const char __pyx_k_gpio_Write[] = "gpio_Write";
static const char __pyx_k_ioStrength[] = "ioStrength";
static const char __pyx_k_multiWrite[] = "multiWrite";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_MemoryError[] = "MemoryError";
static const char __pyx_k_OTHER_ERROR[] = "OTHER_ERROR";
static const char __pyx_k_Rev_unknown[] = "Rev. unknown";
static const char __pyx_k_bytesToRead[] = "bytesToRead";
static const char __pyx_k_clkStrength[] = "clkStrength";
static const char __pyx_k_description[] = "description";
static const char __pyx_k_mro_entries[] = "__mro_entries__";
static const char __pyx_k_setTimeouts[] = "setTimeouts";
static const char __pyx_k_singleWrite[] = "singleWrite";
static const char __pyx_k_ssoStrength[] = "ssoStrength";
static const char __pyx_k_FT4222_close[] = "FT4222.close";
static const char __pyx_k_INVALID_ARGS[] = "INVALID_ARGS";
static const char __pyx_k_TimeoutError[] = "TimeoutError";
static const char __pyx_k_chipRevision[] = "chipRevision";
static const char __pyx_k_closer_close[] = "_closer.close";
static const char __pyx_k_is_coroutine[] = "_is_coroutine";
static const char __pyx_k_openBySerial[] = "openBySerial";
static const char __pyx_k_read_timeout[] = "read_timeout";
static const char __pyx_k_stringsource[] = "<stringsource>";
static const char __pyx_k_vendorCmdGet[] = "vendorCmdGet";
static const char __pyx_k_vendorCmdSet[] = "vendorCmdSet";
static const char __pyx_k_NOT_SUPPORTED[] = "NOT_SUPPORTED";
static const char __pyx_k_class_getitem[] = "__class_getitem__";
static const char __pyx_k_expectedValue[] = "expectedValue";
static const char __pyx_k_ft4222_ft4222[] = "ft4222.ft4222";
static const char __pyx_k_init_subclass[] = "__init_subclass__";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_setSuspendOut[] = "setSuspendOut";
static const char __pyx_k_spiSlave_Init[] = "spiSlave_Init";
static const char __pyx_k_spiSlave_Read[] = "spiSlave_Read";
static const char __pyx_k_write_timeout[] = "write_timeout";
static const char __pyx_k_INVALID_HANDLE[] = "INVALID_HANDLE";
static const char __pyx_k_WRONG_I2C_ADDR[] = "WRONG_I2C_ADDR";
static const char __pyx_k_i2cMaster_Init[] = "i2cMaster_Init";
static const char __pyx_k_i2cMaster_Read[] = "i2cMaster_Read";
static const char __pyx_k_openByLocation[] = "openByLocation";
static const char __pyx_k_spiMaster_Init[] = "spiMaster_Init";
static const char __pyx_k_spiSlave_Write[] = "spiSlave_Write";
static const char __pyx_k_FT4222_getClock[] = "FT4222.getClock";
static const char __pyx_k_FT4222_setClock[] = "FT4222.setClock";
static const char __pyx_k_FUN_NOT_SUPPORT[] = "FUN_NOT_SUPPORT";
static const char __pyx_k_INVALID_POINTER[] = "INVALID_POINTER";
static const char __pyx_k_IS_NOT_I2C_MODE[] = "IS_NOT_I2C_MODE";
static const char __pyx_k_IS_NOT_SPI_MODE[] = "IS_NOT_SPI_MODE";
static const char __pyx_k_expectedValue_c[] = "expectedValue_c";
static const char __pyx_k_getLatencyTimer[] = "getLatencyTimer";
static const char __pyx_k_i2cMaster_Reset[] = "i2cMaster_Reset";
static const char __pyx_k_i2cMaster_Write[] = "i2cMaster_Write";
static const char __pyx_k_maxTransferSize[] = "maxTransferSize";
static const char __pyx_k_setLatencyTimer[] = "setLatencyTimer";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_sizeTransferred[] = "sizeTransferred";
static const char __pyx_k_spiSlave_InitEx[] = "spiSlave_InitEx";
static const char __pyx_k_ControllerStatus[] = "ControllerStatus";
static const char __pyx_k_DEVICE_NOT_FOUND[] = "DEVICE_NOT_FOUND";
static const char __pyx_k_FT2XXDeviceError[] = "FT2XXDeviceError";
static const char __pyx_k_FT4222_chipReset[] = "FT4222.chipReset";
static const char __pyx_k_FT4222_gpio_Init[] = "FT4222.gpio_Init";
static const char __pyx_k_FT4222_gpio_Read[] = "FT4222.gpio_Read";
static const char __pyx_k_FT4222_gpio_Wait[] = "FT4222.gpio_Wait";
static const char __pyx_k_FT4222_spi_Reset[] = "FT4222.spi_Reset";
static const char __pyx_k_INVAILD_FUNCTION[] = "INVAILD_FUNCTION";
static const char __pyx_k_i2cMaster_ReadEx[] = "i2cMaster_ReadEx";
static const char __pyx_k_isEndTransaction[] = "isEndTransaction";
static const char __pyx_k_spiSlave_SetMode[] = "spiSlave_SetMode";
static const char __pyx_k_weakref_finalize[] = "weakref_finalize";
static const char __pyx_k_CLK_NOT_SUPPORTED[] = "CLK_NOT_SUPPORTED";
static const char __pyx_k_DEVICE_NOT_OPENED[] = "DEVICE_NOT_OPENED";
static const char __pyx_k_FT4222DeviceError[] = "FT4222DeviceError";
static const char __pyx_k_FT4222_gpio_Write[] = "FT4222.gpio_Write";
static const char __pyx_k_INVALID_BAUD_RATE[] = "INVALID_BAUD_RATE";
static const char __pyx_k_INVALID_PARAMETER[] = "INVALID_PARAMETER";
static const char __pyx_k_ft4222_ft4222_pyx[] = "ft4222/ft4222.pyx";
static const char __pyx_k_i2cMaster_WriteEx[] = "i2cMaster_WriteEx";
static const char __pyx_k_openByDescription[] = "openByDescription";
static const char __pyx_k_EEPROM_NOT_PRESENT[] = "EEPROM_NOT_PRESENT";
static const char __pyx_k_EEPROM_READ_FAILED[] = "EEPROM_READ_FAILED";
static const char __pyx_k_FT4222_setTimeouts[] = "FT4222.setTimeouts";
static const char __pyx_k_asyncio_coroutines[] = "asyncio.coroutines";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_getMaxTransferSize[] = "getMaxTransferSize";
static const char __pyx_k_setWakeUpInterrupt[] = "setWakeUpInterrupt";
static const char __pyx_k_spiMaster_SetLines[] = "spiMaster_SetLines";
static const char __pyx_k_EEPROM_ERASE_FAILED[] = "EEPROM_ERASE_FAILED";
static const char __pyx_k_EEPROM_WRITE_FAILED[] = "EEPROM_WRITE_FAILED";
static const char __pyx_k_EVENT_NOT_SUPPORTED[] = "EVENT_NOT_SUPPORTED";
static const char __pyx_k_FT4222_vendorCmdGet[] = "FT4222.vendorCmdGet";
static const char __pyx_k_FT4222_vendorCmdSet[] = "FT4222.vendorCmdSet";
static const char __pyx_k_getDeviceInfoDetail[] = "getDeviceInfoDetail";
static const char __pyx_k_i2cMaster_GetStatus[] = "i2cMaster_GetStatus";
static const char __pyx_k_DEVICE_NOT_SUPPORTED[] = "DEVICE_NOT_SUPPORTED";
static const char __pyx_k_FT4222__chip_rev_map[] = "_FT4222__chip_rev_map";
static const char __pyx_k_FT4222_setSuspendOut[] = "FT4222.setSuspendOut";
static const char __pyx_k_FT4222_spiSlave_Init[] = "FT4222.spiSlave_Init";
static const char __pyx_k_FT4222_spiSlave_Read[] = "FT4222.spiSlave_Read";
static const char __pyx_k_createDeviceInfoList[] = "createDeviceInfoList";
static const char __pyx_k_gpio_SetInputTrigger[] = "gpio_SetInputTrigger";
static const char __pyx_k_spiMaster_SingleRead[] = "spiMaster_SingleRead";
static const char __pyx_k_spiSlave_GetRxStatus[] = "spiSlave_GetRxStatus";
static const char __pyx_k_spi_ResetTransaction[] = "spi_ResetTransaction";
static const char __pyx_k_DEVICE_LIST_NOT_READY[] = "DEVICE_LIST_NOT_READY";
static const char __pyx_k_EEPROM_NOT_PROGRAMMED[] = "EEPROM_NOT_PROGRAMMED";
static const char __pyx_k_FAILED_TO_READ_DEVICE[] = "FAILED_TO_READ_DEVICE";
static const char __pyx_k_FT4222_i2cMaster_Init[] = "FT4222.i2cMaster_Init";
static const char __pyx_k_FT4222_i2cMaster_Read[] = "FT4222.i2cMaster_Read";
static const char __pyx_k_FT4222_spiMaster_Init[] = "FT4222.spiMaster_Init";
static const char __pyx_k_FT4222_spiSlave_Write[] = "FT4222.spiSlave_Write";
static const char __pyx_k_IS_NOT_SPI_MULTI_MODE[] = "IS_NOT_SPI_MULTI_MODE";
static const char __pyx_k_gpio_GetTriggerStatus[] = "gpio_GetTriggerStatus";
static const char __pyx_k_gpio_ReadTriggerQueue[] = "gpio_ReadTriggerQueue";
static const char __pyx_k_spiMaster_SingleWrite[] = "spiMaster_SingleWrite";
static const char __pyx_k_FT2XXDeviceError___str[] = "FT2XXDeviceError.__str__";
static const char __pyx_k_FT4222___reduce_cython[] = "FT4222.__reduce_cython__";
static const char __pyx_k_FT4222_getLatencyTimer[] = "FT4222.getLatencyTimer";
static const char __pyx_k_FT4222_i2cMaster_Reset[] = "FT4222.i2cMaster_Reset";
static const char __pyx_k_FT4222_i2cMaster_Write[] = "FT4222.i2cMaster_Write";
static const char __pyx_k_FT4222_setLatencyTimer[] = "FT4222.setLatencyTimer";
static const char __pyx_k_FT4222_spiSlave_InitEx[] = "FT4222.spiSlave_InitEx";
static const char __pyx_k_INSUFFICIENT_RESOURCES[] = "INSUFFICIENT_RESOURCES";
static const char __pyx_k_IS_NOT_SPI_SINGLE_MODE[] = "IS_NOT_SPI_SINGLE_MODE";
static const char __pyx_k_closer___reduce_cython[] = "_closer.__reduce_cython__";
static const char __pyx_k_spi_SetDrivingStrength[] = "spi_SetDrivingStrength";
static const char __pyx_k_FAILED_TO_WRITE_DEVICE0[] = "FAILED_TO_WRITE_DEVICE0";
static const char __pyx_k_FT2XXDeviceError___init[] = "FT2XXDeviceError.__init__";
static const char __pyx_k_FT4222DeviceError___str[] = "FT4222DeviceError.__str__";
static const char __pyx_k_FT4222_i2cMaster_ReadEx[] = "FT4222.i2cMaster_ReadEx";
static const char __pyx_k_FT4222_spiSlave_SetMode[] = "FT4222.spiSlave_SetMode";
static const char __pyx_k_INTERRUPT_NOT_SUPPORTED[] = "INTERRUPT_NOT_SUPPORTED";
static const char __pyx_k_FT4222DeviceError___init[] = "FT4222DeviceError.__init__";
static const char __pyx_k_FT4222___setstate_cython[] = "FT4222.__setstate_cython__";
static const char __pyx_k_FT4222__chip_rev_min_lib[] = "_FT4222__chip_rev_min_lib";
static const char __pyx_k_FT4222_i2cMaster_WriteEx[] = "FT4222.i2cMaster_WriteEx";
static const char __pyx_k_GPIO_INPUT_NOT_SUPPORTED[] = "GPIO_INPUT_NOT_SUPPORTED";
static const char __pyx_k_GPIO_WRITE_NOT_SUPPORTED[] = "GPIO_WRITE_NOT_SUPPORTED";
static const char __pyx_k_VENDER_CMD_NOT_SUPPORTED[] = "VENDER_CMD_NOT_SUPPORTED";
static const char __pyx_k_closer___setstate_cython[] = "_closer.__setstate_cython__";
static const char __pyx_k_spiMaster_EndTransaction[] = "spiMaster_EndTransaction";
static const char __pyx_k_spiMaster_MultiReadWrite[] = "spiMaster_MultiReadWrite";
static const char __pyx_k_FT4222_getMaxTransferSize[] = "FT4222.getMaxTransferSize";
static const char __pyx_k_FT4222_setWakeUpInterrupt[] = "FT4222.setWakeUpInterrupt";
static const char __pyx_k_FT4222_spiMaster_SetLines[] = "FT4222.spiMaster_SetLines";
static const char __pyx_k_GPIO_EXCEEDED_MAX_PORTNUM[] = "GPIO_EXCEEDED_MAX_PORTNUM";
static const char __pyx_k_spiMaster_SingleReadWrite[] = "spiMaster_SingleReadWrite";
static const char __pyx_k_EXCEEDED_MAX_TRANSFER_SIZE[] = "EXCEEDED_MAX_TRANSFER_SIZE";
static const char __pyx_k_FT4222_i2cMaster_GetStatus[] = "FT4222.i2cMaster_GetStatus";
static const char __pyx_k_DEVICE_NOT_OPENED_FOR_ERASE[] = "DEVICE_NOT_OPENED_FOR_ERASE";
static const char __pyx_k_DEVICE_NOT_OPENED_FOR_WRITE[] = "DEVICE_NOT_OPENED_FOR_WRITE";
static const char __pyx_k_FT4222_gpio_SetInputTrigger[] = "FT4222.gpio_SetInputTrigger";
static const char __pyx_k_FT4222_spiMaster_SingleRead[] = "FT4222.spiMaster_SingleRead";
static const char __pyx_k_FT4222_spiSlave_GetRxStatus[] = "FT4222.spiSlave_GetRxStatus";
static const char __pyx_k_FT4222_spi_ResetTransaction[] = "FT4222.spi_ResetTransaction";
static const char __pyx_k_FT4222_gpio_GetTriggerStatus[] = "FT4222.gpio_GetTriggerStatus";
static const char __pyx_k_FT4222_gpio_ReadTriggerQueue[] = "FT4222.gpio_ReadTriggerQueue";
static const char __pyx_k_FT4222_spiMaster_SingleWrite[] = "FT4222.spiMaster_SingleWrite";
static const char __pyx_k_FT2XXDeviceError__ftd2xx_msgs[] = "_FT2XXDeviceError__ftd2xx_msgs";
static const char __pyx_k_FT4222_spi_SetDrivingStrength[] = "FT4222.spi_SetDrivingStrength";
static const char __pyx_k_I2C_NOT_SUPPORTED_IN_THIS_MODE[] = "I2C_NOT_SUPPORTED_IN_THIS_MODE";
static const char __pyx_k_FT4222DeviceError__ftd4222_msgs[] = "_FT4222DeviceError__ftd4222_msgs";
static const char __pyx_k_FT4222_spiMaster_EndTransaction[] = "FT4222.spiMaster_EndTransaction";
static const char __pyx_k_FT4222_spiMaster_MultiReadWrite[] = "FT4222.spiMaster_MultiReadWrite";
static const char __pyx_k_GPIO_NOT_SUPPORTED_IN_THIS_MODE[] = "GPIO_NOT_SUPPORTED_IN_THIS_MODE";
static const char __pyx_k_the_multiWrite_argument_must_be[] = "the multiWrite argument must be of type 'int', 'bytes' or 'bytearray'";
static const char __pyx_k_Chip_system_clock_Attributes_CLK[] = "Chip system clock\n\n    Attributes:\n        CLK_60: 60 MHz\n        CLK_24: 24 MHz\n        CLK_48: 48 MHz\n        CLK_80: 80 MHz\n\n    ";
static const char __pyx_k_Exception_class_for_status_messa[] = "Exception class for status messages";
static const char __pyx_k_FT4222_chipVersion_0x_x_s_libVer[] = "FT4222: chipVersion: 0x{:x} ({:s}), libVersion: 0x{:x}";
static const char __pyx_k_FT4222_spiMaster_SingleReadWrite[] = "FT4222.spiMaster_SingleReadWrite";
static const char __pyx_k_GPIO_OPENDRAIN_INVALID_IN_OUTPUT[] = "GPIO_OPENDRAIN_INVALID_IN_OUTPUTMODE";
static const char __pyx_k_GPIO_PULLDOWN_INVALID_IN_INPUTMO[] = "GPIO_PULLDOWN_INVALID_IN_INPUTMODE";
static const char __pyx_k_GPIO_PULLUP_INVALID_IN_INPUTMODE[] = "GPIO_PULLUP_INVALID_IN_INPUTMODE";
static const char __pyx_k_self__handle_cannot_be_converted[] = "self._handle cannot be converted to a Python object for pickling";
static const char __pyx_k_the_data_argument_must_be_of_typ[] = "the data argument must be of type 'int', 'bytes' or 'bytearray'";
static const char __pyx_k_the_singleWrite_argument_must_be[] = "the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'";
/* #### Code section: decls ### */
static int __pyx_pf_7cpython_5array_5array___getbuffer__(arrayobject *__pyx_v_self, Py_buffer *__pyx_v_info, CYTHON_UNUSED int __pyx_v_flags); /* proto */
static void __pyx_pf_7cpython_5array_5array_2__releasebuffer__(CYTHON_UNUSED arrayobject *__pyx_v_self, Py_buffer *__pyx_v_info); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_16FT2XXDeviceError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_msgnum); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_16FT2XXDeviceError_2__str__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_17FT4222DeviceError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_msgnum); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_17FT4222DeviceError_2__str__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_createDeviceInfoList(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_2getDeviceInfoDetail(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_devnum, PyObject *__pyx_v_update); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_4openBySerial(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_serial); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6openByDescription(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_desc); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_8openByLocation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_locId); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_7_closer_close(struct __pyx_obj_6ft4222_6ft4222__closer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_7_closer_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6ft4222_6ft4222__closer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_7_closer_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6ft4222_6ft4222__closer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ft4222_6ft4222_6FT4222___init__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_handle, CYTHON_UNUSED PyObject *__pyx_v_update); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_2close(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_11chipVersion___get__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_10libVersion___get__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_12chipRevision___get__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_4__repr__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_6setTimeouts(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_read_timeout, PyObject *__pyx_v_write_timeout); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_8setClock(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_clk); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_10getClock(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_12setSuspendOut(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_enable); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_14setWakeUpInterrupt(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_enable); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_16getMaxTransferSize(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_18vendorCmdGet(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_req, PyObject *__pyx_v_bytesToRead); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_20vendorCmdSet(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_req, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_22getLatencyTimer(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_24setLatencyTimer(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_latency); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_26chipReset(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_28gpio_Init(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_gpio0, PyObject *__pyx_v_gpio1, PyObject *__pyx_v_gpio2, PyObject *__pyx_v_gpio3, PyObject *__pyx_v_args); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_30gpio_Read(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_32gpio_Wait(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum, PyObject *__pyx_v_expectedValue, PyObject *__pyx_v_timeout, PyObject *__pyx_v_sleep); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_34gpio_Write(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_36gpio_SetInputTrigger(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum, PyObject *__pyx_v_trigger); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_38gpio_GetTriggerStatus(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_40gpio_ReadTriggerQueue(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum, PyObject *__pyx_v_readSize); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_42i2cMaster_Init(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_kbps); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_44i2cMaster_Read(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_addr, PyObject *__pyx_v_bytesToRead); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_46i2cMaster_Write(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_addr, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_48i2cMaster_ReadEx(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_addr, PyObject *__pyx_v_flag, PyObject *__pyx_v_bytesToRead); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_50i2cMaster_WriteEx(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_addr, PyObject *__pyx_v_flag, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_52i2cMaster_Reset(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_54i2cMaster_GetStatus(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_56spi_Reset(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_58spi_ResetTransaction(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_spiIdx); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_60spi_SetDrivingStrength(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_clkStrength, PyObject *__pyx_v_ioStrength, PyObject *__pyx_v_ssoStrength); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_62spiMaster_Init(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_mode, PyObject *__pyx_v_clock, PyObject *__pyx_v_cpol, PyObject *__pyx_v_cpha, PyObject *__pyx_v_ssoMap); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_64spiMaster_SetLines(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_mode); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_66spiMaster_SingleRead(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_bytesToRead, PyObject *__pyx_v_isEndTransaction); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_68spiMaster_SingleWrite(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_isEndTransaction); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_70spiMaster_SingleReadWrite(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_isEndTransaction); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_72spiMaster_MultiReadWrite(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_singleWrite, PyObject *__pyx_v_multiWrite, PyObject *__pyx_v_bytesToRead); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_74spiMaster_EndTransaction(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_76spiSlave_Init(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_78spiSlave_InitEx(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_proto); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_80spiSlave_Read(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_bytesToRead); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_82spiSlave_SetMode(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_cpol, PyObject *__pyx_v_cpha); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_84spiSlave_GetRxStatus(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_86spiSlave_Write(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_88__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_90__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_6ft4222_6ft4222__closer(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ft4222_6ft4222_FT4222(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
/* #### Code section: late_includes ### */
/* #### Code section: module_state ### */
typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  #ifdef __Pyx_CyFunction_USED
  PyTypeObject *__pyx_CyFunctionType;
  #endif
  #ifdef __Pyx_FusedFunction_USED
  PyTypeObject *__pyx_FusedFunctionType;
  #endif
  #ifdef __Pyx_Generator_USED
  PyTypeObject *__pyx_GeneratorType;
  #endif
  #ifdef __Pyx_IterableCoroutine_USED
  PyTypeObject *__pyx_IterableCoroutineType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineAwaitType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineType;
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyTypeObject *__pyx_ptype_7cpython_4type_type;
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyTypeObject *__pyx_ptype_7cpython_5array_array;
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  PyObject *__pyx_type_6ft4222_6ft4222__closer;
  PyObject *__pyx_type_6ft4222_6ft4222_FT4222;
  #endif
  PyTypeObject *__pyx_ptype_6ft4222_6ft4222__closer;
  PyTypeObject *__pyx_ptype_6ft4222_6ft4222_FT4222;
  PyObject *__pyx_n_u_B;
  PyObject *__pyx_n_s_CLK_24;
  PyObject *__pyx_n_s_CLK_48;
  PyObject *__pyx_n_s_CLK_60;
  PyObject *__pyx_n_s_CLK_80;
  PyObject *__pyx_n_u_CLK_NOT_SUPPORTED;
  PyObject *__pyx_kp_s_Chip_system_clock_Attributes_CLK;
  PyObject *__pyx_n_s_ControllerStatus;
  PyObject *__pyx_n_u_DEVICE_LIST_NOT_READY;
  PyObject *__pyx_n_u_DEVICE_NOT_FOUND;
  PyObject *__pyx_n_u_DEVICE_NOT_OPENED;
  PyObject *__pyx_n_u_DEVICE_NOT_OPENED_FOR_ERASE;
  PyObject *__pyx_n_u_DEVICE_NOT_OPENED_FOR_WRITE;
  PyObject *__pyx_n_u_DEVICE_NOT_SUPPORTED;
  PyObject *__pyx_n_s_Dir;
  PyObject *__pyx_n_u_EEPROM_ERASE_FAILED;
  PyObject *__pyx_n_u_EEPROM_NOT_PRESENT;
  PyObject *__pyx_n_u_EEPROM_NOT_PROGRAMMED;
  PyObject *__pyx_n_u_EEPROM_READ_FAILED;
  PyObject *__pyx_n_u_EEPROM_WRITE_FAILED;
  PyObject *__pyx_n_u_EVENT_NOT_SUPPORTED;
  PyObject *__pyx_n_u_EXCEEDED_MAX_TRANSFER_SIZE;
  PyObject *__pyx_kp_s_Exception_class_for_status_messa;
  PyObject *__pyx_n_u_FAILED_TO_READ_DEVICE;
  PyObject *__pyx_n_u_FAILED_TO_WRITE_DEVICE0;
  PyObject *__pyx_n_s_FT2XXDeviceError;
  PyObject *__pyx_n_s_FT2XXDeviceError___init;
  PyObject *__pyx_n_s_FT2XXDeviceError___str;
  PyObject *__pyx_n_s_FT2XXDeviceError__ftd2xx_msgs;
  PyObject *__pyx_n_s_FT4222;
  PyObject *__pyx_n_s_FT4222DeviceError;
  PyObject *__pyx_n_s_FT4222DeviceError___init;
  PyObject *__pyx_n_s_FT4222DeviceError___str;
  PyObject *__pyx_n_s_FT4222DeviceError__ftd4222_msgs;
  PyObject *__pyx_n_s_FT4222___reduce_cython;
  PyObject *__pyx_n_s_FT4222___setstate_cython;
  PyObject *__pyx_n_s_FT4222__chip_rev_map;
  PyObject *__pyx_n_s_FT4222__chip_rev_min_lib;
  PyObject *__pyx_n_s_FT4222_chipReset;
  PyObject *__pyx_kp_u_FT4222_chipVersion_0x_x_s_libVer;
  PyObject *__pyx_n_s_FT4222_close;
  PyObject *__pyx_n_s_FT4222_getClock;
  PyObject *__pyx_n_s_FT4222_getLatencyTimer;
  PyObject *__pyx_n_s_FT4222_getMaxTransferSize;
  PyObject *__pyx_n_s_FT4222_gpio_GetTriggerStatus;
  PyObject *__pyx_n_s_FT4222_gpio_Init;
  PyObject *__pyx_n_s_FT4222_gpio_Read;
  PyObject *__pyx_n_s_FT4222_gpio_ReadTriggerQueue;
  PyObject *__pyx_n_s_FT4222_gpio_SetInputTrigger;
  PyObject *__pyx_n_s_FT4222_gpio_Wait;
  PyObject *__pyx_n_s_FT4222_gpio_Write;
  PyObject *__pyx_n_s_FT4222_i2cMaster_GetStatus;
  PyObject *__pyx_n_s_FT4222_i2cMaster_Init;
  PyObject *__pyx_n_s_FT4222_i2cMaster_Read;
  PyObject *__pyx_n_s_FT4222_i2cMaster_ReadEx;
  PyObject *__pyx_n_s_FT4222_i2cMaster_Reset;
  PyObject *__pyx_n_s_FT4222_i2cMaster_Write;
  PyObject *__pyx_n_s_FT4222_i2cMaster_WriteEx;
  PyObject *__pyx_n_s_FT4222_setClock;
  PyObject *__pyx_n_s_FT4222_setLatencyTimer;
  PyObject *__pyx_n_s_FT4222_setSuspendOut;
  PyObject *__pyx_n_s_FT4222_setTimeouts;
  PyObject *__pyx_n_s_FT4222_setWakeUpInterrupt;
  PyObject *__pyx_n_s_FT4222_spiMaster_EndTransaction;
  PyObject *__pyx_n_s_FT4222_spiMaster_Init;
  PyObject *__pyx_n_s_FT4222_spiMaster_MultiReadWrite;
  PyObject *__pyx_n_s_FT4222_spiMaster_SetLines;
  PyObject *__pyx_n_s_FT4222_spiMaster_SingleRead;
  PyObject *__pyx_n_s_FT4222_spiMaster_SingleReadWrite;
  PyObject *__pyx_n_s_FT4222_spiMaster_SingleWrite;
  PyObject *__pyx_n_s_FT4222_spiSlave_GetRxStatus;
  PyObject *__pyx_n_s_FT4222_spiSlave_Init;
  PyObject *__pyx_n_s_FT4222_spiSlave_InitEx;
  PyObject *__pyx_n_s_FT4222_spiSlave_Read;
  PyObject *__pyx_n_s_FT4222_spiSlave_SetMode;
  PyObject *__pyx_n_s_FT4222_spiSlave_Write;
  PyObject *__pyx_n_s_FT4222_spi_Reset;
  PyObject *__pyx_n_s_FT4222_spi_ResetTransaction;
  PyObject *__pyx_n_s_FT4222_spi_SetDrivingStrength;
  PyObject *__pyx_n_s_FT4222_vendorCmdGet;
  PyObject *__pyx_n_s_FT4222_vendorCmdSet;
  PyObject *__pyx_n_u_FUN_NOT_SUPPORT;
  PyObject *__pyx_n_s_GPIO;
  PyObject *__pyx_n_u_GPIO_EXCEEDED_MAX_PORTNUM;
  PyObject *__pyx_n_u_GPIO_INPUT_NOT_SUPPORTED;
  PyObject *__pyx_n_u_GPIO_NOT_SUPPORTED_IN_THIS_MODE;
  PyObject *__pyx_n_u_GPIO_OPENDRAIN_INVALID_IN_OUTPUT;
  PyObject *__pyx_n_u_GPIO_PULLDOWN_INVALID_IN_INPUTMO;
  PyObject *__pyx_n_u_GPIO_PULLUP_INVALID_IN_INPUTMODE;
  PyObject *__pyx_n_u_GPIO_WRITE_NOT_SUPPORTED;
  PyObject *__pyx_n_s_I2CMaster;
  PyObject *__pyx_n_u_I2C_NOT_SUPPORTED_IN_THIS_MODE;
  PyObject *__pyx_n_s_INPUT;
  PyObject *__pyx_n_u_INSUFFICIENT_RESOURCES;
  PyObject *__pyx_n_u_INTERRUPT_NOT_SUPPORTED;
  PyObject *__pyx_n_u_INVAILD_FUNCTION;
  PyObject *__pyx_n_u_INVALID_ARGS;
  PyObject *__pyx_n_u_INVALID_BAUD_RATE;
  PyObject *__pyx_n_u_INVALID_HANDLE;
  PyObject *__pyx_n_u_INVALID_PARAMETER;
  PyObject *__pyx_n_u_INVALID_POINTER;
  PyObject *__pyx_n_u_IO_ERROR;
  PyObject *__pyx_n_u_IS_NOT_I2C_MODE;
  PyObject *__pyx_n_u_IS_NOT_SPI_MODE;
  PyObject *__pyx_n_u_IS_NOT_SPI_MULTI_MODE;
  PyObject *__pyx_n_u_IS_NOT_SPI_SINGLE_MODE;
  PyObject *__pyx_n_s_IntEnum;
  PyObject *__pyx_n_s_KeyError;
  PyObject *__pyx_n_s_MemoryError;
  PyObject *__pyx_n_u_NOT_SUPPORTED;
  PyObject *__pyx_n_u_OK;
  PyObject *__pyx_n_u_OTHER_ERROR;
  PyObject *__pyx_kp_u_Rev_A;
  PyObject *__pyx_kp_u_Rev_B;
  PyObject *__pyx_kp_u_Rev_C;
  PyObject *__pyx_kp_u_Rev_D;
  PyObject *__pyx_kp_u_Rev_unknown;
  PyObject *__pyx_n_s_SysClock;
  PyObject *__pyx_n_s_TimeoutError;
  PyObject *__pyx_n_s_Trigger;
  PyObject *__pyx_n_s_TypeError;
  PyObject *__pyx_n_u_VENDER_CMD_NOT_SUPPORTED;
  PyObject *__pyx_n_u_WRONG_I2C_ADDR;
  PyObject *__pyx_n_s__111;
  PyObject *__pyx_kp_u__8;
  PyObject *__pyx_n_s_addr;
  PyObject *__pyx_n_s_args;
  PyObject *__pyx_n_s_asyncio_coroutines;
  PyObject *__pyx_n_s_buf;
  PyObject *__pyx_n_s_bytesRead;
  PyObject *__pyx_n_s_bytesSent;
  PyObject *__pyx_n_s_bytesToRead;
  PyObject *__pyx_n_s_cdata;
  PyObject *__pyx_n_s_cdesc;
  PyObject *__pyx_n_s_chipReset;
  PyObject *__pyx_n_s_chipRevision;
  PyObject *__pyx_n_s_class_getitem;
  PyObject *__pyx_n_s_cline_in_traceback;
  PyObject *__pyx_n_s_clk;
  PyObject *__pyx_n_s_clkStrength;
  PyObject *__pyx_n_s_clock;
  PyObject *__pyx_n_s_close;
  PyObject *__pyx_n_s_closer;
  PyObject *__pyx_n_s_closer___reduce_cython;
  PyObject *__pyx_n_s_closer___setstate_cython;
  PyObject *__pyx_n_s_closer_close;
  PyObject *__pyx_n_s_cpha;
  PyObject *__pyx_n_s_cpol;
  PyObject *__pyx_n_s_createDeviceInfoList;
  PyObject *__pyx_n_s_cs;
  PyObject *__pyx_n_s_cserial;
  PyObject *__pyx_n_s_d;
  PyObject *__pyx_n_s_data;
  PyObject *__pyx_n_s_desc;
  PyObject *__pyx_n_u_description;
  PyObject *__pyx_n_s_devnum;
  PyObject *__pyx_n_s_dict;
  PyObject *__pyx_kp_u_disable;
  PyObject *__pyx_n_s_doc;
  PyObject *__pyx_n_s_enable;
  PyObject *__pyx_kp_u_enable;
  PyObject *__pyx_n_s_encode;
  PyObject *__pyx_n_s_enum;
  PyObject *__pyx_n_s_events;
  PyObject *__pyx_n_s_expectedValue;
  PyObject *__pyx_n_s_expectedValue_c;
  PyObject *__pyx_n_s_f;
  PyObject *__pyx_n_s_finalize;
  PyObject *__pyx_n_s_flag;
  PyObject *__pyx_n_u_flags;
  PyObject *__pyx_n_s_format;
  PyObject *__pyx_n_s_ft4222_ft4222;
  PyObject *__pyx_kp_s_ft4222_ft4222_pyx;
  PyObject *__pyx_kp_u_gc;
  PyObject *__pyx_n_s_getClock;
  PyObject *__pyx_n_s_getDeviceInfoDetail;
  PyObject *__pyx_n_s_getLatencyTimer;
  PyObject *__pyx_n_s_getMaxTransferSize;
  PyObject *__pyx_n_s_getstate;
  PyObject *__pyx_n_s_gpio0;
  PyObject *__pyx_n_s_gpio1;
  PyObject *__pyx_n_s_gpio2;
  PyObject *__pyx_n_s_gpio3;
  PyObject *__pyx_n_s_gpio_GetTriggerStatus;
  PyObject *__pyx_n_s_gpio_Init;
  PyObject *__pyx_n_s_gpio_Read;
  PyObject *__pyx_n_s_gpio_ReadTriggerQueue;
  PyObject *__pyx_n_s_gpio_SetInputTrigger;
  PyObject *__pyx_n_s_gpio_Wait;
  PyObject *__pyx_n_s_gpio_Write;
  PyObject *__pyx_n_s_h;
  PyObject *__pyx_n_s_handle;
  PyObject *__pyx_n_u_handle;
  PyObject *__pyx_n_s_i;
  PyObject *__pyx_n_s_i2cMaster_GetStatus;
  PyObject *__pyx_n_s_i2cMaster_Init;
  PyObject *__pyx_n_s_i2cMaster_Read;
  PyObject *__pyx_n_s_i2cMaster_ReadEx;
  PyObject *__pyx_n_s_i2cMaster_Reset;
  PyObject *__pyx_n_s_i2cMaster_Write;
  PyObject *__pyx_n_s_i2cMaster_WriteEx;
  PyObject *__pyx_n_u_id;
  PyObject *__pyx_n_s_import;
  PyObject *__pyx_n_u_index;
  PyObject *__pyx_n_s_init;
  PyObject *__pyx_n_s_init_subclass;
  PyObject *__pyx_n_s_ioDir;
  PyObject *__pyx_n_s_ioStrength;
  PyObject *__pyx_n_s_isEndTransaction;
  PyObject *__pyx_n_s_is_coroutine;
  PyObject *__pyx_kp_u_isenabled;
  PyObject *__pyx_n_s_kbps;
  PyObject *__pyx_n_s_l;
  PyObject *__pyx_n_s_latency;
  PyObject *__pyx_n_s_locId;
  PyObject *__pyx_n_u_location;
  PyObject *__pyx_n_s_main;
  PyObject *__pyx_n_s_maxTransferSize;
  PyObject *__pyx_n_s_message;
  PyObject *__pyx_n_s_metaclass;
  PyObject *__pyx_n_s_mode;
  PyObject *__pyx_n_s_module;
  PyObject *__pyx_n_s_mro_entries;
  PyObject *__pyx_n_s_msgnum;
  PyObject *__pyx_n_s_multiWrite;
  PyObject *__pyx_n_s_n;
  PyObject *__pyx_n_s_name;
  PyObject *__pyx_n_s_nb;
  PyObject *__pyx_n_s_openByDescription;
  PyObject *__pyx_n_s_openByLocation;
  PyObject *__pyx_n_s_openBySerial;
  PyObject *__pyx_n_s_pRxSize;
  PyObject *__pyx_n_s_portNum;
  PyObject *__pyx_n_s_portNum_c;
  PyObject *__pyx_n_s_prepare;
  PyObject *__pyx_n_s_proto;
  PyObject *__pyx_n_s_pyx_state;
  PyObject *__pyx_n_s_pyx_vtable;
  PyObject *__pyx_n_s_qualname;
  PyObject *__pyx_n_s_queueSize;
  PyObject *__pyx_n_s_range;
  PyObject *__pyx_n_s_readSize;
  PyObject *__pyx_n_s_read_timeout;
  PyObject *__pyx_n_s_reduce;
  PyObject *__pyx_n_s_reduce_cython;
  PyObject *__pyx_n_s_reduce_ex;
  PyObject *__pyx_n_s_req;
  PyObject *__pyx_n_s_res;
  PyObject *__pyx_n_s_round;
  PyObject *__pyx_n_s_self;
  PyObject *__pyx_kp_s_self__handle_cannot_be_converted;
  PyObject *__pyx_n_s_serial;
  PyObject *__pyx_n_u_serial;
  PyObject *__pyx_n_s_setClock;
  PyObject *__pyx_n_s_setLatencyTimer;
  PyObject *__pyx_n_s_setSuspendOut;
  PyObject *__pyx_n_s_setTimeouts;
  PyObject *__pyx_n_s_setWakeUpInterrupt;
  PyObject *__pyx_n_s_set_name;
  PyObject *__pyx_n_s_setstate;
  PyObject *__pyx_n_s_setstate_cython;
  PyObject *__pyx_n_s_singleWrite;
  PyObject *__pyx_n_s_sizeRead;
  PyObject *__pyx_n_s_sizeTransferred;
  PyObject *__pyx_n_s_sleep;
  PyObject *__pyx_n_s_sleep_c;
  PyObject *__pyx_n_s_spiIdx;
  PyObject *__pyx_n_s_spiMaster_EndTransaction;
  PyObject *__pyx_n_s_spiMaster_Init;
  PyObject *__pyx_n_s_spiMaster_MultiReadWrite;
  PyObject *__pyx_n_s_spiMaster_SetLines;
  PyObject *__pyx_n_s_spiMaster_SingleRead;
  PyObject *__pyx_n_s_spiMaster_SingleReadWrite;
  PyObject *__pyx_n_s_spiMaster_SingleWrite;
  PyObject *__pyx_n_s_spiSlave_GetRxStatus;
  PyObject *__pyx_n_s_spiSlave_Init;
  PyObject *__pyx_n_s_spiSlave_InitEx;
  PyObject *__pyx_n_s_spiSlave_Read;
  PyObject *__pyx_n_s_spiSlave_SetMode;
  PyObject *__pyx_n_s_spiSlave_Write;
  PyObject *__pyx_n_s_spi_Reset;
  PyObject *__pyx_n_s_spi_ResetTransaction;
  PyObject *__pyx_n_s_spi_SetDrivingStrength;
  PyObject *__pyx_n_s_ssoMap;
  PyObject *__pyx_n_s_ssoStrength;
  PyObject *__pyx_n_s_start;
  PyObject *__pyx_n_s_status;
  PyObject *__pyx_n_s_str;
  PyObject *__pyx_kp_s_stringsource;
  PyObject *__pyx_n_s_super;
  PyObject *__pyx_n_s_t;
  PyObject *__pyx_n_s_test;
  PyObject *__pyx_kp_u_the_data_argument_must_be_of_typ;
  PyObject *__pyx_kp_u_the_multiWrite_argument_must_be;
  PyObject *__pyx_kp_u_the_singleWrite_argument_must_be;
  PyObject *__pyx_n_s_timeout;
  PyObject *__pyx_n_s_timeout_c;
  PyObject *__pyx_n_s_trigger;
  PyObject *__pyx_n_u_type;
  PyObject *__pyx_n_s_update;
  PyObject *__pyx_kp_u_utf_8;
  PyObject *__pyx_n_s_value;
  PyObject *__pyx_n_s_vendorCmdGet;
  PyObject *__pyx_n_s_vendorCmdSet;
  PyObject *__pyx_n_s_weakref;
  PyObject *__pyx_n_s_weakref_finalize;
  PyObject *__pyx_n_s_write;
  PyObject *__pyx_n_s_write_timeout;
  PyObject *__pyx_n_s_xrange;
  PyObject *__pyx_float_24000000_0;
  PyObject *__pyx_int_0;
  PyObject *__pyx_int_1;
  PyObject *__pyx_int_2;
  PyObject *__pyx_int_3;
  PyObject *__pyx_int_60;
  PyObject *__pyx_int_82;
  PyObject *__pyx_int_100;
  PyObject *__pyx_int_8000;
  PyObject *__pyx_int_16908288;
  PyObject *__pyx_int_16973824;
  PyObject *__pyx_int_17039360;
  PyObject *__pyx_int_1109524736;
  PyObject *__pyx_int_1109524992;
  PyObject *__pyx_int_1109525248;
  PyObject *__pyx_int_1109525504;
  PyObject *__pyx_k__2;
  PyObject *__pyx_k__3;
  PyObject *__pyx_k__4;
  PyObject *__pyx_k__5;
  PyObject *__pyx_tuple_;
  PyObject *__pyx_tuple__6;
  PyObject *__pyx_tuple__7;
  PyObject *__pyx_tuple__9;
  PyObject *__pyx_tuple__11;
  PyObject *__pyx_tuple__15;
  PyObject *__pyx_tuple__17;
  PyObject *__pyx_tuple__19;
  PyObject *__pyx_tuple__20;
  PyObject *__pyx_tuple__22;
  PyObject *__pyx_tuple__24;
  PyObject *__pyx_tuple__28;
  PyObject *__pyx_tuple__30;
  PyObject *__pyx_tuple__32;
  PyObject *__pyx_tuple__34;
  PyObject *__pyx_tuple__37;
  PyObject *__pyx_tuple__40;
  PyObject *__pyx_tuple__42;
  PyObject *__pyx_tuple__44;
  PyObject *__pyx_tuple__46;
  PyObject *__pyx_tuple__50;
  PyObject *__pyx_tuple__52;
  PyObject *__pyx_tuple__54;
  PyObject *__pyx_tuple__56;
  PyObject *__pyx_tuple__58;
  PyObject *__pyx_tuple__60;
  PyObject *__pyx_tuple__62;
  PyObject *__pyx_tuple__64;
  PyObject *__pyx_tuple__65;
  PyObject *__pyx_tuple__67;
  PyObject *__pyx_tuple__68;
  PyObject *__pyx_tuple__70;
  PyObject *__pyx_tuple__72;
  PyObject *__pyx_tuple__74;
  PyObject *__pyx_tuple__77;
  PyObject *__pyx_tuple__80;
  PyObject *__pyx_tuple__82;
  PyObject *__pyx_tuple__84;
  PyObject *__pyx_tuple__86;
  PyObject *__pyx_tuple__88;
  PyObject *__pyx_tuple__90;
  PyObject *__pyx_tuple__92;
  PyObject *__pyx_tuple__94;
  PyObject *__pyx_tuple__96;
  PyObject *__pyx_tuple__99;
  PyObject *__pyx_tuple__101;
  PyObject *__pyx_tuple__103;
  PyObject *__pyx_tuple__105;
  PyObject *__pyx_tuple__107;
  PyObject *__pyx_codeobj__10;
  PyObject *__pyx_codeobj__12;
  PyObject *__pyx_codeobj__13;
  PyObject *__pyx_codeobj__14;
  PyObject *__pyx_codeobj__16;
  PyObject *__pyx_codeobj__18;
  PyObject *__pyx_codeobj__21;
  PyObject *__pyx_codeobj__23;
  PyObject *__pyx_codeobj__25;
  PyObject *__pyx_codeobj__26;
  PyObject *__pyx_codeobj__27;
  PyObject *__pyx_codeobj__29;
  PyObject *__pyx_codeobj__31;
  PyObject *__pyx_codeobj__33;
  PyObject *__pyx_codeobj__35;
  PyObject *__pyx_codeobj__36;
  PyObject *__pyx_codeobj__38;
  PyObject *__pyx_codeobj__39;
  PyObject *__pyx_codeobj__41;
  PyObject *__pyx_codeobj__43;
  PyObject *__pyx_codeobj__45;
  PyObject *__pyx_codeobj__47;
  PyObject *__pyx_codeobj__48;
  PyObject *__pyx_codeobj__49;
  PyObject *__pyx_codeobj__51;
  PyObject *__pyx_codeobj__53;
  PyObject *__pyx_codeobj__55;
  PyObject *__pyx_codeobj__57;
  PyObject *__pyx_codeobj__59;
  PyObject *__pyx_codeobj__61;
  PyObject *__pyx_codeobj__63;
  PyObject *__pyx_codeobj__66;
  PyObject *__pyx_codeobj__69;
  PyObject *__pyx_codeobj__71;
  PyObject *__pyx_codeobj__73;
  PyObject *__pyx_codeobj__75;
  PyObject *__pyx_codeobj__76;
  PyObject *__pyx_codeobj__78;
  PyObject *__pyx_codeobj__79;
  PyObject *__pyx_codeobj__81;
  PyObject *__pyx_codeobj__83;
  PyObject *__pyx_codeobj__85;
  PyObject *__pyx_codeobj__87;
  PyObject *__pyx_codeobj__89;
  PyObject *__pyx_codeobj__91;
  PyObject *__pyx_codeobj__93;
  PyObject *__pyx_codeobj__95;
  PyObject *__pyx_codeobj__97;
  PyObject *__pyx_codeobj__98;
  PyObject *__pyx_codeobj__100;
  PyObject *__pyx_codeobj__102;
  PyObject *__pyx_codeobj__104;
  PyObject *__pyx_codeobj__106;
  PyObject *__pyx_codeobj__108;
  PyObject *__pyx_codeobj__109;
  PyObject *__pyx_codeobj__110;
} __pyx_mstate;

#if CYTHON_USE_MODULE_STATE
#ifdef __cplusplus
namespace {
  extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate(o) ((__pyx_mstate *)__Pyx_PyModule_GetState(o))

#define __pyx_mstate_global (__pyx_mstate(PyState_FindModule(&__pyx_moduledef)))

#define __pyx_m (PyState_FindModule(&__pyx_moduledef))
#else
static __pyx_mstate __pyx_mstate_global_static =
#ifdef __cplusplus
    {};
#else
    {0};
#endif
static __pyx_mstate *__pyx_mstate_global = &__pyx_mstate_global_static;
#endif
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static int __pyx_m_clear(PyObject *m) {
  __pyx_mstate *clear_module_state = __pyx_mstate(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_CLEAR(clear_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_CLEAR(clear_module_state->__pyx_FusedFunctionType);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_4type_type);
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_5array_array);
  Py_CLEAR(clear_module_state->__pyx_ptype_6ft4222_6ft4222__closer);
  Py_CLEAR(clear_module_state->__pyx_type_6ft4222_6ft4222__closer);
  Py_CLEAR(clear_module_state->__pyx_ptype_6ft4222_6ft4222_FT4222);
  Py_CLEAR(clear_module_state->__pyx_type_6ft4222_6ft4222_FT4222);
  Py_CLEAR(clear_module_state->__pyx_n_u_B);
  Py_CLEAR(clear_module_state->__pyx_n_s_CLK_24);
  Py_CLEAR(clear_module_state->__pyx_n_s_CLK_48);
  Py_CLEAR(clear_module_state->__pyx_n_s_CLK_60);
  Py_CLEAR(clear_module_state->__pyx_n_s_CLK_80);
  Py_CLEAR(clear_module_state->__pyx_n_u_CLK_NOT_SUPPORTED);
  Py_CLEAR(clear_module_state->__pyx_kp_s_Chip_system_clock_Attributes_CLK);
  Py_CLEAR(clear_module_state->__pyx_n_s_ControllerStatus);
  Py_CLEAR(clear_module_state->__pyx_n_u_DEVICE_LIST_NOT_READY);
  Py_CLEAR(clear_module_state->__pyx_n_u_DEVICE_NOT_FOUND);
  Py_CLEAR(clear_module_state->__pyx_n_u_DEVICE_NOT_OPENED);
  Py_CLEAR(clear_module_state->__pyx_n_u_DEVICE_NOT_OPENED_FOR_ERASE);
  Py_CLEAR(clear_module_state->__pyx_n_u_DEVICE_NOT_OPENED_FOR_WRITE);
  Py_CLEAR(clear_module_state->__pyx_n_u_DEVICE_NOT_SUPPORTED);
  Py_CLEAR(clear_module_state->__pyx_n_s_Dir);
  Py_CLEAR(clear_module_state->__pyx_n_u_EEPROM_ERASE_FAILED);
  Py_CLEAR(clear_module_state->__pyx_n_u_EEPROM_NOT_PRESENT);
  Py_CLEAR(clear_module_state->__pyx_n_u_EEPROM_NOT_PROGRAMMED);
  Py_CLEAR(clear_module_state->__pyx_n_u_EEPROM_READ_FAILED);
  Py_CLEAR(clear_module_state->__pyx_n_u_EEPROM_WRITE_FAILED);
  Py_CLEAR(clear_module_state->__pyx_n_u_EVENT_NOT_SUPPORTED);
  Py_CLEAR(clear_module_state->__pyx_n_u_EXCEEDED_MAX_TRANSFER_SIZE);
  Py_CLEAR(clear_module_state->__pyx_kp_s_Exception_class_for_status_messa);
  Py_CLEAR(clear_module_state->__pyx_n_u_FAILED_TO_READ_DEVICE);
  Py_CLEAR(clear_module_state->__pyx_n_u_FAILED_TO_WRITE_DEVICE0);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT2XXDeviceError);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT2XXDeviceError___init);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT2XXDeviceError___str);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT2XXDeviceError__ftd2xx_msgs);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222DeviceError);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222DeviceError___init);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222DeviceError___str);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222DeviceError__ftd4222_msgs);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222___reduce_cython);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222___setstate_cython);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222__chip_rev_map);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222__chip_rev_min_lib);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_chipReset);
  Py_CLEAR(clear_module_state->__pyx_kp_u_FT4222_chipVersion_0x_x_s_libVer);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_close);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_getClock);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_getLatencyTimer);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_getMaxTransferSize);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_gpio_GetTriggerStatus);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_gpio_Init);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_gpio_Read);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_gpio_ReadTriggerQueue);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_gpio_SetInputTrigger);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_gpio_Wait);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_gpio_Write);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_i2cMaster_GetStatus);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_i2cMaster_Init);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_i2cMaster_Read);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_i2cMaster_ReadEx);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_i2cMaster_Reset);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_i2cMaster_Write);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_i2cMaster_WriteEx);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_setClock);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_setLatencyTimer);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_setSuspendOut);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_setTimeouts);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_setWakeUpInterrupt);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_spiMaster_EndTransaction);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_spiMaster_Init);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_spiMaster_MultiReadWrite);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_spiMaster_SetLines);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_spiMaster_SingleRead);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_spiMaster_SingleReadWrite);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_spiMaster_SingleWrite);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_spiSlave_GetRxStatus);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_spiSlave_Init);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_spiSlave_InitEx);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_spiSlave_Read);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_spiSlave_SetMode);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_spiSlave_Write);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_spi_Reset);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_spi_ResetTransaction);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_spi_SetDrivingStrength);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_vendorCmdGet);
  Py_CLEAR(clear_module_state->__pyx_n_s_FT4222_vendorCmdSet);
  Py_CLEAR(clear_module_state->__pyx_n_u_FUN_NOT_SUPPORT);
  Py_CLEAR(clear_module_state->__pyx_n_s_GPIO);
  Py_CLEAR(clear_module_state->__pyx_n_u_GPIO_EXCEEDED_MAX_PORTNUM);
  Py_CLEAR(clear_module_state->__pyx_n_u_GPIO_INPUT_NOT_SUPPORTED);
  Py_CLEAR(clear_module_state->__pyx_n_u_GPIO_NOT_SUPPORTED_IN_THIS_MODE);
  Py_CLEAR(clear_module_state->__pyx_n_u_GPIO_OPENDRAIN_INVALID_IN_OUTPUT);
  Py_CLEAR(clear_module_state->__pyx_n_u_GPIO_PULLDOWN_INVALID_IN_INPUTMO);
  Py_CLEAR(clear_module_state->__pyx_n_u_GPIO_PULLUP_INVALID_IN_INPUTMODE);
  Py_CLEAR(clear_module_state->__pyx_n_u_GPIO_WRITE_NOT_SUPPORTED);
  Py_CLEAR(clear_module_state->__pyx_n_s_I2CMaster);
  Py_CLEAR(clear_module_state->__pyx_n_u_I2C_NOT_SUPPORTED_IN_THIS_MODE);
  Py_CLEAR(clear_module_state->__pyx_n_s_INPUT);
  Py_CLEAR(clear_module_state->__pyx_n_u_INSUFFICIENT_RESOURCES);
  Py_CLEAR(clear_module_state->__pyx_n_u_INTERRUPT_NOT_SUPPORTED);
  Py_CLEAR(clear_module_state->__pyx_n_u_INVAILD_FUNCTION);
  Py_CLEAR(clear_module_state->__pyx_n_u_INVALID_ARGS);
  Py_CLEAR(clear_module_state->__pyx_n_u_INVALID_BAUD_RATE);
  Py_CLEAR(clear_module_state->__pyx_n_u_INVALID_HANDLE);
  Py_CLEAR(clear_module_state->__pyx_n_u_INVALID_PARAMETER);
  Py_CLEAR(clear_module_state->__pyx_n_u_INVALID_POINTER);
  Py_CLEAR(clear_module_state->__pyx_n_u_IO_ERROR);
  Py_CLEAR(clear_module_state->__pyx_n_u_IS_NOT_I2C_MODE);
  Py_CLEAR(clear_module_state->__pyx_n_u_IS_NOT_SPI_MODE);
  Py_CLEAR(clear_module_state->__pyx_n_u_IS_NOT_SPI_MULTI_MODE);
  Py_CLEAR(clear_module_state->__pyx_n_u_IS_NOT_SPI_SINGLE_MODE);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntEnum);
  Py_CLEAR(clear_module_state->__pyx_n_s_KeyError);
  Py_CLEAR(clear_module_state->__pyx_n_s_MemoryError);
  Py_CLEAR(clear_module_state->__pyx_n_u_NOT_SUPPORTED);
  Py_CLEAR(clear_module_state->__pyx_n_u_OK);
  Py_CLEAR(clear_module_state->__pyx_n_u_OTHER_ERROR);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Rev_A);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Rev_B);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Rev_C);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Rev_D);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Rev_unknown);
  Py_CLEAR(clear_module_state->__pyx_n_s_SysClock);
  Py_CLEAR(clear_module_state->__pyx_n_s_TimeoutError);
  Py_CLEAR(clear_module_state->__pyx_n_s_Trigger);
  Py_CLEAR(clear_module_state->__pyx_n_s_TypeError);
  Py_CLEAR(clear_module_state->__pyx_n_u_VENDER_CMD_NOT_SUPPORTED);
  Py_CLEAR(clear_module_state->__pyx_n_u_WRONG_I2C_ADDR);
  Py_CLEAR(clear_module_state->__pyx_n_s__111);
  Py_CLEAR(clear_module_state->__pyx_kp_u__8);
  Py_CLEAR(clear_module_state->__pyx_n_s_addr);
  Py_CLEAR(clear_module_state->__pyx_n_s_args);
  Py_CLEAR(clear_module_state->__pyx_n_s_asyncio_coroutines);
  Py_CLEAR(clear_module_state->__pyx_n_s_buf);
  Py_CLEAR(clear_module_state->__pyx_n_s_bytesRead);
  Py_CLEAR(clear_module_state->__pyx_n_s_bytesSent);
  Py_CLEAR(clear_module_state->__pyx_n_s_bytesToRead);
  Py_CLEAR(clear_module_state->__pyx_n_s_cdata);
  Py_CLEAR(clear_module_state->__pyx_n_s_cdesc);
  Py_CLEAR(clear_module_state->__pyx_n_s_chipReset);
  Py_CLEAR(clear_module_state->__pyx_n_s_chipRevision);
  Py_CLEAR(clear_module_state->__pyx_n_s_class_getitem);
  Py_CLEAR(clear_module_state->__pyx_n_s_cline_in_traceback);
  Py_CLEAR(clear_module_state->__pyx_n_s_clk);
  Py_CLEAR(clear_module_state->__pyx_n_s_clkStrength);
  Py_CLEAR(clear_module_state->__pyx_n_s_clock);
  Py_CLEAR(clear_module_state->__pyx_n_s_close);
  Py_CLEAR(clear_module_state->__pyx_n_s_closer);
  Py_CLEAR(clear_module_state->__pyx_n_s_closer___reduce_cython);
  Py_CLEAR(clear_module_state->__pyx_n_s_closer___setstate_cython);
  Py_CLEAR(clear_module_state->__pyx_n_s_closer_close);
  Py_CLEAR(clear_module_state->__pyx_n_s_cpha);
  Py_CLEAR(clear_module_state->__pyx_n_s_cpol);
  Py_CLEAR(clear_module_state->__pyx_n_s_createDeviceInfoList);
  Py_CLEAR(clear_module_state->__pyx_n_s_cs);
  Py_CLEAR(clear_module_state->__pyx_n_s_cserial);
  Py_CLEAR(clear_module_state->__pyx_n_s_d);
  Py_CLEAR(clear_module_state->__pyx_n_s_data);
  Py_CLEAR(clear_module_state->__pyx_n_s_desc);
  Py_CLEAR(clear_module_state->__pyx_n_u_description);
  Py_CLEAR(clear_module_state->__pyx_n_s_devnum);
  Py_CLEAR(clear_module_state->__pyx_n_s_dict);
  Py_CLEAR(clear_module_state->__pyx_kp_u_disable);
  Py_CLEAR(clear_module_state->__pyx_n_s_doc);
  Py_CLEAR(clear_module_state->__pyx_n_s_enable);
  Py_CLEAR(clear_module_state->__pyx_kp_u_enable);
  Py_CLEAR(clear_module_state->__pyx_n_s_encode);
  Py_CLEAR(clear_module_state->__pyx_n_s_enum);
  Py_CLEAR(clear_module_state->__pyx_n_s_events);
  Py_CLEAR(clear_module_state->__pyx_n_s_expectedValue);
  Py_CLEAR(clear_module_state->__pyx_n_s_expectedValue_c);
  Py_CLEAR(clear_module_state->__pyx_n_s_f);
  Py_CLEAR(clear_module_state->__pyx_n_s_finalize);
  Py_CLEAR(clear_module_state->__pyx_n_s_flag);
  Py_CLEAR(clear_module_state->__pyx_n_u_flags);
  Py_CLEAR(clear_module_state->__pyx_n_s_format);
  Py_CLEAR(clear_module_state->__pyx_n_s_ft4222_ft4222);
  Py_CLEAR(clear_module_state->__pyx_kp_s_ft4222_ft4222_pyx);
  Py_CLEAR(clear_module_state->__pyx_kp_u_gc);
  Py_CLEAR(clear_module_state->__pyx_n_s_getClock);
  Py_CLEAR(clear_module_state->__pyx_n_s_getDeviceInfoDetail);
  Py_CLEAR(clear_module_state->__pyx_n_s_getLatencyTimer);
  Py_CLEAR(clear_module_state->__pyx_n_s_getMaxTransferSize);
  Py_CLEAR(clear_module_state->__pyx_n_s_getstate);
  Py_CLEAR(clear_module_state->__pyx_n_s_gpio0);
  Py_CLEAR(clear_module_state->__pyx_n_s_gpio1);
  Py_CLEAR(clear_module_state->__pyx_n_s_gpio2);
  Py_CLEAR(clear_module_state->__pyx_n_s_gpio3);
  Py_CLEAR(clear_module_state->__pyx_n_s_gpio_GetTriggerStatus);
  Py_CLEAR(clear_module_state->__pyx_n_s_gpio_Init);
  Py_CLEAR(clear_module_state->__pyx_n_s_gpio_Read);
  Py_CLEAR(clear_module_state->__pyx_n_s_gpio_ReadTriggerQueue);
  Py_CLEAR(clear_module_state->__pyx_n_s_gpio_SetInputTrigger);
  Py_CLEAR(clear_module_state->__pyx_n_s_gpio_Wait);
  Py_CLEAR(clear_module_state->__pyx_n_s_gpio_Write);
  Py_CLEAR(clear_module_state->__pyx_n_s_h);
  Py_CLEAR(clear_module_state->__pyx_n_s_handle);
  Py_CLEAR(clear_module_state->__pyx_n_u_handle);
  Py_CLEAR(clear_module_state->__pyx_n_s_i);
  Py_CLEAR(clear_module_state->__pyx_n_s_i2cMaster_GetStatus);
  Py_CLEAR(clear_module_state->__pyx_n_s_i2cMaster_Init);
  Py_CLEAR(clear_module_state->__pyx_n_s_i2cMaster_Read);
  Py_CLEAR(clear_module_state->__pyx_n_s_i2cMaster_ReadEx);
  Py_CLEAR(clear_module_state->__pyx_n_s_i2cMaster_Reset);
  Py_CLEAR(clear_module_state->__pyx_n_s_i2cMaster_Write);
  Py_CLEAR(clear_module_state->__pyx_n_s_i2cMaster_WriteEx);
  Py_CLEAR(clear_module_state->__pyx_n_u_id);
  Py_CLEAR(clear_module_state->__pyx_n_s_import);
  Py_CLEAR(clear_module_state->__pyx_n_u_index);
  Py_CLEAR(clear_module_state->__pyx_n_s_init);
  Py_CLEAR(clear_module_state->__pyx_n_s_init_subclass);
  Py_CLEAR(clear_module_state->__pyx_n_s_ioDir);
  Py_CLEAR(clear_module_state->__pyx_n_s_ioStrength);
  Py_CLEAR(clear_module_state->__pyx_n_s_isEndTransaction);
  Py_CLEAR(clear_module_state->__pyx_n_s_is_coroutine);
  Py_CLEAR(clear_module_state->__pyx_kp_u_isenabled);
  Py_CLEAR(clear_module_state->__pyx_n_s_kbps);
  Py_CLEAR(clear_module_state->__pyx_n_s_l);
  Py_CLEAR(clear_module_state->__pyx_n_s_latency);
  Py_CLEAR(clear_module_state->__pyx_n_s_locId);
  Py_CLEAR(clear_module_state->__pyx_n_u_location);
  Py_CLEAR(clear_module_state->__pyx_n_s_main);
  Py_CLEAR(clear_module_state->__pyx_n_s_maxTransferSize);
  Py_CLEAR(clear_module_state->__pyx_n_s_message);
  Py_CLEAR(clear_module_state->__pyx_n_s_metaclass);
  Py_CLEAR(clear_module_state->__pyx_n_s_mode);
  Py_CLEAR(clear_module_state->__pyx_n_s_module);
  Py_CLEAR(clear_module_state->__pyx_n_s_mro_entries);
  Py_CLEAR(clear_module_state->__pyx_n_s_msgnum);
  Py_CLEAR(clear_module_state->__pyx_n_s_multiWrite);
  Py_CLEAR(clear_module_state->__pyx_n_s_n);
  Py_CLEAR(clear_module_state->__pyx_n_s_name);
  Py_CLEAR(clear_module_state->__pyx_n_s_nb);
  Py_CLEAR(clear_module_state->__pyx_n_s_openByDescription);
  Py_CLEAR(clear_module_state->__pyx_n_s_openByLocation);
  Py_CLEAR(clear_module_state->__pyx_n_s_openBySerial);
  Py_CLEAR(clear_module_state->__pyx_n_s_pRxSize);
  Py_CLEAR(clear_module_state->__pyx_n_s_portNum);
  Py_CLEAR(clear_module_state->__pyx_n_s_portNum_c);
  Py_CLEAR(clear_module_state->__pyx_n_s_prepare);
  Py_CLEAR(clear_module_state->__pyx_n_s_proto);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_state);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_vtable);
  Py_CLEAR(clear_module_state->__pyx_n_s_qualname);
  Py_CLEAR(clear_module_state->__pyx_n_s_queueSize);
  Py_CLEAR(clear_module_state->__pyx_n_s_range);
  Py_CLEAR(clear_module_state->__pyx_n_s_readSize);
  Py_CLEAR(clear_module_state->__pyx_n_s_read_timeout);
  Py_CLEAR(clear_module_state->__pyx_n_s_reduce);
  Py_CLEAR(clear_module_state->__pyx_n_s_reduce_cython);
  Py_CLEAR(clear_module_state->__pyx_n_s_reduce_ex);
  Py_CLEAR(clear_module_state->__pyx_n_s_req);
  Py_CLEAR(clear_module_state->__pyx_n_s_res);
  Py_CLEAR(clear_module_state->__pyx_n_s_round);
  Py_CLEAR(clear_module_state->__pyx_n_s_self);
  Py_CLEAR(clear_module_state->__pyx_kp_s_self__handle_cannot_be_converted);
  Py_CLEAR(clear_module_state->__pyx_n_s_serial);
  Py_CLEAR(clear_module_state->__pyx_n_u_serial);
  Py_CLEAR(clear_module_state->__pyx_n_s_setClock);
  Py_CLEAR(clear_module_state->__pyx_n_s_setLatencyTimer);
  Py_CLEAR(clear_module_state->__pyx_n_s_setSuspendOut);
  Py_CLEAR(clear_module_state->__pyx_n_s_setTimeouts);
  Py_CLEAR(clear_module_state->__pyx_n_s_setWakeUpInterrupt);
  Py_CLEAR(clear_module_state->__pyx_n_s_set_name);
  Py_CLEAR(clear_module_state->__pyx_n_s_setstate);
  Py_CLEAR(clear_module_state->__pyx_n_s_setstate_cython);
  Py_CLEAR(clear_module_state->__pyx_n_s_singleWrite);
  Py_CLEAR(clear_module_state->__pyx_n_s_sizeRead);
  Py_CLEAR(clear_module_state->__pyx_n_s_sizeTransferred);
  Py_CLEAR(clear_module_state->__pyx_n_s_sleep);
  Py_CLEAR(clear_module_state->__pyx_n_s_sleep_c);
  Py_CLEAR(clear_module_state->__pyx_n_s_spiIdx);
  Py_CLEAR(clear_module_state->__pyx_n_s_spiMaster_EndTransaction);
  Py_CLEAR(clear_module_state->__pyx_n_s_spiMaster_Init);
  Py_CLEAR(clear_module_state->__pyx_n_s_spiMaster_MultiReadWrite);
  Py_CLEAR(clear_module_state->__pyx_n_s_spiMaster_SetLines);
  Py_CLEAR(clear_module_state->__pyx_n_s_spiMaster_SingleRead);
  Py_CLEAR(clear_module_state->__pyx_n_s_spiMaster_SingleReadWrite);
  Py_CLEAR(clear_module_state->__pyx_n_s_spiMaster_SingleWrite);
  Py_CLEAR(clear_module_state->__pyx_n_s_spiSlave_GetRxStatus);
  Py_CLEAR(clear_module_state->__pyx_n_s_spiSlave_Init);
  Py_CLEAR(clear_module_state->__pyx_n_s_spiSlave_InitEx);
  Py_CLEAR(clear_module_state->__pyx_n_s_spiSlave_Read);
  Py_CLEAR(clear_module_state->__pyx_n_s_spiSlave_SetMode);
  Py_CLEAR(clear_module_state->__pyx_n_s_spiSlave_Write);
  Py_CLEAR(clear_module_state->__pyx_n_s_spi_Reset);
  Py_CLEAR(clear_module_state->__pyx_n_s_spi_ResetTransaction);
  Py_CLEAR(clear_module_state->__pyx_n_s_spi_SetDrivingStrength);
  Py_CLEAR(clear_module_state->__pyx_n_s_ssoMap);
  Py_CLEAR(clear_module_state->__pyx_n_s_ssoStrength);
  Py_CLEAR(clear_module_state->__pyx_n_s_start);
  Py_CLEAR(clear_module_state->__pyx_n_s_status);
  Py_CLEAR(clear_module_state->__pyx_n_s_str);
  Py_CLEAR(clear_module_state->__pyx_kp_s_stringsource);
  Py_CLEAR(clear_module_state->__pyx_n_s_super);
  Py_CLEAR(clear_module_state->__pyx_n_s_t);
  Py_CLEAR(clear_module_state->__pyx_n_s_test);
  Py_CLEAR(clear_module_state->__pyx_kp_u_the_data_argument_must_be_of_typ);
  Py_CLEAR(clear_module_state->__pyx_kp_u_the_multiWrite_argument_must_be);
  Py_CLEAR(clear_module_state->__pyx_kp_u_the_singleWrite_argument_must_be);
  Py_CLEAR(clear_module_state->__pyx_n_s_timeout);
  Py_CLEAR(clear_module_state->__pyx_n_s_timeout_c);
  Py_CLEAR(clear_module_state->__pyx_n_s_trigger);
  Py_CLEAR(clear_module_state->__pyx_n_u_type);
  Py_CLEAR(clear_module_state->__pyx_n_s_update);
  Py_CLEAR(clear_module_state->__pyx_kp_u_utf_8);
  Py_CLEAR(clear_module_state->__pyx_n_s_value);
  Py_CLEAR(clear_module_state->__pyx_n_s_vendorCmdGet);
  Py_CLEAR(clear_module_state->__pyx_n_s_vendorCmdSet);
  Py_CLEAR(clear_module_state->__pyx_n_s_weakref);
  Py_CLEAR(clear_module_state->__pyx_n_s_weakref_finalize);
  Py_CLEAR(clear_module_state->__pyx_n_s_write);
  Py_CLEAR(clear_module_state->__pyx_n_s_write_timeout);
  Py_CLEAR(clear_module_state->__pyx_n_s_xrange);
  Py_CLEAR(clear_module_state->__pyx_float_24000000_0);
  Py_CLEAR(clear_module_state->__pyx_int_0);
  Py_CLEAR(clear_module_state->__pyx_int_1);
  Py_CLEAR(clear_module_state->__pyx_int_2);
  Py_CLEAR(clear_module_state->__pyx_int_3);
  Py_CLEAR(clear_module_state->__pyx_int_60);
  Py_CLEAR(clear_module_state->__pyx_int_82);
  Py_CLEAR(clear_module_state->__pyx_int_100);
  Py_CLEAR(clear_module_state->__pyx_int_8000);
  Py_CLEAR(clear_module_state->__pyx_int_16908288);
  Py_CLEAR(clear_module_state->__pyx_int_16973824);
  Py_CLEAR(clear_module_state->__pyx_int_17039360);
  Py_CLEAR(clear_module_state->__pyx_int_1109524736);
  Py_CLEAR(clear_module_state->__pyx_int_1109524992);
  Py_CLEAR(clear_module_state->__pyx_int_1109525248);
  Py_CLEAR(clear_module_state->__pyx_int_1109525504);
  Py_CLEAR(clear_module_state->__pyx_k__2);
  Py_CLEAR(clear_module_state->__pyx_k__3);
  Py_CLEAR(clear_module_state->__pyx_k__4);
  Py_CLEAR(clear_module_state->__pyx_k__5);
  Py_CLEAR(clear_module_state->__pyx_tuple_);
  Py_CLEAR(clear_module_state->__pyx_tuple__6);
  Py_CLEAR(clear_module_state->__pyx_tuple__7);
  Py_CLEAR(clear_module_state->__pyx_tuple__9);
  Py_CLEAR(clear_module_state->__pyx_tuple__11);
  Py_CLEAR(clear_module_state->__pyx_tuple__15);
  Py_CLEAR(clear_module_state->__pyx_tuple__17);
  Py_CLEAR(clear_module_state->__pyx_tuple__19);
  Py_CLEAR(clear_module_state->__pyx_tuple__20);
  Py_CLEAR(clear_module_state->__pyx_tuple__22);
  Py_CLEAR(clear_module_state->__pyx_tuple__24);
  Py_CLEAR(clear_module_state->__pyx_tuple__28);
  Py_CLEAR(clear_module_state->__pyx_tuple__30);
  Py_CLEAR(clear_module_state->__pyx_tuple__32);
  Py_CLEAR(clear_module_state->__pyx_tuple__34);
  Py_CLEAR(clear_module_state->__pyx_tuple__37);
  Py_CLEAR(clear_module_state->__pyx_tuple__40);
  Py_CLEAR(clear_module_state->__pyx_tuple__42);
  Py_CLEAR(clear_module_state->__pyx_tuple__44);
  Py_CLEAR(clear_module_state->__pyx_tuple__46);
  Py_CLEAR(clear_module_state->__pyx_tuple__50);
  Py_CLEAR(clear_module_state->__pyx_tuple__52);
  Py_CLEAR(clear_module_state->__pyx_tuple__54);
  Py_CLEAR(clear_module_state->__pyx_tuple__56);
  Py_CLEAR(clear_module_state->__pyx_tuple__58);
  Py_CLEAR(clear_module_state->__pyx_tuple__60);
  Py_CLEAR(clear_module_state->__pyx_tuple__62);
  Py_CLEAR(clear_module_state->__pyx_tuple__64);
  Py_CLEAR(clear_module_state->__pyx_tuple__65);
  Py_CLEAR(clear_module_state->__pyx_tuple__67);
  Py_CLEAR(clear_module_state->__pyx_tuple__68);
  Py_CLEAR(clear_module_state->__pyx_tuple__70);
  Py_CLEAR(clear_module_state->__pyx_tuple__72);
  Py_CLEAR(clear_module_state->__pyx_tuple__74);
  Py_CLEAR(clear_module_state->__pyx_tuple__77);
  Py_CLEAR(clear_module_state->__pyx_tuple__80);
  Py_CLEAR(clear_module_state->__pyx_tuple__82);
  Py_CLEAR(clear_module_state->__pyx_tuple__84);
  Py_CLEAR(clear_module_state->__pyx_tuple__86);
  Py_CLEAR(clear_module_state->__pyx_tuple__88);
  Py_CLEAR(clear_module_state->__pyx_tuple__90);
  Py_CLEAR(clear_module_state->__pyx_tuple__92);
  Py_CLEAR(clear_module_state->__pyx_tuple__94);
  Py_CLEAR(clear_module_state->__pyx_tuple__96);
  Py_CLEAR(clear_module_state->__pyx_tuple__99);
  Py_CLEAR(clear_module_state->__pyx_tuple__101);
  Py_CLEAR(clear_module_state->__pyx_tuple__103);
  Py_CLEAR(clear_module_state->__pyx_tuple__105);
  Py_CLEAR(clear_module_state->__pyx_tuple__107);
  Py_CLEAR(clear_module_state->__pyx_codeobj__10);
  Py_CLEAR(clear_module_state->__pyx_codeobj__12);
  Py_CLEAR(clear_module_state->__pyx_codeobj__13);
  Py_CLEAR(clear_module_state->__pyx_codeobj__14);
  Py_CLEAR(clear_module_state->__pyx_codeobj__16);
  Py_CLEAR(clear_module_state->__pyx_codeobj__18);
  Py_CLEAR(clear_module_state->__pyx_codeobj__21);
  Py_CLEAR(clear_module_state->__pyx_codeobj__23);
  Py_CLEAR(clear_module_state->__pyx_codeobj__25);
  Py_CLEAR(clear_module_state->__pyx_codeobj__26);
  Py_CLEAR(clear_module_state->__pyx_codeobj__27);
  Py_CLEAR(clear_module_state->__pyx_codeobj__29);
  Py_CLEAR(clear_module_state->__pyx_codeobj__31);
  Py_CLEAR(clear_module_state->__pyx_codeobj__33);
  Py_CLEAR(clear_module_state->__pyx_codeobj__35);
  Py_CLEAR(clear_module_state->__pyx_codeobj__36);
  Py_CLEAR(clear_module_state->__pyx_codeobj__38);
  Py_CLEAR(clear_module_state->__pyx_codeobj__39);
  Py_CLEAR(clear_module_state->__pyx_codeobj__41);
  Py_CLEAR(clear_module_state->__pyx_codeobj__43);
  Py_CLEAR(clear_module_state->__pyx_codeobj__45);
  Py_CLEAR(clear_module_state->__pyx_codeobj__47);
  Py_CLEAR(clear_module_state->__pyx_codeobj__48);
  Py_CLEAR(clear_module_state->__pyx_codeobj__49);
  Py_CLEAR(clear_module_state->__pyx_codeobj__51);
  Py_CLEAR(clear_module_state->__pyx_codeobj__53);
  Py_CLEAR(clear_module_state->__pyx_codeobj__55);
  Py_CLEAR(clear_module_state->__pyx_codeobj__57);
  Py_CLEAR(clear_module_state->__pyx_codeobj__59);
  Py_CLEAR(clear_module_state->__pyx_codeobj__61);
  Py_CLEAR(clear_module_state->__pyx_codeobj__63);
  Py_CLEAR(clear_module_state->__pyx_codeobj__66);
  Py_CLEAR(clear_module_state->__pyx_codeobj__69);
  Py_CLEAR(clear_module_state->__pyx_codeobj__71);
  Py_CLEAR(clear_module_state->__pyx_codeobj__73);
  Py_CLEAR(clear_module_state->__pyx_codeobj__75);
  Py_CLEAR(clear_module_state->__pyx_codeobj__76);
  Py_CLEAR(clear_module_state->__pyx_codeobj__78);
  Py_CLEAR(clear_module_state->__pyx_codeobj__79);
  Py_CLEAR(clear_module_state->__pyx_codeobj__81);
  Py_CLEAR(clear_module_state->__pyx_codeobj__83);
  Py_CLEAR(clear_module_state->__pyx_codeobj__85);
  Py_CLEAR(clear_module_state->__pyx_codeobj__87);
  Py_CLEAR(clear_module_state->__pyx_codeobj__89);
  Py_CLEAR(clear_module_state->__pyx_codeobj__91);
  Py_CLEAR(clear_module_state->__pyx_codeobj__93);
  Py_CLEAR(clear_module_state->__pyx_codeobj__95);
  Py_CLEAR(clear_module_state->__pyx_codeobj__97);
  Py_CLEAR(clear_module_state->__pyx_codeobj__98);
  Py_CLEAR(clear_module_state->__pyx_codeobj__100);
  Py_CLEAR(clear_module_state->__pyx_codeobj__102);
  Py_CLEAR(clear_module_state->__pyx_codeobj__104);
  Py_CLEAR(clear_module_state->__pyx_codeobj__106);
  Py_CLEAR(clear_module_state->__pyx_codeobj__108);
  Py_CLEAR(clear_module_state->__pyx_codeobj__109);
  Py_CLEAR(clear_module_state->__pyx_codeobj__110);
  return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstate *traverse_module_state = __pyx_mstate(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  Py_VISIT(traverse_module_state->__pyx_empty_tuple);
  Py_VISIT(traverse_module_state->__pyx_empty_bytes);
  Py_VISIT(traverse_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_VISIT(traverse_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_VISIT(traverse_module_state->__pyx_FusedFunctionType);
  #endif
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_4type_type);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_5array_array);
  Py_VISIT(traverse_module_state->__pyx_ptype_6ft4222_6ft4222__closer);
  Py_VISIT(traverse_module_state->__pyx_type_6ft4222_6ft4222__closer);
  Py_VISIT(traverse_module_state->__pyx_ptype_6ft4222_6ft4222_FT4222);
  Py_VISIT(traverse_module_state->__pyx_type_6ft4222_6ft4222_FT4222);
  Py_VISIT(traverse_module_state->__pyx_n_u_B);
  Py_VISIT(traverse_module_state->__pyx_n_s_CLK_24);
  Py_VISIT(traverse_module_state->__pyx_n_s_CLK_48);
  Py_VISIT(traverse_module_state->__pyx_n_s_CLK_60);
  Py_VISIT(traverse_module_state->__pyx_n_s_CLK_80);
  Py_VISIT(traverse_module_state->__pyx_n_u_CLK_NOT_SUPPORTED);
  Py_VISIT(traverse_module_state->__pyx_kp_s_Chip_system_clock_Attributes_CLK);
  Py_VISIT(traverse_module_state->__pyx_n_s_ControllerStatus);
  Py_VISIT(traverse_module_state->__pyx_n_u_DEVICE_LIST_NOT_READY);
  Py_VISIT(traverse_module_state->__pyx_n_u_DEVICE_NOT_FOUND);
  Py_VISIT(traverse_module_state->__pyx_n_u_DEVICE_NOT_OPENED);
  Py_VISIT(traverse_module_state->__pyx_n_u_DEVICE_NOT_OPENED_FOR_ERASE);
  Py_VISIT(traverse_module_state->__pyx_n_u_DEVICE_NOT_OPENED_FOR_WRITE);
  Py_VISIT(traverse_module_state->__pyx_n_u_DEVICE_NOT_SUPPORTED);
  Py_VISIT(traverse_module_state->__pyx_n_s_Dir);
  Py_VISIT(traverse_module_state->__pyx_n_u_EEPROM_ERASE_FAILED);
  Py_VISIT(traverse_module_state->__pyx_n_u_EEPROM_NOT_PRESENT);
  Py_VISIT(traverse_module_state->__pyx_n_u_EEPROM_NOT_PROGRAMMED);
  Py_VISIT(traverse_module_state->__pyx_n_u_EEPROM_READ_FAILED);
  Py_VISIT(traverse_module_state->__pyx_n_u_EEPROM_WRITE_FAILED);
  Py_VISIT(traverse_module_state->__pyx_n_u_EVENT_NOT_SUPPORTED);
  Py_VISIT(traverse_module_state->__pyx_n_u_EXCEEDED_MAX_TRANSFER_SIZE);
  Py_VISIT(traverse_module_state->__pyx_kp_s_Exception_class_for_status_messa);
  Py_VISIT(traverse_module_state->__pyx_n_u_FAILED_TO_READ_DEVICE);
  Py_VISIT(traverse_module_state->__pyx_n_u_FAILED_TO_WRITE_DEVICE0);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT2XXDeviceError);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT2XXDeviceError___init);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT2XXDeviceError___str);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT2XXDeviceError__ftd2xx_msgs);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222DeviceError);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222DeviceError___init);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222DeviceError___str);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222DeviceError__ftd4222_msgs);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222___reduce_cython);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222___setstate_cython);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222__chip_rev_map);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222__chip_rev_min_lib);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_chipReset);
  Py_VISIT(traverse_module_state->__pyx_kp_u_FT4222_chipVersion_0x_x_s_libVer);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_close);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_getClock);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_getLatencyTimer);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_getMaxTransferSize);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_gpio_GetTriggerStatus);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_gpio_Init);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_gpio_Read);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_gpio_ReadTriggerQueue);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_gpio_SetInputTrigger);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_gpio_Wait);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_gpio_Write);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_i2cMaster_GetStatus);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_i2cMaster_Init);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_i2cMaster_Read);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_i2cMaster_ReadEx);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_i2cMaster_Reset);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_i2cMaster_Write);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_i2cMaster_WriteEx);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_setClock);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_setLatencyTimer);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_setSuspendOut);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_setTimeouts);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_setWakeUpInterrupt);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_spiMaster_EndTransaction);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_spiMaster_Init);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_spiMaster_MultiReadWrite);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_spiMaster_SetLines);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_spiMaster_SingleRead);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_spiMaster_SingleReadWrite);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_spiMaster_SingleWrite);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_spiSlave_GetRxStatus);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_spiSlave_Init);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_spiSlave_InitEx);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_spiSlave_Read);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_spiSlave_SetMode);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_spiSlave_Write);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_spi_Reset);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_spi_ResetTransaction);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_spi_SetDrivingStrength);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_vendorCmdGet);
  Py_VISIT(traverse_module_state->__pyx_n_s_FT4222_vendorCmdSet);
  Py_VISIT(traverse_module_state->__pyx_n_u_FUN_NOT_SUPPORT);
  Py_VISIT(traverse_module_state->__pyx_n_s_GPIO);
  Py_VISIT(traverse_module_state->__pyx_n_u_GPIO_EXCEEDED_MAX_PORTNUM);
  Py_VISIT(traverse_module_state->__pyx_n_u_GPIO_INPUT_NOT_SUPPORTED);
  Py_VISIT(traverse_module_state->__pyx_n_u_GPIO_NOT_SUPPORTED_IN_THIS_MODE);
  Py_VISIT(traverse_module_state->__pyx_n_u_GPIO_OPENDRAIN_INVALID_IN_OUTPUT);
  Py_VISIT(traverse_module_state->__pyx_n_u_GPIO_PULLDOWN_INVALID_IN_INPUTMO);
  Py_VISIT(traverse_module_state->__pyx_n_u_GPIO_PULLUP_INVALID_IN_INPUTMODE);
  Py_VISIT(traverse_module_state->__pyx_n_u_GPIO_WRITE_NOT_SUPPORTED);
  Py_VISIT(traverse_module_state->__pyx_n_s_I2CMaster);
  Py_VISIT(traverse_module_state->__pyx_n_u_I2C_NOT_SUPPORTED_IN_THIS_MODE);
  Py_VISIT(traverse_module_state->__pyx_n_s_INPUT);
  Py_VISIT(traverse_module_state->__pyx_n_u_INSUFFICIENT_RESOURCES);
  Py_VISIT(traverse_module_state->__pyx_n_u_INTERRUPT_NOT_SUPPORTED);
  Py_VISIT(traverse_module_state->__pyx_n_u_INVAILD_FUNCTION);
  Py_VISIT(traverse_module_state->__pyx_n_u_INVALID_ARGS);
  Py_VISIT(traverse_module_state->__pyx_n_u_INVALID_BAUD_RATE);
  Py_VISIT(traverse_module_state->__pyx_n_u_INVALID_HANDLE);
  Py_VISIT(traverse_module_state->__pyx_n_u_INVALID_PARAMETER);
  Py_VISIT(traverse_module_state->__pyx_n_u_INVALID_POINTER);
  Py_VISIT(traverse_module_state->__pyx_n_u_IO_ERROR);
  Py_VISIT(traverse_module_state->__pyx_n_u_IS_NOT_I2C_MODE);
  Py_VISIT(traverse_module_state->__pyx_n_u_IS_NOT_SPI_MODE);
  Py_VISIT(traverse_module_state->__pyx_n_u_IS_NOT_SPI_MULTI_MODE);
  Py_VISIT(traverse_module_state->__pyx_n_u_IS_NOT_SPI_SINGLE_MODE);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntEnum);
  Py_VISIT(traverse_module_state->__pyx_n_s_KeyError);
  Py_VISIT(traverse_module_state->__pyx_n_s_MemoryError);
  Py_VISIT(traverse_module_state->__pyx_n_u_NOT_SUPPORTED);
  Py_VISIT(traverse_module_state->__pyx_n_u_OK);
  Py_VISIT(traverse_module_state->__pyx_n_u_OTHER_ERROR);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Rev_A);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Rev_B);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Rev_C);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Rev_D);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Rev_unknown);
  Py_VISIT(traverse_module_state->__pyx_n_s_SysClock);
  Py_VISIT(traverse_module_state->__pyx_n_s_TimeoutError);
  Py_VISIT(traverse_module_state->__pyx_n_s_Trigger);
  Py_VISIT(traverse_module_state->__pyx_n_s_TypeError);
  Py_VISIT(traverse_module_state->__pyx_n_u_VENDER_CMD_NOT_SUPPORTED);
  Py_VISIT(traverse_module_state->__pyx_n_u_WRONG_I2C_ADDR);
  Py_VISIT(traverse_module_state->__pyx_n_s__111);
  Py_VISIT(traverse_module_state->__pyx_kp_u__8);
  Py_VISIT(traverse_module_state->__pyx_n_s_addr);
  Py_VISIT(traverse_module_state->__pyx_n_s_args);
  Py_VISIT(traverse_module_state->__pyx_n_s_asyncio_coroutines);
  Py_VISIT(traverse_module_state->__pyx_n_s_buf);
  Py_VISIT(traverse_module_state->__pyx_n_s_bytesRead);
  Py_VISIT(traverse_module_state->__pyx_n_s_bytesSent);
  Py_VISIT(traverse_module_state->__pyx_n_s_bytesToRead);
  Py_VISIT(traverse_module_state->__pyx_n_s_cdata);
  Py_VISIT(traverse_module_state->__pyx_n_s_cdesc);
  Py_VISIT(traverse_module_state->__pyx_n_s_chipReset);
  Py_VISIT(traverse_module_state->__pyx_n_s_chipRevision);
  Py_VISIT(traverse_module_state->__pyx_n_s_class_getitem);
  Py_VISIT(traverse_module_state->__pyx_n_s_cline_in_traceback);
  Py_VISIT(traverse_module_state->__pyx_n_s_clk);
  Py_VISIT(traverse_module_state->__pyx_n_s_clkStrength);
  Py_VISIT(traverse_module_state->__pyx_n_s_clock);
  Py_VISIT(traverse_module_state->__pyx_n_s_close);
  Py_VISIT(traverse_module_state->__pyx_n_s_closer);
  Py_VISIT(traverse_module_state->__pyx_n_s_closer___reduce_cython);
  Py_VISIT(traverse_module_state->__pyx_n_s_closer___setstate_cython);
  Py_VISIT(traverse_module_state->__pyx_n_s_closer_close);
  Py_VISIT(traverse_module_state->__pyx_n_s_cpha);
  Py_VISIT(traverse_module_state->__pyx_n_s_cpol);
  Py_VISIT(traverse_module_state->__pyx_n_s_createDeviceInfoList);
  Py_VISIT(traverse_module_state->__pyx_n_s_cs);
  Py_VISIT(traverse_module_state->__pyx_n_s_cserial);
  Py_VISIT(traverse_module_state->__pyx_n_s_d);
  Py_VISIT(traverse_module_state->__pyx_n_s_data);
  Py_VISIT(traverse_module_state->__pyx_n_s_desc);
  Py_VISIT(traverse_module_state->__pyx_n_u_description);
  Py_VISIT(traverse_module_state->__pyx_n_s_devnum);
  Py_VISIT(traverse_module_state->__pyx_n_s_dict);
  Py_VISIT(traverse_module_state->__pyx_kp_u_disable);
  Py_VISIT(traverse_module_state->__pyx_n_s_doc);
  Py_VISIT(traverse_module_state->__pyx_n_s_enable);
  Py_VISIT(traverse_module_state->__pyx_kp_u_enable);
  Py_VISIT(traverse_module_state->__pyx_n_s_encode);
  Py_VISIT(traverse_module_state->__pyx_n_s_enum);
  Py_VISIT(traverse_module_state->__pyx_n_s_events);
  Py_VISIT(traverse_module_state->__pyx_n_s_expectedValue);
  Py_VISIT(traverse_module_state->__pyx_n_s_expectedValue_c);
  Py_VISIT(traverse_module_state->__pyx_n_s_f);
  Py_VISIT(traverse_module_state->__pyx_n_s_finalize);
  Py_VISIT(traverse_module_state->__pyx_n_s_flag);
  Py_VISIT(traverse_module_state->__pyx_n_u_flags);
  Py_VISIT(traverse_module_state->__pyx_n_s_format);
  Py_VISIT(traverse_module_state->__pyx_n_s_ft4222_ft4222);
  Py_VISIT(traverse_module_state->__pyx_kp_s_ft4222_ft4222_pyx);
  Py_VISIT(traverse_module_state->__pyx_kp_u_gc);
  Py_VISIT(traverse_module_state->__pyx_n_s_getClock);
  Py_VISIT(traverse_module_state->__pyx_n_s_getDeviceInfoDetail);
  Py_VISIT(traverse_module_state->__pyx_n_s_getLatencyTimer);
  Py_VISIT(traverse_module_state->__pyx_n_s_getMaxTransferSize);
  Py_VISIT(traverse_module_state->__pyx_n_s_getstate);
  Py_VISIT(traverse_module_state->__pyx_n_s_gpio0);
  Py_VISIT(traverse_module_state->__pyx_n_s_gpio1);
  Py_VISIT(traverse_module_state->__pyx_n_s_gpio2);
  Py_VISIT(traverse_module_state->__pyx_n_s_gpio3);
  Py_VISIT(traverse_module_state->__pyx_n_s_gpio_GetTriggerStatus);
  Py_VISIT(traverse_module_state->__pyx_n_s_gpio_Init);
  Py_VISIT(traverse_module_state->__pyx_n_s_gpio_Read);
  Py_VISIT(traverse_module_state->__pyx_n_s_gpio_ReadTriggerQueue);
  Py_VISIT(traverse_module_state->__pyx_n_s_gpio_SetInputTrigger);
  Py_VISIT(traverse_module_state->__pyx_n_s_gpio_Wait);
  Py_VISIT(traverse_module_state->__pyx_n_s_gpio_Write);
  Py_VISIT(traverse_module_state->__pyx_n_s_h);
  Py_VISIT(traverse_module_state->__pyx_n_s_handle);
  Py_VISIT(traverse_module_state->__pyx_n_u_handle);
  Py_VISIT(traverse_module_state->__pyx_n_s_i);
  Py_VISIT(traverse_module_state->__pyx_n_s_i2cMaster_GetStatus);
  Py_VISIT(traverse_module_state->__pyx_n_s_i2cMaster_Init);
  Py_VISIT(traverse_module_state->__pyx_n_s_i2cMaster_Read);
  Py_VISIT(traverse_module_state->__pyx_n_s_i2cMaster_ReadEx);
  Py_VISIT(traverse_module_state->__pyx_n_s_i2cMaster_Reset);
  Py_VISIT(traverse_module_state->__pyx_n_s_i2cMaster_Write);
  Py_VISIT(traverse_module_state->__pyx_n_s_i2cMaster_WriteEx);
  Py_VISIT(traverse_module_state->__pyx_n_u_id);
  Py_VISIT(traverse_module_state->__pyx_n_s_import);
  Py_VISIT(traverse_module_state->__pyx_n_u_index);
  Py_VISIT(traverse_module_state->__pyx_n_s_init);
  Py_VISIT(traverse_module_state->__pyx_n_s_init_subclass);
  Py_VISIT(traverse_module_state->__pyx_n_s_ioDir);
  Py_VISIT(traverse_module_state->__pyx_n_s_ioStrength);
  Py_VISIT(traverse_module_state->__pyx_n_s_isEndTransaction);
  Py_VISIT(traverse_module_state->__pyx_n_s_is_coroutine);
  Py_VISIT(traverse_module_state->__pyx_kp_u_isenabled);
  Py_VISIT(traverse_module_state->__pyx_n_s_kbps);
  Py_VISIT(traverse_module_state->__pyx_n_s_l);
  Py_VISIT(traverse_module_state->__pyx_n_s_latency);
  Py_VISIT(traverse_module_state->__pyx_n_s_locId);
  Py_VISIT(traverse_module_state->__pyx_n_u_location);
  Py_VISIT(traverse_module_state->__pyx_n_s_main);
  Py_VISIT(traverse_module_state->__pyx_n_s_maxTransferSize);
  Py_VISIT(traverse_module_state->__pyx_n_s_message);
  Py_VISIT(traverse_module_state->__pyx_n_s_metaclass);
  Py_VISIT(traverse_module_state->__pyx_n_s_mode);
  Py_VISIT(traverse_module_state->__pyx_n_s_module);
  Py_VISIT(traverse_module_state->__pyx_n_s_mro_entries);
  Py_VISIT(traverse_module_state->__pyx_n_s_msgnum);
  Py_VISIT(traverse_module_state->__pyx_n_s_multiWrite);
  Py_VISIT(traverse_module_state->__pyx_n_s_n);
  Py_VISIT(traverse_module_state->__pyx_n_s_name);
  Py_VISIT(traverse_module_state->__pyx_n_s_nb);
  Py_VISIT(traverse_module_state->__pyx_n_s_openByDescription);
  Py_VISIT(traverse_module_state->__pyx_n_s_openByLocation);
  Py_VISIT(traverse_module_state->__pyx_n_s_openBySerial);
  Py_VISIT(traverse_module_state->__pyx_n_s_pRxSize);
  Py_VISIT(traverse_module_state->__pyx_n_s_portNum);
  Py_VISIT(traverse_module_state->__pyx_n_s_portNum_c);
  Py_VISIT(traverse_module_state->__pyx_n_s_prepare);
  Py_VISIT(traverse_module_state->__pyx_n_s_proto);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_state);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_vtable);
  Py_VISIT(traverse_module_state->__pyx_n_s_qualname);
  Py_VISIT(traverse_module_state->__pyx_n_s_queueSize);
  Py_VISIT(traverse_module_state->__pyx_n_s_range);
  Py_VISIT(traverse_module_state->__pyx_n_s_readSize);
  Py_VISIT(traverse_module_state->__pyx_n_s_read_timeout);
  Py_VISIT(traverse_module_state->__pyx_n_s_reduce);
  Py_VISIT(traverse_module_state->__pyx_n_s_reduce_cython);
  Py_VISIT(traverse_module_state->__pyx_n_s_reduce_ex);
  Py_VISIT(traverse_module_state->__pyx_n_s_req);
  Py_VISIT(traverse_module_state->__pyx_n_s_res);
  Py_VISIT(traverse_module_state->__pyx_n_s_round);
  Py_VISIT(traverse_module_state->__pyx_n_s_self);
  Py_VISIT(traverse_module_state->__pyx_kp_s_self__handle_cannot_be_converted);
  Py_VISIT(traverse_module_state->__pyx_n_s_serial);
  Py_VISIT(traverse_module_state->__pyx_n_u_serial);
  Py_VISIT(traverse_module_state->__pyx_n_s_setClock);
  Py_VISIT(traverse_module_state->__pyx_n_s_setLatencyTimer);
  Py_VISIT(traverse_module_state->__pyx_n_s_setSuspendOut);
  Py_VISIT(traverse_module_state->__pyx_n_s_setTimeouts);
  Py_VISIT(traverse_module_state->__pyx_n_s_setWakeUpInterrupt);
  Py_VISIT(traverse_module_state->__pyx_n_s_set_name);
  Py_VISIT(traverse_module_state->__pyx_n_s_setstate);
  Py_VISIT(traverse_module_state->__pyx_n_s_setstate_cython);
  Py_VISIT(traverse_module_state->__pyx_n_s_singleWrite);
  Py_VISIT(traverse_module_state->__pyx_n_s_sizeRead);
  Py_VISIT(traverse_module_state->__pyx_n_s_sizeTransferred);
  Py_VISIT(traverse_module_state->__pyx_n_s_sleep);
  Py_VISIT(traverse_module_state->__pyx_n_s_sleep_c);
  Py_VISIT(traverse_module_state->__pyx_n_s_spiIdx);
  Py_VISIT(traverse_module_state->__pyx_n_s_spiMaster_EndTransaction);
  Py_VISIT(traverse_module_state->__pyx_n_s_spiMaster_Init);
  Py_VISIT(traverse_module_state->__pyx_n_s_spiMaster_MultiReadWrite);
  Py_VISIT(traverse_module_state->__pyx_n_s_spiMaster_SetLines);
  Py_VISIT(traverse_module_state->__pyx_n_s_spiMaster_SingleRead);
  Py_VISIT(traverse_module_state->__pyx_n_s_spiMaster_SingleReadWrite);
  Py_VISIT(traverse_module_state->__pyx_n_s_spiMaster_SingleWrite);
  Py_VISIT(traverse_module_state->__pyx_n_s_spiSlave_GetRxStatus);
  Py_VISIT(traverse_module_state->__pyx_n_s_spiSlave_Init);
  Py_VISIT(traverse_module_state->__pyx_n_s_spiSlave_InitEx);
  Py_VISIT(traverse_module_state->__pyx_n_s_spiSlave_Read);
  Py_VISIT(traverse_module_state->__pyx_n_s_spiSlave_SetMode);
  Py_VISIT(traverse_module_state->__pyx_n_s_spiSlave_Write);
  Py_VISIT(traverse_module_state->__pyx_n_s_spi_Reset);
  Py_VISIT(traverse_module_state->__pyx_n_s_spi_ResetTransaction);
  Py_VISIT(traverse_module_state->__pyx_n_s_spi_SetDrivingStrength);
  Py_VISIT(traverse_module_state->__pyx_n_s_ssoMap);
  Py_VISIT(traverse_module_state->__pyx_n_s_ssoStrength);
  Py_VISIT(traverse_module_state->__pyx_n_s_start);
  Py_VISIT(traverse_module_state->__pyx_n_s_status);
  Py_VISIT(traverse_module_state->__pyx_n_s_str);
  Py_VISIT(traverse_module_state->__pyx_kp_s_stringsource);
  Py_VISIT(traverse_module_state->__pyx_n_s_super);
  Py_VISIT(traverse_module_state->__pyx_n_s_t);
  Py_VISIT(traverse_module_state->__pyx_n_s_test);
  Py_VISIT(traverse_module_state->__pyx_kp_u_the_data_argument_must_be_of_typ);
  Py_VISIT(traverse_module_state->__pyx_kp_u_the_multiWrite_argument_must_be);
  Py_VISIT(traverse_module_state->__pyx_kp_u_the_singleWrite_argument_must_be);
  Py_VISIT(traverse_module_state->__pyx_n_s_timeout);
  Py_VISIT(traverse_module_state->__pyx_n_s_timeout_c);
  Py_VISIT(traverse_module_state->__pyx_n_s_trigger);
  Py_VISIT(traverse_module_state->__pyx_n_u_type);
  Py_VISIT(traverse_module_state->__pyx_n_s_update);
  Py_VISIT(traverse_module_state->__pyx_kp_u_utf_8);
  Py_VISIT(traverse_module_state->__pyx_n_s_value);
  Py_VISIT(traverse_module_state->__pyx_n_s_vendorCmdGet);
  Py_VISIT(traverse_module_state->__pyx_n_s_vendorCmdSet);
  Py_VISIT(traverse_module_state->__pyx_n_s_weakref);
  Py_VISIT(traverse_module_state->__pyx_n_s_weakref_finalize);
  Py_VISIT(traverse_module_state->__pyx_n_s_write);
  Py_VISIT(traverse_module_state->__pyx_n_s_write_timeout);
  Py_VISIT(traverse_module_state->__pyx_n_s_xrange);
  Py_VISIT(traverse_module_state->__pyx_float_24000000_0);
  Py_VISIT(traverse_module_state->__pyx_int_0);
  Py_VISIT(traverse_module_state->__pyx_int_1);
  Py_VISIT(traverse_module_state->__pyx_int_2);
  Py_VISIT(traverse_module_state->__pyx_int_3);
  Py_VISIT(traverse_module_state->__pyx_int_60);
  Py_VISIT(traverse_module_state->__pyx_int_82);
  Py_VISIT(traverse_module_state->__pyx_int_100);
  Py_VISIT(traverse_module_state->__pyx_int_8000);
  Py_VISIT(traverse_module_state->__pyx_int_16908288);
  Py_VISIT(traverse_module_state->__pyx_int_16973824);
  Py_VISIT(traverse_module_state->__pyx_int_17039360);
  Py_VISIT(traverse_module_state->__pyx_int_1109524736);
  Py_VISIT(traverse_module_state->__pyx_int_1109524992);
  Py_VISIT(traverse_module_state->__pyx_int_1109525248);
  Py_VISIT(traverse_module_state->__pyx_int_1109525504);
  Py_VISIT(traverse_module_state->__pyx_k__2);
  Py_VISIT(traverse_module_state->__pyx_k__3);
  Py_VISIT(traverse_module_state->__pyx_k__4);
  Py_VISIT(traverse_module_state->__pyx_k__5);
  Py_VISIT(traverse_module_state->__pyx_tuple_);
  Py_VISIT(traverse_module_state->__pyx_tuple__6);
  Py_VISIT(traverse_module_state->__pyx_tuple__7);
  Py_VISIT(traverse_module_state->__pyx_tuple__9);
  Py_VISIT(traverse_module_state->__pyx_tuple__11);
  Py_VISIT(traverse_module_state->__pyx_tuple__15);
  Py_VISIT(traverse_module_state->__pyx_tuple__17);
  Py_VISIT(traverse_module_state->__pyx_tuple__19);
  Py_VISIT(traverse_module_state->__pyx_tuple__20);
  Py_VISIT(traverse_module_state->__pyx_tuple__22);
  Py_VISIT(traverse_module_state->__pyx_tuple__24);
  Py_VISIT(traverse_module_state->__pyx_tuple__28);
  Py_VISIT(traverse_module_state->__pyx_tuple__30);
  Py_VISIT(traverse_module_state->__pyx_tuple__32);
  Py_VISIT(traverse_module_state->__pyx_tuple__34);
  Py_VISIT(traverse_module_state->__pyx_tuple__37);
  Py_VISIT(traverse_module_state->__pyx_tuple__40);
  Py_VISIT(traverse_module_state->__pyx_tuple__42);
  Py_VISIT(traverse_module_state->__pyx_tuple__44);
  Py_VISIT(traverse_module_state->__pyx_tuple__46);
  Py_VISIT(traverse_module_state->__pyx_tuple__50);
  Py_VISIT(traverse_module_state->__pyx_tuple__52);
  Py_VISIT(traverse_module_state->__pyx_tuple__54);
  Py_VISIT(traverse_module_state->__pyx_tuple__56);
  Py_VISIT(traverse_module_state->__pyx_tuple__58);
  Py_VISIT(traverse_module_state->__pyx_tuple__60);
  Py_VISIT(traverse_module_state->__pyx_tuple__62);
  Py_VISIT(traverse_module_state->__pyx_tuple__64);
  Py_VISIT(traverse_module_state->__pyx_tuple__65);
  Py_VISIT(traverse_module_state->__pyx_tuple__67);
  Py_VISIT(traverse_module_state->__pyx_tuple__68);
  Py_VISIT(traverse_module_state->__pyx_tuple__70);
  Py_VISIT(traverse_module_state->__pyx_tuple__72);
  Py_VISIT(traverse_module_state->__pyx_tuple__74);
  Py_VISIT(traverse_module_state->__pyx_tuple__77);
  Py_VISIT(traverse_module_state->__pyx_tuple__80);
  Py_VISIT(traverse_module_state->__pyx_tuple__82);
  Py_VISIT(traverse_module_state->__pyx_tuple__84);
  Py_VISIT(traverse_module_state->__pyx_tuple__86);
  Py_VISIT(traverse_module_state->__pyx_tuple__88);
  Py_VISIT(traverse_module_state->__pyx_tuple__90);
  Py_VISIT(traverse_module_state->__pyx_tuple__92);
  Py_VISIT(traverse_module_state->__pyx_tuple__94);
  Py_VISIT(traverse_module_state->__pyx_tuple__96);
  Py_VISIT(traverse_module_state->__pyx_tuple__99);
  Py_VISIT(traverse_module_state->__pyx_tuple__101);
  Py_VISIT(traverse_module_state->__pyx_tuple__103);
  Py_VISIT(traverse_module_state->__pyx_tuple__105);
  Py_VISIT(traverse_module_state->__pyx_tuple__107);
  Py_VISIT(traverse_module_state->__pyx_codeobj__10);
  Py_VISIT(traverse_module_state->__pyx_codeobj__12);
  Py_VISIT(traverse_module_state->__pyx_codeobj__13);
  Py_VISIT(traverse_module_state->__pyx_codeobj__14);
  Py_VISIT(traverse_module_state->__pyx_codeobj__16);
  Py_VISIT(traverse_module_state->__pyx_codeobj__18);
  Py_VISIT(traverse_module_state->__pyx_codeobj__21);
  Py_VISIT(traverse_module_state->__pyx_codeobj__23);
  Py_VISIT(traverse_module_state->__pyx_codeobj__25);
  Py_VISIT(traverse_module_state->__pyx_codeobj__26);
  Py_VISIT(traverse_module_state->__pyx_codeobj__27);
  Py_VISIT(traverse_module_state->__pyx_codeobj__29);
  Py_VISIT(traverse_module_state->__pyx_codeobj__31);
  Py_VISIT(traverse_module_state->__pyx_codeobj__33);
  Py_VISIT(traverse_module_state->__pyx_codeobj__35);
  Py_VISIT(traverse_module_state->__pyx_codeobj__36);
  Py_VISIT(traverse_module_state->__pyx_codeobj__38);
  Py_VISIT(traverse_module_state->__pyx_codeobj__39);
  Py_VISIT(traverse_module_state->__pyx_codeobj__41);
  Py_VISIT(traverse_module_state->__pyx_codeobj__43);
  Py_VISIT(traverse_module_state->__pyx_codeobj__45);
  Py_VISIT(traverse_module_state->__pyx_codeobj__47);
  Py_VISIT(traverse_module_state->__pyx_codeobj__48);
  Py_VISIT(traverse_module_state->__pyx_codeobj__49);
  Py_VISIT(traverse_module_state->__pyx_codeobj__51);
  Py_VISIT(traverse_module_state->__pyx_codeobj__53);
  Py_VISIT(traverse_module_state->__pyx_codeobj__55);
  Py_VISIT(traverse_module_state->__pyx_codeobj__57);
  Py_VISIT(traverse_module_state->__pyx_codeobj__59);
  Py_VISIT(traverse_module_state->__pyx_codeobj__61);
  Py_VISIT(traverse_module_state->__pyx_codeobj__63);
  Py_VISIT(traverse_module_state->__pyx_codeobj__66);
  Py_VISIT(traverse_module_state->__pyx_codeobj__69);
  Py_VISIT(traverse_module_state->__pyx_codeobj__71);
  Py_VISIT(traverse_module_state->__pyx_codeobj__73);
  Py_VISIT(traverse_module_state->__pyx_codeobj__75);
  Py_VISIT(traverse_module_state->__pyx_codeobj__76);
  Py_VISIT(traverse_module_state->__pyx_codeobj__78);
  Py_VISIT(traverse_module_state->__pyx_codeobj__79);
  Py_VISIT(traverse_module_state->__pyx_codeobj__81);
  Py_VISIT(traverse_module_state->__pyx_codeobj__83);
  Py_VISIT(traverse_module_state->__pyx_codeobj__85);
  Py_VISIT(traverse_module_state->__pyx_codeobj__87);
  Py_VISIT(traverse_module_state->__pyx_codeobj__89);
  Py_VISIT(traverse_module_state->__pyx_codeobj__91);
  Py_VISIT(traverse_module_state->__pyx_codeobj__93);
  Py_VISIT(traverse_module_state->__pyx_codeobj__95);
  Py_VISIT(traverse_module_state->__pyx_codeobj__97);
  Py_VISIT(traverse_module_state->__pyx_codeobj__98);
  Py_VISIT(traverse_module_state->__pyx_codeobj__100);
  Py_VISIT(traverse_module_state->__pyx_codeobj__102);
  Py_VISIT(traverse_module_state->__pyx_codeobj__104);
  Py_VISIT(traverse_module_state->__pyx_codeobj__106);
  Py_VISIT(traverse_module_state->__pyx_codeobj__108);
  Py_VISIT(traverse_module_state->__pyx_codeobj__109);
  Py_VISIT(traverse_module_state->__pyx_codeobj__110);
  return 0;
}
#endif
/* #### Code section: module_state_defines ### */
#define __pyx_d __pyx_mstate_global->__pyx_d
#define __pyx_b __pyx_mstate_global->__pyx_b
#define __pyx_cython_runtime __pyx_mstate_global->__pyx_cython_runtime
#define __pyx_empty_tuple __pyx_mstate_global->__pyx_empty_tuple
#define __pyx_empty_bytes __pyx_mstate_global->__pyx_empty_bytes
#define __pyx_empty_unicode __pyx_mstate_global->__pyx_empty_unicode
#ifdef __Pyx_CyFunction_USED
#define __pyx_CyFunctionType __pyx_mstate_global->__pyx_CyFunctionType
#endif
#ifdef __Pyx_FusedFunction_USED
#define __pyx_FusedFunctionType __pyx_mstate_global->__pyx_FusedFunctionType
#endif
#ifdef __Pyx_Generator_USED
#define __pyx_GeneratorType __pyx_mstate_global->__pyx_GeneratorType
#endif
#ifdef __Pyx_IterableCoroutine_USED
#define __pyx_IterableCoroutineType __pyx_mstate_global->__pyx_IterableCoroutineType
#endif
#ifdef __Pyx_Coroutine_USED
#define __pyx_CoroutineAwaitType __pyx_mstate_global->__pyx_CoroutineAwaitType
#endif
#ifdef __Pyx_Coroutine_USED
#define __pyx_CoroutineType __pyx_mstate_global->__pyx_CoroutineType
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_ptype_7cpython_4type_type __pyx_mstate_global->__pyx_ptype_7cpython_4type_type
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_ptype_7cpython_5array_array __pyx_mstate_global->__pyx_ptype_7cpython_5array_array
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#define __pyx_type_6ft4222_6ft4222__closer __pyx_mstate_global->__pyx_type_6ft4222_6ft4222__closer
#define __pyx_type_6ft4222_6ft4222_FT4222 __pyx_mstate_global->__pyx_type_6ft4222_6ft4222_FT4222
#endif
#define __pyx_ptype_6ft4222_6ft4222__closer __pyx_mstate_global->__pyx_ptype_6ft4222_6ft4222__closer
#define __pyx_ptype_6ft4222_6ft4222_FT4222 __pyx_mstate_global->__pyx_ptype_6ft4222_6ft4222_FT4222
#define __pyx_n_u_B __pyx_mstate_global->__pyx_n_u_B
#define __pyx_n_s_CLK_24 __pyx_mstate_global->__pyx_n_s_CLK_24
#define __pyx_n_s_CLK_48 __pyx_mstate_global->__pyx_n_s_CLK_48
#define __pyx_n_s_CLK_60 __pyx_mstate_global->__pyx_n_s_CLK_60
#define __pyx_n_s_CLK_80 __pyx_mstate_global->__pyx_n_s_CLK_80
#define __pyx_n_u_CLK_NOT_SUPPORTED __pyx_mstate_global->__pyx_n_u_CLK_NOT_SUPPORTED
#define __pyx_kp_s_Chip_system_clock_Attributes_CLK __pyx_mstate_global->__pyx_kp_s_Chip_system_clock_Attributes_CLK
#define __pyx_n_s_ControllerStatus __pyx_mstate_global->__pyx_n_s_ControllerStatus
#define __pyx_n_u_DEVICE_LIST_NOT_READY __pyx_mstate_global->__pyx_n_u_DEVICE_LIST_NOT_READY
#define __pyx_n_u_DEVICE_NOT_FOUND __pyx_mstate_global->__pyx_n_u_DEVICE_NOT_FOUND
#define __pyx_n_u_DEVICE_NOT_OPENED __pyx_mstate_global->__pyx_n_u_DEVICE_NOT_OPENED
#define __pyx_n_u_DEVICE_NOT_OPENED_FOR_ERASE __pyx_mstate_global->__pyx_n_u_DEVICE_NOT_OPENED_FOR_ERASE
#define __pyx_n_u_DEVICE_NOT_OPENED_FOR_WRITE __pyx_mstate_global->__pyx_n_u_DEVICE_NOT_OPENED_FOR_WRITE
#define __pyx_n_u_DEVICE_NOT_SUPPORTED __pyx_mstate_global->__pyx_n_u_DEVICE_NOT_SUPPORTED
#define __pyx_n_s_Dir __pyx_mstate_global->__pyx_n_s_Dir
#define __pyx_n_u_EEPROM_ERASE_FAILED __pyx_mstate_global->__pyx_n_u_EEPROM_ERASE_FAILED
#define __pyx_n_u_EEPROM_NOT_PRESENT __pyx_mstate_global->__pyx_n_u_EEPROM_NOT_PRESENT
#define __pyx_n_u_EEPROM_NOT_PROGRAMMED __pyx_mstate_global->__pyx_n_u_EEPROM_NOT_PROGRAMMED
#define __pyx_n_u_EEPROM_READ_FAILED __pyx_mstate_global->__pyx_n_u_EEPROM_READ_FAILED
#define __pyx_n_u_EEPROM_WRITE_FAILED __pyx_mstate_global->__pyx_n_u_EEPROM_WRITE_FAILED
#define __pyx_n_u_EVENT_NOT_SUPPORTED __pyx_mstate_global->__pyx_n_u_EVENT_NOT_SUPPORTED
#define __pyx_n_u_EXCEEDED_MAX_TRANSFER_SIZE __pyx_mstate_global->__pyx_n_u_EXCEEDED_MAX_TRANSFER_SIZE
#define __pyx_kp_s_Exception_class_for_status_messa __pyx_mstate_global->__pyx_kp_s_Exception_class_for_status_messa
#define __pyx_n_u_FAILED_TO_READ_DEVICE __pyx_mstate_global->__pyx_n_u_FAILED_TO_READ_DEVICE
#define __pyx_n_u_FAILED_TO_WRITE_DEVICE0 __pyx_mstate_global->__pyx_n_u_FAILED_TO_WRITE_DEVICE0
#define __pyx_n_s_FT2XXDeviceError __pyx_mstate_global->__pyx_n_s_FT2XXDeviceError
#define __pyx_n_s_FT2XXDeviceError___init __pyx_mstate_global->__pyx_n_s_FT2XXDeviceError___init
#define __pyx_n_s_FT2XXDeviceError___str __pyx_mstate_global->__pyx_n_s_FT2XXDeviceError___str
#define __pyx_n_s_FT2XXDeviceError__ftd2xx_msgs __pyx_mstate_global->__pyx_n_s_FT2XXDeviceError__ftd2xx_msgs
#define __pyx_n_s_FT4222 __pyx_mstate_global->__pyx_n_s_FT4222
#define __pyx_n_s_FT4222DeviceError __pyx_mstate_global->__pyx_n_s_FT4222DeviceError
#define __pyx_n_s_FT4222DeviceError___init __pyx_mstate_global->__pyx_n_s_FT4222DeviceError___init
#define __pyx_n_s_FT4222DeviceError___str __pyx_mstate_global->__pyx_n_s_FT4222DeviceError___str
#define __pyx_n_s_FT4222DeviceError__ftd4222_msgs __pyx_mstate_global->__pyx_n_s_FT4222DeviceError__ftd4222_msgs
#define __pyx_n_s_FT4222___reduce_cython __pyx_mstate_global->__pyx_n_s_FT4222___reduce_cython
#define __pyx_n_s_FT4222___setstate_cython __pyx_mstate_global->__pyx_n_s_FT4222___setstate_cython
#define __pyx_n_s_FT4222__chip_rev_map __pyx_mstate_global->__pyx_n_s_FT4222__chip_rev_map
#define __pyx_n_s_FT4222__chip_rev_min_lib __pyx_mstate_global->__pyx_n_s_FT4222__chip_rev_min_lib
#define __pyx_n_s_FT4222_chipReset __pyx_mstate_global->__pyx_n_s_FT4222_chipReset
#define __pyx_kp_u_FT4222_chipVersion_0x_x_s_libVer __pyx_mstate_global->__pyx_kp_u_FT4222_chipVersion_0x_x_s_libVer
#define __pyx_n_s_FT4222_close __pyx_mstate_global->__pyx_n_s_FT4222_close
#define __pyx_n_s_FT4222_getClock __pyx_mstate_global->__pyx_n_s_FT4222_getClock
#define __pyx_n_s_FT4222_getLatencyTimer __pyx_mstate_global->__pyx_n_s_FT4222_getLatencyTimer
#define __pyx_n_s_FT4222_getMaxTransferSize __pyx_mstate_global->__pyx_n_s_FT4222_getMaxTransferSize
#define __pyx_n_s_FT4222_gpio_GetTriggerStatus __pyx_mstate_global->__pyx_n_s_FT4222_gpio_GetTriggerStatus
#define __pyx_n_s_FT4222_gpio_Init __pyx_mstate_global->__pyx_n_s_FT4222_gpio_Init
#define __pyx_n_s_FT4222_gpio_Read __pyx_mstate_global->__pyx_n_s_FT4222_gpio_Read
#define __pyx_n_s_FT4222_gpio_ReadTriggerQueue __pyx_mstate_global->__pyx_n_s_FT4222_gpio_ReadTriggerQueue
#define __pyx_n_s_FT4222_gpio_SetInputTrigger __pyx_mstate_global->__pyx_n_s_FT4222_gpio_SetInputTrigger
#define __pyx_n_s_FT4222_gpio_Wait __pyx_mstate_global->__pyx_n_s_FT4222_gpio_Wait
#define __pyx_n_s_FT4222_gpio_Write __pyx_mstate_global->__pyx_n_s_FT4222_gpio_Write
#define __pyx_n_s_FT4222_i2cMaster_GetStatus __pyx_mstate_global->__pyx_n_s_FT4222_i2cMaster_GetStatus
#define __pyx_n_s_FT4222_i2cMaster_Init __pyx_mstate_global->__pyx_n_s_FT4222_i2cMaster_Init
#define __pyx_n_s_FT4222_i2cMaster_Read __pyx_mstate_global->__pyx_n_s_FT4222_i2cMaster_Read
#define __pyx_n_s_FT4222_i2cMaster_ReadEx __pyx_mstate_global->__pyx_n_s_FT4222_i2cMaster_ReadEx
#define __pyx_n_s_FT4222_i2cMaster_Reset __pyx_mstate_global->__pyx_n_s_FT4222_i2cMaster_Reset
#define __pyx_n_s_FT4222_i2cMaster_Write __pyx_mstate_global->__pyx_n_s_FT4222_i2cMaster_Write
#define __pyx_n_s_FT4222_i2cMaster_WriteEx __pyx_mstate_global->__pyx_n_s_FT4222_i2cMaster_WriteEx
#define __pyx_n_s_FT4222_setClock __pyx_mstate_global->__pyx_n_s_FT4222_setClock
#define __pyx_n_s_FT4222_setLatencyTimer __pyx_mstate_global->__pyx_n_s_FT4222_setLatencyTimer
#define __pyx_n_s_FT4222_setSuspendOut __pyx_mstate_global->__pyx_n_s_FT4222_setSuspendOut
#define __pyx_n_s_FT4222_setTimeouts __pyx_mstate_global->__pyx_n_s_FT4222_setTimeouts
#define __pyx_n_s_FT4222_setWakeUpInterrupt __pyx_mstate_global->__pyx_n_s_FT4222_setWakeUpInterrupt
#define __pyx_n_s_FT4222_spiMaster_EndTransaction __pyx_mstate_global->__pyx_n_s_FT4222_spiMaster_EndTransaction
#define __pyx_n_s_FT4222_spiMaster_Init __pyx_mstate_global->__pyx_n_s_FT4222_spiMaster_Init
#define __pyx_n_s_FT4222_spiMaster_MultiReadWrite __pyx_mstate_global->__pyx_n_s_FT4222_spiMaster_MultiReadWrite
#define __pyx_n_s_FT4222_spiMaster_SetLines __pyx_mstate_global->__pyx_n_s_FT4222_spiMaster_SetLines
#define __pyx_n_s_FT4222_spiMaster_SingleRead __pyx_mstate_global->__pyx_n_s_FT4222_spiMaster_SingleRead
#define __pyx_n_s_FT4222_spiMaster_SingleReadWrite __pyx_mstate_global->__pyx_n_s_FT4222_spiMaster_SingleReadWrite
#define __pyx_n_s_FT4222_spiMaster_SingleWrite __pyx_mstate_global->__pyx_n_s_FT4222_spiMaster_SingleWrite
#define __pyx_n_s_FT4222_spiSlave_GetRxStatus __pyx_mstate_global->__pyx_n_s_FT4222_spiSlave_GetRxStatus
#define __pyx_n_s_FT4222_spiSlave_Init __pyx_mstate_global->__pyx_n_s_FT4222_spiSlave_Init
#define __pyx_n_s_FT4222_spiSlave_InitEx __pyx_mstate_global->__pyx_n_s_FT4222_spiSlave_InitEx
#define __pyx_n_s_FT4222_spiSlave_Read __pyx_mstate_global->__pyx_n_s_FT4222_spiSlave_Read
#define __pyx_n_s_FT4222_spiSlave_SetMode __pyx_mstate_global->__pyx_n_s_FT4222_spiSlave_SetMode
#define __pyx_n_s_FT4222_spiSlave_Write __pyx_mstate_global->__pyx_n_s_FT4222_spiSlave_Write
#define __pyx_n_s_FT4222_spi_Reset __pyx_mstate_global->__pyx_n_s_FT4222_spi_Reset
#define __pyx_n_s_FT4222_spi_ResetTransaction __pyx_mstate_global->__pyx_n_s_FT4222_spi_ResetTransaction
#define __pyx_n_s_FT4222_spi_SetDrivingStrength __pyx_mstate_global->__pyx_n_s_FT4222_spi_SetDrivingStrength
#define __pyx_n_s_FT4222_vendorCmdGet __pyx_mstate_global->__pyx_n_s_FT4222_vendorCmdGet
#define __pyx_n_s_FT4222_vendorCmdSet __pyx_mstate_global->__pyx_n_s_FT4222_vendorCmdSet
#define __pyx_n_u_FUN_NOT_SUPPORT __pyx_mstate_global->__pyx_n_u_FUN_NOT_SUPPORT
#define __pyx_n_s_GPIO __pyx_mstate_global->__pyx_n_s_GPIO
#define __pyx_n_u_GPIO_EXCEEDED_MAX_PORTNUM __pyx_mstate_global->__pyx_n_u_GPIO_EXCEEDED_MAX_PORTNUM
#define __pyx_n_u_GPIO_INPUT_NOT_SUPPORTED __pyx_mstate_global->__pyx_n_u_GPIO_INPUT_NOT_SUPPORTED
#define __pyx_n_u_GPIO_NOT_SUPPORTED_IN_THIS_MODE __pyx_mstate_global->__pyx_n_u_GPIO_NOT_SUPPORTED_IN_THIS_MODE
#define __pyx_n_u_GPIO_OPENDRAIN_INVALID_IN_OUTPUT __pyx_mstate_global->__pyx_n_u_GPIO_OPENDRAIN_INVALID_IN_OUTPUT
#define __pyx_n_u_GPIO_PULLDOWN_INVALID_IN_INPUTMO __pyx_mstate_global->__pyx_n_u_GPIO_PULLDOWN_INVALID_IN_INPUTMO
#define __pyx_n_u_GPIO_PULLUP_INVALID_IN_INPUTMODE __pyx_mstate_global->__pyx_n_u_GPIO_PULLUP_INVALID_IN_INPUTMODE
#define __pyx_n_u_GPIO_WRITE_NOT_SUPPORTED __pyx_mstate_global->__pyx_n_u_GPIO_WRITE_NOT_SUPPORTED
#define __pyx_n_s_I2CMaster __pyx_mstate_global->__pyx_n_s_I2CMaster
#define __pyx_n_u_I2C_NOT_SUPPORTED_IN_THIS_MODE __pyx_mstate_global->__pyx_n_u_I2C_NOT_SUPPORTED_IN_THIS_MODE
#define __pyx_n_s_INPUT __pyx_mstate_global->__pyx_n_s_INPUT
#define __pyx_n_u_INSUFFICIENT_RESOURCES __pyx_mstate_global->__pyx_n_u_INSUFFICIENT_RESOURCES
#define __pyx_n_u_INTERRUPT_NOT_SUPPORTED __pyx_mstate_global->__pyx_n_u_INTERRUPT_NOT_SUPPORTED
#define __pyx_n_u_INVAILD_FUNCTION __pyx_mstate_global->__pyx_n_u_INVAILD_FUNCTION
#define __pyx_n_u_INVALID_ARGS __pyx_mstate_global->__pyx_n_u_INVALID_ARGS
#define __pyx_n_u_INVALID_BAUD_RATE __pyx_mstate_global->__pyx_n_u_INVALID_BAUD_RATE
#define __pyx_n_u_INVALID_HANDLE __pyx_mstate_global->__pyx_n_u_INVALID_HANDLE
#define __pyx_n_u_INVALID_PARAMETER __pyx_mstate_global->__pyx_n_u_INVALID_PARAMETER
#define __pyx_n_u_INVALID_POINTER __pyx_mstate_global->__pyx_n_u_INVALID_POINTER
#define __pyx_n_u_IO_ERROR __pyx_mstate_global->__pyx_n_u_IO_ERROR
#define __pyx_n_u_IS_NOT_I2C_MODE __pyx_mstate_global->__pyx_n_u_IS_NOT_I2C_MODE
#define __pyx_n_u_IS_NOT_SPI_MODE __pyx_mstate_global->__pyx_n_u_IS_NOT_SPI_MODE
#define __pyx_n_u_IS_NOT_SPI_MULTI_MODE __pyx_mstate_global->__pyx_n_u_IS_NOT_SPI_MULTI_MODE
#define __pyx_n_u_IS_NOT_SPI_SINGLE_MODE __pyx_mstate_global->__pyx_n_u_IS_NOT_SPI_SINGLE_MODE
#define __pyx_n_s_IntEnum __pyx_mstate_global->__pyx_n_s_IntEnum
#define __pyx_n_s_KeyError __pyx_mstate_global->__pyx_n_s_KeyError
#define __pyx_n_s_MemoryError __pyx_mstate_global->__pyx_n_s_MemoryError
#define __pyx_n_u_NOT_SUPPORTED __pyx_mstate_global->__pyx_n_u_NOT_SUPPORTED
#define __pyx_n_u_OK __pyx_mstate_global->__pyx_n_u_OK
#define __pyx_n_u_OTHER_ERROR __pyx_mstate_global->__pyx_n_u_OTHER_ERROR
#define __pyx_kp_u_Rev_A __pyx_mstate_global->__pyx_kp_u_Rev_A
#define __pyx_kp_u_Rev_B __pyx_mstate_global->__pyx_kp_u_Rev_B
#define __pyx_kp_u_Rev_C __pyx_mstate_global->__pyx_kp_u_Rev_C
#define __pyx_kp_u_Rev_D __pyx_mstate_global->__pyx_kp_u_Rev_D
#define __pyx_kp_u_Rev_unknown __pyx_mstate_global->__pyx_kp_u_Rev_unknown
#define __pyx_n_s_SysClock __pyx_mstate_global->__pyx_n_s_SysClock
#define __pyx_n_s_TimeoutError __pyx_mstate_global->__pyx_n_s_TimeoutError
#define __pyx_n_s_Trigger __pyx_mstate_global->__pyx_n_s_Trigger
#define __pyx_n_s_TypeError __pyx_mstate_global->__pyx_n_s_TypeError
#define __pyx_n_u_VENDER_CMD_NOT_SUPPORTED __pyx_mstate_global->__pyx_n_u_VENDER_CMD_NOT_SUPPORTED
#define __pyx_n_u_WRONG_I2C_ADDR __pyx_mstate_global->__pyx_n_u_WRONG_I2C_ADDR
#define __pyx_n_s__111 __pyx_mstate_global->__pyx_n_s__111
#define __pyx_kp_u__8 __pyx_mstate_global->__pyx_kp_u__8
#define __pyx_n_s_addr __pyx_mstate_global->__pyx_n_s_addr
#define __pyx_n_s_args __pyx_mstate_global->__pyx_n_s_args
#define __pyx_n_s_asyncio_coroutines __pyx_mstate_global->__pyx_n_s_asyncio_coroutines
#define __pyx_n_s_buf __pyx_mstate_global->__pyx_n_s_buf
#define __pyx_n_s_bytesRead __pyx_mstate_global->__pyx_n_s_bytesRead
#define __pyx_n_s_bytesSent __pyx_mstate_global->__pyx_n_s_bytesSent
#define __pyx_n_s_bytesToRead __pyx_mstate_global->__pyx_n_s_bytesToRead
#define __pyx_n_s_cdata __pyx_mstate_global->__pyx_n_s_cdata
#define __pyx_n_s_cdesc __pyx_mstate_global->__pyx_n_s_cdesc
#define __pyx_n_s_chipReset __pyx_mstate_global->__pyx_n_s_chipReset
#define __pyx_n_s_chipRevision __pyx_mstate_global->__pyx_n_s_chipRevision
#define __pyx_n_s_class_getitem __pyx_mstate_global->__pyx_n_s_class_getitem
#define __pyx_n_s_cline_in_traceback __pyx_mstate_global->__pyx_n_s_cline_in_traceback
#define __pyx_n_s_clk __pyx_mstate_global->__pyx_n_s_clk
#define __pyx_n_s_clkStrength __pyx_mstate_global->__pyx_n_s_clkStrength
#define __pyx_n_s_clock __pyx_mstate_global->__pyx_n_s_clock
#define __pyx_n_s_close __pyx_mstate_global->__pyx_n_s_close
#define __pyx_n_s_closer __pyx_mstate_global->__pyx_n_s_closer
#define __pyx_n_s_closer___reduce_cython __pyx_mstate_global->__pyx_n_s_closer___reduce_cython
#define __pyx_n_s_closer___setstate_cython __pyx_mstate_global->__pyx_n_s_closer___setstate_cython
#define __pyx_n_s_closer_close __pyx_mstate_global->__pyx_n_s_closer_close
#define __pyx_n_s_cpha __pyx_mstate_global->__pyx_n_s_cpha
#define __pyx_n_s_cpol __pyx_mstate_global->__pyx_n_s_cpol
#define __pyx_n_s_createDeviceInfoList __pyx_mstate_global->__pyx_n_s_createDeviceInfoList
#define __pyx_n_s_cs __pyx_mstate_global->__pyx_n_s_cs
#define __pyx_n_s_cserial __pyx_mstate_global->__pyx_n_s_cserial
#define __pyx_n_s_d __pyx_mstate_global->__pyx_n_s_d
#define __pyx_n_s_data __pyx_mstate_global->__pyx_n_s_data
#define __pyx_n_s_desc __pyx_mstate_global->__pyx_n_s_desc
#define __pyx_n_u_description __pyx_mstate_global->__pyx_n_u_description
#define __pyx_n_s_devnum __pyx_mstate_global->__pyx_n_s_devnum
#define __pyx_n_s_dict __pyx_mstate_global->__pyx_n_s_dict
#define __pyx_kp_u_disable __pyx_mstate_global->__pyx_kp_u_disable
#define __pyx_n_s_doc __pyx_mstate_global->__pyx_n_s_doc
#define __pyx_n_s_enable __pyx_mstate_global->__pyx_n_s_enable
#define __pyx_kp_u_enable __pyx_mstate_global->__pyx_kp_u_enable
#define __pyx_n_s_encode __pyx_mstate_global->__pyx_n_s_encode
#define __pyx_n_s_enum __pyx_mstate_global->__pyx_n_s_enum
#define __pyx_n_s_events __pyx_mstate_global->__pyx_n_s_events
#define __pyx_n_s_expectedValue __pyx_mstate_global->__pyx_n_s_expectedValue
#define __pyx_n_s_expectedValue_c __pyx_mstate_global->__pyx_n_s_expectedValue_c
#define __pyx_n_s_f __pyx_mstate_global->__pyx_n_s_f
#define __pyx_n_s_finalize __pyx_mstate_global->__pyx_n_s_finalize
#define __pyx_n_s_flag __pyx_mstate_global->__pyx_n_s_flag
#define __pyx_n_u_flags __pyx_mstate_global->__pyx_n_u_flags
#define __pyx_n_s_format __pyx_mstate_global->__pyx_n_s_format
#define __pyx_n_s_ft4222_ft4222 __pyx_mstate_global->__pyx_n_s_ft4222_ft4222
#define __pyx_kp_s_ft4222_ft4222_pyx __pyx_mstate_global->__pyx_kp_s_ft4222_ft4222_pyx
#define __pyx_kp_u_gc __pyx_mstate_global->__pyx_kp_u_gc
#define __pyx_n_s_getClock __pyx_mstate_global->__pyx_n_s_getClock
#define __pyx_n_s_getDeviceInfoDetail __pyx_mstate_global->__pyx_n_s_getDeviceInfoDetail
#define __pyx_n_s_getLatencyTimer __pyx_mstate_global->__pyx_n_s_getLatencyTimer
#define __pyx_n_s_getMaxTransferSize __pyx_mstate_global->__pyx_n_s_getMaxTransferSize
#define __pyx_n_s_getstate __pyx_mstate_global->__pyx_n_s_getstate
#define __pyx_n_s_gpio0 __pyx_mstate_global->__pyx_n_s_gpio0
#define __pyx_n_s_gpio1 __pyx_mstate_global->__pyx_n_s_gpio1
#define __pyx_n_s_gpio2 __pyx_mstate_global->__pyx_n_s_gpio2
#define __pyx_n_s_gpio3 __pyx_mstate_global->__pyx_n_s_gpio3
#define __pyx_n_s_gpio_GetTriggerStatus __pyx_mstate_global->__pyx_n_s_gpio_GetTriggerStatus
#define __pyx_n_s_gpio_Init __pyx_mstate_global->__pyx_n_s_gpio_Init
#define __pyx_n_s_gpio_Read __pyx_mstate_global->__pyx_n_s_gpio_Read
#define __pyx_n_s_gpio_ReadTriggerQueue __pyx_mstate_global->__pyx_n_s_gpio_ReadTriggerQueue
#define __pyx_n_s_gpio_SetInputTrigger __pyx_mstate_global->__pyx_n_s_gpio_SetInputTrigger
#define __pyx_n_s_gpio_Wait __pyx_mstate_global->__pyx_n_s_gpio_Wait
#define __pyx_n_s_gpio_Write __pyx_mstate_global->__pyx_n_s_gpio_Write
#define __pyx_n_s_h __pyx_mstate_global->__pyx_n_s_h
#define __pyx_n_s_handle __pyx_mstate_global->__pyx_n_s_handle
#define __pyx_n_u_handle __pyx_mstate_global->__pyx_n_u_handle
#define __pyx_n_s_i __pyx_mstate_global->__pyx_n_s_i
#define __pyx_n_s_i2cMaster_GetStatus __pyx_mstate_global->__pyx_n_s_i2cMaster_GetStatus
#define __pyx_n_s_i2cMaster_Init __pyx_mstate_global->__pyx_n_s_i2cMaster_Init
#define __pyx_n_s_i2cMaster_Read __pyx_mstate_global->__pyx_n_s_i2cMaster_Read
#define __pyx_n_s_i2cMaster_ReadEx __pyx_mstate_global->__pyx_n_s_i2cMaster_ReadEx
#define __pyx_n_s_i2cMaster_Reset __pyx_mstate_global->__pyx_n_s_i2cMaster_Reset
#define __pyx_n_s_i2cMaster_Write __pyx_mstate_global->__pyx_n_s_i2cMaster_Write
#define __pyx_n_s_i2cMaster_WriteEx __pyx_mstate_global->__pyx_n_s_i2cMaster_WriteEx
#define __pyx_n_u_id __pyx_mstate_global->__pyx_n_u_id
#define __pyx_n_s_import __pyx_mstate_global->__pyx_n_s_import
#define __pyx_n_u_index __pyx_mstate_global->__pyx_n_u_index
#define __pyx_n_s_init __pyx_mstate_global->__pyx_n_s_init
#define __pyx_n_s_init_subclass __pyx_mstate_global->__pyx_n_s_init_subclass
#define __pyx_n_s_ioDir __pyx_mstate_global->__pyx_n_s_ioDir
#define __pyx_n_s_ioStrength __pyx_mstate_global->__pyx_n_s_ioStrength
#define __pyx_n_s_isEndTransaction __pyx_mstate_global->__pyx_n_s_isEndTransaction
#define __pyx_n_s_is_coroutine __pyx_mstate_global->__pyx_n_s_is_coroutine
#define __pyx_kp_u_isenabled __pyx_mstate_global->__pyx_kp_u_isenabled
#define __pyx_n_s_kbps __pyx_mstate_global->__pyx_n_s_kbps
#define __pyx_n_s_l __pyx_mstate_global->__pyx_n_s_l
#define __pyx_n_s_latency __pyx_mstate_global->__pyx_n_s_latency
#define __pyx_n_s_locId __pyx_mstate_global->__pyx_n_s_locId
#define __pyx_n_u_location __pyx_mstate_global->__pyx_n_u_location
#define __pyx_n_s_main __pyx_mstate_global->__pyx_n_s_main
#define __pyx_n_s_maxTransferSize __pyx_mstate_global->__pyx_n_s_maxTransferSize
#define __pyx_n_s_message __pyx_mstate_global->__pyx_n_s_message
#define __pyx_n_s_metaclass __pyx_mstate_global->__pyx_n_s_metaclass
#define __pyx_n_s_mode __pyx_mstate_global->__pyx_n_s_mode
#define __pyx_n_s_module __pyx_mstate_global->__pyx_n_s_module
#define __pyx_n_s_mro_entries __pyx_mstate_global->__pyx_n_s_mro_entries
#define __pyx_n_s_msgnum __pyx_mstate_global->__pyx_n_s_msgnum
#define __pyx_n_s_multiWrite __pyx_mstate_global->__pyx_n_s_multiWrite
#define __pyx_n_s_n __pyx_mstate_global->__pyx_n_s_n
#define __pyx_n_s_name __pyx_mstate_global->__pyx_n_s_name
#define __pyx_n_s_nb __pyx_mstate_global->__pyx_n_s_nb
#define __pyx_n_s_openByDescription __pyx_mstate_global->__pyx_n_s_openByDescription
#define __pyx_n_s_openByLocation __pyx_mstate_global->__pyx_n_s_openByLocation
#define __pyx_n_s_openBySerial __pyx_mstate_global->__pyx_n_s_openBySerial
#define __pyx_n_s_pRxSize __pyx_mstate_global->__pyx_n_s_pRxSize
#define __pyx_n_s_portNum __pyx_mstate_global->__pyx_n_s_portNum
#define __pyx_n_s_portNum_c __pyx_mstate_global->__pyx_n_s_portNum_c
#define __pyx_n_s_prepare __pyx_mstate_global->__pyx_n_s_prepare
#define __pyx_n_s_proto __pyx_mstate_global->__pyx_n_s_proto
#define __pyx_n_s_pyx_state __pyx_mstate_global->__pyx_n_s_pyx_state
#define __pyx_n_s_pyx_vtable __pyx_mstate_global->__pyx_n_s_pyx_vtable
#define __pyx_n_s_qualname __pyx_mstate_global->__pyx_n_s_qualname
#define __pyx_n_s_queueSize __pyx_mstate_global->__pyx_n_s_queueSize
#define __pyx_n_s_range __pyx_mstate_global->__pyx_n_s_range
#define __pyx_n_s_readSize __pyx_mstate_global->__pyx_n_s_readSize
#define __pyx_n_s_read_timeout __pyx_mstate_global->__pyx_n_s_read_timeout
#define __pyx_n_s_reduce __pyx_mstate_global->__pyx_n_s_reduce
#define __pyx_n_s_reduce_cython __pyx_mstate_global->__pyx_n_s_reduce_cython
#define __pyx_n_s_reduce_ex __pyx_mstate_global->__pyx_n_s_reduce_ex
#define __pyx_n_s_req __pyx_mstate_global->__pyx_n_s_req
#define __pyx_n_s_res __pyx_mstate_global->__pyx_n_s_res
#define __pyx_n_s_round __pyx_mstate_global->__pyx_n_s_round
#define __pyx_n_s_self __pyx_mstate_global->__pyx_n_s_self
#define __pyx_kp_s_self__handle_cannot_be_converted __pyx_mstate_global->__pyx_kp_s_self__handle_cannot_be_converted
#define __pyx_n_s_serial __pyx_mstate_global->__pyx_n_s_serial
#define __pyx_n_u_serial __pyx_mstate_global->__pyx_n_u_serial
#define __pyx_n_s_setClock __pyx_mstate_global->__pyx_n_s_setClock
#define __pyx_n_s_setLatencyTimer __pyx_mstate_global->__pyx_n_s_setLatencyTimer
#define __pyx_n_s_setSuspendOut __pyx_mstate_global->__pyx_n_s_setSuspendOut
#define __pyx_n_s_setTimeouts __pyx_mstate_global->__pyx_n_s_setTimeouts
#define __pyx_n_s_setWakeUpInterrupt __pyx_mstate_global->__pyx_n_s_setWakeUpInterrupt
#define __pyx_n_s_set_name __pyx_mstate_global->__pyx_n_s_set_name
#define __pyx_n_s_setstate __pyx_mstate_global->__pyx_n_s_setstate
#define __pyx_n_s_setstate_cython __pyx_mstate_global->__pyx_n_s_setstate_cython
#define __pyx_n_s_singleWrite __pyx_mstate_global->__pyx_n_s_singleWrite
#define __pyx_n_s_sizeRead __pyx_mstate_global->__pyx_n_s_sizeRead
#define __pyx_n_s_sizeTransferred __pyx_mstate_global->__pyx_n_s_sizeTransferred
#define __pyx_n_s_sleep __pyx_mstate_global->__pyx_n_s_sleep
#define __pyx_n_s_sleep_c __pyx_mstate_global->__pyx_n_s_sleep_c
#define __pyx_n_s_spiIdx __pyx_mstate_global->__pyx_n_s_spiIdx
#define __pyx_n_s_spiMaster_EndTransaction __pyx_mstate_global->__pyx_n_s_spiMaster_EndTransaction
#define __pyx_n_s_spiMaster_Init __pyx_mstate_global->__pyx_n_s_spiMaster_Init
#define __pyx_n_s_spiMaster_MultiReadWrite __pyx_mstate_global->__pyx_n_s_spiMaster_MultiReadWrite
#define __pyx_n_s_spiMaster_SetLines __pyx_mstate_global->__pyx_n_s_spiMaster_SetLines
#define __pyx_n_s_spiMaster_SingleRead __pyx_mstate_global->__pyx_n_s_spiMaster_SingleRead
#define __pyx_n_s_spiMaster_SingleReadWrite __pyx_mstate_global->__pyx_n_s_spiMaster_SingleReadWrite
#define __pyx_n_s_spiMaster_SingleWrite __pyx_mstate_global->__pyx_n_s_spiMaster_SingleWrite
#define __pyx_n_s_spiSlave_GetRxStatus __pyx_mstate_global->__pyx_n_s_spiSlave_GetRxStatus
#define __pyx_n_s_spiSlave_Init __pyx_mstate_global->__pyx_n_s_spiSlave_Init
#define __pyx_n_s_spiSlave_InitEx __pyx_mstate_global->__pyx_n_s_spiSlave_InitEx
#define __pyx_n_s_spiSlave_Read __pyx_mstate_global->__pyx_n_s_spiSlave_Read
#define __pyx_n_s_spiSlave_SetMode __pyx_mstate_global->__pyx_n_s_spiSlave_SetMode
#define __pyx_n_s_spiSlave_Write __pyx_mstate_global->__pyx_n_s_spiSlave_Write
#define __pyx_n_s_spi_Reset __pyx_mstate_global->__pyx_n_s_spi_Reset
#define __pyx_n_s_spi_ResetTransaction __pyx_mstate_global->__pyx_n_s_spi_ResetTransaction
#define __pyx_n_s_spi_SetDrivingStrength __pyx_mstate_global->__pyx_n_s_spi_SetDrivingStrength
#define __pyx_n_s_ssoMap __pyx_mstate_global->__pyx_n_s_ssoMap
#define __pyx_n_s_ssoStrength __pyx_mstate_global->__pyx_n_s_ssoStrength
#define __pyx_n_s_start __pyx_mstate_global->__pyx_n_s_start
#define __pyx_n_s_status __pyx_mstate_global->__pyx_n_s_status
#define __pyx_n_s_str __pyx_mstate_global->__pyx_n_s_str
#define __pyx_kp_s_stringsource __pyx_mstate_global->__pyx_kp_s_stringsource
#define __pyx_n_s_super __pyx_mstate_global->__pyx_n_s_super
#define __pyx_n_s_t __pyx_mstate_global->__pyx_n_s_t
#define __pyx_n_s_test __pyx_mstate_global->__pyx_n_s_test
#define __pyx_kp_u_the_data_argument_must_be_of_typ __pyx_mstate_global->__pyx_kp_u_the_data_argument_must_be_of_typ
#define __pyx_kp_u_the_multiWrite_argument_must_be __pyx_mstate_global->__pyx_kp_u_the_multiWrite_argument_must_be
#define __pyx_kp_u_the_singleWrite_argument_must_be __pyx_mstate_global->__pyx_kp_u_the_singleWrite_argument_must_be
#define __pyx_n_s_timeout __pyx_mstate_global->__pyx_n_s_timeout
#define __pyx_n_s_timeout_c __pyx_mstate_global->__pyx_n_s_timeout_c
#define __pyx_n_s_trigger __pyx_mstate_global->__pyx_n_s_trigger
#define __pyx_n_u_type __pyx_mstate_global->__pyx_n_u_type
#define __pyx_n_s_update __pyx_mstate_global->__pyx_n_s_update
#define __pyx_kp_u_utf_8 __pyx_mstate_global->__pyx_kp_u_utf_8
#define __pyx_n_s_value __pyx_mstate_global->__pyx_n_s_value
#define __pyx_n_s_vendorCmdGet __pyx_mstate_global->__pyx_n_s_vendorCmdGet
#define __pyx_n_s_vendorCmdSet __pyx_mstate_global->__pyx_n_s_vendorCmdSet
#define __pyx_n_s_weakref __pyx_mstate_global->__pyx_n_s_weakref
#define __pyx_n_s_weakref_finalize __pyx_mstate_global->__pyx_n_s_weakref_finalize
#define __pyx_n_s_write __pyx_mstate_global->__pyx_n_s_write
#define __pyx_n_s_write_timeout __pyx_mstate_global->__pyx_n_s_write_timeout
#define __pyx_n_s_xrange __pyx_mstate_global->__pyx_n_s_xrange
#define __pyx_float_24000000_0 __pyx_mstate_global->__pyx_float_24000000_0
#define __pyx_int_0 __pyx_mstate_global->__pyx_int_0
#define __pyx_int_1 __pyx_mstate_global->__pyx_int_1
#define __pyx_int_2 __pyx_mstate_global->__pyx_int_2
#define __pyx_int_3 __pyx_mstate_global->__pyx_int_3
#define __pyx_int_60 __pyx_mstate_global->__pyx_int_60
#define __pyx_int_82 __pyx_mstate_global->__pyx_int_82
#define __pyx_int_100 __pyx_mstate_global->__pyx_int_100
#define __pyx_int_8000 __pyx_mstate_global->__pyx_int_8000
#define __pyx_int_16908288 __pyx_mstate_global->__pyx_int_16908288
#define __pyx_int_16973824 __pyx_mstate_global->__pyx_int_16973824
#define __pyx_int_17039360 __pyx_mstate_global->__pyx_int_17039360
#define __pyx_int_1109524736 __pyx_mstate_global->__pyx_int_1109524736
#define __pyx_int_1109524992 __pyx_mstate_global->__pyx_int_1109524992
#define __pyx_int_1109525248 __pyx_mstate_global->__pyx_int_1109525248
#define __pyx_int_1109525504 __pyx_mstate_global->__pyx_int_1109525504
#define __pyx_k__2 __pyx_mstate_global->__pyx_k__2
#define __pyx_k__3 __pyx_mstate_global->__pyx_k__3
#define __pyx_k__4 __pyx_mstate_global->__pyx_k__4
#define __pyx_k__5 __pyx_mstate_global->__pyx_k__5
#define __pyx_tuple_ __pyx_mstate_global->__pyx_tuple_
#define __pyx_tuple__6 __pyx_mstate_global->__pyx_tuple__6
#define __pyx_tuple__7 __pyx_mstate_global->__pyx_tuple__7
#define __pyx_tuple__9 __pyx_mstate_global->__pyx_tuple__9
#define __pyx_tuple__11 __pyx_mstate_global->__pyx_tuple__11
#define __pyx_tuple__15 __pyx_mstate_global->__pyx_tuple__15
#define __pyx_tuple__17 __pyx_mstate_global->__pyx_tuple__17
#define __pyx_tuple__19 __pyx_mstate_global->__pyx_tuple__19
#define __pyx_tuple__20 __pyx_mstate_global->__pyx_tuple__20
#define __pyx_tuple__22 __pyx_mstate_global->__pyx_tuple__22
#define __pyx_tuple__24 __pyx_mstate_global->__pyx_tuple__24
#define __pyx_tuple__28 __pyx_mstate_global->__pyx_tuple__28
#define __pyx_tuple__30 __pyx_mstate_global->__pyx_tuple__30
#define __pyx_tuple__32 __pyx_mstate_global->__pyx_tuple__32
#define __pyx_tuple__34 __pyx_mstate_global->__pyx_tuple__34
#define __pyx_tuple__37 __pyx_mstate_global->__pyx_tuple__37
#define __pyx_tuple__40 __pyx_mstate_global->__pyx_tuple__40
#define __pyx_tuple__42 __pyx_mstate_global->__pyx_tuple__42
#define __pyx_tuple__44 __pyx_mstate_global->__pyx_tuple__44
#define __pyx_tuple__46 __pyx_mstate_global->__pyx_tuple__46
#define __pyx_tuple__50 __pyx_mstate_global->__pyx_tuple__50
#define __pyx_tuple__52 __pyx_mstate_global->__pyx_tuple__52
#define __pyx_tuple__54 __pyx_mstate_global->__pyx_tuple__54
#define __pyx_tuple__56 __pyx_mstate_global->__pyx_tuple__56
#define __pyx_tuple__58 __pyx_mstate_global->__pyx_tuple__58
#define __pyx_tuple__60 __pyx_mstate_global->__pyx_tuple__60
#define __pyx_tuple__62 __pyx_mstate_global->__pyx_tuple__62
#define __pyx_tuple__64 __pyx_mstate_global->__pyx_tuple__64
#define __pyx_tuple__65 __pyx_mstate_global->__pyx_tuple__65
#define __pyx_tuple__67 __pyx_mstate_global->__pyx_tuple__67
#define __pyx_tuple__68 __pyx_mstate_global->__pyx_tuple__68
#define __pyx_tuple__70 __pyx_mstate_global->__pyx_tuple__70
#define __pyx_tuple__72 __pyx_mstate_global->__pyx_tuple__72
#define __pyx_tuple__74 __pyx_mstate_global->__pyx_tuple__74
#define __pyx_tuple__77 __pyx_mstate_global->__pyx_tuple__77
#define __pyx_tuple__80 __pyx_mstate_global->__pyx_tuple__80
#define __pyx_tuple__82 __pyx_mstate_global->__pyx_tuple__82
#define __pyx_tuple__84 __pyx_mstate_global->__pyx_tuple__84
#define __pyx_tuple__86 __pyx_mstate_global->__pyx_tuple__86
#define __pyx_tuple__88 __pyx_mstate_global->__pyx_tuple__88
#define __pyx_tuple__90 __pyx_mstate_global->__pyx_tuple__90
#define __pyx_tuple__92 __pyx_mstate_global->__pyx_tuple__92
#define __pyx_tuple__94 __pyx_mstate_global->__pyx_tuple__94
#define __pyx_tuple__96 __pyx_mstate_global->__pyx_tuple__96
#define __pyx_tuple__99 __pyx_mstate_global->__pyx_tuple__99
#define __pyx_tuple__101 __pyx_mstate_global->__pyx_tuple__101
#define __pyx_tuple__103 __pyx_mstate_global->__pyx_tuple__103
#define __pyx_tuple__105 __pyx_mstate_global->__pyx_tuple__105
#define __pyx_tuple__107 __pyx_mstate_global->__pyx_tuple__107
#define __pyx_codeobj__10 __pyx_mstate_global->__pyx_codeobj__10
#define __pyx_codeobj__12 __pyx_mstate_global->__pyx_codeobj__12
#define __pyx_codeobj__13 __pyx_mstate_global->__pyx_codeobj__13
#define __pyx_codeobj__14 __pyx_mstate_global->__pyx_codeobj__14
#define __pyx_codeobj__16 __pyx_mstate_global->__pyx_codeobj__16
#define __pyx_codeobj__18 __pyx_mstate_global->__pyx_codeobj__18
#define __pyx_codeobj__21 __pyx_mstate_global->__pyx_codeobj__21
#define __pyx_codeobj__23 __pyx_mstate_global->__pyx_codeobj__23
#define __pyx_codeobj__25 __pyx_mstate_global->__pyx_codeobj__25
#define __pyx_codeobj__26 __pyx_mstate_global->__pyx_codeobj__26
#define __pyx_codeobj__27 __pyx_mstate_global->__pyx_codeobj__27
#define __pyx_codeobj__29 __pyx_mstate_global->__pyx_codeobj__29
#define __pyx_codeobj__31 __pyx_mstate_global->__pyx_codeobj__31
#define __pyx_codeobj__33 __pyx_mstate_global->__pyx_codeobj__33
#define __pyx_codeobj__35 __pyx_mstate_global->__pyx_codeobj__35
#define __pyx_codeobj__36 __pyx_mstate_global->__pyx_codeobj__36
#define __pyx_codeobj__38 __pyx_mstate_global->__pyx_codeobj__38
#define __pyx_codeobj__39 __pyx_mstate_global->__pyx_codeobj__39
#define __pyx_codeobj__41 __pyx_mstate_global->__pyx_codeobj__41
#define __pyx_codeobj__43 __pyx_mstate_global->__pyx_codeobj__43
#define __pyx_codeobj__45 __pyx_mstate_global->__pyx_codeobj__45
#define __pyx_codeobj__47 __pyx_mstate_global->__pyx_codeobj__47
#define __pyx_codeobj__48 __pyx_mstate_global->__pyx_codeobj__48
#define __pyx_codeobj__49 __pyx_mstate_global->__pyx_codeobj__49
#define __pyx_codeobj__51 __pyx_mstate_global->__pyx_codeobj__51
#define __pyx_codeobj__53 __pyx_mstate_global->__pyx_codeobj__53
#define __pyx_codeobj__55 __pyx_mstate_global->__pyx_codeobj__55
#define __pyx_codeobj__57 __pyx_mstate_global->__pyx_codeobj__57
#define __pyx_codeobj__59 __pyx_mstate_global->__pyx_codeobj__59
#define __pyx_codeobj__61 __pyx_mstate_global->__pyx_codeobj__61
#define __pyx_codeobj__63 __pyx_mstate_global->__pyx_codeobj__63
#define __pyx_codeobj__66 __pyx_mstate_global->__pyx_codeobj__66
#define __pyx_codeobj__69 __pyx_mstate_global->__pyx_codeobj__69
#define __pyx_codeobj__71 __pyx_mstate_global->__pyx_codeobj__71
#define __pyx_codeobj__73 __pyx_mstate_global->__pyx_codeobj__73
#define __pyx_codeobj__75 __pyx_mstate_global->__pyx_codeobj__75
#define __pyx_codeobj__76 __pyx_mstate_global->__pyx_codeobj__76
#define __pyx_codeobj__78 __pyx_mstate_global->__pyx_codeobj__78
#define __pyx_codeobj__79 __pyx_mstate_global->__pyx_codeobj__79
#define __pyx_codeobj__81 __pyx_mstate_global->__pyx_codeobj__81
#define __pyx_codeobj__83 __pyx_mstate_global->__pyx_codeobj__83
#define __pyx_codeobj__85 __pyx_mstate_global->__pyx_codeobj__85
#define __pyx_codeobj__87 __pyx_mstate_global->__pyx_codeobj__87
#define __pyx_codeobj__89 __pyx_mstate_global->__pyx_codeobj__89
#define __pyx_codeobj__91 __pyx_mstate_global->__pyx_codeobj__91
#define __pyx_codeobj__93 __pyx_mstate_global->__pyx_codeobj__93
#define __pyx_codeobj__95 __pyx_mstate_global->__pyx_codeobj__95
#define __pyx_codeobj__97 __pyx_mstate_global->__pyx_codeobj__97
#define __pyx_codeobj__98 __pyx_mstate_global->__pyx_codeobj__98
#define __pyx_codeobj__100 __pyx_mstate_global->__pyx_codeobj__100
#define __pyx_codeobj__102 __pyx_mstate_global->__pyx_codeobj__102
#define __pyx_codeobj__104 __pyx_mstate_global->__pyx_codeobj__104
#define __pyx_codeobj__106 __pyx_mstate_global->__pyx_codeobj__106
#define __pyx_codeobj__108 __pyx_mstate_global->__pyx_codeobj__108
#define __pyx_codeobj__109 __pyx_mstate_global->__pyx_codeobj__109
#define __pyx_codeobj__110 __pyx_mstate_global->__pyx_codeobj__110
/* #### Code section: module_code ### */

/* "cpython/array.pxd":104
 *             __data_union data
 * 
 *         def __getbuffer__(self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fulfill the PEP.
 */

/* Python wrapper */
CYTHON_UNUSED static int __pyx_pw_7cpython_5array_5array_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
CYTHON_UNUSED static int __pyx_pw_7cpython_5array_5array_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7cpython_5array_5array___getbuffer__(((arrayobject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7cpython_5array_5array___getbuffer__(arrayobject *__pyx_v_self, Py_buffer *__pyx_v_info, CYTHON_UNUSED int __pyx_v_flags) {
  PyObject *__pyx_v_item_count = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  char *__pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  char __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  if (unlikely(__pyx_v_info == NULL)) {
    PyErr_SetString(PyExc_BufferError, "PyObject_GetBuffer: view==NULL argument is obsolete");
    return -1;
  }
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(__pyx_v_info->obj);

  /* "cpython/array.pxd":109
 *             # In particular strided access is always provided regardless
 *             # of flags
 *             item_count = Py_SIZE(self)             # <<<<<<<<<<<<<<
 * 
 *             info.suboffsets = NULL
 */
  __pyx_t_1 = PyInt_FromSsize_t(Py_SIZE(((PyObject *)__pyx_v_self))); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_item_count = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cpython/array.pxd":111
 *             item_count = Py_SIZE(self)
 * 
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *             info.buf = self.data.as_chars
 *             info.readonly = 0
 */
  __pyx_v_info->suboffsets = NULL;

  /* "cpython/array.pxd":112
 * 
 *             info.suboffsets = NULL
 *             info.buf = self.data.as_chars             # <<<<<<<<<<<<<<
 *             info.readonly = 0
 *             info.ndim = 1
 */
  __pyx_t_2 = __pyx_v_self->data.as_chars;
  __pyx_v_info->buf = __pyx_t_2;

  /* "cpython/array.pxd":113
 *             info.suboffsets = NULL
 *             info.buf = self.data.as_chars
 *             info.readonly = 0             # <<<<<<<<<<<<<<
 *             info.ndim = 1
 *             info.itemsize = self.ob_descr.itemsize   # e.g. sizeof(float)
 */
  __pyx_v_info->readonly = 0;

  /* "cpython/array.pxd":114
 *             info.buf = self.data.as_chars
 *             info.readonly = 0
 *             info.ndim = 1             # <<<<<<<<<<<<<<
 *             info.itemsize = self.ob_descr.itemsize   # e.g. sizeof(float)
 *             info.len = info.itemsize * item_count
 */
  __pyx_v_info->ndim = 1;

  /* "cpython/array.pxd":115
 *             info.readonly = 0
 *             info.ndim = 1
 *             info.itemsize = self.ob_descr.itemsize   # e.g. sizeof(float)             # <<<<<<<<<<<<<<
 *             info.len = info.itemsize * item_count
 * 
 */
  __pyx_t_3 = __pyx_v_self->ob_descr->itemsize;
  __pyx_v_info->itemsize = __pyx_t_3;

  /* "cpython/array.pxd":116
 *             info.ndim = 1
 *             info.itemsize = self.ob_descr.itemsize   # e.g. sizeof(float)
 *             info.len = info.itemsize * item_count             # <<<<<<<<<<<<<<
 * 
 *             info.shape = <Py_ssize_t*> PyObject_Malloc(sizeof(Py_ssize_t) + 2)
 */
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_info->itemsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyNumber_Multiply(__pyx_t_1, __pyx_v_item_count); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_4); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 116, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_info->len = __pyx_t_5;

  /* "cpython/array.pxd":118
 *             info.len = info.itemsize * item_count
 * 
 *             info.shape = <Py_ssize_t*> PyObject_Malloc(sizeof(Py_ssize_t) + 2)             # <<<<<<<<<<<<<<
 *             if not info.shape:
 *                 raise MemoryError()
 */
  __pyx_v_info->shape = ((Py_ssize_t *)PyObject_Malloc(((sizeof(Py_ssize_t)) + 2)));

  /* "cpython/array.pxd":119
 * 
 *             info.shape = <Py_ssize_t*> PyObject_Malloc(sizeof(Py_ssize_t) + 2)
 *             if not info.shape:             # <<<<<<<<<<<<<<
 *                 raise MemoryError()
 *             info.shape[0] = item_count      # constant regardless of resizing
 */
  __pyx_t_6 = (!(__pyx_v_info->shape != 0));
  if (unlikely(__pyx_t_6)) {

    /* "cpython/array.pxd":120
 *             info.shape = <Py_ssize_t*> PyObject_Malloc(sizeof(Py_ssize_t) + 2)
 *             if not info.shape:
 *                 raise MemoryError()             # <<<<<<<<<<<<<<
 *             info.shape[0] = item_count      # constant regardless of resizing
 *             info.strides = &info.itemsize
 */
    PyErr_NoMemory(); __PYX_ERR(2, 120, __pyx_L1_error)

    /* "cpython/array.pxd":119
 * 
 *             info.shape = <Py_ssize_t*> PyObject_Malloc(sizeof(Py_ssize_t) + 2)
 *             if not info.shape:             # <<<<<<<<<<<<<<
 *                 raise MemoryError()
 *             info.shape[0] = item_count      # constant regardless of resizing
 */
  }

  /* "cpython/array.pxd":121
 *             if not info.shape:
 *                 raise MemoryError()
 *             info.shape[0] = item_count      # constant regardless of resizing             # <<<<<<<<<<<<<<
 *             info.strides = &info.itemsize
 * 
 */
  __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_v_item_count); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 121, __pyx_L1_error)
  (__pyx_v_info->shape[0]) = __pyx_t_5;

  /* "cpython/array.pxd":122
 *                 raise MemoryError()
 *             info.shape[0] = item_count      # constant regardless of resizing
 *             info.strides = &info.itemsize             # <<<<<<<<<<<<<<
 * 
 *             info.format = <char*> (info.shape + 1)
 */
  __pyx_v_info->strides = (&__pyx_v_info->itemsize);

  /* "cpython/array.pxd":124
 *             info.strides = &info.itemsize
 * 
 *             info.format = <char*> (info.shape + 1)             # <<<<<<<<<<<<<<
 *             info.format[0] = self.ob_descr.typecode
 *             info.format[1] = 0
 */
  __pyx_v_info->format = ((char *)(__pyx_v_info->shape + 1));

  /* "cpython/array.pxd":125
 * 
 *             info.format = <char*> (info.shape + 1)
 *             info.format[0] = self.ob_descr.typecode             # <<<<<<<<<<<<<<
 *             info.format[1] = 0
 *             info.obj = self
 */
  __pyx_t_7 = __pyx_v_self->ob_descr->typecode;
  (__pyx_v_info->format[0]) = __pyx_t_7;

  /* "cpython/array.pxd":126
 *             info.format = <char*> (info.shape + 1)
 *             info.format[0] = self.ob_descr.typecode
 *             info.format[1] = 0             # <<<<<<<<<<<<<<
 *             info.obj = self
 * 
 */
  (__pyx_v_info->format[1]) = 0;

  /* "cpython/array.pxd":127
 *             info.format[0] = self.ob_descr.typecode
 *             info.format[1] = 0
 *             info.obj = self             # <<<<<<<<<<<<<<
 * 
 *         def __releasebuffer__(self, Py_buffer* info):
 */
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self);
  __Pyx_GOTREF(__pyx_v_info->obj);
  __Pyx_DECREF(__pyx_v_info->obj);
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "cpython/array.pxd":104
 *             __data_union data
 * 
 *         def __getbuffer__(self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fulfill the PEP.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cpython.array.array.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  __pyx_L2:;
  __Pyx_XDECREF(__pyx_v_item_count);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cpython/array.pxd":129
 *             info.obj = self
 * 
 *         def __releasebuffer__(self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             PyObject_Free(info.shape)
 * 
 */

/* Python wrapper */
CYTHON_UNUSED static void __pyx_pw_7cpython_5array_5array_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info); /*proto*/
CYTHON_UNUSED static void __pyx_pw_7cpython_5array_5array_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__releasebuffer__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_7cpython_5array_5array_2__releasebuffer__(((arrayobject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_7cpython_5array_5array_2__releasebuffer__(CYTHON_UNUSED arrayobject *__pyx_v_self, Py_buffer *__pyx_v_info) {

  /* "cpython/array.pxd":130
 * 
 *         def __releasebuffer__(self, Py_buffer* info):
 *             PyObject_Free(info.shape)             # <<<<<<<<<<<<<<
 * 
 *     array newarrayobject(PyTypeObject* type, Py_ssize_t size, arraydescr *descr)
 */
  PyObject_Free(__pyx_v_info->shape);

  /* "cpython/array.pxd":129
 *             info.obj = self
 * 
 *         def __releasebuffer__(self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             PyObject_Free(info.shape)
 * 
 */

  /* function exit code */
}

/* "cpython/array.pxd":141
 * 
 * 
 * cdef inline array clone(array template, Py_ssize_t length, bint zero):             # <<<<<<<<<<<<<<
 *     """ fast creation of a new array, given a template array.
 *     type will be same as template.
 */

static CYTHON_INLINE arrayobject *__pyx_f_7cpython_5array_clone(arrayobject *__pyx_v_template, Py_ssize_t __pyx_v_length, int __pyx_v_zero) {
  arrayobject *__pyx_v_op = 0;
  arrayobject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 1);

  /* "cpython/array.pxd":145
 *     type will be same as template.
 *     if zero is true, new array will be initialized with zeroes."""
 *     cdef array op = newarrayobject(Py_TYPE(template), length, template.ob_descr)             # <<<<<<<<<<<<<<
 *     if zero and op is not None:
 *         memset(op.data.as_chars, 0, length * op.ob_descr.itemsize)
 */
  __pyx_t_1 = ((PyObject *)newarrayobject(Py_TYPE(((PyObject *)__pyx_v_template)), __pyx_v_length, __pyx_v_template->ob_descr)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_op = ((arrayobject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cpython/array.pxd":146
 *     if zero is true, new array will be initialized with zeroes."""
 *     cdef array op = newarrayobject(Py_TYPE(template), length, template.ob_descr)
 *     if zero and op is not None:             # <<<<<<<<<<<<<<
 *         memset(op.data.as_chars, 0, length * op.ob_descr.itemsize)
 *     return op
 */
  if (__pyx_v_zero) {
  } else {
    __pyx_t_2 = __pyx_v_zero;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (((PyObject *)__pyx_v_op) != Py_None);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "cpython/array.pxd":147
 *     cdef array op = newarrayobject(Py_TYPE(template), length, template.ob_descr)
 *     if zero and op is not None:
 *         memset(op.data.as_chars, 0, length * op.ob_descr.itemsize)             # <<<<<<<<<<<<<<
 *     return op
 * 
 */
    (void)(memset(__pyx_v_op->data.as_chars, 0, (__pyx_v_length * __pyx_v_op->ob_descr->itemsize)));

    /* "cpython/array.pxd":146
 *     if zero is true, new array will be initialized with zeroes."""
 *     cdef array op = newarrayobject(Py_TYPE(template), length, template.ob_descr)
 *     if zero and op is not None:             # <<<<<<<<<<<<<<
 *         memset(op.data.as_chars, 0, length * op.ob_descr.itemsize)
 *     return op
 */
  }

  /* "cpython/array.pxd":148
 *     if zero and op is not None:
 *         memset(op.data.as_chars, 0, length * op.ob_descr.itemsize)
 *     return op             # <<<<<<<<<<<<<<
 * 
 * cdef inline array copy(array self):
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_op);
  __pyx_r = __pyx_v_op;
  goto __pyx_L0;

  /* "cpython/array.pxd":141
 * 
 * 
 * cdef inline array clone(array template, Py_ssize_t length, bint zero):             # <<<<<<<<<<<<<<
 *     """ fast creation of a new array, given a template array.
 *     type will be same as template.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cpython.array.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_op);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cpython/array.pxd":150
 *     return op
 * 
 * cdef inline array copy(array self):             # <<<<<<<<<<<<<<
 *     """ make a copy of an array. """
 *     cdef array op = newarrayobject(Py_TYPE(self), Py_SIZE(self), self.ob_descr)
 */

static CYTHON_INLINE arrayobject *__pyx_f_7cpython_5array_copy(arrayobject *__pyx_v_self) {
  arrayobject *__pyx_v_op = 0;
  arrayobject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy", 1);

  /* "cpython/array.pxd":152
 * cdef inline array copy(array self):
 *     """ make a copy of an array. """
 *     cdef array op = newarrayobject(Py_TYPE(self), Py_SIZE(self), self.ob_descr)             # <<<<<<<<<<<<<<
 *     memcpy(op.data.as_chars, self.data.as_chars, Py_SIZE(op) * op.ob_descr.itemsize)
 *     return op
 */
  __pyx_t_1 = ((PyObject *)newarrayobject(Py_TYPE(((PyObject *)__pyx_v_self)), Py_SIZE(((PyObject *)__pyx_v_self)), __pyx_v_self->ob_descr)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_op = ((arrayobject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cpython/array.pxd":153
 *     """ make a copy of an array. """
 *     cdef array op = newarrayobject(Py_TYPE(self), Py_SIZE(self), self.ob_descr)
 *     memcpy(op.data.as_chars, self.data.as_chars, Py_SIZE(op) * op.ob_descr.itemsize)             # <<<<<<<<<<<<<<
 *     return op
 * 
 */
  (void)(memcpy(__pyx_v_op->data.as_chars, __pyx_v_self->data.as_chars, (Py_SIZE(((PyObject *)__pyx_v_op)) * __pyx_v_op->ob_descr->itemsize)));

  /* "cpython/array.pxd":154
 *     cdef array op = newarrayobject(Py_TYPE(self), Py_SIZE(self), self.ob_descr)
 *     memcpy(op.data.as_chars, self.data.as_chars, Py_SIZE(op) * op.ob_descr.itemsize)
 *     return op             # <<<<<<<<<<<<<<
 * 
 * cdef inline int extend_buffer(array self, char* stuff, Py_ssize_t n) except -1:
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_op);
  __pyx_r = __pyx_v_op;
  goto __pyx_L0;

  /* "cpython/array.pxd":150
 *     return op
 * 
 * cdef inline array copy(array self):             # <<<<<<<<<<<<<<
 *     """ make a copy of an array. """
 *     cdef array op = newarrayobject(Py_TYPE(self), Py_SIZE(self), self.ob_descr)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cpython.array.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_op);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cpython/array.pxd":156
 *     return op
 * 
 * cdef inline int extend_buffer(array self, char* stuff, Py_ssize_t n) except -1:             # <<<<<<<<<<<<<<
 *     """ efficient appending of new stuff of same type
 *     (e.g. of same array type)
 */

static CYTHON_INLINE int __pyx_f_7cpython_5array_extend_buffer(arrayobject *__pyx_v_self, char *__pyx_v_stuff, Py_ssize_t __pyx_v_n) {
  Py_ssize_t __pyx_v_itemsize;
  Py_ssize_t __pyx_v_origsize;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "cpython/array.pxd":160
 *     (e.g. of same array type)
 *     n: number of elements (not number of bytes!) """
 *     cdef Py_ssize_t itemsize = self.ob_descr.itemsize             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t origsize = Py_SIZE(self)
 *     resize_smart(self, origsize + n)
 */
  __pyx_t_1 = __pyx_v_self->ob_descr->itemsize;
  __pyx_v_itemsize = __pyx_t_1;

  /* "cpython/array.pxd":161
 *     n: number of elements (not number of bytes!) """
 *     cdef Py_ssize_t itemsize = self.ob_descr.itemsize
 *     cdef Py_ssize_t origsize = Py_SIZE(self)             # <<<<<<<<<<<<<<
 *     resize_smart(self, origsize + n)
 *     memcpy(self.data.as_chars + origsize * itemsize, stuff, n * itemsize)
 */
  __pyx_v_origsize = Py_SIZE(((PyObject *)__pyx_v_self));

  /* "cpython/array.pxd":162
 *     cdef Py_ssize_t itemsize = self.ob_descr.itemsize
 *     cdef Py_ssize_t origsize = Py_SIZE(self)
 *     resize_smart(self, origsize + n)             # <<<<<<<<<<<<<<
 *     memcpy(self.data.as_chars + origsize * itemsize, stuff, n * itemsize)
 *     return 0
 */
  __pyx_t_1 = resize_smart(__pyx_v_self, (__pyx_v_origsize + __pyx_v_n)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 162, __pyx_L1_error)

  /* "cpython/array.pxd":163
 *     cdef Py_ssize_t origsize = Py_SIZE(self)
 *     resize_smart(self, origsize + n)
 *     memcpy(self.data.as_chars + origsize * itemsize, stuff, n * itemsize)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  (void)(memcpy((__pyx_v_self->data.as_chars + (__pyx_v_origsize * __pyx_v_itemsize)), __pyx_v_stuff, (__pyx_v_n * __pyx_v_itemsize)));

  /* "cpython/array.pxd":164
 *     resize_smart(self, origsize + n)
 *     memcpy(self.data.as_chars + origsize * itemsize, stuff, n * itemsize)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef inline int extend(array self, array other) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "cpython/array.pxd":156
 *     return op
 * 
 * cdef inline int extend_buffer(array self, char* stuff, Py_ssize_t n) except -1:             # <<<<<<<<<<<<<<
 *     """ efficient appending of new stuff of same type
 *     (e.g. of same array type)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cpython.array.extend_buffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "cpython/array.pxd":166
 *     return 0
 * 
 * cdef inline int extend(array self, array other) except -1:             # <<<<<<<<<<<<<<
 *     """ extend array with data from another array; types must match. """
 *     if self.ob_descr.typecode != other.ob_descr.typecode:
 */

static CYTHON_INLINE int __pyx_f_7cpython_5array_extend(arrayobject *__pyx_v_self, arrayobject *__pyx_v_other) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "cpython/array.pxd":168
 * cdef inline int extend(array self, array other) except -1:
 *     """ extend array with data from another array; types must match. """
 *     if self.ob_descr.typecode != other.ob_descr.typecode:             # <<<<<<<<<<<<<<
 *         PyErr_BadArgument()
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))
 */
  __pyx_t_1 = (__pyx_v_self->ob_descr->typecode != __pyx_v_other->ob_descr->typecode);
  if (__pyx_t_1) {

    /* "cpython/array.pxd":169
 *     """ extend array with data from another array; types must match. """
 *     if self.ob_descr.typecode != other.ob_descr.typecode:
 *         PyErr_BadArgument()             # <<<<<<<<<<<<<<
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))
 * 
 */
    __pyx_t_2 = PyErr_BadArgument(); if (unlikely(__pyx_t_2 == ((int)0))) __PYX_ERR(2, 169, __pyx_L1_error)

    /* "cpython/array.pxd":168
 * cdef inline int extend(array self, array other) except -1:
 *     """ extend array with data from another array; types must match. """
 *     if self.ob_descr.typecode != other.ob_descr.typecode:             # <<<<<<<<<<<<<<
 *         PyErr_BadArgument()
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))
 */
  }

  /* "cpython/array.pxd":170
 *     if self.ob_descr.typecode != other.ob_descr.typecode:
 *         PyErr_BadArgument()
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))             # <<<<<<<<<<<<<<
 * 
 * cdef inline void zero(array self):
 */
  __pyx_t_2 = __pyx_f_7cpython_5array_extend_buffer(__pyx_v_self, __pyx_v_other->data.as_chars, Py_SIZE(((PyObject *)__pyx_v_other))); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 170, __pyx_L1_error)
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "cpython/array.pxd":166
 *     return 0
 * 
 * cdef inline int extend(array self, array other) except -1:             # <<<<<<<<<<<<<<
 *     """ extend array with data from another array; types must match. """
 *     if self.ob_descr.typecode != other.ob_descr.typecode:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cpython.array.extend", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "cpython/array.pxd":172
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))
 * 
 * cdef inline void zero(array self):             # <<<<<<<<<<<<<<
 *     """ set all elements of array to zero. """
 *     memset(self.data.as_chars, 0, Py_SIZE(self) * self.ob_descr.itemsize)
 */

static CYTHON_INLINE void __pyx_f_7cpython_5array_zero(arrayobject *__pyx_v_self) {

  /* "cpython/array.pxd":174
 * cdef inline void zero(array self):
 *     """ set all elements of array to zero. """
 *     memset(self.data.as_chars, 0, Py_SIZE(self) * self.ob_descr.itemsize)             # <<<<<<<<<<<<<<
 */
  (void)(memset(__pyx_v_self->data.as_chars, 0, (Py_SIZE(((PyObject *)__pyx_v_self)) * __pyx_v_self->ob_descr->itemsize)));

  /* "cpython/array.pxd":172
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))
 * 
 * cdef inline void zero(array self):             # <<<<<<<<<<<<<<
 *     """ set all elements of array to zero. """
 *     memset(self.data.as_chars, 0, Py_SIZE(self) * self.ob_descr.itemsize)
 */

  /* function exit code */
}

/* "ft4222/ft4222.pyx":50
 *                  'NOT_SUPPORTED', 'OTHER_ERROR', 'DEVICE_LIST_NOT_READY']
 * 
 *     def __init__(self, msgnum):             # <<<<<<<<<<<<<<
 *         self.message = self.__ftd2xx_msgs[msgnum]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_16FT2XXDeviceError_1__init__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_16FT2XXDeviceError_1__init__ = {"__init__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_16FT2XXDeviceError_1__init__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6ft4222_6ft4222_16FT2XXDeviceError_1__init__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_msgnum = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_msgnum,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 50, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_msgnum)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 50, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 50, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__init__") < 0)) __PYX_ERR(0, 50, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_msgnum = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 50, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT2XXDeviceError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_16FT2XXDeviceError___init__(__pyx_self, __pyx_v_self, __pyx_v_msgnum);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_16FT2XXDeviceError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_msgnum) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 1);

  /* "ft4222/ft4222.pyx":51
 * 
 *     def __init__(self, msgnum):
 *         self.message = self.__ftd2xx_msgs[msgnum]             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_FT2XXDeviceError__ftd2xx_msgs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_v_msgnum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_message, __pyx_t_2) < 0) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":50
 *                  'NOT_SUPPORTED', 'OTHER_ERROR', 'DEVICE_LIST_NOT_READY']
 * 
 *     def __init__(self, msgnum):             # <<<<<<<<<<<<<<
 *         self.message = self.__ftd2xx_msgs[msgnum]
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ft4222.ft4222.FT2XXDeviceError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":53
 *         self.message = self.__ftd2xx_msgs[msgnum]
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_16FT2XXDeviceError_3__str__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_16FT2XXDeviceError_3__str__ = {"__str__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_16FT2XXDeviceError_3__str__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6ft4222_6ft4222_16FT2XXDeviceError_3__str__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 53, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__str__") < 0)) __PYX_ERR(0, 53, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_self = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__str__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 53, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT2XXDeviceError.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_16FT2XXDeviceError_2__str__(__pyx_self, __pyx_v_self);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_16FT2XXDeviceError_2__str__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 1);

  /* "ft4222/ft4222.pyx":54
 * 
 *     def __str__(self):
 *         return self.message             # <<<<<<<<<<<<<<
 * 
 * class FT4222DeviceError(FT2XXDeviceError):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_message); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ft4222/ft4222.pyx":53
 *         self.message = self.__ftd2xx_msgs[msgnum]
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ft4222.ft4222.FT2XXDeviceError.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":69
 *                   'GPIO_INPUT_NOT_SUPPORTED', 'EVENT_NOT_SUPPORTED', 'FUN_NOT_SUPPORT']
 * 
 *     def __init__(self, msgnum):             # <<<<<<<<<<<<<<
 *         if msgnum >= FT4222_DEVICE_NOT_SUPPORTED:
 *             self.message = self.__ftd4222_msgs[msgnum - FT4222_DEVICE_NOT_SUPPORTED]
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_17FT4222DeviceError_1__init__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_17FT4222DeviceError_1__init__ = {"__init__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_17FT4222DeviceError_1__init__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6ft4222_6ft4222_17FT4222DeviceError_1__init__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_msgnum = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_msgnum,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 69, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_msgnum)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 69, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 69, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__init__") < 0)) __PYX_ERR(0, 69, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_msgnum = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 69, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222DeviceError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_17FT4222DeviceError___init__(__pyx_self, __pyx_v_self, __pyx_v_msgnum);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_17FT4222DeviceError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_msgnum) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 1);

  /* "ft4222/ft4222.pyx":70
 * 
 *     def __init__(self, msgnum):
 *         if msgnum >= FT4222_DEVICE_NOT_SUPPORTED:             # <<<<<<<<<<<<<<
 *             self.message = self.__ftd4222_msgs[msgnum - FT4222_DEVICE_NOT_SUPPORTED]
 *         else:
 */
  __pyx_t_1 = __Pyx_PyInt_From_FT4222_STATUS(FT4222_DEVICE_NOT_SUPPORTED); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_msgnum, __pyx_t_1, Py_GE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "ft4222/ft4222.pyx":71
 *     def __init__(self, msgnum):
 *         if msgnum >= FT4222_DEVICE_NOT_SUPPORTED:
 *             self.message = self.__ftd4222_msgs[msgnum - FT4222_DEVICE_NOT_SUPPORTED]             # <<<<<<<<<<<<<<
 *         else:
 *             super(FT4222DeviceError, self).__init__(msgnum)
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_FT4222DeviceError__ftd4222_msgs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 71, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyInt_From_FT4222_STATUS(FT4222_DEVICE_NOT_SUPPORTED); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 71, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyNumber_Subtract(__pyx_v_msgnum, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 71, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 71, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_message, __pyx_t_1) < 0) __PYX_ERR(0, 71, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ft4222/ft4222.pyx":70
 * 
 *     def __init__(self, msgnum):
 *         if msgnum >= FT4222_DEVICE_NOT_SUPPORTED:             # <<<<<<<<<<<<<<
 *             self.message = self.__ftd4222_msgs[msgnum - FT4222_DEVICE_NOT_SUPPORTED]
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":73
 *             self.message = self.__ftd4222_msgs[msgnum - FT4222_DEVICE_NOT_SUPPORTED]
 *         else:
 *             super(FT4222DeviceError, self).__init__(msgnum)             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 73, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 73, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4)) __PYX_ERR(0, 73, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_self);
    __Pyx_GIVEREF(__pyx_v_self);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_self)) __PYX_ERR(0, 73, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 73, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 73, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    __pyx_t_5 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_5 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_msgnum};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 73, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":69
 *                   'GPIO_INPUT_NOT_SUPPORTED', 'EVENT_NOT_SUPPORTED', 'FUN_NOT_SUPPORT']
 * 
 *     def __init__(self, msgnum):             # <<<<<<<<<<<<<<
 *         if msgnum >= FT4222_DEVICE_NOT_SUPPORTED:
 *             self.message = self.__ftd4222_msgs[msgnum - FT4222_DEVICE_NOT_SUPPORTED]
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222DeviceError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":75
 *             super(FT4222DeviceError, self).__init__(msgnum)
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_17FT4222DeviceError_3__str__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_17FT4222DeviceError_3__str__ = {"__str__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_17FT4222DeviceError_3__str__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6ft4222_6ft4222_17FT4222DeviceError_3__str__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 75, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__str__") < 0)) __PYX_ERR(0, 75, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_self = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__str__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 75, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222DeviceError.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_17FT4222DeviceError_2__str__(__pyx_self, __pyx_v_self);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_17FT4222DeviceError_2__str__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 1);

  /* "ft4222/ft4222.pyx":76
 * 
 *     def __str__(self):
 *         return self.message             # <<<<<<<<<<<<<<
 * 
 * class SysClock(IntEnum):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_message); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ft4222/ft4222.pyx":75
 *             super(FT4222DeviceError, self).__init__(msgnum)
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222DeviceError.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":93
 *     CLK_80 = 3
 * 
 * def createDeviceInfoList():             # <<<<<<<<<<<<<<
 *     """Create the internal device info list and return number of entries"""
 *     cdef DWORD nb
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_1createDeviceInfoList(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_createDeviceInfoList, "Create the internal device info list and return number of entries");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_1createDeviceInfoList = {"createDeviceInfoList", (PyCFunction)__pyx_pw_6ft4222_6ft4222_1createDeviceInfoList, METH_NOARGS, __pyx_doc_6ft4222_6ft4222_createDeviceInfoList};
static PyObject *__pyx_pw_6ft4222_6ft4222_1createDeviceInfoList(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("createDeviceInfoList (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_createDeviceInfoList(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_createDeviceInfoList(CYTHON_UNUSED PyObject *__pyx_self) {
  DWORD __pyx_v_nb;
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("createDeviceInfoList", 1);

  /* "ft4222/ft4222.pyx":96
 *     """Create the internal device info list and return number of entries"""
 *     cdef DWORD nb
 *     status = FT_CreateDeviceInfoList(&nb)             # <<<<<<<<<<<<<<
 *     if status == FT_OK:
 *         return nb
 */
  __pyx_v_status = FT_CreateDeviceInfoList((&__pyx_v_nb));

  /* "ft4222/ft4222.pyx":97
 *     cdef DWORD nb
 *     status = FT_CreateDeviceInfoList(&nb)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         return nb
 *     raise FT2XXDeviceError, status
 */
  __pyx_t_1 = (__pyx_v_status == FT_OK);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":98
 *     status = FT_CreateDeviceInfoList(&nb)
 *     if status == FT_OK:
 *         return nb             # <<<<<<<<<<<<<<
 *     raise FT2XXDeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_DWORD(__pyx_v_nb); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 98, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":97
 *     cdef DWORD nb
 *     status = FT_CreateDeviceInfoList(&nb)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         return nb
 *     raise FT2XXDeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":99
 *     if status == FT_OK:
 *         return nb
 *     raise FT2XXDeviceError, status             # <<<<<<<<<<<<<<
 * 
 * def getDeviceInfoDetail(devnum=0, update=True):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT2XXDeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 99, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":93
 *     CLK_80 = 3
 * 
 * def createDeviceInfoList():             # <<<<<<<<<<<<<<
 *     """Create the internal device info list and return number of entries"""
 *     cdef DWORD nb
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.createDeviceInfoList", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":101
 *     raise FT2XXDeviceError, status
 * 
 * def getDeviceInfoDetail(devnum=0, update=True):             # <<<<<<<<<<<<<<
 *     """Get an entry from the internal device info list. Set update to
 *     False to avoid a slow call to createDeviceInfoList."""
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_3getDeviceInfoDetail(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_2getDeviceInfoDetail, "Get an entry from the internal device info list. Set update to\n    False to avoid a slow call to createDeviceInfoList.");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_3getDeviceInfoDetail = {"getDeviceInfoDetail", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_3getDeviceInfoDetail, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_2getDeviceInfoDetail};
static PyObject *__pyx_pw_6ft4222_6ft4222_3getDeviceInfoDetail(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_devnum = 0;
  PyObject *__pyx_v_update = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getDeviceInfoDetail (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_devnum,&__pyx_n_s_update,0};
    values[0] = __Pyx_Arg_NewRef_FASTCALL(((PyObject *)((PyObject *)__pyx_int_0)));
    values[1] = __Pyx_Arg_NewRef_FASTCALL(((PyObject *)((PyObject *)Py_True)));
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_devnum);
          if (value) { values[0] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 101, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_update);
          if (value) { values[1] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 101, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "getDeviceInfoDetail") < 0)) __PYX_ERR(0, 101, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_devnum = values[0];
    __pyx_v_update = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("getDeviceInfoDetail", 0, 0, 2, __pyx_nargs); __PYX_ERR(0, 101, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.getDeviceInfoDetail", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_2getDeviceInfoDetail(__pyx_self, __pyx_v_devnum, __pyx_v_update);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_2getDeviceInfoDetail(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_devnum, PyObject *__pyx_v_update) {
  DWORD __pyx_v_f;
  DWORD __pyx_v_t;
  DWORD __pyx_v_i;
  DWORD __pyx_v_l;
  FT_HANDLE __pyx_v_h;
  char __pyx_v_n[MAX_DESCRIPTION_SIZE];
  char __pyx_v_d[MAX_DESCRIPTION_SIZE];
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  DWORD __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getDeviceInfoDetail", 1);

  /* "ft4222/ft4222.pyx":110
 *         char d[MAX_DESCRIPTION_SIZE]
 *     # createDeviceInfoList is slow, only run if update is True
 *     if update: createDeviceInfoList()             # <<<<<<<<<<<<<<
 *     status = FT_GetDeviceInfoDetail(devnum, &f, &t, &i, &l, n, d, &h)
 *     if status == FT_OK:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_update); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 110, __pyx_L1_error)
  if (__pyx_t_1) {
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_createDeviceInfoList); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    __pyx_t_5 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_5 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 0+__pyx_t_5);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 110, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }

  /* "ft4222/ft4222.pyx":111
 *     # createDeviceInfoList is slow, only run if update is True
 *     if update: createDeviceInfoList()
 *     status = FT_GetDeviceInfoDetail(devnum, &f, &t, &i, &l, n, d, &h)             # <<<<<<<<<<<<<<
 *     if status == FT_OK:
 *         return {'index': devnum, 'flags': f, 'type': t,
 */
  __pyx_t_6 = __Pyx_PyInt_As_DWORD(__pyx_v_devnum); if (unlikely((__pyx_t_6 == ((DWORD)-1)) && PyErr_Occurred())) __PYX_ERR(0, 111, __pyx_L1_error)
  __pyx_v_status = FT_GetDeviceInfoDetail(__pyx_t_6, (&__pyx_v_f), (&__pyx_v_t), (&__pyx_v_i), (&__pyx_v_l), __pyx_v_n, __pyx_v_d, (&__pyx_v_h));

  /* "ft4222/ft4222.pyx":112
 *     if update: createDeviceInfoList()
 *     status = FT_GetDeviceInfoDetail(devnum, &f, &t, &i, &l, n, d, &h)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         return {'index': devnum, 'flags': f, 'type': t,
 *                 'id': i, 'location': l, 'serial': n,
 */
  __pyx_t_1 = (__pyx_v_status == FT_OK);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":113
 *     status = FT_GetDeviceInfoDetail(devnum, &f, &t, &i, &l, n, d, &h)
 *     if status == FT_OK:
 *         return {'index': devnum, 'flags': f, 'type': t,             # <<<<<<<<<<<<<<
 *                 'id': i, 'location': l, 'serial': n,
 *                 'description': d, 'handle': <size_t>h}
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyDict_NewPresized(8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_index, __pyx_v_devnum) < 0) __PYX_ERR(0, 113, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyInt_From_DWORD(__pyx_v_f); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_flags, __pyx_t_3) < 0) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyInt_From_DWORD(__pyx_v_t); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_type, __pyx_t_3) < 0) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "ft4222/ft4222.pyx":114
 *     if status == FT_OK:
 *         return {'index': devnum, 'flags': f, 'type': t,
 *                 'id': i, 'location': l, 'serial': n,             # <<<<<<<<<<<<<<
 *                 'description': d, 'handle': <size_t>h}
 *     raise FT2XXDeviceError, status
 */
    __pyx_t_3 = __Pyx_PyInt_From_DWORD(__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 114, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_id, __pyx_t_3) < 0) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyInt_From_DWORD(__pyx_v_l); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 114, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_location, __pyx_t_3) < 0) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_FromString(__pyx_v_n); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 114, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_serial, __pyx_t_3) < 0) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "ft4222/ft4222.pyx":115
 *         return {'index': devnum, 'flags': f, 'type': t,
 *                 'id': i, 'location': l, 'serial': n,
 *                 'description': d, 'handle': <size_t>h}             # <<<<<<<<<<<<<<
 *     raise FT2XXDeviceError, status
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_FromString(__pyx_v_d); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 115, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_description, __pyx_t_3) < 0) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyInt_FromSize_t(((size_t)__pyx_v_h)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 115, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_handle, __pyx_t_3) < 0) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":112
 *     if update: createDeviceInfoList()
 *     status = FT_GetDeviceInfoDetail(devnum, &f, &t, &i, &l, n, d, &h)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         return {'index': devnum, 'flags': f, 'type': t,
 *                 'id': i, 'location': l, 'serial': n,
 */
  }

  /* "ft4222/ft4222.pyx":116
 *                 'id': i, 'location': l, 'serial': n,
 *                 'description': d, 'handle': <size_t>h}
 *     raise FT2XXDeviceError, status             # <<<<<<<<<<<<<<
 * 
 * def openBySerial(serial):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT2XXDeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 116, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":101
 *     raise FT2XXDeviceError, status
 * 
 * def getDeviceInfoDetail(devnum=0, update=True):             # <<<<<<<<<<<<<<
 *     """Get an entry from the internal device info list. Set update to
 *     False to avoid a slow call to createDeviceInfoList."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.getDeviceInfoDetail", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":118
 *     raise FT2XXDeviceError, status
 * 
 * def openBySerial(serial):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by serial number
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_5openBySerial(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_4openBySerial, "Open a handle to a usb device by serial number\n\n    Args:\n        serial (bytes, str): Serial number\n\n    Returns:\n        :obj:`FT4222`: Opened device\n\n    Raises:\n        FT2XXDeviceError: on error\n\n    ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_5openBySerial = {"openBySerial", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_5openBySerial, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_4openBySerial};
static PyObject *__pyx_pw_6ft4222_6ft4222_5openBySerial(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_serial = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("openBySerial (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_serial,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_serial)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 118, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "openBySerial") < 0)) __PYX_ERR(0, 118, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_serial = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("openBySerial", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 118, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.openBySerial", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_4openBySerial(__pyx_self, __pyx_v_serial);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_4openBySerial(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_serial) {
  FT_HANDLE __pyx_v_handle;
  char *__pyx_v_cserial;
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  char *__pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("openBySerial", 0);
  __Pyx_INCREF(__pyx_v_serial);

  /* "ft4222/ft4222.pyx":131
 * 
 *     """
 *     if isinstance(serial, str):             # <<<<<<<<<<<<<<
 *         serial = serial.encode('utf-8')
 *     cdef FT_HANDLE handle
 */
  __pyx_t_1 = PyUnicode_Check(__pyx_v_serial); 
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":132
 *     """
 *     if isinstance(serial, str):
 *         serial = serial.encode('utf-8')             # <<<<<<<<<<<<<<
 *     cdef FT_HANDLE handle
 *     cdef char* cserial = serial
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_serial, __pyx_n_s_encode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 132, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    __pyx_t_5 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_5 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_kp_u_utf_8};
      __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 132, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF_SET(__pyx_v_serial, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "ft4222/ft4222.pyx":131
 * 
 *     """
 *     if isinstance(serial, str):             # <<<<<<<<<<<<<<
 *         serial = serial.encode('utf-8')
 *     cdef FT_HANDLE handle
 */
  }

  /* "ft4222/ft4222.pyx":134
 *         serial = serial.encode('utf-8')
 *     cdef FT_HANDLE handle
 *     cdef char* cserial = serial             # <<<<<<<<<<<<<<
 *     status = FT_OpenEx(<PVOID>cserial, FT_OPEN_BY_SERIAL_NUMBER, &handle)
 *     if status == FT_OK:
 */
  __pyx_t_6 = __Pyx_PyObject_AsWritableString(__pyx_v_serial); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 134, __pyx_L1_error)
  __pyx_v_cserial = __pyx_t_6;

  /* "ft4222/ft4222.pyx":135
 *     cdef FT_HANDLE handle
 *     cdef char* cserial = serial
 *     status = FT_OpenEx(<PVOID>cserial, FT_OPEN_BY_SERIAL_NUMBER, &handle)             # <<<<<<<<<<<<<<
 *     if status == FT_OK:
 *         return FT4222(<uintptr_t>handle, update=False)
 */
  __pyx_v_status = FT_OpenEx(((PVOID)__pyx_v_cserial), FT_OPEN_BY_SERIAL_NUMBER, (&__pyx_v_handle));

  /* "ft4222/ft4222.pyx":136
 *     cdef char* cserial = serial
 *     status = FT_OpenEx(<PVOID>cserial, FT_OPEN_BY_SERIAL_NUMBER, &handle)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         return FT4222(<uintptr_t>handle, update=False)
 *     raise FT2XXDeviceError, status
 */
  __pyx_t_1 = (__pyx_v_status == FT_OK);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":137
 *     status = FT_OpenEx(<PVOID>cserial, FT_OPEN_BY_SERIAL_NUMBER, &handle)
 *     if status == FT_OK:
 *         return FT4222(<uintptr_t>handle, update=False)             # <<<<<<<<<<<<<<
 *     raise FT2XXDeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_FromSize_t(((uintptr_t)__pyx_v_handle)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 137, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 137, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2)) __PYX_ERR(0, 137, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 137, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_update, Py_False) < 0) __PYX_ERR(0, 137, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222), __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 137, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":136
 *     cdef char* cserial = serial
 *     status = FT_OpenEx(<PVOID>cserial, FT_OPEN_BY_SERIAL_NUMBER, &handle)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         return FT4222(<uintptr_t>handle, update=False)
 *     raise FT2XXDeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":138
 *     if status == FT_OK:
 *         return FT4222(<uintptr_t>handle, update=False)
 *     raise FT2XXDeviceError, status             # <<<<<<<<<<<<<<
 * 
 * def openByDescription(desc):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT2XXDeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_Raise(__pyx_t_4, __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 138, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":118
 *     raise FT2XXDeviceError, status
 * 
 * def openBySerial(serial):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by serial number
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.openBySerial", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_serial);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":140
 *     raise FT2XXDeviceError, status
 * 
 * def openByDescription(desc):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by description
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_7openByDescription(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6openByDescription, "Open a handle to a usb device by description\n\n    Args:\n        desc (bytes, str): Description of the device\n\n    Returns:\n        :obj:`FT4222`: Opened device\n\n    Raises:\n        FT2XXDeviceError: on error\n\n    ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_7openByDescription = {"openByDescription", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_7openByDescription, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6openByDescription};
static PyObject *__pyx_pw_6ft4222_6ft4222_7openByDescription(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_desc = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("openByDescription (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_desc,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_desc)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 140, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "openByDescription") < 0)) __PYX_ERR(0, 140, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_desc = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("openByDescription", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 140, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.openByDescription", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6openByDescription(__pyx_self, __pyx_v_desc);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6openByDescription(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_desc) {
  FT_HANDLE __pyx_v_handle;
  char *__pyx_v_cdesc;
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  char *__pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("openByDescription", 0);
  __Pyx_INCREF(__pyx_v_desc);

  /* "ft4222/ft4222.pyx":153
 * 
 *     """
 *     if isinstance(desc, str):             # <<<<<<<<<<<<<<
 *         desc = desc.encode('utf-8')
 *     cdef FT_HANDLE handle
 */
  __pyx_t_1 = PyUnicode_Check(__pyx_v_desc); 
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":154
 *     """
 *     if isinstance(desc, str):
 *         desc = desc.encode('utf-8')             # <<<<<<<<<<<<<<
 *     cdef FT_HANDLE handle
 *     cdef char* cdesc = desc
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_desc, __pyx_n_s_encode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    __pyx_t_5 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_5 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_kp_u_utf_8};
      __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 154, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF_SET(__pyx_v_desc, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "ft4222/ft4222.pyx":153
 * 
 *     """
 *     if isinstance(desc, str):             # <<<<<<<<<<<<<<
 *         desc = desc.encode('utf-8')
 *     cdef FT_HANDLE handle
 */
  }

  /* "ft4222/ft4222.pyx":156
 *         desc = desc.encode('utf-8')
 *     cdef FT_HANDLE handle
 *     cdef char* cdesc = desc             # <<<<<<<<<<<<<<
 *     status = FT_OpenEx(<PVOID>cdesc, FT_OPEN_BY_DESCRIPTION, &handle)
 *     if status == FT_OK:
 */
  __pyx_t_6 = __Pyx_PyObject_AsWritableString(__pyx_v_desc); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 156, __pyx_L1_error)
  __pyx_v_cdesc = __pyx_t_6;

  /* "ft4222/ft4222.pyx":157
 *     cdef FT_HANDLE handle
 *     cdef char* cdesc = desc
 *     status = FT_OpenEx(<PVOID>cdesc, FT_OPEN_BY_DESCRIPTION, &handle)             # <<<<<<<<<<<<<<
 *     if status == FT_OK:
 *         #printf("handle: %d\n", handle)
 */
  __pyx_v_status = FT_OpenEx(((PVOID)__pyx_v_cdesc), FT_OPEN_BY_DESCRIPTION, (&__pyx_v_handle));

  /* "ft4222/ft4222.pyx":158
 *     cdef char* cdesc = desc
 *     status = FT_OpenEx(<PVOID>cdesc, FT_OPEN_BY_DESCRIPTION, &handle)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         #printf("handle: %d\n", handle)
 *         return FT4222(<uintptr_t>handle, update=False)
 */
  __pyx_t_1 = (__pyx_v_status == FT_OK);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":160
 *     if status == FT_OK:
 *         #printf("handle: %d\n", handle)
 *         return FT4222(<uintptr_t>handle, update=False)             # <<<<<<<<<<<<<<
 *     raise FT2XXDeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_FromSize_t(((uintptr_t)__pyx_v_handle)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_update, Py_False) < 0) __PYX_ERR(0, 160, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222), __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":158
 *     cdef char* cdesc = desc
 *     status = FT_OpenEx(<PVOID>cdesc, FT_OPEN_BY_DESCRIPTION, &handle)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         #printf("handle: %d\n", handle)
 *         return FT4222(<uintptr_t>handle, update=False)
 */
  }

  /* "ft4222/ft4222.pyx":161
 *         #printf("handle: %d\n", handle)
 *         return FT4222(<uintptr_t>handle, update=False)
 *     raise FT2XXDeviceError, status             # <<<<<<<<<<<<<<
 * 
 * def openByLocation(locId):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT2XXDeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_Raise(__pyx_t_4, __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 161, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":140
 *     raise FT2XXDeviceError, status
 * 
 * def openByDescription(desc):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by description
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.openByDescription", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_desc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":163
 *     raise FT2XXDeviceError, status
 * 
 * def openByLocation(locId):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by location
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_9openByLocation(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_8openByLocation, "Open a handle to a usb device by location\n\n    Args:\n        locId (int): Location id\n\n    Returns:\n        :obj:`FT4222`: Opened device\n\n    Raises:\n        FT2XXDeviceError: on error\n\n    ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_9openByLocation = {"openByLocation", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_9openByLocation, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_8openByLocation};
static PyObject *__pyx_pw_6ft4222_6ft4222_9openByLocation(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_locId = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("openByLocation (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_locId,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_locId)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 163, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "openByLocation") < 0)) __PYX_ERR(0, 163, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_locId = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("openByLocation", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 163, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.openByLocation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_8openByLocation(__pyx_self, __pyx_v_locId);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_8openByLocation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_locId) {
  FT_HANDLE __pyx_v_handle;
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  uintptr_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("openByLocation", 1);

  /* "ft4222/ft4222.pyx":177
 *     """
 *     cdef FT_HANDLE handle
 *     status = FT_OpenEx(<PVOID><uintptr_t>locId, FT_OPEN_BY_LOCATION, &handle)             # <<<<<<<<<<<<<<
 *     if status == FT_OK:
 *         return FT4222(<uintptr_t>handle, update=False)
 */
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_locId); if (unlikely((__pyx_t_1 == ((uintptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 177, __pyx_L1_error)
  __pyx_v_status = FT_OpenEx(((PVOID)((uintptr_t)__pyx_t_1)), FT_OPEN_BY_LOCATION, (&__pyx_v_handle));

  /* "ft4222/ft4222.pyx":178
 *     cdef FT_HANDLE handle
 *     status = FT_OpenEx(<PVOID><uintptr_t>locId, FT_OPEN_BY_LOCATION, &handle)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         return FT4222(<uintptr_t>handle, update=False)
 *     raise FT2XXDeviceError, status
 */
  __pyx_t_2 = (__pyx_v_status == FT_OK);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":179
 *     status = FT_OpenEx(<PVOID><uintptr_t>locId, FT_OPEN_BY_LOCATION, &handle)
 *     if status == FT_OK:
 *         return FT4222(<uintptr_t>handle, update=False)             # <<<<<<<<<<<<<<
 *     raise FT2XXDeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_FromSize_t(((uintptr_t)__pyx_v_handle)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3)) __PYX_ERR(0, 179, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_update, Py_False) < 0) __PYX_ERR(0, 179, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222), __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":178
 *     cdef FT_HANDLE handle
 *     status = FT_OpenEx(<PVOID><uintptr_t>locId, FT_OPEN_BY_LOCATION, &handle)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         return FT4222(<uintptr_t>handle, update=False)
 *     raise FT2XXDeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":180
 *     if status == FT_OK:
 *         return FT4222(<uintptr_t>handle, update=False)
 *     raise FT2XXDeviceError, status             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_FT2XXDeviceError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_5, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 180, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":163
 *     raise FT2XXDeviceError, status
 * 
 * def openByLocation(locId):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by location
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ft4222.ft4222.openByLocation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":191
 * 
 *     @staticmethod
 *     cdef wrap(FT_HANDLE h):             # <<<<<<<<<<<<<<
 *         cdef _closer closer = _closer()
 *         closer._handle = h
 */

static PyObject *__pyx_f_6ft4222_6ft4222_7_closer_wrap(FT_HANDLE __pyx_v_h) {
  struct __pyx_obj_6ft4222_6ft4222__closer *__pyx_v_closer = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("wrap", 1);

  /* "ft4222/ft4222.pyx":192
 *     @staticmethod
 *     cdef wrap(FT_HANDLE h):
 *         cdef _closer closer = _closer()             # <<<<<<<<<<<<<<
 *         closer._handle = h
 *         return closer
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_6ft4222_6ft4222__closer)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_closer = ((struct __pyx_obj_6ft4222_6ft4222__closer *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":193
 *     cdef wrap(FT_HANDLE h):
 *         cdef _closer closer = _closer()
 *         closer._handle = h             # <<<<<<<<<<<<<<
 *         return closer
 * 
 */
  __pyx_v_closer->_handle = __pyx_v_h;

  /* "ft4222/ft4222.pyx":194
 *         cdef _closer closer = _closer()
 *         closer._handle = h
 *         return closer             # <<<<<<<<<<<<<<
 * 
 *     def close(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_closer);
  __pyx_r = ((PyObject *)__pyx_v_closer);
  goto __pyx_L0;

  /* "ft4222/ft4222.pyx":191
 * 
 *     @staticmethod
 *     cdef wrap(FT_HANDLE h):             # <<<<<<<<<<<<<<
 *         cdef _closer closer = _closer()
 *         closer._handle = h
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ft4222.ft4222._closer.wrap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_closer);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":196
 *         return closer
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         FT4222_UnInitialize(self._handle)
 *         FT_Close(self._handle)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_7_closer_1close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_7_closer_1close = {"close", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_7_closer_1close, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6ft4222_6ft4222_7_closer_1close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("close", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "close", 0))) return NULL;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_7_closer_close(((struct __pyx_obj_6ft4222_6ft4222__closer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_7_closer_close(struct __pyx_obj_6ft4222_6ft4222__closer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close", 1);

  /* "ft4222/ft4222.pyx":197
 * 
 *     def close(self):
 *         FT4222_UnInitialize(self._handle)             # <<<<<<<<<<<<<<
 *         FT_Close(self._handle)
 * 
 */
  (void)(FT4222_UnInitialize(__pyx_v_self->_handle));

  /* "ft4222/ft4222.pyx":198
 *     def close(self):
 *         FT4222_UnInitialize(self._handle)
 *         FT_Close(self._handle)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  (void)(FT_Close(__pyx_v_self->_handle));

  /* "ft4222/ft4222.pyx":196
 *         return closer
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         FT4222_UnInitialize(self._handle)
 *         FT_Close(self._handle)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_7_closer_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_7_closer_3__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_7_closer_3__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6ft4222_6ft4222_7_closer_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__reduce_cython__", 0))) return NULL;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_7_closer_2__reduce_cython__(((struct __pyx_obj_6ft4222_6ft4222__closer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_7_closer_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6ft4222_6ft4222__closer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 1);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 */
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_kp_s_self__handle_cannot_be_converted, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("ft4222.ft4222._closer.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_7_closer_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_7_closer_5__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_7_closer_5__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6ft4222_6ft4222_7_closer_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_state,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_state)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 3, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__setstate_cython__") < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222._closer.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_7_closer_4__setstate_cython__(((struct __pyx_obj_6ft4222_6ft4222__closer *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_7_closer_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6ft4222_6ft4222__closer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 1);

  /* "(tree fragment)":4
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 */
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_kp_s_self__handle_cannot_be_converted, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("ft4222.ft4222._closer.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":219
 *     cdef object _close
 * 
 *     def __init__(self, handle, update=True):             # <<<<<<<<<<<<<<
 *         self._handle = <FT_HANDLE><uintptr_t>handle
 *         self._chip_version = 0
 */

/* Python wrapper */
static int __pyx_pw_6ft4222_6ft4222_6FT4222_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ft4222_6ft4222_6FT4222_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_handle = 0;
  CYTHON_UNUSED PyObject *__pyx_v_update = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_handle,&__pyx_n_s_update,0};
    values[1] = __Pyx_Arg_NewRef_VARARGS(((PyObject *)Py_True));
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_VARARGS(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_VARARGS(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_VARARGS(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_VARARGS(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_handle)) != 0)) {
          (void)__Pyx_Arg_NewRef_VARARGS(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 219, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_VARARGS(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_update);
          if (value) { values[1] = __Pyx_Arg_NewRef_VARARGS(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 219, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__init__") < 0)) __PYX_ERR(0, 219, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_VARARGS(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_VARARGS(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_handle = values[0];
    __pyx_v_update = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 219, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_VARARGS(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222___init__(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_handle, __pyx_v_update);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_VARARGS(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ft4222_6ft4222_6FT4222___init__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_handle, CYTHON_UNUSED PyObject *__pyx_v_update) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  uintptr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 1);

  /* "ft4222/ft4222.pyx":220
 * 
 *     def __init__(self, handle, update=True):
 *         self._handle = <FT_HANDLE><uintptr_t>handle             # <<<<<<<<<<<<<<
 *         self._chip_version = 0
 *         self._dll_version = 0
 */
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_handle); if (unlikely((__pyx_t_1 == ((uintptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 220, __pyx_L1_error)
  __pyx_v_self->_handle = ((FT_HANDLE)((uintptr_t)__pyx_t_1));

  /* "ft4222/ft4222.pyx":221
 *     def __init__(self, handle, update=True):
 *         self._handle = <FT_HANDLE><uintptr_t>handle
 *         self._chip_version = 0             # <<<<<<<<<<<<<<
 *         self._dll_version = 0
 *         self._get_version()
 */
  __pyx_v_self->_chip_version = 0;

  /* "ft4222/ft4222.pyx":222
 *         self._handle = <FT_HANDLE><uintptr_t>handle
 *         self._chip_version = 0
 *         self._dll_version = 0             # <<<<<<<<<<<<<<
 *         self._get_version()
 *         self._close = weakref_finalize(self, _closer.wrap(self._handle).close)
 */
  __pyx_v_self->_dll_version = 0;

  /* "ft4222/ft4222.pyx":223
 *         self._chip_version = 0
 *         self._dll_version = 0
 *         self._get_version()             # <<<<<<<<<<<<<<
 *         self._close = weakref_finalize(self, _closer.wrap(self._handle).close)
 * 
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_6ft4222_6ft4222_FT4222 *)__pyx_v_self->__pyx_vtab)->_get_version(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 223, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":224
 *         self._dll_version = 0
 *         self._get_version()
 *         self._close = weakref_finalize(self, _closer.wrap(self._handle).close)             # <<<<<<<<<<<<<<
 * 
 *     def close(self):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_weakref_finalize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_f_6ft4222_6ft4222_7_closer_wrap(__pyx_v_self->_handle); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_close); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_6 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_4, ((PyObject *)__pyx_v_self), __pyx_t_5};
    __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_6, 2+__pyx_t_6);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 224, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_close);
  __Pyx_DECREF(__pyx_v_self->_close);
  __pyx_v_self->_close = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":219
 *     cdef object _close
 * 
 *     def __init__(self, handle, update=True):             # <<<<<<<<<<<<<<
 *         self._handle = <FT_HANDLE><uintptr_t>handle
 *         self._chip_version = 0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":226
 *         self._close = weakref_finalize(self, _closer.wrap(self._handle).close)
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """Closes the device."""
 *         FT4222_UnInitialize(self._handle)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_3close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_2close, "Closes the device.");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_3close = {"close", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_3close, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_2close};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_3close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("close", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "close", 0))) return NULL;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_2close(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_2close(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 1);

  /* "ft4222/ft4222.pyx":228
 *     def close(self):
 *         """Closes the device."""
 *         FT4222_UnInitialize(self._handle)             # <<<<<<<<<<<<<<
 *         status = FT_Close(self._handle)
 *         if status != FT4222_OK:
 */
  (void)(FT4222_UnInitialize(__pyx_v_self->_handle));

  /* "ft4222/ft4222.pyx":229
 *         """Closes the device."""
 *         FT4222_UnInitialize(self._handle)
 *         status = FT_Close(self._handle)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_v_status = FT_Close(__pyx_v_self->_handle);

  /* "ft4222/ft4222.pyx":230
 *         FT4222_UnInitialize(self._handle)
 *         status = FT_Close(self._handle)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 *         self._handle = NULL
 */
  __pyx_t_1 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":231
 *         status = FT_Close(self._handle)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 *         self._handle = NULL
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 231, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 231, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 231, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":230
 *         FT4222_UnInitialize(self._handle)
 *         status = FT_Close(self._handle)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 *         self._handle = NULL
 */
  }

  /* "ft4222/ft4222.pyx":232
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 *         self._handle = NULL             # <<<<<<<<<<<<<<
 * 
 *     cdef _get_version(self):
 */
  __pyx_v_self->_handle = NULL;

  /* "ft4222/ft4222.pyx":226
 *         self._close = weakref_finalize(self, _closer.wrap(self._handle).close)
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """Closes the device."""
 *         FT4222_UnInitialize(self._handle)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":234
 *         self._handle = NULL
 * 
 *     cdef _get_version(self):             # <<<<<<<<<<<<<<
 *         cdef FT4222_Version ver
 *         status = FT4222_GetVersion(self._handle, &ver)
 */

static PyObject *__pyx_f_6ft4222_6ft4222_6FT4222__get_version(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  FT4222_Version __pyx_v_ver;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  DWORD __pyx_t_2;
  __Pyx_RefNannySetupContext("_get_version", 1);

  /* "ft4222/ft4222.pyx":236
 *     cdef _get_version(self):
 *         cdef FT4222_Version ver
 *         status = FT4222_GetVersion(self._handle, &ver)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             self._chip_version = ver.chipVersion
 */
  __pyx_v_status = FT4222_GetVersion(__pyx_v_self->_handle, (&__pyx_v_ver));

  /* "ft4222/ft4222.pyx":237
 *         cdef FT4222_Version ver
 *         status = FT4222_GetVersion(self._handle, &ver)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             self._chip_version = ver.chipVersion
 *             self._dll_version = ver.dllVersion
 */
  __pyx_t_1 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":238
 *         status = FT4222_GetVersion(self._handle, &ver)
 *         if status == FT4222_OK:
 *             self._chip_version = ver.chipVersion             # <<<<<<<<<<<<<<
 *             self._dll_version = ver.dllVersion
 * 
 */
    __pyx_t_2 = __pyx_v_ver.chipVersion;
    __pyx_v_self->_chip_version = __pyx_t_2;

    /* "ft4222/ft4222.pyx":239
 *         if status == FT4222_OK:
 *             self._chip_version = ver.chipVersion
 *             self._dll_version = ver.dllVersion             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
    __pyx_t_2 = __pyx_v_ver.dllVersion;
    __pyx_v_self->_dll_version = __pyx_t_2;

    /* "ft4222/ft4222.pyx":237
 *         cdef FT4222_Version ver
 *         status = FT4222_GetVersion(self._handle, &ver)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             self._chip_version = ver.chipVersion
 *             self._dll_version = ver.dllVersion
 */
  }

  /* "ft4222/ft4222.pyx":234
 *         self._handle = NULL
 * 
 *     cdef _get_version(self):             # <<<<<<<<<<<<<<
 *         cdef FT4222_Version ver
 *         status = FT4222_GetVersion(self._handle, &ver)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":241
 *             self._dll_version = ver.dllVersion
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def chipVersion(self) -> int:
 *         """Chip version as number"""
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_11chipVersion_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_11chipVersion_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_11chipVersion___get__(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_11chipVersion___get__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 1);

  /* "ft4222/ft4222.pyx":244
 *     def chipVersion(self) -> int:
 *         """Chip version as number"""
 *         return self._chip_version             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_DWORD(__pyx_v_self->_chip_version); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ft4222/ft4222.pyx":241
 *             self._dll_version = ver.dllVersion
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def chipVersion(self) -> int:
 *         """Chip version as number"""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.chipVersion.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":246
 *         return self._chip_version
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def libVersion(self) -> int:
 *         """Library version as number"""
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_10libVersion_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_10libVersion_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_10libVersion___get__(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_10libVersion___get__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 1);

  /* "ft4222/ft4222.pyx":249
 *     def libVersion(self) -> int:
 *         """Library version as number"""
 *         return self._dll_version             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_DWORD(__pyx_v_self->_dll_version); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ft4222/ft4222.pyx":246
 *         return self._chip_version
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def libVersion(self) -> int:
 *         """Library version as number"""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.libVersion.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":251
 *         return self._dll_version
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def chipRevision(self) -> str:
 *         """The revision of the chip in human readable format"""
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_12chipRevision_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_12chipRevision_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_12chipRevision___get__(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_12chipRevision___get__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 1);

  /* "ft4222/ft4222.pyx":254
 *     def chipRevision(self) -> str:
 *         """The revision of the chip in human readable format"""
 *         try:             # <<<<<<<<<<<<<<
 *             return self.__chip_rev_map[self._chip_version]
 *         except KeyError:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "ft4222/ft4222.pyx":255
 *         """The revision of the chip in human readable format"""
 *         try:
 *             return self.__chip_rev_map[self._chip_version]             # <<<<<<<<<<<<<<
 *         except KeyError:
 *             return "Rev. unknown"
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_FT4222__chip_rev_map); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 255, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_GetItemInt(__pyx_t_4, __pyx_v_self->_chip_version, DWORD, 0, __Pyx_PyInt_From_DWORD, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 255, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_r = __pyx_t_5;
      __pyx_t_5 = 0;
      goto __pyx_L7_try_return;

      /* "ft4222/ft4222.pyx":254
 *     def chipRevision(self) -> str:
 *         """The revision of the chip in human readable format"""
 *         try:             # <<<<<<<<<<<<<<
 *             return self.__chip_rev_map[self._chip_version]
 *         except KeyError:
 */
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "ft4222/ft4222.pyx":256
 *         try:
 *             return self.__chip_rev_map[self._chip_version]
 *         except KeyError:             # <<<<<<<<<<<<<<
 *             return "Rev. unknown"
 * 
 */
    __pyx_t_6 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError);
    if (__pyx_t_6) {
      __Pyx_AddTraceback("ft4222.ft4222.FT4222.chipRevision.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_4, &__pyx_t_7) < 0) __PYX_ERR(0, 256, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_7);

      /* "ft4222/ft4222.pyx":257
 *             return self.__chip_rev_map[self._chip_version]
 *         except KeyError:
 *             return "Rev. unknown"             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_Rev_unknown);
      __pyx_r = __pyx_kp_u_Rev_unknown;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_except_return;
    }
    goto __pyx_L5_except_error;

    /* "ft4222/ft4222.pyx":254
 *     def chipRevision(self) -> str:
 *         """The revision of the chip in human readable format"""
 *         try:             # <<<<<<<<<<<<<<
 *             return self.__chip_rev_map[self._chip_version]
 *         except KeyError:
 */
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
  }

  /* "ft4222/ft4222.pyx":251
 *         return self._dll_version
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def chipRevision(self) -> str:
 *         """The revision of the chip in human readable format"""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.chipRevision.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":259
 *             return "Rev. unknown"
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "FT4222: chipVersion: 0x{:x} ({:s}), libVersion: 0x{:x}".format(self._chip_version, self.chipRevision, self._dll_version)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_5__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_5__repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_4__repr__(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_4__repr__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 1);

  /* "ft4222/ft4222.pyx":260
 * 
 *     def __repr__(self):
 *         return "FT4222: chipVersion: 0x{:x} ({:s}), libVersion: 0x{:x}".format(self._chip_version, self.chipRevision, self._dll_version)             # <<<<<<<<<<<<<<
 * 
 *     def setTimeouts(self, read_timeout, write_timeout):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_FT4222_chipVersion_0x_x_s_libVer, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_DWORD(__pyx_v_self->_chip_version); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_chipRevision); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_DWORD(__pyx_v_self->_dll_version); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  __pyx_t_7 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_7, 3+__pyx_t_7);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 260, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ft4222/ft4222.pyx":259
 *             return "Rev. unknown"
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "FT4222: chipVersion: 0x{:x} ({:s}), libVersion: 0x{:x}".format(self._chip_version, self.chipRevision, self._dll_version)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":262
 *         return "FT4222: chipVersion: 0x{:x} ({:s}), libVersion: 0x{:x}".format(self._chip_version, self.chipRevision, self._dll_version)
 * 
 *     def setTimeouts(self, read_timeout, write_timeout):             # <<<<<<<<<<<<<<
 *         """Set the read and write timeouts
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_7setTimeouts(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_6setTimeouts, "Set the read and write timeouts\n\n        Args:\n            read_timeout (int): Read timeout in milliseconds.\n            write_timeout (int): Write timeout in milliseconds.\n\n        Raises:\n            FT2XXDeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_7setTimeouts = {"setTimeouts", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_7setTimeouts, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_6setTimeouts};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_7setTimeouts(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_read_timeout = 0;
  PyObject *__pyx_v_write_timeout = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setTimeouts (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_read_timeout,&__pyx_n_s_write_timeout,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_read_timeout)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 262, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_write_timeout)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 262, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("setTimeouts", 1, 2, 2, 1); __PYX_ERR(0, 262, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "setTimeouts") < 0)) __PYX_ERR(0, 262, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_read_timeout = values[0];
    __pyx_v_write_timeout = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("setTimeouts", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 262, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.setTimeouts", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_6setTimeouts(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_read_timeout, __pyx_v_write_timeout);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_6setTimeouts(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_read_timeout, PyObject *__pyx_v_write_timeout) {
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  ULONG __pyx_t_1;
  ULONG __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setTimeouts", 1);

  /* "ft4222/ft4222.pyx":273
 * 
 *         """
 *         status = FT_SetTimeouts(self._handle, read_timeout, write_timeout)             # <<<<<<<<<<<<<<
 *         if status != FT_OK:
 *             raise FT2XXDeviceError, status
 */
  __pyx_t_1 = __Pyx_PyInt_As_ULONG(__pyx_v_read_timeout); if (unlikely((__pyx_t_1 == ((ULONG)-1)) && PyErr_Occurred())) __PYX_ERR(0, 273, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_ULONG(__pyx_v_write_timeout); if (unlikely((__pyx_t_2 == ((ULONG)-1)) && PyErr_Occurred())) __PYX_ERR(0, 273, __pyx_L1_error)
  __pyx_v_status = FT_SetTimeouts(__pyx_v_self->_handle, __pyx_t_1, __pyx_t_2);

  /* "ft4222/ft4222.pyx":274
 *         """
 *         status = FT_SetTimeouts(self._handle, read_timeout, write_timeout)
 *         if status != FT_OK:             # <<<<<<<<<<<<<<
 *             raise FT2XXDeviceError, status
 * 
 */
  __pyx_t_3 = (__pyx_v_status != FT_OK);
  if (unlikely(__pyx_t_3)) {

    /* "ft4222/ft4222.pyx":275
 *         status = FT_SetTimeouts(self._handle, read_timeout, write_timeout)
 *         if status != FT_OK:
 *             raise FT2XXDeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def setClock(self, clk):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT2XXDeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 275, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 275, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_4, __pyx_t_5, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 275, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":274
 *         """
 *         status = FT_SetTimeouts(self._handle, read_timeout, write_timeout)
 *         if status != FT_OK:             # <<<<<<<<<<<<<<
 *             raise FT2XXDeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":262
 *         return "FT4222: chipVersion: 0x{:x} ({:s}), libVersion: 0x{:x}".format(self._chip_version, self.chipRevision, self._dll_version)
 * 
 *     def setTimeouts(self, read_timeout, write_timeout):             # <<<<<<<<<<<<<<
 *         """Set the read and write timeouts
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.setTimeouts", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":277
 *             raise FT2XXDeviceError, status
 * 
 *     def setClock(self, clk):             # <<<<<<<<<<<<<<
 *         """Set the system clock
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_9setClock(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_8setClock, "Set the system clock\n\n        Args:\n            clk (:obj:`ft4222.SysClock`): Desired clock\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_9setClock = {"setClock", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_9setClock, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_8setClock};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_9setClock(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_clk = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setClock (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_clk,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_clk)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 277, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "setClock") < 0)) __PYX_ERR(0, 277, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_clk = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("setClock", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 277, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.setClock", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_8setClock(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_clk);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_8setClock(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_clk) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  FT4222_ClockRate __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setClock", 1);

  /* "ft4222/ft4222.pyx":287
 * 
 *         """
 *         status = FT4222_SetClock(self._handle, clk)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((FT4222_ClockRate)__Pyx_PyInt_As_FT4222_ClockRate(__pyx_v_clk)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 287, __pyx_L1_error)
  __pyx_v_status = FT4222_SetClock(__pyx_v_self->_handle, __pyx_t_1);

  /* "ft4222/ft4222.pyx":288
 *         """
 *         status = FT4222_SetClock(self._handle, clk)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_2 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_2)) {

    /* "ft4222/ft4222.pyx":289
 *         status = FT4222_SetClock(self._handle, clk)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def getClock(self):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 289, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 289, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 289, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":288
 *         """
 *         status = FT4222_SetClock(self._handle, clk)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":277
 *             raise FT2XXDeviceError, status
 * 
 *     def setClock(self, clk):             # <<<<<<<<<<<<<<
 *         """Set the system clock
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.setClock", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":291
 *             raise FT4222DeviceError, status
 * 
 *     def getClock(self):             # <<<<<<<<<<<<<<
 *         """Get the system clock
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_11getClock(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_10getClock, "Get the system clock\n\n        Returns:\n            :obj:`ft4222.SysClock`: Clock\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_11getClock = {"getClock", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_11getClock, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_10getClock};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_11getClock(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getClock (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("getClock", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "getClock", 0))) return NULL;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_10getClock(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_10getClock(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  FT4222_ClockRate __pyx_v_clk;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getClock", 1);

  /* "ft4222/ft4222.pyx":302
 *         """
 *         cdef FT4222_ClockRate clk
 *         status = FT4222_GetClock(self._handle, &clk)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return SysClock(clk)
 */
  __pyx_v_status = FT4222_GetClock(__pyx_v_self->_handle, (&__pyx_v_clk));

  /* "ft4222/ft4222.pyx":303
 *         cdef FT4222_ClockRate clk
 *         status = FT4222_GetClock(self._handle, &clk)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return SysClock(clk)
 *         raise FT4222DeviceError, status
 */
  __pyx_t_1 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":304
 *         status = FT4222_GetClock(self._handle, &clk)
 *         if status == FT4222_OK:
 *             return SysClock(clk)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_SysClock); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 304, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_FT4222_ClockRate(__pyx_v_clk); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 304, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    __pyx_t_6 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_6 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_4};
      __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_6, 1+__pyx_t_6);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 304, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":303
 *         cdef FT4222_ClockRate clk
 *         status = FT4222_GetClock(self._handle, &clk)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return SysClock(clk)
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":305
 *         if status == FT4222_OK:
 *             return SysClock(clk)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def setSuspendOut(self, enable):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 305, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":291
 *             raise FT4222DeviceError, status
 * 
 *     def getClock(self):             # <<<<<<<<<<<<<<
 *         """Get the system clock
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.getClock", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":307
 *         raise FT4222DeviceError, status
 * 
 *     def setSuspendOut(self, enable):             # <<<<<<<<<<<<<<
 *         """Enable or disable, suspend out, which will emit a signal when FT4222H enters suspend mode.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_13setSuspendOut(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_12setSuspendOut, "Enable or disable, suspend out, which will emit a signal when FT4222H enters suspend mode.\n\n        Args:\n            enable (bool): TRUE to enable suspend out and configure GPIO2 as an output pin for emitting a signal when suspended. FALSE to switch back to GPIO2\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_13setSuspendOut = {"setSuspendOut", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_13setSuspendOut, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_12setSuspendOut};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_13setSuspendOut(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_enable = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setSuspendOut (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_enable,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_enable)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 307, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "setSuspendOut") < 0)) __PYX_ERR(0, 307, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_enable = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("setSuspendOut", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 307, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.setSuspendOut", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_12setSuspendOut(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_enable);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_12setSuspendOut(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_enable) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  BOOL __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setSuspendOut", 1);

  /* "ft4222/ft4222.pyx":317
 * 
 *         """
 *         status = FT4222_SetSuspendOut(self._handle, enable)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = __Pyx_PyInt_As_BOOL(__pyx_v_enable); if (unlikely((__pyx_t_1 == ((BOOL)-1)) && PyErr_Occurred())) __PYX_ERR(0, 317, __pyx_L1_error)
  __pyx_v_status = FT4222_SetSuspendOut(__pyx_v_self->_handle, __pyx_t_1);

  /* "ft4222/ft4222.pyx":318
 *         """
 *         status = FT4222_SetSuspendOut(self._handle, enable)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_2 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_2)) {

    /* "ft4222/ft4222.pyx":319
 *         status = FT4222_SetSuspendOut(self._handle, enable)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def setWakeUpInterrupt(self, enable):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 319, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 319, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 319, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":318
 *         """
 *         status = FT4222_SetSuspendOut(self._handle, enable)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":307
 *         raise FT4222DeviceError, status
 * 
 *     def setSuspendOut(self, enable):             # <<<<<<<<<<<<<<
 *         """Enable or disable, suspend out, which will emit a signal when FT4222H enters suspend mode.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.setSuspendOut", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":321
 *             raise FT4222DeviceError, status
 * 
 *     def setWakeUpInterrupt(self, enable):             # <<<<<<<<<<<<<<
 *         """Enable or disable the wakeup/interrupt
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_15setWakeUpInterrupt(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_14setWakeUpInterrupt, "Enable or disable the wakeup/interrupt\n\n        Args:\n            enable (bool): True to configure GPIO3 as an input pin for wakeup/interrupt. FALSE to switch back to GPIO3.\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_15setWakeUpInterrupt = {"setWakeUpInterrupt", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_15setWakeUpInterrupt, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_14setWakeUpInterrupt};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_15setWakeUpInterrupt(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_enable = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setWakeUpInterrupt (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_enable,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_enable)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 321, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "setWakeUpInterrupt") < 0)) __PYX_ERR(0, 321, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_enable = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("setWakeUpInterrupt", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 321, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.setWakeUpInterrupt", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_14setWakeUpInterrupt(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_enable);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_14setWakeUpInterrupt(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_enable) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  BOOL __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setWakeUpInterrupt", 1);

  /* "ft4222/ft4222.pyx":331
 * 
 *         """
 *         status = FT4222_SetWakeUpInterrupt(self._handle, enable)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = __Pyx_PyInt_As_BOOL(__pyx_v_enable); if (unlikely((__pyx_t_1 == ((BOOL)-1)) && PyErr_Occurred())) __PYX_ERR(0, 331, __pyx_L1_error)
  __pyx_v_status = FT4222_SetWakeUpInterrupt(__pyx_v_self->_handle, __pyx_t_1);

  /* "ft4222/ft4222.pyx":332
 *         """
 *         status = FT4222_SetWakeUpInterrupt(self._handle, enable)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_2 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_2)) {

    /* "ft4222/ft4222.pyx":333
 *         status = FT4222_SetWakeUpInterrupt(self._handle, enable)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def getMaxTransferSize(self):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 333, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 333, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 333, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":332
 *         """
 *         status = FT4222_SetWakeUpInterrupt(self._handle, enable)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":321
 *             raise FT4222DeviceError, status
 * 
 *     def setWakeUpInterrupt(self, enable):             # <<<<<<<<<<<<<<
 *         """Enable or disable the wakeup/interrupt
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.setWakeUpInterrupt", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":335
 *             raise FT4222DeviceError, status
 * 
 *     def getMaxTransferSize(self):             # <<<<<<<<<<<<<<
 *         """Get the maximum transfer size for the currently configured operating mode
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_17getMaxTransferSize(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_16getMaxTransferSize, "Get the maximum transfer size for the currently configured operating mode\n\n        Returns:\n            int: Maximum number of bytes for each transfer.\n\n        Raises:\n            FT4222DeviceError: on error\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_17getMaxTransferSize = {"getMaxTransferSize", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_17getMaxTransferSize, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_16getMaxTransferSize};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_17getMaxTransferSize(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getMaxTransferSize (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("getMaxTransferSize", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "getMaxTransferSize", 0))) return NULL;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_16getMaxTransferSize(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_16getMaxTransferSize(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  uint16 __pyx_v_maxTransferSize;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getMaxTransferSize", 1);

  /* "ft4222/ft4222.pyx":344
 *             FT4222DeviceError: on error
 *         """
 *         cdef uint16 maxTransferSize = 0             # <<<<<<<<<<<<<<
 *         status = FT4222_GetMaxTransferSize(self._handle, &maxTransferSize)
 *         if status == FT4222_OK:
 */
  __pyx_v_maxTransferSize = 0;

  /* "ft4222/ft4222.pyx":345
 *         """
 *         cdef uint16 maxTransferSize = 0
 *         status = FT4222_GetMaxTransferSize(self._handle, &maxTransferSize)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return maxTransferSize
 */
  __pyx_v_status = FT4222_GetMaxTransferSize(__pyx_v_self->_handle, (&__pyx_v_maxTransferSize));

  /* "ft4222/ft4222.pyx":346
 *         cdef uint16 maxTransferSize = 0
 *         status = FT4222_GetMaxTransferSize(self._handle, &maxTransferSize)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return maxTransferSize
 *         raise FT4222DeviceError, status
 */
  __pyx_t_1 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":347
 *         status = FT4222_GetMaxTransferSize(self._handle, &maxTransferSize)
 *         if status == FT4222_OK:
 *             return maxTransferSize             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_uint16_t(__pyx_v_maxTransferSize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 347, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":346
 *         cdef uint16 maxTransferSize = 0
 *         status = FT4222_GetMaxTransferSize(self._handle, &maxTransferSize)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return maxTransferSize
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":348
 *         if status == FT4222_OK:
 *             return maxTransferSize
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def vendorCmdGet(self, req, bytesToRead):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 348, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":335
 *             raise FT4222DeviceError, status
 * 
 *     def getMaxTransferSize(self):             # <<<<<<<<<<<<<<
 *         """Get the maximum transfer size for the currently configured operating mode
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.getMaxTransferSize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":350
 *         raise FT4222DeviceError, status
 * 
 *     def vendorCmdGet(self, req, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Vendor get command"""
 *         cdef:
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_19vendorCmdGet(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_18vendorCmdGet, "Vendor get command");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_19vendorCmdGet = {"vendorCmdGet", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_19vendorCmdGet, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_18vendorCmdGet};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_19vendorCmdGet(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_req = 0;
  PyObject *__pyx_v_bytesToRead = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("vendorCmdGet (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_req,&__pyx_n_s_bytesToRead,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_req)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 350, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_bytesToRead)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 350, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("vendorCmdGet", 1, 2, 2, 1); __PYX_ERR(0, 350, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "vendorCmdGet") < 0)) __PYX_ERR(0, 350, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_req = values[0];
    __pyx_v_bytesToRead = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("vendorCmdGet", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 350, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.vendorCmdGet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_18vendorCmdGet(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_req, __pyx_v_bytesToRead);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_18vendorCmdGet(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_req, PyObject *__pyx_v_bytesToRead) {
  arrayobject *__pyx_v_buf = 0;
  FT_STATUS __pyx_v_status;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_buf;
  __Pyx_Buffer __pyx_pybuffer_buf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  UCHAR __pyx_t_5;
  USHORT __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("vendorCmdGet", 1);
  __pyx_pybuffer_buf.pybuffer.buf = NULL;
  __pyx_pybuffer_buf.refcount = 0;
  __pyx_pybuffernd_buf.data = NULL;
  __pyx_pybuffernd_buf.rcbuffer = &__pyx_pybuffer_buf;

  /* "ft4222/ft4222.pyx":353
 *         """Vendor get command"""
 *         cdef:
 *             array[uint8] buf = array('B', [])             # <<<<<<<<<<<<<<
 *         resize(buf, bytesToRead)
 *         status = FT_VendorCmdGet(self._handle, req, buf.data.as_uchars, bytesToRead)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 353, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 353, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_n_u_B);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_B)) __PYX_ERR(0, 353, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1)) __PYX_ERR(0, 353, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7cpython_5array_array), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 353, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_buf.rcbuffer->pybuffer, (PyObject*)((arrayobject *)__pyx_t_1), &__Pyx_TypeInfo_nn_uint8, PyBUF_FORMAT| PyBUF_INDIRECT, 1, 0, __pyx_stack) == -1)) {
      __pyx_v_buf = ((arrayobject *)Py_None); __Pyx_INCREF(Py_None); __pyx_pybuffernd_buf.rcbuffer->pybuffer.buf = NULL;
      __PYX_ERR(0, 353, __pyx_L1_error)
    } else {__pyx_pybuffernd_buf.diminfo[0].strides = __pyx_pybuffernd_buf.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_buf.diminfo[0].shape = __pyx_pybuffernd_buf.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_buf.diminfo[0].suboffsets = __pyx_pybuffernd_buf.rcbuffer->pybuffer.suboffsets[0];
    }
  }
  __pyx_v_buf = ((arrayobject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":354
 *         cdef:
 *             array[uint8] buf = array('B', [])
 *         resize(buf, bytesToRead)             # <<<<<<<<<<<<<<
 *         status = FT_VendorCmdGet(self._handle, req, buf.data.as_uchars, bytesToRead)
 *         if status == FT_OK:
 */
  __pyx_t_3 = __Pyx_PyIndex_AsSsize_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_3 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 354, __pyx_L1_error)
  __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 354, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":355
 *             array[uint8] buf = array('B', [])
 *         resize(buf, bytesToRead)
 *         status = FT_VendorCmdGet(self._handle, req, buf.data.as_uchars, bytesToRead)             # <<<<<<<<<<<<<<
 *         if status == FT_OK:
 *             return bytes(buf)
 */
  __pyx_t_5 = __Pyx_PyInt_As_UCHAR(__pyx_v_req); if (unlikely((__pyx_t_5 == ((UCHAR)-1)) && PyErr_Occurred())) __PYX_ERR(0, 355, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyInt_As_USHORT(__pyx_v_bytesToRead); if (unlikely((__pyx_t_6 == ((USHORT)-1)) && PyErr_Occurred())) __PYX_ERR(0, 355, __pyx_L1_error)
  __pyx_v_status = FT_VendorCmdGet(__pyx_v_self->_handle, __pyx_t_5, __pyx_v_buf->data.as_uchars, __pyx_t_6);

  /* "ft4222/ft4222.pyx":356
 *         resize(buf, bytesToRead)
 *         status = FT_VendorCmdGet(self._handle, req, buf.data.as_uchars, bytesToRead)
 *         if status == FT_OK:             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
  __pyx_t_7 = (__pyx_v_status == FT_OK);
  if (__pyx_t_7) {

    /* "ft4222/ft4222.pyx":357
 *         status = FT_VendorCmdGet(self._handle, req, buf.data.as_uchars, bytesToRead)
 *         if status == FT_OK:
 *             return bytes(buf)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), ((PyObject *)__pyx_v_buf)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 357, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":356
 *         resize(buf, bytesToRead)
 *         status = FT_VendorCmdGet(self._handle, req, buf.data.as_uchars, bytesToRead)
 *         if status == FT_OK:             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":358
 *         if status == FT_OK:
 *             return bytes(buf)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def vendorCmdSet(self, req, data):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_Raise(__pyx_t_1, __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 358, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":350
 *         raise FT4222DeviceError, status
 * 
 *     def vendorCmdGet(self, req, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Vendor get command"""
 *         cdef:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.vendorCmdGet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_buf);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":360
 *         raise FT4222DeviceError, status
 * 
 *     def vendorCmdSet(self, req, data):             # <<<<<<<<<<<<<<
 *         """Vendor set command"""
 *         if isinstance(data, int):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_21vendorCmdSet(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_20vendorCmdSet, "Vendor set command");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_21vendorCmdSet = {"vendorCmdSet", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_21vendorCmdSet, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_20vendorCmdSet};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_21vendorCmdSet(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_req = 0;
  PyObject *__pyx_v_data = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("vendorCmdSet (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_req,&__pyx_n_s_data,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_req)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 360, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_data)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 360, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("vendorCmdSet", 1, 2, 2, 1); __PYX_ERR(0, 360, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "vendorCmdSet") < 0)) __PYX_ERR(0, 360, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_req = values[0];
    __pyx_v_data = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("vendorCmdSet", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 360, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.vendorCmdSet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_20vendorCmdSet(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_req, __pyx_v_data);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_20vendorCmdSet(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_req, PyObject *__pyx_v_data) {
  uint8 *__pyx_v_cdata;
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  uint8 *__pyx_t_5;
  UCHAR __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("vendorCmdSet", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "ft4222/ft4222.pyx":362
 *     def vendorCmdSet(self, req, data):
 *         """Vendor set command"""
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_data); 
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":363
 *         """Vendor set command"""
 *         if isinstance(data, int):
 *             data = bytes([data])             # <<<<<<<<<<<<<<
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_data);
    __Pyx_GIVEREF(__pyx_v_data);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_v_data)) __PYX_ERR(0, 363, __pyx_L1_error);
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "ft4222/ft4222.pyx":362
 *     def vendorCmdSet(self, req, data):
 *         """Vendor set command"""
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":364
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  __pyx_t_4 = PyBytes_Check(__pyx_v_data); 
  if (!__pyx_t_4) {
  } else {
    __pyx_t_1 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = PyByteArray_Check(__pyx_v_data); 
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  __pyx_t_4 = (!__pyx_t_1);
  if (unlikely(__pyx_t_4)) {

    /* "ft4222/ft4222.pyx":365
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint16 bytesSent
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 365, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 365, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":364
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":368
 *         cdef:
 *             uint16 bytesSent
 *             uint8* cdata = data             # <<<<<<<<<<<<<<
 *         status = FT_VendorCmdSet(self._handle, req, cdata, len(data))
 *         if status != FT_OK:
 */
  __pyx_t_5 = __Pyx_PyObject_AsWritableUString(__pyx_v_data); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 368, __pyx_L1_error)
  __pyx_v_cdata = __pyx_t_5;

  /* "ft4222/ft4222.pyx":369
 *             uint16 bytesSent
 *             uint8* cdata = data
 *         status = FT_VendorCmdSet(self._handle, req, cdata, len(data))             # <<<<<<<<<<<<<<
 *         if status != FT_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_6 = __Pyx_PyInt_As_UCHAR(__pyx_v_req); if (unlikely((__pyx_t_6 == ((UCHAR)-1)) && PyErr_Occurred())) __PYX_ERR(0, 369, __pyx_L1_error)
  __pyx_t_7 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 369, __pyx_L1_error)
  __pyx_v_status = FT_VendorCmdSet(__pyx_v_self->_handle, __pyx_t_6, __pyx_v_cdata, __pyx_t_7);

  /* "ft4222/ft4222.pyx":370
 *             uint8* cdata = data
 *         status = FT_VendorCmdSet(self._handle, req, cdata, len(data))
 *         if status != FT_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_4 = (__pyx_v_status != FT_OK);
  if (unlikely(__pyx_t_4)) {

    /* "ft4222/ft4222.pyx":371
 *         status = FT_VendorCmdSet(self._handle, req, cdata, len(data))
 *         if status != FT_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def getLatencyTimer(self):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 371, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 371, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_3, __pyx_t_2, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 371, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":370
 *             uint8* cdata = data
 *         status = FT_VendorCmdSet(self._handle, req, cdata, len(data))
 *         if status != FT_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":360
 *         raise FT4222DeviceError, status
 * 
 *     def vendorCmdSet(self, req, data):             # <<<<<<<<<<<<<<
 *         """Vendor set command"""
 *         if isinstance(data, int):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.vendorCmdSet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":373
 *             raise FT4222DeviceError, status
 * 
 *     def getLatencyTimer(self):             # <<<<<<<<<<<<<<
 *         """Get the current value of the latency timer.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_23getLatencyTimer(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_22getLatencyTimer, "Get the current value of the latency timer.\n\n        Returns:\n            int: Latency timer value in milliseconds\n\n        Raises:\n            FT2XXDeviceError: on error\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_23getLatencyTimer = {"getLatencyTimer", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_23getLatencyTimer, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_22getLatencyTimer};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_23getLatencyTimer(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getLatencyTimer (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("getLatencyTimer", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "getLatencyTimer", 0))) return NULL;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_22getLatencyTimer(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_22getLatencyTimer(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  UCHAR __pyx_v_latency;
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getLatencyTimer", 1);

  /* "ft4222/ft4222.pyx":383
 *         """
 *         cdef UCHAR latency
 *         status = FT_GetLatencyTimer(self._handle, &latency)             # <<<<<<<<<<<<<<
 *         if status == FT_OK:
 *             return latency
 */
  __pyx_v_status = FT_GetLatencyTimer(__pyx_v_self->_handle, (&__pyx_v_latency));

  /* "ft4222/ft4222.pyx":384
 *         cdef UCHAR latency
 *         status = FT_GetLatencyTimer(self._handle, &latency)
 *         if status == FT_OK:             # <<<<<<<<<<<<<<
 *             return latency
 *         raise FT2XXDeviceError, status
 */
  __pyx_t_1 = (__pyx_v_status == FT_OK);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":385
 *         status = FT_GetLatencyTimer(self._handle, &latency)
 *         if status == FT_OK:
 *             return latency             # <<<<<<<<<<<<<<
 *         raise FT2XXDeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_UCHAR(__pyx_v_latency); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 385, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":384
 *         cdef UCHAR latency
 *         status = FT_GetLatencyTimer(self._handle, &latency)
 *         if status == FT_OK:             # <<<<<<<<<<<<<<
 *             return latency
 *         raise FT2XXDeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":386
 *         if status == FT_OK:
 *             return latency
 *         raise FT2XXDeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def setLatencyTimer(self, latency):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT2XXDeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 386, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":373
 *             raise FT4222DeviceError, status
 * 
 *     def getLatencyTimer(self):             # <<<<<<<<<<<<<<
 *         """Get the current value of the latency timer.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.getLatencyTimer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":388
 *         raise FT2XXDeviceError, status
 * 
 *     def setLatencyTimer(self, latency):             # <<<<<<<<<<<<<<
 *         """Set the current value of the latency timer.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_25setLatencyTimer(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_24setLatencyTimer, "Set the current value of the latency timer.\n\n        Args:\n            latency (int): Latency timer value in milliseconds. Valid range is 2 - 255\n\n        Raises:\n            FT2XXDeviceError: on error\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_25setLatencyTimer = {"setLatencyTimer", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_25setLatencyTimer, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_24setLatencyTimer};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_25setLatencyTimer(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_latency = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setLatencyTimer (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_latency,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_latency)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 388, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "setLatencyTimer") < 0)) __PYX_ERR(0, 388, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_latency = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("setLatencyTimer", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 388, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.setLatencyTimer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_24setLatencyTimer(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_latency);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_24setLatencyTimer(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_latency) {
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  UCHAR __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setLatencyTimer", 1);

  /* "ft4222/ft4222.pyx":397
 *             FT2XXDeviceError: on error
 *         """
 *         status = FT_SetLatencyTimer(self._handle, latency)             # <<<<<<<<<<<<<<
 *         if status != FT_OK:
 *             raise FT2XXDeviceError, status
 */
  __pyx_t_1 = __Pyx_PyInt_As_UCHAR(__pyx_v_latency); if (unlikely((__pyx_t_1 == ((UCHAR)-1)) && PyErr_Occurred())) __PYX_ERR(0, 397, __pyx_L1_error)
  __pyx_v_status = FT_SetLatencyTimer(__pyx_v_self->_handle, __pyx_t_1);

  /* "ft4222/ft4222.pyx":398
 *         """
 *         status = FT_SetLatencyTimer(self._handle, latency)
 *         if status != FT_OK:             # <<<<<<<<<<<<<<
 *             raise FT2XXDeviceError, status
 * 
 */
  __pyx_t_2 = (__pyx_v_status != FT_OK);
  if (unlikely(__pyx_t_2)) {

    /* "ft4222/ft4222.pyx":399
 *         status = FT_SetLatencyTimer(self._handle, latency)
 *         if status != FT_OK:
 *             raise FT2XXDeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def chipReset(self):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT2XXDeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 399, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 399, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 399, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":398
 *         """
 *         status = FT_SetLatencyTimer(self._handle, latency)
 *         if status != FT_OK:             # <<<<<<<<<<<<<<
 *             raise FT2XXDeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":388
 *         raise FT2XXDeviceError, status
 * 
 *     def setLatencyTimer(self, latency):             # <<<<<<<<<<<<<<
 *         """Set the current value of the latency timer.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.setLatencyTimer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":401
 *             raise FT2XXDeviceError, status
 * 
 *     def chipReset(self):             # <<<<<<<<<<<<<<
 *         """Software reset for device.
 *         This function is used to attempt to recover system after a failure. It is a software reset for device.
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_27chipReset(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_26chipReset, "Software reset for device.\n        This function is used to attempt to recover system after a failure. It is a software reset for device.\n\n        Re-opening the device may be needed after calling this function.\n\n        Raises:\n            FT2XXDeviceError: on error\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_27chipReset = {"chipReset", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_27chipReset, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_26chipReset};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_27chipReset(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("chipReset (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("chipReset", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "chipReset", 0))) return NULL;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_26chipReset(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_26chipReset(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("chipReset", 1);

  /* "ft4222/ft4222.pyx":410
 *             FT2XXDeviceError: on error
 *         """
 *         status = FT4222_ChipReset(self._handle)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT2XXDeviceError, status
 */
  __pyx_v_status = FT4222_ChipReset(__pyx_v_self->_handle);

  /* "ft4222/ft4222.pyx":411
 *         """
 *         status = FT4222_ChipReset(self._handle)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT2XXDeviceError, status
 * 
 */
  __pyx_t_1 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":412
 *         status = FT4222_ChipReset(self._handle)
 *         if status != FT4222_OK:
 *             raise FT2XXDeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def gpio_Init(self, *args, gpio0=Dir.INPUT, gpio1=Dir.INPUT, gpio2=Dir.INPUT, gpio3=Dir.INPUT):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT2XXDeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 412, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 412, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 412, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":411
 *         """
 *         status = FT4222_ChipReset(self._handle)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT2XXDeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":401
 *             raise FT2XXDeviceError, status
 * 
 *     def chipReset(self):             # <<<<<<<<<<<<<<
 *         """Software reset for device.
 *         This function is used to attempt to recover system after a failure. It is a software reset for device.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.chipReset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":414
 *             raise FT2XXDeviceError, status
 * 
 *     def gpio_Init(self, *args, gpio0=Dir.INPUT, gpio1=Dir.INPUT, gpio2=Dir.INPUT, gpio3=Dir.INPUT):             # <<<<<<<<<<<<<<
 *         """Initialize the GPIO interface.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_29gpio_Init(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_28gpio_Init, "Initialize the GPIO interface.\n\n        Args:\n            *args (:obj:`list` of :obj:`ft4222.GPIO.Dir`, optional): List containing a direction for each port.\n            gpio0 (:obj:`ft4222.GPIO.Dir`, optional): Direction of gpio0\n            gpio1 (:obj:`ft4222.GPIO.Dir`, optional): Direction of gpio1\n            gpio2 (:obj:`ft4222.GPIO.Dir`, optional): Direction of gpio2\n            gpio3 (:obj:`ft4222.GPIO.Dir`, optional): Direction of gpio3\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_29gpio_Init = {"gpio_Init", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_29gpio_Init, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_28gpio_Init};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_29gpio_Init(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_gpio0 = 0;
  PyObject *__pyx_v_gpio1 = 0;
  PyObject *__pyx_v_gpio2 = 0;
  PyObject *__pyx_v_gpio3 = 0;
  PyObject *__pyx_v_args = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gpio_Init (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_gpio0,&__pyx_n_s_gpio1,&__pyx_n_s_gpio2,&__pyx_n_s_gpio3,0};
    values[0] = __Pyx_Arg_NewRef_VARARGS(__pyx_k__2);
    values[1] = __Pyx_Arg_NewRef_VARARGS(__pyx_k__3);
    values[2] = __Pyx_Arg_NewRef_VARARGS(__pyx_k__4);
    values[3] = __Pyx_Arg_NewRef_VARARGS(__pyx_k__5);
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        default:
        case  0: break;
      }
      kw_args = __Pyx_NumKwargs_VARARGS(__pyx_kwds);
      if (kw_args > 0 && likely(kw_args <= 4)) {
        Py_ssize_t index;
        for (index = 0; index < 4 && kw_args > 0; index++) {
          PyObject* value = __Pyx_GetKwValue_VARARGS(__pyx_kwds, __pyx_kwvalues, *__pyx_pyargnames[index]);
          if (value) { values[index] = __Pyx_Arg_NewRef_VARARGS(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 414, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, 0, "gpio_Init") < 0)) __PYX_ERR(0, 414, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs < 0)) {
      goto __pyx_L5_argtuple_error;
    } else {
    }
    __pyx_v_gpio0 = values[0];
    __pyx_v_gpio1 = values[1];
    __pyx_v_gpio2 = values[2];
    __pyx_v_gpio3 = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gpio_Init", 0, 0, 0, __pyx_nargs); __PYX_ERR(0, 414, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_VARARGS(values[__pyx_temp]);
    }
  }
  __Pyx_DECREF(__pyx_v_args); __pyx_v_args = 0;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_Init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_28gpio_Init(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_gpio0, __pyx_v_gpio1, __pyx_v_gpio2, __pyx_v_gpio3, __pyx_v_args);

  /* function exit code */
  __Pyx_DECREF(__pyx_v_args);
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_VARARGS(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_28gpio_Init(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_gpio0, PyObject *__pyx_v_gpio1, PyObject *__pyx_v_gpio2, PyObject *__pyx_v_gpio3, PyObject *__pyx_v_args) {
  GPIO_Dir __pyx_v_ioDir[4];
  Py_ssize_t __pyx_v_i;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  GPIO_Dir __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gpio_Init", 1);

  /* "ft4222/ft4222.pyx":430
 *         cdef:
 *             GPIO_Dir ioDir[4]
 *         if len(args) > 0:             # <<<<<<<<<<<<<<
 *             for i in xrange(len(args)):
 *                 ioDir[i] = args[i]
 */
  __pyx_t_1 = __Pyx_PyTuple_GET_SIZE(__pyx_v_args); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 430, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_1 > 0);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":431
 *             GPIO_Dir ioDir[4]
 *         if len(args) > 0:
 *             for i in xrange(len(args)):             # <<<<<<<<<<<<<<
 *                 ioDir[i] = args[i]
 *             for i in xrange(len(args), 4 - len(args)):
 */
    __pyx_t_1 = __Pyx_PyTuple_GET_SIZE(__pyx_v_args); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 431, __pyx_L1_error)
    __pyx_t_3 = __pyx_t_1;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "ft4222/ft4222.pyx":432
 *         if len(args) > 0:
 *             for i in xrange(len(args)):
 *                 ioDir[i] = args[i]             # <<<<<<<<<<<<<<
 *             for i in xrange(len(args), 4 - len(args)):
 *                 ioDir[i] = GPIO_INPUT
 */
      __pyx_t_5 = __Pyx_GetItemInt_Tuple(__pyx_v_args, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 432, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = ((GPIO_Dir)__Pyx_PyInt_As_GPIO_Dir(__pyx_t_5)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 432, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      (__pyx_v_ioDir[__pyx_v_i]) = __pyx_t_6;
    }

    /* "ft4222/ft4222.pyx":433
 *             for i in xrange(len(args)):
 *                 ioDir[i] = args[i]
 *             for i in xrange(len(args), 4 - len(args)):             # <<<<<<<<<<<<<<
 *                 ioDir[i] = GPIO_INPUT
 *         else:
 */
    __pyx_t_1 = __Pyx_PyTuple_GET_SIZE(__pyx_v_args); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 433, __pyx_L1_error)
    __pyx_t_3 = (4 - __pyx_t_1);
    __pyx_t_1 = __Pyx_PyTuple_GET_SIZE(__pyx_v_args); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 433, __pyx_L1_error)
    __pyx_t_4 = __pyx_t_3;
    for (__pyx_t_7 = __pyx_t_1; __pyx_t_7 < __pyx_t_4; __pyx_t_7+=1) {
      __pyx_v_i = __pyx_t_7;

      /* "ft4222/ft4222.pyx":434
 *                 ioDir[i] = args[i]
 *             for i in xrange(len(args), 4 - len(args)):
 *                 ioDir[i] = GPIO_INPUT             # <<<<<<<<<<<<<<
 *         else:
 *             ioDir[0] = gpio0
 */
      (__pyx_v_ioDir[__pyx_v_i]) = GPIO_INPUT;
    }

    /* "ft4222/ft4222.pyx":430
 *         cdef:
 *             GPIO_Dir ioDir[4]
 *         if len(args) > 0:             # <<<<<<<<<<<<<<
 *             for i in xrange(len(args)):
 *                 ioDir[i] = args[i]
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":436
 *                 ioDir[i] = GPIO_INPUT
 *         else:
 *             ioDir[0] = gpio0             # <<<<<<<<<<<<<<
 *             ioDir[1] = gpio1
 *             ioDir[2] = gpio2
 */
  /*else*/ {
    __pyx_t_6 = ((GPIO_Dir)__Pyx_PyInt_As_GPIO_Dir(__pyx_v_gpio0)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 436, __pyx_L1_error)
    (__pyx_v_ioDir[0]) = __pyx_t_6;

    /* "ft4222/ft4222.pyx":437
 *         else:
 *             ioDir[0] = gpio0
 *             ioDir[1] = gpio1             # <<<<<<<<<<<<<<
 *             ioDir[2] = gpio2
 *             ioDir[3] = gpio3
 */
    __pyx_t_6 = ((GPIO_Dir)__Pyx_PyInt_As_GPIO_Dir(__pyx_v_gpio1)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 437, __pyx_L1_error)
    (__pyx_v_ioDir[1]) = __pyx_t_6;

    /* "ft4222/ft4222.pyx":438
 *             ioDir[0] = gpio0
 *             ioDir[1] = gpio1
 *             ioDir[2] = gpio2             # <<<<<<<<<<<<<<
 *             ioDir[3] = gpio3
 *         status = FT4222_GPIO_Init(self._handle, ioDir)
 */
    __pyx_t_6 = ((GPIO_Dir)__Pyx_PyInt_As_GPIO_Dir(__pyx_v_gpio2)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 438, __pyx_L1_error)
    (__pyx_v_ioDir[2]) = __pyx_t_6;

    /* "ft4222/ft4222.pyx":439
 *             ioDir[1] = gpio1
 *             ioDir[2] = gpio2
 *             ioDir[3] = gpio3             # <<<<<<<<<<<<<<
 *         status = FT4222_GPIO_Init(self._handle, ioDir)
 *         if status != FT4222_OK:
 */
    __pyx_t_6 = ((GPIO_Dir)__Pyx_PyInt_As_GPIO_Dir(__pyx_v_gpio3)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 439, __pyx_L1_error)
    (__pyx_v_ioDir[3]) = __pyx_t_6;
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":440
 *             ioDir[2] = gpio2
 *             ioDir[3] = gpio3
 *         status = FT4222_GPIO_Init(self._handle, ioDir)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_v_status = FT4222_GPIO_Init(__pyx_v_self->_handle, __pyx_v_ioDir);

  /* "ft4222/ft4222.pyx":441
 *             ioDir[3] = gpio3
 *         status = FT4222_GPIO_Init(self._handle, ioDir)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_2 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_2)) {

    /* "ft4222/ft4222.pyx":442
 *         status = FT4222_GPIO_Init(self._handle, ioDir)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def gpio_Read(self, portNum):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 442, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 442, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_5, __pyx_t_8, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __PYX_ERR(0, 442, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":441
 *             ioDir[3] = gpio3
 *         status = FT4222_GPIO_Init(self._handle, ioDir)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":414
 *             raise FT2XXDeviceError, status
 * 
 *     def gpio_Init(self, *args, gpio0=Dir.INPUT, gpio1=Dir.INPUT, gpio2=Dir.INPUT, gpio3=Dir.INPUT):             # <<<<<<<<<<<<<<
 *         """Initialize the GPIO interface.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_Init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":444
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_Read(self, portNum):             # <<<<<<<<<<<<<<
 *         """Read value from selected GPIO
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_31gpio_Read(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_30gpio_Read, "Read value from selected GPIO\n\n        Args:\n            portNum (:obj:`ft4222.GPIO.Port`): GPIO Port number\n\n        Returns:\n            bool: True if high, False otherwise\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_31gpio_Read = {"gpio_Read", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_31gpio_Read, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_30gpio_Read};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_31gpio_Read(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_portNum = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gpio_Read (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_portNum,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_portNum)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 444, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "gpio_Read") < 0)) __PYX_ERR(0, 444, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_portNum = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gpio_Read", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 444, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_Read", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_30gpio_Read(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_portNum);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_30gpio_Read(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum) {
  BOOL __pyx_v_value;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GPIO_Port __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gpio_Read", 1);

  /* "ft4222/ft4222.pyx":459
 *         cdef:
 *             BOOL value
 *         status = FT4222_GPIO_Read(self._handle, portNum, &value)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return value
 */
  __pyx_t_1 = ((GPIO_Port)__Pyx_PyInt_As_GPIO_Port(__pyx_v_portNum)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 459, __pyx_L1_error)
  __pyx_v_status = FT4222_GPIO_Read(__pyx_v_self->_handle, __pyx_t_1, (&__pyx_v_value));

  /* "ft4222/ft4222.pyx":460
 *             BOOL value
 *         status = FT4222_GPIO_Read(self._handle, portNum, &value)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return value
 *         raise FT4222DeviceError, status
 */
  __pyx_t_2 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":461
 *         status = FT4222_GPIO_Read(self._handle, portNum, &value)
 *         if status == FT4222_OK:
 *             return value             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_BOOL(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 461, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":460
 *             BOOL value
 *         status = FT4222_GPIO_Read(self._handle, portNum, &value)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return value
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":462
 *         if status == FT4222_OK:
 *             return value
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def gpio_Wait(self, portNum, expectedValue, timeout=0, sleep=1):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __PYX_ERR(0, 462, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":444
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_Read(self, portNum):             # <<<<<<<<<<<<<<
 *         """Read value from selected GPIO
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_Read", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":464
 *         raise FT4222DeviceError, status
 * 
 *     def gpio_Wait(self, portNum, expectedValue, timeout=0, sleep=1):             # <<<<<<<<<<<<<<
 *         """Blocks until the selected GPIO reaches a certain value.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_33gpio_Wait(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_32gpio_Wait, "Blocks until the selected GPIO reaches a certain value.\n\n        Will release the GIL while waiting.\n\n        Args:\n            portNum (:obj:`ft4222.GPIO.Port`): GPIO Port number\n            expectedValue (bool): True for high, False for low\n            timeout (int): timeout in milliseconds, or 0 for no timeout\n            sleep (int): sleep time in milliseconds between each polling attempt, or 0 for no sleep\n\n        Raises:\n            FT4222DeviceError: on error\n            TimeoutError: when timeout is exceeded\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_33gpio_Wait = {"gpio_Wait", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_33gpio_Wait, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_32gpio_Wait};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_33gpio_Wait(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_portNum = 0;
  PyObject *__pyx_v_expectedValue = 0;
  PyObject *__pyx_v_timeout = 0;
  PyObject *__pyx_v_sleep = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gpio_Wait (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_portNum,&__pyx_n_s_expectedValue,&__pyx_n_s_timeout,&__pyx_n_s_sleep,0};
    values[2] = __Pyx_Arg_NewRef_FASTCALL(((PyObject *)__pyx_int_0));
    values[3] = __Pyx_Arg_NewRef_FASTCALL(((PyObject *)__pyx_int_1));
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  4: values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_portNum)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 464, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_expectedValue)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 464, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("gpio_Wait", 0, 2, 4, 1); __PYX_ERR(0, 464, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_timeout);
          if (value) { values[2] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 464, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_sleep);
          if (value) { values[3] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 464, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "gpio_Wait") < 0)) __PYX_ERR(0, 464, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  4: values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_portNum = values[0];
    __pyx_v_expectedValue = values[1];
    __pyx_v_timeout = values[2];
    __pyx_v_sleep = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gpio_Wait", 0, 2, 4, __pyx_nargs); __PYX_ERR(0, 464, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_Wait", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_32gpio_Wait(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_portNum, __pyx_v_expectedValue, __pyx_v_timeout, __pyx_v_sleep);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_32gpio_Wait(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum, PyObject *__pyx_v_expectedValue, PyObject *__pyx_v_timeout, PyObject *__pyx_v_sleep) {
  BOOL __pyx_v_value;
  FT4222_STATUS __pyx_v_status;
  GPIO_Port __pyx_v_portNum_c;
  BOOL __pyx_v_expectedValue_c;
  clock_t __pyx_v_start;
  int __pyx_v_timeout_c;
  int __pyx_v_sleep_c;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GPIO_Port __pyx_t_1;
  BOOL __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  clock_t __pyx_t_6;
  double __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gpio_Wait", 1);

  /* "ft4222/ft4222.pyx":483
 *             BOOL value
 *             FT4222_STATUS status
 *             GPIO_Port portNum_c = portNum             # <<<<<<<<<<<<<<
 *             BOOL expectedValue_c = expectedValue
 *             clock_t start = clock()
 */
  __pyx_t_1 = ((GPIO_Port)__Pyx_PyInt_As_GPIO_Port(__pyx_v_portNum)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 483, __pyx_L1_error)
  __pyx_v_portNum_c = __pyx_t_1;

  /* "ft4222/ft4222.pyx":484
 *             FT4222_STATUS status
 *             GPIO_Port portNum_c = portNum
 *             BOOL expectedValue_c = expectedValue             # <<<<<<<<<<<<<<
 *             clock_t start = clock()
 *             int timeout_c = timeout
 */
  __pyx_t_2 = __Pyx_PyInt_As_BOOL(__pyx_v_expectedValue); if (unlikely((__pyx_t_2 == ((BOOL)-1)) && PyErr_Occurred())) __PYX_ERR(0, 484, __pyx_L1_error)
  __pyx_v_expectedValue_c = __pyx_t_2;

  /* "ft4222/ft4222.pyx":485
 *             GPIO_Port portNum_c = portNum
 *             BOOL expectedValue_c = expectedValue
 *             clock_t start = clock()             # <<<<<<<<<<<<<<
 *             int timeout_c = timeout
 *             int sleep_c = sleep
 */
  __pyx_v_start = clock();

  /* "ft4222/ft4222.pyx":486
 *             BOOL expectedValue_c = expectedValue
 *             clock_t start = clock()
 *             int timeout_c = timeout             # <<<<<<<<<<<<<<
 *             int sleep_c = sleep
 * 
 */
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_v_timeout); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 486, __pyx_L1_error)
  __pyx_v_timeout_c = __pyx_t_3;

  /* "ft4222/ft4222.pyx":487
 *             clock_t start = clock()
 *             int timeout_c = timeout
 *             int sleep_c = sleep             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
 */
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_v_sleep); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 487, __pyx_L1_error)
  __pyx_v_sleep_c = __pyx_t_3;

  /* "ft4222/ft4222.pyx":489
 *             int sleep_c = sleep
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             while True:
 *                 if timeout_c > 0 and (clock() - start) / (CLOCKS_PER_SEC / 1000) > timeout_c:
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      _save = NULL;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "ft4222/ft4222.pyx":490
 * 
 *         with nogil:
 *             while True:             # <<<<<<<<<<<<<<
 *                 if timeout_c > 0 and (clock() - start) / (CLOCKS_PER_SEC / 1000) > timeout_c:
 *                     raise TimeoutError
 */
        while (1) {

          /* "ft4222/ft4222.pyx":491
 *         with nogil:
 *             while True:
 *                 if timeout_c > 0 and (clock() - start) / (CLOCKS_PER_SEC / 1000) > timeout_c:             # <<<<<<<<<<<<<<
 *                     raise TimeoutError
 * 
 */
          __pyx_t_5 = (__pyx_v_timeout_c > 0);
          if (__pyx_t_5) {
          } else {
            __pyx_t_4 = __pyx_t_5;
            goto __pyx_L9_bool_binop_done;
          }
          __pyx_t_6 = (clock() - __pyx_v_start);
          __pyx_t_7 = (((double)CLOCKS_PER_SEC) / 1000.0);
          if (unlikely(__pyx_t_7 == 0)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
            #endif
            PyErr_SetString(PyExc_ZeroDivisionError, "float division");
            #ifdef WITH_THREAD
            __Pyx_PyGILState_Release(__pyx_gilstate_save);
            #endif
            __PYX_ERR(0, 491, __pyx_L4_error)
          }
          __pyx_t_5 = ((((double)__pyx_t_6) / __pyx_t_7) > __pyx_v_timeout_c);
          __pyx_t_4 = __pyx_t_5;
          __pyx_L9_bool_binop_done:;
          if (unlikely(__pyx_t_4)) {

            /* "ft4222/ft4222.pyx":492
 *             while True:
 *                 if timeout_c > 0 and (clock() - start) / (CLOCKS_PER_SEC / 1000) > timeout_c:
 *                     raise TimeoutError             # <<<<<<<<<<<<<<
 * 
 *                 status = FT4222_GPIO_Read(self._handle, portNum_c, &value)
 */
            {
                #ifdef WITH_THREAD
                PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
                #endif
                /*try:*/ {
                  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_TimeoutError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 492, __pyx_L14_error)
                  __Pyx_GOTREF(__pyx_t_8);
                  __Pyx_Raise(__pyx_t_8, 0, 0, 0);
                  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
                  __PYX_ERR(0, 492, __pyx_L14_error)
                }
                /*finally:*/ {
                  __pyx_L14_error: {
                    #ifdef WITH_THREAD
                    __Pyx_PyGILState_Release(__pyx_gilstate_save);
                    #endif
                    goto __pyx_L4_error;
                  }
                }
            }

            /* "ft4222/ft4222.pyx":491
 *         with nogil:
 *             while True:
 *                 if timeout_c > 0 and (clock() - start) / (CLOCKS_PER_SEC / 1000) > timeout_c:             # <<<<<<<<<<<<<<
 *                     raise TimeoutError
 * 
 */
          }

          /* "ft4222/ft4222.pyx":494
 *                     raise TimeoutError
 * 
 *                 status = FT4222_GPIO_Read(self._handle, portNum_c, &value)             # <<<<<<<<<<<<<<
 *                 if status == FT4222_OK:
 *                     if value == expectedValue_c:
 */
          __pyx_v_status = FT4222_GPIO_Read(__pyx_v_self->_handle, __pyx_v_portNum_c, (&__pyx_v_value));

          /* "ft4222/ft4222.pyx":495
 * 
 *                 status = FT4222_GPIO_Read(self._handle, portNum_c, &value)
 *                 if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *                     if value == expectedValue_c:
 *                         break
 */
          __pyx_t_4 = (__pyx_v_status == FT4222_OK);
          if (likely(__pyx_t_4)) {

            /* "ft4222/ft4222.pyx":496
 *                 status = FT4222_GPIO_Read(self._handle, portNum_c, &value)
 *                 if status == FT4222_OK:
 *                     if value == expectedValue_c:             # <<<<<<<<<<<<<<
 *                         break
 *                 else:
 */
            __pyx_t_4 = (__pyx_v_value == __pyx_v_expectedValue_c);
            if (__pyx_t_4) {

              /* "ft4222/ft4222.pyx":497
 *                 if status == FT4222_OK:
 *                     if value == expectedValue_c:
 *                         break             # <<<<<<<<<<<<<<
 *                 else:
 *                     raise FT4222DeviceError, status
 */
              goto __pyx_L7_break;

              /* "ft4222/ft4222.pyx":496
 *                 status = FT4222_GPIO_Read(self._handle, portNum_c, &value)
 *                 if status == FT4222_OK:
 *                     if value == expectedValue_c:             # <<<<<<<<<<<<<<
 *                         break
 *                 else:
 */
            }

            /* "ft4222/ft4222.pyx":495
 * 
 *                 status = FT4222_GPIO_Read(self._handle, portNum_c, &value)
 *                 if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *                     if value == expectedValue_c:
 *                         break
 */
            goto __pyx_L16;
          }

          /* "ft4222/ft4222.pyx":499
 *                         break
 *                 else:
 *                     raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *                 if sleep_c > 0:
 */
          /*else*/ {
            {
                #ifdef WITH_THREAD
                PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
                #endif
                /*try:*/ {
                  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 499, __pyx_L21_error)
                  __Pyx_GOTREF(__pyx_t_8);
                  __pyx_t_9 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 499, __pyx_L21_error)
                  __Pyx_GOTREF(__pyx_t_9);
                  __Pyx_Raise(__pyx_t_8, __pyx_t_9, 0, 0);
                  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
                  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                  __PYX_ERR(0, 499, __pyx_L21_error)
                }
                /*finally:*/ {
                  __pyx_L21_error: {
                    #ifdef WITH_THREAD
                    __Pyx_PyGILState_Release(__pyx_gilstate_save);
                    #endif
                    goto __pyx_L4_error;
                  }
                }
            }
          }
          __pyx_L16:;

          /* "ft4222/ft4222.pyx":501
 *                     raise FT4222DeviceError, status
 * 
 *                 if sleep_c > 0:             # <<<<<<<<<<<<<<
 *                     msleep(sleep_c)
 * 
 */
          __pyx_t_4 = (__pyx_v_sleep_c > 0);
          if (__pyx_t_4) {

            /* "ft4222/ft4222.pyx":502
 * 
 *                 if sleep_c > 0:
 *                     msleep(sleep_c)             # <<<<<<<<<<<<<<
 * 
 *     def gpio_Write(self, portNum, value):
 */
            msleep(__pyx_v_sleep_c);

            /* "ft4222/ft4222.pyx":501
 *                     raise FT4222DeviceError, status
 * 
 *                 if sleep_c > 0:             # <<<<<<<<<<<<<<
 *                     msleep(sleep_c)
 * 
 */
          }
        }
        __pyx_L7_break:;
      }

      /* "ft4222/ft4222.pyx":489
 *             int sleep_c = sleep
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             while True:
 *                 if timeout_c > 0 and (clock() - start) / (CLOCKS_PER_SEC / 1000) > timeout_c:
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L4_error: {
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L1_error;
        }
        __pyx_L5:;
      }
  }

  /* "ft4222/ft4222.pyx":464
 *         raise FT4222DeviceError, status
 * 
 *     def gpio_Wait(self, portNum, expectedValue, timeout=0, sleep=1):             # <<<<<<<<<<<<<<
 *         """Blocks until the selected GPIO reaches a certain value.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_Wait", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":504
 *                     msleep(sleep_c)
 * 
 *     def gpio_Write(self, portNum, value):             # <<<<<<<<<<<<<<
 *         """Write value to given GPIO
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_35gpio_Write(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_34gpio_Write, "Write value to given GPIO\n\n        Args:\n            portNum (:obj:`ft4222.GPIO.Port`): GPIO Port number\n            value (bool): True for high, False for low\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_35gpio_Write = {"gpio_Write", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_35gpio_Write, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_34gpio_Write};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_35gpio_Write(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_portNum = 0;
  PyObject *__pyx_v_value = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gpio_Write (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_portNum,&__pyx_n_s_value,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_portNum)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 504, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_value)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 504, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("gpio_Write", 1, 2, 2, 1); __PYX_ERR(0, 504, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "gpio_Write") < 0)) __PYX_ERR(0, 504, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_portNum = values[0];
    __pyx_v_value = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gpio_Write", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 504, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_Write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_34gpio_Write(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_portNum, __pyx_v_value);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_34gpio_Write(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum, PyObject *__pyx_v_value) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GPIO_Port __pyx_t_1;
  BOOL __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gpio_Write", 1);

  /* "ft4222/ft4222.pyx":515
 * 
 *         """
 *         status = FT4222_GPIO_Write(self._handle, portNum, value)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((GPIO_Port)__Pyx_PyInt_As_GPIO_Port(__pyx_v_portNum)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 515, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_BOOL(__pyx_v_value); if (unlikely((__pyx_t_2 == ((BOOL)-1)) && PyErr_Occurred())) __PYX_ERR(0, 515, __pyx_L1_error)
  __pyx_v_status = FT4222_GPIO_Write(__pyx_v_self->_handle, __pyx_t_1, __pyx_t_2);

  /* "ft4222/ft4222.pyx":516
 *         """
 *         status = FT4222_GPIO_Write(self._handle, portNum, value)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_3 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_3)) {

    /* "ft4222/ft4222.pyx":517
 *         status = FT4222_GPIO_Write(self._handle, portNum, value)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def gpio_SetInputTrigger(self, portNum, trigger):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 517, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 517, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_4, __pyx_t_5, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 517, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":516
 *         """
 *         status = FT4222_GPIO_Write(self._handle, portNum, value)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":504
 *                     msleep(sleep_c)
 * 
 *     def gpio_Write(self, portNum, value):             # <<<<<<<<<<<<<<
 *         """Write value to given GPIO
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_Write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":519
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_SetInputTrigger(self, portNum, trigger):             # <<<<<<<<<<<<<<
 *         """Set software trigger conditions on the specified GPIO pin.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_37gpio_SetInputTrigger(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_36gpio_SetInputTrigger, "Set software trigger conditions on the specified GPIO pin.\n\n        Args:\n            portNum (:obj:`ft4222.GPIO.Port`): GPIO Port number\n            trigger (:obj:`ft4222.GPIO.Trigger`): Combination of trigger conditions\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_37gpio_SetInputTrigger = {"gpio_SetInputTrigger", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_37gpio_SetInputTrigger, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_36gpio_SetInputTrigger};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_37gpio_SetInputTrigger(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_portNum = 0;
  PyObject *__pyx_v_trigger = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gpio_SetInputTrigger (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_portNum,&__pyx_n_s_trigger,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_portNum)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 519, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_trigger)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 519, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("gpio_SetInputTrigger", 1, 2, 2, 1); __PYX_ERR(0, 519, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "gpio_SetInputTrigger") < 0)) __PYX_ERR(0, 519, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_portNum = values[0];
    __pyx_v_trigger = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gpio_SetInputTrigger", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 519, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_SetInputTrigger", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_36gpio_SetInputTrigger(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_portNum, __pyx_v_trigger);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_36gpio_SetInputTrigger(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum, PyObject *__pyx_v_trigger) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GPIO_Port __pyx_t_1;
  GPIO_Trigger __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gpio_SetInputTrigger", 1);

  /* "ft4222/ft4222.pyx":530
 * 
 *         """
 *         status = FT4222_GPIO_SetInputTrigger(self._handle, portNum, trigger)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((GPIO_Port)__Pyx_PyInt_As_GPIO_Port(__pyx_v_portNum)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 530, __pyx_L1_error)
  __pyx_t_2 = ((GPIO_Trigger)__Pyx_PyInt_As_GPIO_Trigger(__pyx_v_trigger)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 530, __pyx_L1_error)
  __pyx_v_status = FT4222_GPIO_SetInputTrigger(__pyx_v_self->_handle, __pyx_t_1, __pyx_t_2);

  /* "ft4222/ft4222.pyx":531
 *         """
 *         status = FT4222_GPIO_SetInputTrigger(self._handle, portNum, trigger)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_3 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_3)) {

    /* "ft4222/ft4222.pyx":532
 *         status = FT4222_GPIO_SetInputTrigger(self._handle, portNum, trigger)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def gpio_GetTriggerStatus(self, portNum):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 532, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 532, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_4, __pyx_t_5, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 532, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":531
 *         """
 *         status = FT4222_GPIO_SetInputTrigger(self._handle, portNum, trigger)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":519
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_SetInputTrigger(self, portNum, trigger):             # <<<<<<<<<<<<<<
 *         """Set software trigger conditions on the specified GPIO pin.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_SetInputTrigger", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":534
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_GetTriggerStatus(self, portNum):             # <<<<<<<<<<<<<<
 *         """Get the size of trigger event queue.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_39gpio_GetTriggerStatus(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_38gpio_GetTriggerStatus, "Get the size of trigger event queue.\n\n        Args:\n            portNum (:obj:`ft4222.GPIO.Port`): GPIO Port number\n\n        Returns:\n            int: Size of the trigger queue\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_39gpio_GetTriggerStatus = {"gpio_GetTriggerStatus", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_39gpio_GetTriggerStatus, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_38gpio_GetTriggerStatus};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_39gpio_GetTriggerStatus(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_portNum = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gpio_GetTriggerStatus (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_portNum,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_portNum)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 534, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "gpio_GetTriggerStatus") < 0)) __PYX_ERR(0, 534, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_portNum = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gpio_GetTriggerStatus", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 534, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_GetTriggerStatus", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_38gpio_GetTriggerStatus(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_portNum);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_38gpio_GetTriggerStatus(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum) {
  uint16 __pyx_v_queueSize;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GPIO_Port __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gpio_GetTriggerStatus", 1);

  /* "ft4222/ft4222.pyx":549
 *         cdef:
 *             uint16 queueSize
 *         status = FT4222_GPIO_GetTriggerStatus(self._handle, portNum, &queueSize)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return queueSize
 */
  __pyx_t_1 = ((GPIO_Port)__Pyx_PyInt_As_GPIO_Port(__pyx_v_portNum)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 549, __pyx_L1_error)
  __pyx_v_status = FT4222_GPIO_GetTriggerStatus(__pyx_v_self->_handle, __pyx_t_1, (&__pyx_v_queueSize));

  /* "ft4222/ft4222.pyx":550
 *             uint16 queueSize
 *         status = FT4222_GPIO_GetTriggerStatus(self._handle, portNum, &queueSize)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return queueSize
 *         raise FT4222DeviceError, status
 */
  __pyx_t_2 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":551
 *         status = FT4222_GPIO_GetTriggerStatus(self._handle, portNum, &queueSize)
 *         if status == FT4222_OK:
 *             return queueSize             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_uint16_t(__pyx_v_queueSize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 551, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":550
 *             uint16 queueSize
 *         status = FT4222_GPIO_GetTriggerStatus(self._handle, portNum, &queueSize)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return queueSize
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":552
 *         if status == FT4222_OK:
 *             return queueSize
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def gpio_ReadTriggerQueue(self, portNum, readSize=None):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __PYX_ERR(0, 552, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":534
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_GetTriggerStatus(self, portNum):             # <<<<<<<<<<<<<<
 *         """Get the size of trigger event queue.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_GetTriggerStatus", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":554
 *         raise FT4222DeviceError, status
 * 
 *     def gpio_ReadTriggerQueue(self, portNum, readSize=None):             # <<<<<<<<<<<<<<
 *         """Get events recorded in the trigger event queue.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_41gpio_ReadTriggerQueue(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_40gpio_ReadTriggerQueue, "Get events recorded in the trigger event queue.\n\n        Args:\n            portNum (:obj:`ft4222.GPIO.Port`): GPIO Port number\n            readSize (:obj:`int`, optional): Size of the queue, :obj:`gpio_GetTriggerStatus()` gets called if omitted\n\n        Returns:\n            :obj:`list` of :obj:`ft4222.GPIO.Trigger`: Trigger queue as list\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_41gpio_ReadTriggerQueue = {"gpio_ReadTriggerQueue", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_41gpio_ReadTriggerQueue, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_40gpio_ReadTriggerQueue};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_41gpio_ReadTriggerQueue(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_portNum = 0;
  PyObject *__pyx_v_readSize = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gpio_ReadTriggerQueue (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_portNum,&__pyx_n_s_readSize,0};
    values[1] = __Pyx_Arg_NewRef_FASTCALL(((PyObject *)Py_None));
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_portNum)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 554, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_readSize);
          if (value) { values[1] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 554, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "gpio_ReadTriggerQueue") < 0)) __PYX_ERR(0, 554, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_portNum = values[0];
    __pyx_v_readSize = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gpio_ReadTriggerQueue", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 554, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_ReadTriggerQueue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_40gpio_ReadTriggerQueue(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_portNum, __pyx_v_readSize);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_40gpio_ReadTriggerQueue(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum, PyObject *__pyx_v_readSize) {
  GPIO_Trigger *__pyx_v_events;
  uint16 __pyx_v_sizeRead;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  size_t __pyx_t_6;
  GPIO_Port __pyx_t_7;
  uint16 __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  PyObject *(*__pyx_t_10)(PyObject *);
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gpio_ReadTriggerQueue", 1);

  /* "ft4222/ft4222.pyx":568
 * 
 *         """
 *         if readSize == None:             # <<<<<<<<<<<<<<
 *             self.gpio_GetTriggerStatus(portNum)
 *         cdef:
 */
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_readSize, Py_None, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 568, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":569
 *         """
 *         if readSize == None:
 *             self.gpio_GetTriggerStatus(portNum)             # <<<<<<<<<<<<<<
 *         cdef:
 *             GPIO_Trigger *events = <GPIO_Trigger*>alloca(portNum * sizeof(GPIO_Trigger))
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_gpio_GetTriggerStatus); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 569, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    __pyx_t_5 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_5 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_portNum};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 569, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ft4222/ft4222.pyx":568
 * 
 *         """
 *         if readSize == None:             # <<<<<<<<<<<<<<
 *             self.gpio_GetTriggerStatus(portNum)
 *         cdef:
 */
  }

  /* "ft4222/ft4222.pyx":571
 *             self.gpio_GetTriggerStatus(portNum)
 *         cdef:
 *             GPIO_Trigger *events = <GPIO_Trigger*>alloca(portNum * sizeof(GPIO_Trigger))             # <<<<<<<<<<<<<<
 *             uint16 sizeRead
 *         status = FT4222_GPIO_ReadTriggerQueue(self._handle, portNum, events, readSize, &sizeRead)
 */
  __pyx_t_1 = __Pyx_PyInt_FromSize_t((sizeof(GPIO_Trigger))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 571, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Multiply(__pyx_v_portNum, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 571, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyInt_As_size_t(__pyx_t_3); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 571, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_events = ((GPIO_Trigger *)alloca(__pyx_t_6));

  /* "ft4222/ft4222.pyx":573
 *             GPIO_Trigger *events = <GPIO_Trigger*>alloca(portNum * sizeof(GPIO_Trigger))
 *             uint16 sizeRead
 *         status = FT4222_GPIO_ReadTriggerQueue(self._handle, portNum, events, readSize, &sizeRead)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             res = []
 */
  __pyx_t_7 = ((GPIO_Port)__Pyx_PyInt_As_GPIO_Port(__pyx_v_portNum)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 573, __pyx_L1_error)
  __pyx_t_8 = __Pyx_PyInt_As_uint16_t(__pyx_v_readSize); if (unlikely((__pyx_t_8 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 573, __pyx_L1_error)
  __pyx_v_status = FT4222_GPIO_ReadTriggerQueue(__pyx_v_self->_handle, __pyx_t_7, __pyx_v_events, __pyx_t_8, (&__pyx_v_sizeRead));

  /* "ft4222/ft4222.pyx":574
 *             uint16 sizeRead
 *         status = FT4222_GPIO_ReadTriggerQueue(self._handle, portNum, events, readSize, &sizeRead)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             res = []
 *             for i in xrange(readSize):
 */
  __pyx_t_2 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":575
 *         status = FT4222_GPIO_ReadTriggerQueue(self._handle, portNum, events, readSize, &sizeRead)
 *         if status == FT4222_OK:
 *             res = []             # <<<<<<<<<<<<<<
 *             for i in xrange(readSize):
 *                 res.append(Trigger(events[i]))
 */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 575, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_res = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "ft4222/ft4222.pyx":576
 *         if status == FT4222_OK:
 *             res = []
 *             for i in xrange(readSize):             # <<<<<<<<<<<<<<
 *                 res.append(Trigger(events[i]))
 *             return res
 */
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_xrange, __pyx_v_readSize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 576, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
      __pyx_t_1 = __pyx_t_3; __Pyx_INCREF(__pyx_t_1);
      __pyx_t_9 = 0;
      __pyx_t_10 = NULL;
    } else {
      __pyx_t_9 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 576, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_10 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 576, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    for (;;) {
      if (likely(!__pyx_t_10)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 576, __pyx_L1_error)
            #endif
            if (__pyx_t_9 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_3); __pyx_t_9++; if (unlikely((0 < 0))) __PYX_ERR(0, 576, __pyx_L1_error)
          #else
          __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 576, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 576, __pyx_L1_error)
            #endif
            if (__pyx_t_9 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_3); __pyx_t_9++; if (unlikely((0 < 0))) __PYX_ERR(0, 576, __pyx_L1_error)
          #else
          __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 576, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        }
      } else {
        __pyx_t_3 = __pyx_t_10(__pyx_t_1);
        if (unlikely(!__pyx_t_3)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 576, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "ft4222/ft4222.pyx":577
 *             res = []
 *             for i in xrange(readSize):
 *                 res.append(Trigger(events[i]))             # <<<<<<<<<<<<<<
 *             return res
 *         raise FT4222DeviceError, status
 */
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_Trigger); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 577, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_11 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_11 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 577, __pyx_L1_error)
      __pyx_t_12 = __Pyx_PyInt_From_GPIO_Trigger((__pyx_v_events[__pyx_t_11])); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 577, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_13 = NULL;
      __pyx_t_5 = 0;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
          __pyx_t_5 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_t_12};
        __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 577, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      }
      __pyx_t_14 = __Pyx_PyList_Append(__pyx_v_res, __pyx_t_3); if (unlikely(__pyx_t_14 == ((int)-1))) __PYX_ERR(0, 577, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "ft4222/ft4222.pyx":576
 *         if status == FT4222_OK:
 *             res = []
 *             for i in xrange(readSize):             # <<<<<<<<<<<<<<
 *                 res.append(Trigger(events[i]))
 *             return res
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ft4222/ft4222.pyx":578
 *             for i in xrange(readSize):
 *                 res.append(Trigger(events[i]))
 *             return res             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_res);
    __pyx_r = __pyx_v_res;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":574
 *             uint16 sizeRead
 *         status = FT4222_GPIO_ReadTriggerQueue(self._handle, portNum, events, readSize, &sizeRead)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             res = []
 *             for i in xrange(readSize):
 */
  }

  /* "ft4222/ft4222.pyx":579
 *                 res.append(Trigger(events[i]))
 *             return res
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 579, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 579, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_1, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 579, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":554
 *         raise FT4222DeviceError, status
 * 
 *     def gpio_ReadTriggerQueue(self, portNum, readSize=None):             # <<<<<<<<<<<<<<
 *         """Get events recorded in the trigger event queue.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_ReadTriggerQueue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":582
 * 
 * 
 *     def i2cMaster_Init(self, kbps=100):             # <<<<<<<<<<<<<<
 *         """Initialize the FT4222H as an I2C master with the requested I2C speed.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_43i2cMaster_Init(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_42i2cMaster_Init, "Initialize the FT4222H as an I2C master with the requested I2C speed.\n\n        Args:\n            kbps (int): Speed in kb/s\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_43i2cMaster_Init = {"i2cMaster_Init", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_43i2cMaster_Init, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_42i2cMaster_Init};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_43i2cMaster_Init(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_kbps = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("i2cMaster_Init (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_kbps,0};
    values[0] = __Pyx_Arg_NewRef_FASTCALL(((PyObject *)__pyx_int_100));
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_kbps);
          if (value) { values[0] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 582, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "i2cMaster_Init") < 0)) __PYX_ERR(0, 582, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_kbps = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("i2cMaster_Init", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 582, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_Init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_42i2cMaster_Init(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_kbps);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_42i2cMaster_Init(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_kbps) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_v_n = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  uint32 __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  long __pyx_t_5;
  long __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("i2cMaster_Init", 1);

  /* "ft4222/ft4222.pyx":592
 * 
 *         """
 *         status = FT4222_I2CMaster_Init(self._handle, kbps)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = __Pyx_PyInt_As_uint32_t(__pyx_v_kbps); if (unlikely((__pyx_t_1 == ((uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 592, __pyx_L1_error)
  __pyx_v_status = FT4222_I2CMaster_Init(__pyx_v_self->_handle, __pyx_t_1);

  /* "ft4222/ft4222.pyx":593
 *         """
 *         status = FT4222_I2CMaster_Init(self._handle, kbps)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 *         # current version (v1.3) of ftdi's lib can only handle clock rates down to 60kHz
 */
  __pyx_t_2 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_2)) {

    /* "ft4222/ft4222.pyx":594
 *         status = FT4222_I2CMaster_Init(self._handle, kbps)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 *         # current version (v1.3) of ftdi's lib can only handle clock rates down to 60kHz
 *         # although the chip can handle clock rates down to ~23.6kHz
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 594, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 594, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 594, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":593
 *         """
 *         status = FT4222_I2CMaster_Init(self._handle, kbps)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 *         # current version (v1.3) of ftdi's lib can only handle clock rates down to 60kHz
 */
  }

  /* "ft4222/ft4222.pyx":598
 *         # although the chip can handle clock rates down to ~23.6kHz
 *         # there's a undocumented ways to achieve this
 *         if kbps < 60:             # <<<<<<<<<<<<<<
 *             #             Operating Clock Freq
 *             # SCL Freq = -----------------------  | M = 6 or 8; N = 1, 2, 3, , 127
 */
  __pyx_t_4 = PyObject_RichCompare(__pyx_v_kbps, __pyx_int_60, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 598, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 598, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":607
 *             #            M * SCL Freq               8 * kbps
 *             #
 *             n = max(min(int(round(24000000.0 / (8 * 1000 * kbps) - 1)), 127), 1)             # <<<<<<<<<<<<<<
 *             self.setClock(SysClock.CLK_24)
 *             self.vendorCmdSet(0x52, n)
 */
    __pyx_t_5 = 1;
    __pyx_t_6 = 0x7F;
    __pyx_t_4 = __Pyx_PyInt_MultiplyCObj(__pyx_int_8000, __pyx_v_kbps, 0x1f40, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyFloat_TrueDivideCObj(__pyx_float_24000000_0, __pyx_t_4, 24000000.0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyInt_SubtractObjC(__pyx_t_3, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_round, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyNumber_Int(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_7 = __Pyx_PyInt_From_long(__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = PyObject_RichCompare(__pyx_t_7, __pyx_t_4, Py_LT); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 607, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 607, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_2) {
      __pyx_t_8 = __Pyx_PyInt_From_long(__pyx_t_6); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_3 = __pyx_t_8;
      __pyx_t_8 = 0;
    } else {
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_3 = __pyx_t_4;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = __pyx_t_3;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_8 = __Pyx_PyInt_From_long(__pyx_t_5); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_7 = PyObject_RichCompare(__pyx_t_8, __pyx_t_4, Py_GT); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 607, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 607, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (__pyx_t_2) {
      __pyx_t_7 = __Pyx_PyInt_From_long(__pyx_t_5); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_3 = __pyx_t_7;
      __pyx_t_7 = 0;
    } else {
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_3 = __pyx_t_4;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __pyx_t_3;
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_n = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "ft4222/ft4222.pyx":608
 *             #
 *             n = max(min(int(round(24000000.0 / (8 * 1000 * kbps) - 1)), 127), 1)
 *             self.setClock(SysClock.CLK_24)             # <<<<<<<<<<<<<<
 *             self.vendorCmdSet(0x52, n)
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setClock); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 608, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_SysClock); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 608, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_CLK_24); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 608, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = NULL;
    __pyx_t_9 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_9 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_8};
      __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_9, 1+__pyx_t_9);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 608, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "ft4222/ft4222.pyx":609
 *             n = max(min(int(round(24000000.0 / (8 * 1000 * kbps) - 1)), 127), 1)
 *             self.setClock(SysClock.CLK_24)
 *             self.vendorCmdSet(0x52, n)             # <<<<<<<<<<<<<<
 * 
 *     def i2cMaster_Read(self, addr, bytesToRead):
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_vendorCmdSet); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 609, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = NULL;
    __pyx_t_9 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_9 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_8, __pyx_int_82, __pyx_v_n};
      __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_9, 2+__pyx_t_9);
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 609, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "ft4222/ft4222.pyx":598
 *         # although the chip can handle clock rates down to ~23.6kHz
 *         # there's a undocumented ways to achieve this
 *         if kbps < 60:             # <<<<<<<<<<<<<<
 *             #             Operating Clock Freq
 *             # SCL Freq = -----------------------  | M = 6 or 8; N = 1, 2, 3, , 127
 */
  }

  /* "ft4222/ft4222.pyx":582
 * 
 * 
 *     def i2cMaster_Init(self, kbps=100):             # <<<<<<<<<<<<<<
 *         """Initialize the FT4222H as an I2C master with the requested I2C speed.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_Init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_n);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":611
 *             self.vendorCmdSet(0x52, n)
 * 
 *     def i2cMaster_Read(self, addr, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the specified I2C slave device with START and STOP conditions.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_45i2cMaster_Read(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_44i2cMaster_Read, "Read data from the specified I2C slave device with START and STOP conditions.\n\n        Args:\n            addr (int): I2C slave address\n            bytesToRead (int): Number of bytes to read from slave\n\n        Returns:\n            bytes: Bytes read from slave\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_45i2cMaster_Read = {"i2cMaster_Read", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_45i2cMaster_Read, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_44i2cMaster_Read};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_45i2cMaster_Read(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_addr = 0;
  PyObject *__pyx_v_bytesToRead = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("i2cMaster_Read (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_addr,&__pyx_n_s_bytesToRead,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_addr)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 611, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_bytesToRead)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 611, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("i2cMaster_Read", 1, 2, 2, 1); __PYX_ERR(0, 611, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "i2cMaster_Read") < 0)) __PYX_ERR(0, 611, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_addr = values[0];
    __pyx_v_bytesToRead = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("i2cMaster_Read", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 611, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_Read", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_44i2cMaster_Read(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_addr, __pyx_v_bytesToRead);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_44i2cMaster_Read(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_addr, PyObject *__pyx_v_bytesToRead) {
  arrayobject *__pyx_v_buf = 0;
  uint16 __pyx_v_bytesRead;
  FT4222_STATUS __pyx_v_status;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_buf;
  __Pyx_Buffer __pyx_pybuffer_buf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  uint16 __pyx_t_5;
  uint16 __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("i2cMaster_Read", 1);
  __pyx_pybuffer_buf.pybuffer.buf = NULL;
  __pyx_pybuffer_buf.refcount = 0;
  __pyx_pybuffernd_buf.data = NULL;
  __pyx_pybuffernd_buf.rcbuffer = &__pyx_pybuffer_buf;

  /* "ft4222/ft4222.pyx":626
 *         """
 *         cdef:
 *             array[uint8] buf = array('B', [])             # <<<<<<<<<<<<<<
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 626, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 626, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_n_u_B);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_B)) __PYX_ERR(0, 626, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1)) __PYX_ERR(0, 626, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7cpython_5array_array), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 626, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_buf.rcbuffer->pybuffer, (PyObject*)((arrayobject *)__pyx_t_1), &__Pyx_TypeInfo_nn_uint8, PyBUF_FORMAT| PyBUF_INDIRECT, 1, 0, __pyx_stack) == -1)) {
      __pyx_v_buf = ((arrayobject *)Py_None); __Pyx_INCREF(Py_None); __pyx_pybuffernd_buf.rcbuffer->pybuffer.buf = NULL;
      __PYX_ERR(0, 626, __pyx_L1_error)
    } else {__pyx_pybuffernd_buf.diminfo[0].strides = __pyx_pybuffernd_buf.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_buf.diminfo[0].shape = __pyx_pybuffernd_buf.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_buf.diminfo[0].suboffsets = __pyx_pybuffernd_buf.rcbuffer->pybuffer.suboffsets[0];
    }
  }
  __pyx_v_buf = ((arrayobject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":628
 *             array[uint8] buf = array('B', [])
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)             # <<<<<<<<<<<<<<
 *         status = FT4222_I2CMaster_Read(self._handle, addr, buf.data.as_uchars, bytesToRead, &bytesRead)
 *         resize(buf, bytesRead)
 */
  __pyx_t_3 = __Pyx_PyIndex_AsSsize_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_3 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 628, __pyx_L1_error)
  __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 628, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":629
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)
 *         status = FT4222_I2CMaster_Read(self._handle, addr, buf.data.as_uchars, bytesToRead, &bytesRead)             # <<<<<<<<<<<<<<
 *         resize(buf, bytesRead)
 *         if status == FT4222_OK:
 */
  __pyx_t_5 = __Pyx_PyInt_As_uint16_t(__pyx_v_addr); if (unlikely((__pyx_t_5 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 629, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyInt_As_uint16_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_6 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 629, __pyx_L1_error)
  __pyx_v_status = FT4222_I2CMaster_Read(__pyx_v_self->_handle, __pyx_t_5, __pyx_v_buf->data.as_uchars, __pyx_t_6, (&__pyx_v_bytesRead));

  /* "ft4222/ft4222.pyx":630
 *         resize(buf, bytesToRead)
 *         status = FT4222_I2CMaster_Read(self._handle, addr, buf.data.as_uchars, bytesToRead, &bytesRead)
 *         resize(buf, bytesRead)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return bytes(buf)
 */
  __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_v_bytesRead); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 630, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":631
 *         status = FT4222_I2CMaster_Read(self._handle, addr, buf.data.as_uchars, bytesToRead, &bytesRead)
 *         resize(buf, bytesRead)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
  __pyx_t_7 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_7) {

    /* "ft4222/ft4222.pyx":632
 *         resize(buf, bytesRead)
 *         if status == FT4222_OK:
 *             return bytes(buf)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), ((PyObject *)__pyx_v_buf)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 632, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":631
 *         status = FT4222_I2CMaster_Read(self._handle, addr, buf.data.as_uchars, bytesToRead, &bytesRead)
 *         resize(buf, bytesRead)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":633
 *         if status == FT4222_OK:
 *             return bytes(buf)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def i2cMaster_Write(self, addr, data):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 633, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 633, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_Raise(__pyx_t_1, __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 633, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":611
 *             self.vendorCmdSet(0x52, n)
 * 
 *     def i2cMaster_Read(self, addr, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the specified I2C slave device with START and STOP conditions.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_Read", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_buf);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":635
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_Write(self, addr, data):             # <<<<<<<<<<<<<<
 *         """Write data to the specified I2C slave device with START and STOP conditions.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_47i2cMaster_Write(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_46i2cMaster_Write, "Write data to the specified I2C slave device with START and STOP conditions.\n\n        Args:\n            addr (int): I2C slave address\n            data (int, bytes, bytearray): Data to write to slave\n\n        Returns:\n            int: Bytes sent to slave\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_47i2cMaster_Write = {"i2cMaster_Write", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_47i2cMaster_Write, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_46i2cMaster_Write};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_47i2cMaster_Write(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_addr = 0;
  PyObject *__pyx_v_data = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("i2cMaster_Write (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_addr,&__pyx_n_s_data,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_addr)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 635, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_data)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 635, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("i2cMaster_Write", 1, 2, 2, 1); __PYX_ERR(0, 635, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "i2cMaster_Write") < 0)) __PYX_ERR(0, 635, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_addr = values[0];
    __pyx_v_data = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("i2cMaster_Write", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 635, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_Write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_46i2cMaster_Write(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_addr, __pyx_v_data);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_46i2cMaster_Write(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_addr, PyObject *__pyx_v_data) {
  uint16 __pyx_v_bytesSent;
  uint8 *__pyx_v_cdata;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  uint8 *__pyx_t_5;
  uint16 __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("i2cMaster_Write", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "ft4222/ft4222.pyx":649
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_data); 
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":650
 *         """
 *         if isinstance(data, int):
 *             data = bytes([data])             # <<<<<<<<<<<<<<
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 650, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_data);
    __Pyx_GIVEREF(__pyx_v_data);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_v_data)) __PYX_ERR(0, 650, __pyx_L1_error);
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 650, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "ft4222/ft4222.pyx":649
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":651
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  __pyx_t_4 = PyBytes_Check(__pyx_v_data); 
  if (!__pyx_t_4) {
  } else {
    __pyx_t_1 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = PyByteArray_Check(__pyx_v_data); 
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  __pyx_t_4 = (!__pyx_t_1);
  if (unlikely(__pyx_t_4)) {

    /* "ft4222/ft4222.pyx":652
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint16 bytesSent
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 652, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 652, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":651
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":655
 *         cdef:
 *             uint16 bytesSent
 *             uint8* cdata = data             # <<<<<<<<<<<<<<
 *         status = FT4222_I2CMaster_Write(self._handle, addr, cdata, len(data), &bytesSent)
 *         if status == FT4222_OK:
 */
  __pyx_t_5 = __Pyx_PyObject_AsWritableUString(__pyx_v_data); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 655, __pyx_L1_error)
  __pyx_v_cdata = __pyx_t_5;

  /* "ft4222/ft4222.pyx":656
 *             uint16 bytesSent
 *             uint8* cdata = data
 *         status = FT4222_I2CMaster_Write(self._handle, addr, cdata, len(data), &bytesSent)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return bytesSent
 */
  __pyx_t_6 = __Pyx_PyInt_As_uint16_t(__pyx_v_addr); if (unlikely((__pyx_t_6 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 656, __pyx_L1_error)
  __pyx_t_7 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 656, __pyx_L1_error)
  __pyx_v_status = FT4222_I2CMaster_Write(__pyx_v_self->_handle, __pyx_t_6, __pyx_v_cdata, __pyx_t_7, (&__pyx_v_bytesSent));

  /* "ft4222/ft4222.pyx":657
 *             uint8* cdata = data
 *         status = FT4222_I2CMaster_Write(self._handle, addr, cdata, len(data), &bytesSent)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytesSent
 *         raise FT4222DeviceError, status
 */
  __pyx_t_4 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_4) {

    /* "ft4222/ft4222.pyx":658
 *         status = FT4222_I2CMaster_Write(self._handle, addr, cdata, len(data), &bytesSent)
 *         if status == FT4222_OK:
 *             return bytesSent             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_uint16_t(__pyx_v_bytesSent); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 658, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":657
 *             uint8* cdata = data
 *         status = FT4222_I2CMaster_Write(self._handle, addr, cdata, len(data), &bytesSent)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytesSent
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":659
 *         if status == FT4222_OK:
 *             return bytesSent
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def i2cMaster_ReadEx(self, addr, flag, bytesToRead):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 659, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 659, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_Raise(__pyx_t_3, __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 659, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":635
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_Write(self, addr, data):             # <<<<<<<<<<<<<<
 *         """Write data to the specified I2C slave device with START and STOP conditions.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_Write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":661
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_ReadEx(self, addr, flag, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the specified I2C slave device with the specified I2C condition.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_49i2cMaster_ReadEx(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_48i2cMaster_ReadEx, "Read data from the specified I2C slave device with the specified I2C condition.\n\n        Args:\n            addr (int): I2C slave address\n            flag (:obj:`ft4222.I2CMaster.Flag`): Flag to control start- and stopbit generation\n            bytesToRead (int): Number of bytes to read from slave\n\n        Returns:\n            bytes: Bytes read from slave\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_49i2cMaster_ReadEx = {"i2cMaster_ReadEx", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_49i2cMaster_ReadEx, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_48i2cMaster_ReadEx};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_49i2cMaster_ReadEx(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_addr = 0;
  PyObject *__pyx_v_flag = 0;
  PyObject *__pyx_v_bytesToRead = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("i2cMaster_ReadEx (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_addr,&__pyx_n_s_flag,&__pyx_n_s_bytesToRead,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_addr)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 661, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_flag)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 661, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("i2cMaster_ReadEx", 1, 3, 3, 1); __PYX_ERR(0, 661, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_bytesToRead)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[2]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 661, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("i2cMaster_ReadEx", 1, 3, 3, 2); __PYX_ERR(0, 661, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "i2cMaster_ReadEx") < 0)) __PYX_ERR(0, 661, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
      values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
    }
    __pyx_v_addr = values[0];
    __pyx_v_flag = values[1];
    __pyx_v_bytesToRead = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("i2cMaster_ReadEx", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 661, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_ReadEx", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_48i2cMaster_ReadEx(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_addr, __pyx_v_flag, __pyx_v_bytesToRead);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_48i2cMaster_ReadEx(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_addr, PyObject *__pyx_v_flag, PyObject *__pyx_v_bytesToRead) {
  arrayobject *__pyx_v_buf = 0;
  uint16 __pyx_v_bytesRead;
  FT4222_STATUS __pyx_v_status;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_buf;
  __Pyx_Buffer __pyx_pybuffer_buf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  uint16 __pyx_t_5;
  uint8 __pyx_t_6;
  uint16 __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("i2cMaster_ReadEx", 1);
  __pyx_pybuffer_buf.pybuffer.buf = NULL;
  __pyx_pybuffer_buf.refcount = 0;
  __pyx_pybuffernd_buf.data = NULL;
  __pyx_pybuffernd_buf.rcbuffer = &__pyx_pybuffer_buf;

  /* "ft4222/ft4222.pyx":677
 *         """
 *         cdef:
 *             array[uint8] buf = array('B', [])             # <<<<<<<<<<<<<<
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 677, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 677, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_n_u_B);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_B)) __PYX_ERR(0, 677, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1)) __PYX_ERR(0, 677, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7cpython_5array_array), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 677, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_buf.rcbuffer->pybuffer, (PyObject*)((arrayobject *)__pyx_t_1), &__Pyx_TypeInfo_nn_uint8, PyBUF_FORMAT| PyBUF_INDIRECT, 1, 0, __pyx_stack) == -1)) {
      __pyx_v_buf = ((arrayobject *)Py_None); __Pyx_INCREF(Py_None); __pyx_pybuffernd_buf.rcbuffer->pybuffer.buf = NULL;
      __PYX_ERR(0, 677, __pyx_L1_error)
    } else {__pyx_pybuffernd_buf.diminfo[0].strides = __pyx_pybuffernd_buf.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_buf.diminfo[0].shape = __pyx_pybuffernd_buf.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_buf.diminfo[0].suboffsets = __pyx_pybuffernd_buf.rcbuffer->pybuffer.suboffsets[0];
    }
  }
  __pyx_v_buf = ((arrayobject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":679
 *             array[uint8] buf = array('B', [])
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)             # <<<<<<<<<<<<<<
 *         status = FT4222_I2CMaster_ReadEx(self._handle, addr, flag, buf.data.as_uchars, bytesToRead, &bytesRead)
 *         resize(buf, bytesRead)
 */
  __pyx_t_3 = __Pyx_PyIndex_AsSsize_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_3 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 679, __pyx_L1_error)
  __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 679, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":680
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)
 *         status = FT4222_I2CMaster_ReadEx(self._handle, addr, flag, buf.data.as_uchars, bytesToRead, &bytesRead)             # <<<<<<<<<<<<<<
 *         resize(buf, bytesRead)
 *         if status == FT4222_OK:
 */
  __pyx_t_5 = __Pyx_PyInt_As_uint16_t(__pyx_v_addr); if (unlikely((__pyx_t_5 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 680, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyInt_As_uint8_t(__pyx_v_flag); if (unlikely((__pyx_t_6 == ((uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 680, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_As_uint16_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_7 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 680, __pyx_L1_error)
  __pyx_v_status = FT4222_I2CMaster_ReadEx(__pyx_v_self->_handle, __pyx_t_5, __pyx_t_6, __pyx_v_buf->data.as_uchars, __pyx_t_7, (&__pyx_v_bytesRead));

  /* "ft4222/ft4222.pyx":681
 *         resize(buf, bytesToRead)
 *         status = FT4222_I2CMaster_ReadEx(self._handle, addr, flag, buf.data.as_uchars, bytesToRead, &bytesRead)
 *         resize(buf, bytesRead)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return bytes(buf)
 */
  __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_v_bytesRead); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 681, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":682
 *         status = FT4222_I2CMaster_ReadEx(self._handle, addr, flag, buf.data.as_uchars, bytesToRead, &bytesRead)
 *         resize(buf, bytesRead)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
  __pyx_t_8 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_8) {

    /* "ft4222/ft4222.pyx":683
 *         resize(buf, bytesRead)
 *         if status == FT4222_OK:
 *             return bytes(buf)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), ((PyObject *)__pyx_v_buf)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 683, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":682
 *         status = FT4222_I2CMaster_ReadEx(self._handle, addr, flag, buf.data.as_uchars, bytesToRead, &bytesRead)
 *         resize(buf, bytesRead)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":684
 *         if status == FT4222_OK:
 *             return bytes(buf)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def i2cMaster_WriteEx(self, addr, flag, data):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 684, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 684, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_Raise(__pyx_t_1, __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 684, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":661
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_ReadEx(self, addr, flag, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the specified I2C slave device with the specified I2C condition.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_ReadEx", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_buf);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":686
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_WriteEx(self, addr, flag, data):             # <<<<<<<<<<<<<<
 *         """Write data to the specified I2C slave device with the specified I2C condition.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_51i2cMaster_WriteEx(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_50i2cMaster_WriteEx, "Write data to the specified I2C slave device with the specified I2C condition.\n\n        Args:\n            addr (int): I2C slave address\n            flag (:obj:`ft4222.I2CMaster.Flag`): Flag to control start- and stopbit generation\n            data (int, bytes, bytearray): Data to write to slave\n\n        Returns:\n            int: Bytes sent to slave\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_51i2cMaster_WriteEx = {"i2cMaster_WriteEx", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_51i2cMaster_WriteEx, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_50i2cMaster_WriteEx};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_51i2cMaster_WriteEx(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_addr = 0;
  PyObject *__pyx_v_flag = 0;
  PyObject *__pyx_v_data = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("i2cMaster_WriteEx (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_addr,&__pyx_n_s_flag,&__pyx_n_s_data,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_addr)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 686, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_flag)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 686, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("i2cMaster_WriteEx", 1, 3, 3, 1); __PYX_ERR(0, 686, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_data)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[2]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 686, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("i2cMaster_WriteEx", 1, 3, 3, 2); __PYX_ERR(0, 686, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "i2cMaster_WriteEx") < 0)) __PYX_ERR(0, 686, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
      values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
    }
    __pyx_v_addr = values[0];
    __pyx_v_flag = values[1];
    __pyx_v_data = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("i2cMaster_WriteEx", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 686, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_WriteEx", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_50i2cMaster_WriteEx(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_addr, __pyx_v_flag, __pyx_v_data);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_50i2cMaster_WriteEx(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_addr, PyObject *__pyx_v_flag, PyObject *__pyx_v_data) {
  uint16 __pyx_v_bytesSent;
  uint8 *__pyx_v_cdata;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  uint8 *__pyx_t_5;
  uint16 __pyx_t_6;
  uint8 __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("i2cMaster_WriteEx", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "ft4222/ft4222.pyx":701
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_data); 
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":702
 *         """
 *         if isinstance(data, int):
 *             data = bytes([data])             # <<<<<<<<<<<<<<
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 702, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_data);
    __Pyx_GIVEREF(__pyx_v_data);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_v_data)) __PYX_ERR(0, 702, __pyx_L1_error);
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 702, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "ft4222/ft4222.pyx":701
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":703
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  __pyx_t_4 = PyBytes_Check(__pyx_v_data); 
  if (!__pyx_t_4) {
  } else {
    __pyx_t_1 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = PyByteArray_Check(__pyx_v_data); 
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  __pyx_t_4 = (!__pyx_t_1);
  if (unlikely(__pyx_t_4)) {

    /* "ft4222/ft4222.pyx":704
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint16 bytesSent
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 704, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 704, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":703
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":707
 *         cdef:
 *             uint16 bytesSent
 *             uint8* cdata = data             # <<<<<<<<<<<<<<
 *         status = FT4222_I2CMaster_WriteEx(self._handle, addr, flag, cdata, len(data), &bytesSent)
 *         if status == FT4222_OK:
 */
  __pyx_t_5 = __Pyx_PyObject_AsWritableUString(__pyx_v_data); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 707, __pyx_L1_error)
  __pyx_v_cdata = __pyx_t_5;

  /* "ft4222/ft4222.pyx":708
 *             uint16 bytesSent
 *             uint8* cdata = data
 *         status = FT4222_I2CMaster_WriteEx(self._handle, addr, flag, cdata, len(data), &bytesSent)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return bytesSent
 */
  __pyx_t_6 = __Pyx_PyInt_As_uint16_t(__pyx_v_addr); if (unlikely((__pyx_t_6 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 708, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_As_uint8_t(__pyx_v_flag); if (unlikely((__pyx_t_7 == ((uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 708, __pyx_L1_error)
  __pyx_t_8 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 708, __pyx_L1_error)
  __pyx_v_status = FT4222_I2CMaster_WriteEx(__pyx_v_self->_handle, __pyx_t_6, __pyx_t_7, __pyx_v_cdata, __pyx_t_8, (&__pyx_v_bytesSent));

  /* "ft4222/ft4222.pyx":709
 *             uint8* cdata = data
 *         status = FT4222_I2CMaster_WriteEx(self._handle, addr, flag, cdata, len(data), &bytesSent)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytesSent
 *         raise FT4222DeviceError, status
 */
  __pyx_t_4 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_4) {

    /* "ft4222/ft4222.pyx":710
 *         status = FT4222_I2CMaster_WriteEx(self._handle, addr, flag, cdata, len(data), &bytesSent)
 *         if status == FT4222_OK:
 *             return bytesSent             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_uint16_t(__pyx_v_bytesSent); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 710, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":709
 *             uint8* cdata = data
 *         status = FT4222_I2CMaster_WriteEx(self._handle, addr, flag, cdata, len(data), &bytesSent)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytesSent
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":711
 *         if status == FT4222_OK:
 *             return bytesSent
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def i2cMaster_Reset(self):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 711, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 711, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_Raise(__pyx_t_3, __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 711, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":686
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_WriteEx(self, addr, flag, data):             # <<<<<<<<<<<<<<
 *         """Write data to the specified I2C slave device with the specified I2C condition.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_WriteEx", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":713
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_Reset(self):             # <<<<<<<<<<<<<<
 *         """Reset the I2C master device.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_53i2cMaster_Reset(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_52i2cMaster_Reset, "Reset the I2C master device.\n\n        If the I2C bus encounters errors or works abnormally, this function will reset the I2C device.\n        It is not necessary to call I2CMaster_Init again after calling this reset function.\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_53i2cMaster_Reset = {"i2cMaster_Reset", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_53i2cMaster_Reset, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_52i2cMaster_Reset};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_53i2cMaster_Reset(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("i2cMaster_Reset (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("i2cMaster_Reset", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "i2cMaster_Reset", 0))) return NULL;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_52i2cMaster_Reset(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_52i2cMaster_Reset(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("i2cMaster_Reset", 1);

  /* "ft4222/ft4222.pyx":723
 * 
 *         """
 *         status = FT4222_I2CMaster_Reset(self._handle)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_v_status = FT4222_I2CMaster_Reset(__pyx_v_self->_handle);

  /* "ft4222/ft4222.pyx":724
 *         """
 *         status = FT4222_I2CMaster_Reset(self._handle)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_1 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":725
 *         status = FT4222_I2CMaster_Reset(self._handle)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def i2cMaster_GetStatus(self):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 725, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 725, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 725, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":724
 *         """
 *         status = FT4222_I2CMaster_Reset(self._handle)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":713
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_Reset(self):             # <<<<<<<<<<<<<<
 *         """Reset the I2C master device.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_Reset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":727
 *             raise FT4222DeviceError, status
 * 
 *     def i2cMaster_GetStatus(self):             # <<<<<<<<<<<<<<
 *         """Read the status of the I2C master controller.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_55i2cMaster_GetStatus(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_54i2cMaster_GetStatus, "Read the status of the I2C master controller.\n\n        This can be used to poll a slave until its write-cycle is complete.\n\n        Returns:\n            :obj:`ft4222.I2CMaster.ControllerStatus`: Controller status\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_55i2cMaster_GetStatus = {"i2cMaster_GetStatus", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_55i2cMaster_GetStatus, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_54i2cMaster_GetStatus};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_55i2cMaster_GetStatus(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("i2cMaster_GetStatus (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("i2cMaster_GetStatus", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "i2cMaster_GetStatus", 0))) return NULL;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_54i2cMaster_GetStatus(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_54i2cMaster_GetStatus(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  uint8 __pyx_v_cs;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("i2cMaster_GetStatus", 1);

  /* "ft4222/ft4222.pyx":740
 *         """
 *         cdef uint8 cs
 *         status = FT4222_I2CMaster_GetStatus(self._handle, &cs)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return ControllerStatus(cs)
 */
  __pyx_v_status = FT4222_I2CMaster_GetStatus(__pyx_v_self->_handle, (&__pyx_v_cs));

  /* "ft4222/ft4222.pyx":741
 *         cdef uint8 cs
 *         status = FT4222_I2CMaster_GetStatus(self._handle, &cs)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return ControllerStatus(cs)
 *         raise FT4222DeviceError, status
 */
  __pyx_t_1 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":742
 *         status = FT4222_I2CMaster_GetStatus(self._handle, &cs)
 *         if status == FT4222_OK:
 *             return ControllerStatus(cs)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_ControllerStatus); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 742, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_uint8_t(__pyx_v_cs); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 742, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    __pyx_t_6 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_6 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_4};
      __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_6, 1+__pyx_t_6);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 742, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":741
 *         cdef uint8 cs
 *         status = FT4222_I2CMaster_GetStatus(self._handle, &cs)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return ControllerStatus(cs)
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":743
 *         if status == FT4222_OK:
 *             return ControllerStatus(cs)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 743, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 743, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 743, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":727
 *             raise FT4222DeviceError, status
 * 
 *     def i2cMaster_GetStatus(self):             # <<<<<<<<<<<<<<
 *         """Read the status of the I2C master controller.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_GetStatus", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":746
 * 
 * 
 *     def spi_Reset(self):             # <<<<<<<<<<<<<<
 *         """Reset the SPI master or slave device
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_57spi_Reset(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_56spi_Reset, "Reset the SPI master or slave device\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_57spi_Reset = {"spi_Reset", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_57spi_Reset, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_56spi_Reset};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_57spi_Reset(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spi_Reset (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("spi_Reset", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "spi_Reset", 0))) return NULL;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_56spi_Reset(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_56spi_Reset(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spi_Reset", 1);

  /* "ft4222/ft4222.pyx":753
 * 
 *         """
 *         status = FT4222_SPI_Reset(self._handle);             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_v_status = FT4222_SPI_Reset(__pyx_v_self->_handle);

  /* "ft4222/ft4222.pyx":754
 *         """
 *         status = FT4222_SPI_Reset(self._handle);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_1 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":755
 *         status = FT4222_SPI_Reset(self._handle);
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spi_ResetTransaction(self, spiIdx):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 755, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 755, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 755, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":754
 *         """
 *         status = FT4222_SPI_Reset(self._handle);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":746
 * 
 * 
 *     def spi_Reset(self):             # <<<<<<<<<<<<<<
 *         """Reset the SPI master or slave device
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spi_Reset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":757
 *             raise FT4222DeviceError, status
 * 
 *     def spi_ResetTransaction(self, spiIdx):             # <<<<<<<<<<<<<<
 *         """Reset the SPI transaction
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_59spi_ResetTransaction(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_58spi_ResetTransaction, "Reset the SPI transaction\n\n        Args:\n            spiIdx (int): The index of the SPI transaction, which ranges from 0~3 depending on the mode of the chip.\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_59spi_ResetTransaction = {"spi_ResetTransaction", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_59spi_ResetTransaction, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_58spi_ResetTransaction};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_59spi_ResetTransaction(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_spiIdx = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spi_ResetTransaction (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_spiIdx,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_spiIdx)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 757, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "spi_ResetTransaction") < 0)) __PYX_ERR(0, 757, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_spiIdx = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spi_ResetTransaction", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 757, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spi_ResetTransaction", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_58spi_ResetTransaction(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_spiIdx);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_58spi_ResetTransaction(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_spiIdx) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  uint8 __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spi_ResetTransaction", 1);

  /* "ft4222/ft4222.pyx":767
 * 
 *         """
 *         status = FT4222_SPI_ResetTransaction(self._handle, spiIdx);             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = __Pyx_PyInt_As_uint8_t(__pyx_v_spiIdx); if (unlikely((__pyx_t_1 == ((uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 767, __pyx_L1_error)
  __pyx_v_status = FT4222_SPI_ResetTransaction(__pyx_v_self->_handle, __pyx_t_1);

  /* "ft4222/ft4222.pyx":768
 *         """
 *         status = FT4222_SPI_ResetTransaction(self._handle, spiIdx);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_2 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_2)) {

    /* "ft4222/ft4222.pyx":769
 *         status = FT4222_SPI_ResetTransaction(self._handle, spiIdx);
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spi_SetDrivingStrength(self, clkStrength, ioStrength, ssoStrength):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 769, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 769, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 769, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":768
 *         """
 *         status = FT4222_SPI_ResetTransaction(self._handle, spiIdx);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":757
 *             raise FT4222DeviceError, status
 * 
 *     def spi_ResetTransaction(self, spiIdx):             # <<<<<<<<<<<<<<
 *         """Reset the SPI transaction
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spi_ResetTransaction", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":771
 *             raise FT4222DeviceError, status
 * 
 *     def spi_SetDrivingStrength(self, clkStrength, ioStrength, ssoStrength):             # <<<<<<<<<<<<<<
 *         """Reset the SPI master or slave device.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_61spi_SetDrivingStrength(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_60spi_SetDrivingStrength, "Reset the SPI master or slave device.\n\n        Args:\n            clkStrength (:obj:`ft4222.SPI.DrivingStrength`): Driving strength clock pin (master only)\n            ioStrength (:obj:`ft4222.SPI.DrivingStrength`): Driving strength io pin\n            ssoStrength (:obj:`ft4222.SPI.DrivingStrength`): Driving strength sso pin (master only)\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_61spi_SetDrivingStrength = {"spi_SetDrivingStrength", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_61spi_SetDrivingStrength, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_60spi_SetDrivingStrength};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_61spi_SetDrivingStrength(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_clkStrength = 0;
  PyObject *__pyx_v_ioStrength = 0;
  PyObject *__pyx_v_ssoStrength = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spi_SetDrivingStrength (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_clkStrength,&__pyx_n_s_ioStrength,&__pyx_n_s_ssoStrength,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_clkStrength)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 771, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_ioStrength)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 771, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("spi_SetDrivingStrength", 1, 3, 3, 1); __PYX_ERR(0, 771, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_ssoStrength)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[2]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 771, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("spi_SetDrivingStrength", 1, 3, 3, 2); __PYX_ERR(0, 771, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "spi_SetDrivingStrength") < 0)) __PYX_ERR(0, 771, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
      values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
    }
    __pyx_v_clkStrength = values[0];
    __pyx_v_ioStrength = values[1];
    __pyx_v_ssoStrength = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spi_SetDrivingStrength", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 771, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spi_SetDrivingStrength", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_60spi_SetDrivingStrength(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_clkStrength, __pyx_v_ioStrength, __pyx_v_ssoStrength);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_60spi_SetDrivingStrength(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_clkStrength, PyObject *__pyx_v_ioStrength, PyObject *__pyx_v_ssoStrength) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  SPI_DrivingStrength __pyx_t_1;
  SPI_DrivingStrength __pyx_t_2;
  SPI_DrivingStrength __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spi_SetDrivingStrength", 1);

  /* "ft4222/ft4222.pyx":783
 * 
 *         """
 *         status = FT4222_SPI_SetDrivingStrength(self._handle, clkStrength, ioStrength, ssoStrength);             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((SPI_DrivingStrength)__Pyx_PyInt_As_SPI_DrivingStrength(__pyx_v_clkStrength)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 783, __pyx_L1_error)
  __pyx_t_2 = ((SPI_DrivingStrength)__Pyx_PyInt_As_SPI_DrivingStrength(__pyx_v_ioStrength)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 783, __pyx_L1_error)
  __pyx_t_3 = ((SPI_DrivingStrength)__Pyx_PyInt_As_SPI_DrivingStrength(__pyx_v_ssoStrength)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 783, __pyx_L1_error)
  __pyx_v_status = FT4222_SPI_SetDrivingStrength(__pyx_v_self->_handle, __pyx_t_1, __pyx_t_2, __pyx_t_3);

  /* "ft4222/ft4222.pyx":784
 *         """
 *         status = FT4222_SPI_SetDrivingStrength(self._handle, clkStrength, ioStrength, ssoStrength);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_4 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_4)) {

    /* "ft4222/ft4222.pyx":785
 *         status = FT4222_SPI_SetDrivingStrength(self._handle, clkStrength, ioStrength, ssoStrength);
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiMaster_Init(self, mode, clock, cpol, cpha, ssoMap):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 785, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 785, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_Raise(__pyx_t_5, __pyx_t_6, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 785, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":784
 *         """
 *         status = FT4222_SPI_SetDrivingStrength(self._handle, clkStrength, ioStrength, ssoStrength);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":771
 *             raise FT4222DeviceError, status
 * 
 *     def spi_SetDrivingStrength(self, clkStrength, ioStrength, ssoStrength):             # <<<<<<<<<<<<<<
 *         """Reset the SPI master or slave device.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spi_SetDrivingStrength", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":787
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_Init(self, mode, clock, cpol, cpha, ssoMap):             # <<<<<<<<<<<<<<
 *         """Initialize as an SPI master under all modes.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_63spiMaster_Init(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_62spiMaster_Init, "Initialize as an SPI master under all modes.\n\n        Args:\n            mode (:obj:`ft4222.SPIMaster.Mode`): SPI transmission lines / mode\n            clock (:obj:`ft4222.SPIMaster.Clock`): Clock divider\n            cpol (:obj:`ft4222.SPI.Cpol`): Clock polarity\n            cpha (:obj:`ft4222.SPI.Cpha`): Clock phase\n            ssoMap (:obj:`ft4222.SPIMaster.SlaveSelect`): Slave selection output pins\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_63spiMaster_Init = {"spiMaster_Init", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_63spiMaster_Init, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_62spiMaster_Init};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_63spiMaster_Init(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_mode = 0;
  PyObject *__pyx_v_clock = 0;
  PyObject *__pyx_v_cpol = 0;
  PyObject *__pyx_v_cpha = 0;
  PyObject *__pyx_v_ssoMap = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiMaster_Init (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_mode,&__pyx_n_s_clock,&__pyx_n_s_cpol,&__pyx_n_s_cpha,&__pyx_n_s_ssoMap,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  5: values[4] = __Pyx_Arg_FASTCALL(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_mode)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 787, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_clock)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 787, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_Init", 1, 5, 5, 1); __PYX_ERR(0, 787, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_cpol)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[2]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 787, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_Init", 1, 5, 5, 2); __PYX_ERR(0, 787, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_cpha)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[3]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 787, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_Init", 1, 5, 5, 3); __PYX_ERR(0, 787, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_ssoMap)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[4]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 787, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_Init", 1, 5, 5, 4); __PYX_ERR(0, 787, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "spiMaster_Init") < 0)) __PYX_ERR(0, 787, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 5)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
      values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
      values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
      values[4] = __Pyx_Arg_FASTCALL(__pyx_args, 4);
    }
    __pyx_v_mode = values[0];
    __pyx_v_clock = values[1];
    __pyx_v_cpol = values[2];
    __pyx_v_cpha = values[3];
    __pyx_v_ssoMap = values[4];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spiMaster_Init", 1, 5, 5, __pyx_nargs); __PYX_ERR(0, 787, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_Init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_62spiMaster_Init(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_mode, __pyx_v_clock, __pyx_v_cpol, __pyx_v_cpha, __pyx_v_ssoMap);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_62spiMaster_Init(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_mode, PyObject *__pyx_v_clock, PyObject *__pyx_v_cpol, PyObject *__pyx_v_cpha, PyObject *__pyx_v_ssoMap) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  FT4222_SPIMode __pyx_t_1;
  FT4222_SPIClock __pyx_t_2;
  FT4222_SPICPOL __pyx_t_3;
  FT4222_SPICPHA __pyx_t_4;
  uint8 __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiMaster_Init", 1);

  /* "ft4222/ft4222.pyx":801
 * 
 *         """
 *         status = FT4222_SPIMaster_Init(self._handle, mode, clock, cpol, cpha, ssoMap);             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((FT4222_SPIMode)__Pyx_PyInt_As_FT4222_SPIMode(__pyx_v_mode)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 801, __pyx_L1_error)
  __pyx_t_2 = ((FT4222_SPIClock)__Pyx_PyInt_As_FT4222_SPIClock(__pyx_v_clock)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 801, __pyx_L1_error)
  __pyx_t_3 = ((FT4222_SPICPOL)__Pyx_PyInt_As_FT4222_SPICPOL(__pyx_v_cpol)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 801, __pyx_L1_error)
  __pyx_t_4 = ((FT4222_SPICPHA)__Pyx_PyInt_As_FT4222_SPICPHA(__pyx_v_cpha)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 801, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyInt_As_uint8_t(__pyx_v_ssoMap); if (unlikely((__pyx_t_5 == ((uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 801, __pyx_L1_error)
  __pyx_v_status = FT4222_SPIMaster_Init(__pyx_v_self->_handle, __pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4, __pyx_t_5);

  /* "ft4222/ft4222.pyx":802
 *         """
 *         status = FT4222_SPIMaster_Init(self._handle, mode, clock, cpol, cpha, ssoMap);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_6 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_6)) {

    /* "ft4222/ft4222.pyx":803
 *         status = FT4222_SPIMaster_Init(self._handle, mode, clock, cpol, cpha, ssoMap);
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiMaster_SetLines(self, mode):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 803, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 803, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_7, __pyx_t_8, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __PYX_ERR(0, 803, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":802
 *         """
 *         status = FT4222_SPIMaster_Init(self._handle, mode, clock, cpol, cpha, ssoMap);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":787
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_Init(self, mode, clock, cpol, cpha, ssoMap):             # <<<<<<<<<<<<<<
 *         """Initialize as an SPI master under all modes.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_Init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":805
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_SetLines(self, mode):             # <<<<<<<<<<<<<<
 *         """Switch the FT4222H SPI master to single, dual, or quad mode.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_65spiMaster_SetLines(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_64spiMaster_SetLines, "Switch the FT4222H SPI master to single, dual, or quad mode.\n\n        This overrides the mode passed to FT4222_SPIMaster_init. This might be needed if a\n        device accepts commands in single mode but data transfer is to use dual or quad mode.\n\n        Args:\n            mode (:obj:`ft4222.SPIMaster.Mode`): SPI transmission lines / mode\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_65spiMaster_SetLines = {"spiMaster_SetLines", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_65spiMaster_SetLines, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_64spiMaster_SetLines};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_65spiMaster_SetLines(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_mode = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiMaster_SetLines (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_mode,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_mode)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 805, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "spiMaster_SetLines") < 0)) __PYX_ERR(0, 805, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_mode = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spiMaster_SetLines", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 805, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_SetLines", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_64spiMaster_SetLines(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_mode);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_64spiMaster_SetLines(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_mode) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  FT4222_SPIMode __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiMaster_SetLines", 1);

  /* "ft4222/ft4222.pyx":818
 * 
 *         """
 *         status = FT4222_SPIMaster_SetLines(self._handle, mode);             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((FT4222_SPIMode)__Pyx_PyInt_As_FT4222_SPIMode(__pyx_v_mode)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 818, __pyx_L1_error)
  __pyx_v_status = FT4222_SPIMaster_SetLines(__pyx_v_self->_handle, __pyx_t_1);

  /* "ft4222/ft4222.pyx":819
 *         """
 *         status = FT4222_SPIMaster_SetLines(self._handle, mode);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_2 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_2)) {

    /* "ft4222/ft4222.pyx":820
 *         status = FT4222_SPIMaster_SetLines(self._handle, mode);
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiMaster_SingleRead(self, bytesToRead, isEndTransaction):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 820, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 820, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 820, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":819
 *         """
 *         status = FT4222_SPIMaster_SetLines(self._handle, mode);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":805
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_SetLines(self, mode):             # <<<<<<<<<<<<<<
 *         """Switch the FT4222H SPI master to single, dual, or quad mode.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_SetLines", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":822
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleRead(self, bytesToRead, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Read data from a SPI slave in single mode
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_67spiMaster_SingleRead(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_66spiMaster_SingleRead, "Read data from a SPI slave in single mode\n\n        Args:\n            bytesToRead (int): Number of bytes to read\n            isEndTransaction (bool): If True the slave select pin will be raised at the end\n\n        Returns:\n            bytes: Bytes read from slave\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_67spiMaster_SingleRead = {"spiMaster_SingleRead", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_67spiMaster_SingleRead, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_66spiMaster_SingleRead};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_67spiMaster_SingleRead(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_bytesToRead = 0;
  PyObject *__pyx_v_isEndTransaction = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiMaster_SingleRead (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_bytesToRead,&__pyx_n_s_isEndTransaction,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_bytesToRead)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 822, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_isEndTransaction)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 822, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_SingleRead", 1, 2, 2, 1); __PYX_ERR(0, 822, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "spiMaster_SingleRead") < 0)) __PYX_ERR(0, 822, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_bytesToRead = values[0];
    __pyx_v_isEndTransaction = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spiMaster_SingleRead", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 822, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_SingleRead", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_66spiMaster_SingleRead(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_bytesToRead, __pyx_v_isEndTransaction);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_66spiMaster_SingleRead(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_bytesToRead, PyObject *__pyx_v_isEndTransaction) {
  arrayobject *__pyx_v_buf = 0;
  uint16 __pyx_v_bytesRead;
  FT4222_STATUS __pyx_v_status;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_buf;
  __Pyx_Buffer __pyx_pybuffer_buf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  uint16 __pyx_t_5;
  BOOL __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiMaster_SingleRead", 1);
  __pyx_pybuffer_buf.pybuffer.buf = NULL;
  __pyx_pybuffer_buf.refcount = 0;
  __pyx_pybuffernd_buf.data = NULL;
  __pyx_pybuffernd_buf.rcbuffer = &__pyx_pybuffer_buf;

  /* "ft4222/ft4222.pyx":837
 *         """
 *         cdef:
 *             array[uint8] buf = array('B', [])             # <<<<<<<<<<<<<<
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 837, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 837, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_n_u_B);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_B)) __PYX_ERR(0, 837, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1)) __PYX_ERR(0, 837, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7cpython_5array_array), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 837, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_buf.rcbuffer->pybuffer, (PyObject*)((arrayobject *)__pyx_t_1), &__Pyx_TypeInfo_nn_uint8, PyBUF_FORMAT| PyBUF_INDIRECT, 1, 0, __pyx_stack) == -1)) {
      __pyx_v_buf = ((arrayobject *)Py_None); __Pyx_INCREF(Py_None); __pyx_pybuffernd_buf.rcbuffer->pybuffer.buf = NULL;
      __PYX_ERR(0, 837, __pyx_L1_error)
    } else {__pyx_pybuffernd_buf.diminfo[0].strides = __pyx_pybuffernd_buf.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_buf.diminfo[0].shape = __pyx_pybuffernd_buf.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_buf.diminfo[0].suboffsets = __pyx_pybuffernd_buf.rcbuffer->pybuffer.suboffsets[0];
    }
  }
  __pyx_v_buf = ((arrayobject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":839
 *             array[uint8] buf = array('B', [])
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)             # <<<<<<<<<<<<<<
 *         status = FT4222_SPIMaster_SingleRead(self._handle, buf.data.as_uchars, bytesToRead, &bytesRead, isEndTransaction)
 *         if status == FT4222_OK:
 */
  __pyx_t_3 = __Pyx_PyIndex_AsSsize_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_3 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 839, __pyx_L1_error)
  __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 839, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":840
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)
 *         status = FT4222_SPIMaster_SingleRead(self._handle, buf.data.as_uchars, bytesToRead, &bytesRead, isEndTransaction)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             resize(buf, bytesRead)
 */
  __pyx_t_5 = __Pyx_PyInt_As_uint16_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_5 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 840, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyInt_As_BOOL(__pyx_v_isEndTransaction); if (unlikely((__pyx_t_6 == ((BOOL)-1)) && PyErr_Occurred())) __PYX_ERR(0, 840, __pyx_L1_error)
  __pyx_v_status = FT4222_SPIMaster_SingleRead(__pyx_v_self->_handle, __pyx_v_buf->data.as_uchars, __pyx_t_5, (&__pyx_v_bytesRead), __pyx_t_6);

  /* "ft4222/ft4222.pyx":841
 *         resize(buf, bytesToRead)
 *         status = FT4222_SPIMaster_SingleRead(self._handle, buf.data.as_uchars, bytesToRead, &bytesRead, isEndTransaction)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             resize(buf, bytesRead)
 *             return bytes(buf)
 */
  __pyx_t_7 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_7) {

    /* "ft4222/ft4222.pyx":842
 *         status = FT4222_SPIMaster_SingleRead(self._handle, buf.data.as_uchars, bytesToRead, &bytesRead, isEndTransaction)
 *         if status == FT4222_OK:
 *             resize(buf, bytesRead)             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
    __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_v_bytesRead); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 842, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":843
 *         if status == FT4222_OK:
 *             resize(buf, bytesRead)
 *             return bytes(buf)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), ((PyObject *)__pyx_v_buf)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 843, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":841
 *         resize(buf, bytesToRead)
 *         status = FT4222_SPIMaster_SingleRead(self._handle, buf.data.as_uchars, bytesToRead, &bytesRead, isEndTransaction)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             resize(buf, bytesRead)
 *             return bytes(buf)
 */
  }

  /* "ft4222/ft4222.pyx":844
 *             resize(buf, bytesRead)
 *             return bytes(buf)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiMaster_SingleWrite(self, data, isEndTransaction):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 844, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 844, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_Raise(__pyx_t_1, __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 844, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":822
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleRead(self, bytesToRead, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Read data from a SPI slave in single mode
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_SingleRead", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_buf);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":846
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleWrite(self, data, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Write data to a SPI slave in single mode
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_69spiMaster_SingleWrite(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_68spiMaster_SingleWrite, "Write data to a SPI slave in single mode\n\n        Args:\n            data (bytes, bytearray, int): Data to write to slave\n            isEndTransaction (bool): If True the slave select pin will be raised at the end\n\n        Returns:\n            int: Bytes sent to slave\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_69spiMaster_SingleWrite = {"spiMaster_SingleWrite", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_69spiMaster_SingleWrite, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_68spiMaster_SingleWrite};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_69spiMaster_SingleWrite(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_isEndTransaction = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiMaster_SingleWrite (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_isEndTransaction,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_data)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 846, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_isEndTransaction)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 846, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_SingleWrite", 1, 2, 2, 1); __PYX_ERR(0, 846, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "spiMaster_SingleWrite") < 0)) __PYX_ERR(0, 846, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_isEndTransaction = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spiMaster_SingleWrite", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 846, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_SingleWrite", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_68spiMaster_SingleWrite(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_data, __pyx_v_isEndTransaction);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_68spiMaster_SingleWrite(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_isEndTransaction) {
  uint16 __pyx_v_bytesSent;
  uint8 *__pyx_v_cdata;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  uint8 *__pyx_t_5;
  Py_ssize_t __pyx_t_6;
  BOOL __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiMaster_SingleWrite", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "ft4222/ft4222.pyx":860
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_data); 
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":861
 *         """
 *         if isinstance(data, int):
 *             data = bytes([data])             # <<<<<<<<<<<<<<
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 861, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_data);
    __Pyx_GIVEREF(__pyx_v_data);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_v_data)) __PYX_ERR(0, 861, __pyx_L1_error);
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 861, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "ft4222/ft4222.pyx":860
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":862
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  __pyx_t_4 = PyBytes_Check(__pyx_v_data); 
  if (!__pyx_t_4) {
  } else {
    __pyx_t_1 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = PyByteArray_Check(__pyx_v_data); 
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  __pyx_t_4 = (!__pyx_t_1);
  if (unlikely(__pyx_t_4)) {

    /* "ft4222/ft4222.pyx":863
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint16 bytesSent
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 863, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 863, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":862
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":866
 *         cdef:
 *             uint16 bytesSent
 *             uint8* cdata = data             # <<<<<<<<<<<<<<
 *         status = FT4222_SPIMaster_SingleWrite(self._handle, cdata, len(data), &bytesSent, isEndTransaction);
 *         if status == FT4222_OK:
 */
  __pyx_t_5 = __Pyx_PyObject_AsWritableUString(__pyx_v_data); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 866, __pyx_L1_error)
  __pyx_v_cdata = __pyx_t_5;

  /* "ft4222/ft4222.pyx":867
 *             uint16 bytesSent
 *             uint8* cdata = data
 *         status = FT4222_SPIMaster_SingleWrite(self._handle, cdata, len(data), &bytesSent, isEndTransaction);             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return bytesSent
 */
  __pyx_t_6 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 867, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_As_BOOL(__pyx_v_isEndTransaction); if (unlikely((__pyx_t_7 == ((BOOL)-1)) && PyErr_Occurred())) __PYX_ERR(0, 867, __pyx_L1_error)
  __pyx_v_status = FT4222_SPIMaster_SingleWrite(__pyx_v_self->_handle, __pyx_v_cdata, __pyx_t_6, (&__pyx_v_bytesSent), __pyx_t_7);

  /* "ft4222/ft4222.pyx":868
 *             uint8* cdata = data
 *         status = FT4222_SPIMaster_SingleWrite(self._handle, cdata, len(data), &bytesSent, isEndTransaction);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytesSent
 *         raise FT4222DeviceError, status
 */
  __pyx_t_4 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_4) {

    /* "ft4222/ft4222.pyx":869
 *         status = FT4222_SPIMaster_SingleWrite(self._handle, cdata, len(data), &bytesSent, isEndTransaction);
 *         if status == FT4222_OK:
 *             return bytesSent             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_uint16_t(__pyx_v_bytesSent); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 869, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":868
 *             uint8* cdata = data
 *         status = FT4222_SPIMaster_SingleWrite(self._handle, cdata, len(data), &bytesSent, isEndTransaction);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytesSent
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":870
 *         if status == FT4222_OK:
 *             return bytesSent
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiMaster_SingleReadWrite(self, data, isEndTransaction):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 870, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 870, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_Raise(__pyx_t_3, __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 870, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":846
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleWrite(self, data, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Write data to a SPI slave in single mode
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_SingleWrite", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":872
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleReadWrite(self, data, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Write and read data to and from a SPI slave in single mode
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_71spiMaster_SingleReadWrite(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_70spiMaster_SingleReadWrite, "Write and read data to and from a SPI slave in single mode\n\n        Args:\n            data (bytes, bytearray, int): Data to write to slave\n            isEndTransaction (bool): If True the slave select pin will be raised at the end\n\n        Returns:\n            bytes: Bytes read from slave\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_71spiMaster_SingleReadWrite = {"spiMaster_SingleReadWrite", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_71spiMaster_SingleReadWrite, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_70spiMaster_SingleReadWrite};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_71spiMaster_SingleReadWrite(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_isEndTransaction = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiMaster_SingleReadWrite (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_isEndTransaction,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_data)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 872, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_isEndTransaction)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 872, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_SingleReadWrite", 1, 2, 2, 1); __PYX_ERR(0, 872, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "spiMaster_SingleReadWrite") < 0)) __PYX_ERR(0, 872, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_isEndTransaction = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spiMaster_SingleReadWrite", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 872, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_SingleReadWrite", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_70spiMaster_SingleReadWrite(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_data, __pyx_v_isEndTransaction);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_70spiMaster_SingleReadWrite(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_isEndTransaction) {
  uint16 __pyx_v_sizeTransferred;
  uint8 *__pyx_v_cdata;
  arrayobject *__pyx_v_buf = 0;
  FT4222_STATUS __pyx_v_status;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_buf;
  __Pyx_Buffer __pyx_pybuffer_buf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  uint8 *__pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  BOOL __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiMaster_SingleReadWrite", 0);
  __Pyx_INCREF(__pyx_v_data);
  __pyx_pybuffer_buf.pybuffer.buf = NULL;
  __pyx_pybuffer_buf.refcount = 0;
  __pyx_pybuffernd_buf.data = NULL;
  __pyx_pybuffernd_buf.rcbuffer = &__pyx_pybuffer_buf;

  /* "ft4222/ft4222.pyx":886
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_data); 
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":887
 *         """
 *         if isinstance(data, int):
 *             data = bytes([data])             # <<<<<<<<<<<<<<
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 887, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_data);
    __Pyx_GIVEREF(__pyx_v_data);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_v_data)) __PYX_ERR(0, 887, __pyx_L1_error);
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 887, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "ft4222/ft4222.pyx":886
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":888
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  __pyx_t_4 = PyBytes_Check(__pyx_v_data); 
  if (!__pyx_t_4) {
  } else {
    __pyx_t_1 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = PyByteArray_Check(__pyx_v_data); 
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  __pyx_t_4 = (!__pyx_t_1);
  if (unlikely(__pyx_t_4)) {

    /* "ft4222/ft4222.pyx":889
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint16 sizeTransferred
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 889, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 889, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":888
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":892
 *         cdef:
 *             uint16 sizeTransferred
 *             uint8* cdata = data             # <<<<<<<<<<<<<<
 *             array[uint8] buf = array('B', [])
 *         resize(buf, len(data))
 */
  __pyx_t_5 = __Pyx_PyObject_AsWritableUString(__pyx_v_data); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 892, __pyx_L1_error)
  __pyx_v_cdata = __pyx_t_5;

  /* "ft4222/ft4222.pyx":893
 *             uint16 sizeTransferred
 *             uint8* cdata = data
 *             array[uint8] buf = array('B', [])             # <<<<<<<<<<<<<<
 *         resize(buf, len(data))
 *         status = FT4222_SPIMaster_SingleReadWrite(self._handle, buf.data.as_uchars, cdata, len(data), &sizeTransferred, isEndTransaction);
 */
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 893, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 893, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_n_u_B);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_B)) __PYX_ERR(0, 893, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3)) __PYX_ERR(0, 893, __pyx_L1_error);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7cpython_5array_array), __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 893, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_buf.rcbuffer->pybuffer, (PyObject*)((arrayobject *)__pyx_t_3), &__Pyx_TypeInfo_nn_uint8, PyBUF_FORMAT| PyBUF_INDIRECT, 1, 0, __pyx_stack) == -1)) {
      __pyx_v_buf = ((arrayobject *)Py_None); __Pyx_INCREF(Py_None); __pyx_pybuffernd_buf.rcbuffer->pybuffer.buf = NULL;
      __PYX_ERR(0, 893, __pyx_L1_error)
    } else {__pyx_pybuffernd_buf.diminfo[0].strides = __pyx_pybuffernd_buf.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_buf.diminfo[0].shape = __pyx_pybuffernd_buf.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_buf.diminfo[0].suboffsets = __pyx_pybuffernd_buf.rcbuffer->pybuffer.suboffsets[0];
    }
  }
  __pyx_v_buf = ((arrayobject *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ft4222/ft4222.pyx":894
 *             uint8* cdata = data
 *             array[uint8] buf = array('B', [])
 *         resize(buf, len(data))             # <<<<<<<<<<<<<<
 *         status = FT4222_SPIMaster_SingleReadWrite(self._handle, buf.data.as_uchars, cdata, len(data), &sizeTransferred, isEndTransaction);
 *         if status == FT4222_OK:
 */
  __pyx_t_6 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 894, __pyx_L1_error)
  __pyx_t_7 = resize(((arrayobject *)__pyx_v_buf), __pyx_t_6); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 894, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":895
 *             array[uint8] buf = array('B', [])
 *         resize(buf, len(data))
 *         status = FT4222_SPIMaster_SingleReadWrite(self._handle, buf.data.as_uchars, cdata, len(data), &sizeTransferred, isEndTransaction);             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             resize(buf, sizeTransferred)
 */
  __pyx_t_6 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 895, __pyx_L1_error)
  __pyx_t_8 = __Pyx_PyInt_As_BOOL(__pyx_v_isEndTransaction); if (unlikely((__pyx_t_8 == ((BOOL)-1)) && PyErr_Occurred())) __PYX_ERR(0, 895, __pyx_L1_error)
  __pyx_v_status = FT4222_SPIMaster_SingleReadWrite(__pyx_v_self->_handle, __pyx_v_buf->data.as_uchars, __pyx_v_cdata, __pyx_t_6, (&__pyx_v_sizeTransferred), __pyx_t_8);

  /* "ft4222/ft4222.pyx":896
 *         resize(buf, len(data))
 *         status = FT4222_SPIMaster_SingleReadWrite(self._handle, buf.data.as_uchars, cdata, len(data), &sizeTransferred, isEndTransaction);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             resize(buf, sizeTransferred)
 *             return bytes(buf)
 */
  __pyx_t_4 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_4) {

    /* "ft4222/ft4222.pyx":897
 *         status = FT4222_SPIMaster_SingleReadWrite(self._handle, buf.data.as_uchars, cdata, len(data), &sizeTransferred, isEndTransaction);
 *         if status == FT4222_OK:
 *             resize(buf, sizeTransferred)             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
    __pyx_t_7 = resize(((arrayobject *)__pyx_v_buf), __pyx_v_sizeTransferred); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 897, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":898
 *         if status == FT4222_OK:
 *             resize(buf, sizeTransferred)
 *             return bytes(buf)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), ((PyObject *)__pyx_v_buf)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 898, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":896
 *         resize(buf, len(data))
 *         status = FT4222_SPIMaster_SingleReadWrite(self._handle, buf.data.as_uchars, cdata, len(data), &sizeTransferred, isEndTransaction);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             resize(buf, sizeTransferred)
 *             return bytes(buf)
 */
  }

  /* "ft4222/ft4222.pyx":899
 *             resize(buf, sizeTransferred)
 *             return bytes(buf)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiMaster_MultiReadWrite(self, singleWrite, multiWrite, bytesToRead):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 899, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 899, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_Raise(__pyx_t_3, __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 899, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":872
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleReadWrite(self, data, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Write and read data to and from a SPI slave in single mode
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_SingleReadWrite", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_buf);
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":901
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_MultiReadWrite(self, singleWrite, multiWrite, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Write and read data to and from a SPI slave in dual- or quad-mode (multi-mode).
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_73spiMaster_MultiReadWrite(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_72spiMaster_MultiReadWrite, "Write and read data to and from a SPI slave in dual- or quad-mode (multi-mode).\n\n        Args:\n            singleWrite (bytes, bytearray, int): Data to write to slave in signle-line mode (max. 15 bytes)\n            multiWrite (bytes, bytearray, int): Data to write to slave in multi-line mode (max. 65535 bytes)\n            bytesToRead (int):  Number of bytes to read on multi-line (max. 65535 bytes)\n\n        Returns:\n            bytes: Bytes read from slave in multi-line mode\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_73spiMaster_MultiReadWrite = {"spiMaster_MultiReadWrite", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_73spiMaster_MultiReadWrite, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_72spiMaster_MultiReadWrite};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_73spiMaster_MultiReadWrite(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_singleWrite = 0;
  PyObject *__pyx_v_multiWrite = 0;
  PyObject *__pyx_v_bytesToRead = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiMaster_MultiReadWrite (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_singleWrite,&__pyx_n_s_multiWrite,&__pyx_n_s_bytesToRead,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_singleWrite)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 901, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_multiWrite)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 901, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_MultiReadWrite", 1, 3, 3, 1); __PYX_ERR(0, 901, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_bytesToRead)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[2]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 901, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_MultiReadWrite", 1, 3, 3, 2); __PYX_ERR(0, 901, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "spiMaster_MultiReadWrite") < 0)) __PYX_ERR(0, 901, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
      values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
    }
    __pyx_v_singleWrite = values[0];
    __pyx_v_multiWrite = values[1];
    __pyx_v_bytesToRead = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spiMaster_MultiReadWrite", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 901, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_MultiReadWrite", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_72spiMaster_MultiReadWrite(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_singleWrite, __pyx_v_multiWrite, __pyx_v_bytesToRead);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_72spiMaster_MultiReadWrite(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_singleWrite, PyObject *__pyx_v_multiWrite, PyObject *__pyx_v_bytesToRead) {
  PyObject *__pyx_v_write = NULL;
  uint8 *__pyx_v_cdata;
  arrayobject *__pyx_v_buf = 0;
  uint32 __pyx_v_bytesRead;
  FT4222_STATUS __pyx_v_status;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_buf;
  __Pyx_Buffer __pyx_pybuffer_buf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  uint8 *__pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  uint16 __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiMaster_MultiReadWrite", 0);
  __Pyx_INCREF(__pyx_v_singleWrite);
  __Pyx_INCREF(__pyx_v_multiWrite);
  __pyx_pybuffer_buf.pybuffer.buf = NULL;
  __pyx_pybuffer_buf.refcount = 0;
  __pyx_pybuffernd_buf.data = NULL;
  __pyx_pybuffernd_buf.rcbuffer = &__pyx_pybuffer_buf;

  /* "ft4222/ft4222.pyx":916
 * 
 *         """
 *         if isinstance(singleWrite, int):             # <<<<<<<<<<<<<<
 *             singleWrite = bytes([singleWrite])
 *         elif not isinstance(singleWrite, (bytes, bytearray)):
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_singleWrite); 
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":917
 *         """
 *         if isinstance(singleWrite, int):
 *             singleWrite = bytes([singleWrite])             # <<<<<<<<<<<<<<
 *         elif not isinstance(singleWrite, (bytes, bytearray)):
 *             raise TypeError("the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 917, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_singleWrite);
    __Pyx_GIVEREF(__pyx_v_singleWrite);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_v_singleWrite)) __PYX_ERR(0, 917, __pyx_L1_error);
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 917, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_singleWrite, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "ft4222/ft4222.pyx":916
 * 
 *         """
 *         if isinstance(singleWrite, int):             # <<<<<<<<<<<<<<
 *             singleWrite = bytes([singleWrite])
 *         elif not isinstance(singleWrite, (bytes, bytearray)):
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":918
 *         if isinstance(singleWrite, int):
 *             singleWrite = bytes([singleWrite])
 *         elif not isinstance(singleWrite, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 *         if isinstance(multiWrite, int):
 */
  __pyx_t_4 = PyBytes_Check(__pyx_v_singleWrite); 
  if (!__pyx_t_4) {
  } else {
    __pyx_t_1 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = PyByteArray_Check(__pyx_v_singleWrite); 
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  __pyx_t_4 = (!__pyx_t_1);
  if (unlikely(__pyx_t_4)) {

    /* "ft4222/ft4222.pyx":919
 *             singleWrite = bytes([singleWrite])
 *         elif not isinstance(singleWrite, (bytes, bytearray)):
 *             raise TypeError("the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         if isinstance(multiWrite, int):
 *             multiWrite = bytes([multiWrite])
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 919, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 919, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":918
 *         if isinstance(singleWrite, int):
 *             singleWrite = bytes([singleWrite])
 *         elif not isinstance(singleWrite, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 *         if isinstance(multiWrite, int):
 */
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":920
 *         elif not isinstance(singleWrite, (bytes, bytearray)):
 *             raise TypeError("the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 *         if isinstance(multiWrite, int):             # <<<<<<<<<<<<<<
 *             multiWrite = bytes([multiWrite])
 *         elif not isinstance(multiWrite, (bytes, bytearray)):
 */
  __pyx_t_4 = PyInt_Check(__pyx_v_multiWrite); 
  if (__pyx_t_4) {

    /* "ft4222/ft4222.pyx":921
 *             raise TypeError("the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 *         if isinstance(multiWrite, int):
 *             multiWrite = bytes([multiWrite])             # <<<<<<<<<<<<<<
 *         elif not isinstance(multiWrite, (bytes, bytearray)):
 *             raise TypeError("the multiWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 */
    __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 921, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_multiWrite);
    __Pyx_GIVEREF(__pyx_v_multiWrite);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 0, __pyx_v_multiWrite)) __PYX_ERR(0, 921, __pyx_L1_error);
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 921, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_multiWrite, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "ft4222/ft4222.pyx":920
 *         elif not isinstance(singleWrite, (bytes, bytearray)):
 *             raise TypeError("the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 *         if isinstance(multiWrite, int):             # <<<<<<<<<<<<<<
 *             multiWrite = bytes([multiWrite])
 *         elif not isinstance(multiWrite, (bytes, bytearray)):
 */
    goto __pyx_L6;
  }

  /* "ft4222/ft4222.pyx":922
 *         if isinstance(multiWrite, int):
 *             multiWrite = bytes([multiWrite])
 *         elif not isinstance(multiWrite, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the multiWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 *         write = singleWrite + multiWrite
 */
  __pyx_t_1 = PyBytes_Check(__pyx_v_multiWrite); 
  if (!__pyx_t_1) {
  } else {
    __pyx_t_4 = __pyx_t_1;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_1 = PyByteArray_Check(__pyx_v_multiWrite); 
  __pyx_t_4 = __pyx_t_1;
  __pyx_L7_bool_binop_done:;
  __pyx_t_1 = (!__pyx_t_4);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":923
 *             multiWrite = bytes([multiWrite])
 *         elif not isinstance(multiWrite, (bytes, bytearray)):
 *             raise TypeError("the multiWrite argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         write = singleWrite + multiWrite
 *         cdef:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 923, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 923, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":922
 *         if isinstance(multiWrite, int):
 *             multiWrite = bytes([multiWrite])
 *         elif not isinstance(multiWrite, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the multiWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 *         write = singleWrite + multiWrite
 */
  }
  __pyx_L6:;

  /* "ft4222/ft4222.pyx":924
 *         elif not isinstance(multiWrite, (bytes, bytearray)):
 *             raise TypeError("the multiWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 *         write = singleWrite + multiWrite             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint8* cdata = write
 */
  __pyx_t_2 = PyNumber_Add(__pyx_v_singleWrite, __pyx_v_multiWrite); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 924, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_write = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":926
 *         write = singleWrite + multiWrite
 *         cdef:
 *             uint8* cdata = write             # <<<<<<<<<<<<<<
 *             array[uint8] buf = array('B', [])
 *             uint32 bytesRead
 */
  __pyx_t_5 = __Pyx_PyObject_AsWritableUString(__pyx_v_write); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 926, __pyx_L1_error)
  __pyx_v_cdata = __pyx_t_5;

  /* "ft4222/ft4222.pyx":927
 *         cdef:
 *             uint8* cdata = write
 *             array[uint8] buf = array('B', [])             # <<<<<<<<<<<<<<
 *             uint32 bytesRead
 *         resize(buf, bytesToRead)
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 927, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 927, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_n_u_B);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_n_u_B)) __PYX_ERR(0, 927, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2)) __PYX_ERR(0, 927, __pyx_L1_error);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7cpython_5array_array), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 927, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_buf.rcbuffer->pybuffer, (PyObject*)((arrayobject *)__pyx_t_2), &__Pyx_TypeInfo_nn_uint8, PyBUF_FORMAT| PyBUF_INDIRECT, 1, 0, __pyx_stack) == -1)) {
      __pyx_v_buf = ((arrayobject *)Py_None); __Pyx_INCREF(Py_None); __pyx_pybuffernd_buf.rcbuffer->pybuffer.buf = NULL;
      __PYX_ERR(0, 927, __pyx_L1_error)
    } else {__pyx_pybuffernd_buf.diminfo[0].strides = __pyx_pybuffernd_buf.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_buf.diminfo[0].shape = __pyx_pybuffernd_buf.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_buf.diminfo[0].suboffsets = __pyx_pybuffernd_buf.rcbuffer->pybuffer.suboffsets[0];
    }
  }
  __pyx_v_buf = ((arrayobject *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":929
 *             array[uint8] buf = array('B', [])
 *             uint32 bytesRead
 *         resize(buf, bytesToRead)             # <<<<<<<<<<<<<<
 *         status = FT4222_SPIMaster_MultiReadWrite(self._handle, buf.data.as_uchars, cdata, len(singleWrite), len(multiWrite), bytesToRead, &bytesRead);
 *         if status == FT4222_OK:
 */
  __pyx_t_6 = __Pyx_PyIndex_AsSsize_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_6 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 929, __pyx_L1_error)
  __pyx_t_7 = resize(((arrayobject *)__pyx_v_buf), __pyx_t_6); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 929, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":930
 *             uint32 bytesRead
 *         resize(buf, bytesToRead)
 *         status = FT4222_SPIMaster_MultiReadWrite(self._handle, buf.data.as_uchars, cdata, len(singleWrite), len(multiWrite), bytesToRead, &bytesRead);             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             resize(buf, bytesRead)
 */
  __pyx_t_6 = PyObject_Length(__pyx_v_singleWrite); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 930, __pyx_L1_error)
  __pyx_t_8 = PyObject_Length(__pyx_v_multiWrite); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 930, __pyx_L1_error)
  __pyx_t_9 = __Pyx_PyInt_As_uint16_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_9 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 930, __pyx_L1_error)
  __pyx_v_status = FT4222_SPIMaster_MultiReadWrite(__pyx_v_self->_handle, __pyx_v_buf->data.as_uchars, __pyx_v_cdata, __pyx_t_6, __pyx_t_8, __pyx_t_9, (&__pyx_v_bytesRead));

  /* "ft4222/ft4222.pyx":931
 *         resize(buf, bytesToRead)
 *         status = FT4222_SPIMaster_MultiReadWrite(self._handle, buf.data.as_uchars, cdata, len(singleWrite), len(multiWrite), bytesToRead, &bytesRead);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             resize(buf, bytesRead)
 *             return bytes(buf)
 */
  __pyx_t_1 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":932
 *         status = FT4222_SPIMaster_MultiReadWrite(self._handle, buf.data.as_uchars, cdata, len(singleWrite), len(multiWrite), bytesToRead, &bytesRead);
 *         if status == FT4222_OK:
 *             resize(buf, bytesRead)             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
    __pyx_t_7 = resize(((arrayobject *)__pyx_v_buf), __pyx_v_bytesRead); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 932, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":933
 *         if status == FT4222_OK:
 *             resize(buf, bytesRead)
 *             return bytes(buf)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), ((PyObject *)__pyx_v_buf)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 933, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":931
 *         resize(buf, bytesToRead)
 *         status = FT4222_SPIMaster_MultiReadWrite(self._handle, buf.data.as_uchars, cdata, len(singleWrite), len(multiWrite), bytesToRead, &bytesRead);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             resize(buf, bytesRead)
 *             return bytes(buf)
 */
  }

  /* "ft4222/ft4222.pyx":934
 *             resize(buf, bytesRead)
 *             return bytes(buf)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiMaster_EndTransaction(self):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 934, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 934, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 934, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":901
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_MultiReadWrite(self, singleWrite, multiWrite, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Write and read data to and from a SPI slave in dual- or quad-mode (multi-mode).
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_MultiReadWrite", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF(__pyx_v_write);
  __Pyx_XDECREF((PyObject *)__pyx_v_buf);
  __Pyx_XDECREF(__pyx_v_singleWrite);
  __Pyx_XDECREF(__pyx_v_multiWrite);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":936
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_EndTransaction(self):             # <<<<<<<<<<<<<<
 *         """End the current SPI transaction.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_75spiMaster_EndTransaction(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_74spiMaster_EndTransaction, "End the current SPI transaction.\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_75spiMaster_EndTransaction = {"spiMaster_EndTransaction", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_75spiMaster_EndTransaction, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_74spiMaster_EndTransaction};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_75spiMaster_EndTransaction(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiMaster_EndTransaction (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("spiMaster_EndTransaction", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "spiMaster_EndTransaction", 0))) return NULL;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_74spiMaster_EndTransaction(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_74spiMaster_EndTransaction(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  DWORD __pyx_v_bytesSent;
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiMaster_EndTransaction", 1);

  /* "ft4222/ft4222.pyx":945
 *         cdef:
 *             DWORD bytesSent;
 *         status = FT_Write(self._handle, <unsigned char*>NULL, 0, &bytesSent);             # <<<<<<<<<<<<<<
 *         if status == FT_OK:
 *             return
 */
  __pyx_v_status = FT_Write(__pyx_v_self->_handle, ((unsigned char *)NULL), 0, (&__pyx_v_bytesSent));

  /* "ft4222/ft4222.pyx":946
 *             DWORD bytesSent;
 *         status = FT_Write(self._handle, <unsigned char*>NULL, 0, &bytesSent);
 *         if status == FT_OK:             # <<<<<<<<<<<<<<
 *             return
 *         raise FT4222DeviceError, status
 */
  __pyx_t_1 = (__pyx_v_status == FT_OK);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":947
 *         status = FT_Write(self._handle, <unsigned char*>NULL, 0, &bytesSent);
 *         if status == FT_OK:
 *             return             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":946
 *             DWORD bytesSent;
 *         status = FT_Write(self._handle, <unsigned char*>NULL, 0, &bytesSent);
 *         if status == FT_OK:             # <<<<<<<<<<<<<<
 *             return
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":948
 *         if status == FT_OK:
 *             return
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiSlave_Init(self):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 948, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 948, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 948, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":936
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_EndTransaction(self):             # <<<<<<<<<<<<<<
 *         """End the current SPI transaction.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_EndTransaction", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":950
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_Init(self):             # <<<<<<<<<<<<<<
 *         """Initialize the FT4222H as an SPI slave. Default SPI_SlaveProtocol is SPI_SLAVE_WITH_PROTOCOL.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_77spiSlave_Init(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_76spiSlave_Init, "Initialize the FT4222H as an SPI slave. Default SPI_SlaveProtocol is SPI_SLAVE_WITH_PROTOCOL.\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_77spiSlave_Init = {"spiSlave_Init", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_77spiSlave_Init, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_76spiSlave_Init};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_77spiSlave_Init(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiSlave_Init (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("spiSlave_Init", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "spiSlave_Init", 0))) return NULL;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_76spiSlave_Init(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_76spiSlave_Init(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiSlave_Init", 1);

  /* "ft4222/ft4222.pyx":957
 * 
 *         """
 *         status = FT4222_SPISlave_Init(self._handle);             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_v_status = FT4222_SPISlave_Init(__pyx_v_self->_handle);

  /* "ft4222/ft4222.pyx":958
 *         """
 *         status = FT4222_SPISlave_Init(self._handle);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_1 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":959
 *         status = FT4222_SPISlave_Init(self._handle);
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiSlave_InitEx(self, proto):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 959, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 959, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 959, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":958
 *         """
 *         status = FT4222_SPISlave_Init(self._handle);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":950
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_Init(self):             # <<<<<<<<<<<<<<
 *         """Initialize the FT4222H as an SPI slave. Default SPI_SlaveProtocol is SPI_SLAVE_WITH_PROTOCOL.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiSlave_Init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":961
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_InitEx(self, proto):             # <<<<<<<<<<<<<<
 *         """Initialize as an SPI slave under all modes.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_79spiSlave_InitEx(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_78spiSlave_InitEx, "Initialize as an SPI slave under all modes.\n\n        Args:\n            mode (:obj:`ft4222.SPISlave.Protocol`): SPI slave protocol\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_79spiSlave_InitEx = {"spiSlave_InitEx", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_79spiSlave_InitEx, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_78spiSlave_InitEx};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_79spiSlave_InitEx(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_proto = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiSlave_InitEx (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_proto,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_proto)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 961, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "spiSlave_InitEx") < 0)) __PYX_ERR(0, 961, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_proto = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spiSlave_InitEx", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 961, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiSlave_InitEx", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_78spiSlave_InitEx(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_proto);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_78spiSlave_InitEx(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_proto) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  SPI_SlaveProtocol __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiSlave_InitEx", 1);

  /* "ft4222/ft4222.pyx":971
 * 
 *         """
 *         status = FT4222_SPISlave_InitEx(self._handle,proto);             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((SPI_SlaveProtocol)__Pyx_PyInt_As_SPI_SlaveProtocol(__pyx_v_proto)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 971, __pyx_L1_error)
  __pyx_v_status = FT4222_SPISlave_InitEx(__pyx_v_self->_handle, __pyx_t_1);

  /* "ft4222/ft4222.pyx":972
 *         """
 *         status = FT4222_SPISlave_InitEx(self._handle,proto);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_2 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_2)) {

    /* "ft4222/ft4222.pyx":973
 *         status = FT4222_SPISlave_InitEx(self._handle,proto);
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiSlave_Read(self, bytesToRead):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 973, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 973, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 973, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":972
 *         """
 *         status = FT4222_SPISlave_InitEx(self._handle,proto);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":961
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_InitEx(self, proto):             # <<<<<<<<<<<<<<
 *         """Initialize as an SPI slave under all modes.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiSlave_InitEx", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":975
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_Read(self, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the receive queue of the SPI slave device.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_81spiSlave_Read(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_80spiSlave_Read, "Read data from the receive queue of the SPI slave device.\n\n        Args:\n            bytesToRead (int): Number of bytes to read\n\n        Returns:\n            bytes: Bytes read from slave\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_81spiSlave_Read = {"spiSlave_Read", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_81spiSlave_Read, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_80spiSlave_Read};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_81spiSlave_Read(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_bytesToRead = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiSlave_Read (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_bytesToRead,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_bytesToRead)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 975, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "spiSlave_Read") < 0)) __PYX_ERR(0, 975, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_bytesToRead = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spiSlave_Read", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 975, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiSlave_Read", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_80spiSlave_Read(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_bytesToRead);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_80spiSlave_Read(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_bytesToRead) {
  arrayobject *__pyx_v_buf = 0;
  uint16 __pyx_v_sizeRead;
  FT4222_STATUS __pyx_v_status;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_buf;
  __Pyx_Buffer __pyx_pybuffer_buf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  uint16 __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiSlave_Read", 1);
  __pyx_pybuffer_buf.pybuffer.buf = NULL;
  __pyx_pybuffer_buf.refcount = 0;
  __pyx_pybuffernd_buf.data = NULL;
  __pyx_pybuffernd_buf.rcbuffer = &__pyx_pybuffer_buf;

  /* "ft4222/ft4222.pyx":989
 *         """
 *         cdef:
 *             array[uint8] buf = array('B', [])             # <<<<<<<<<<<<<<
 *             uint16 sizeRead
 *         resize(buf, bytesToRead)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 989, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 989, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_n_u_B);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_B)) __PYX_ERR(0, 989, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1)) __PYX_ERR(0, 989, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7cpython_5array_array), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 989, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_buf.rcbuffer->pybuffer, (PyObject*)((arrayobject *)__pyx_t_1), &__Pyx_TypeInfo_nn_uint8, PyBUF_FORMAT| PyBUF_INDIRECT, 1, 0, __pyx_stack) == -1)) {
      __pyx_v_buf = ((arrayobject *)Py_None); __Pyx_INCREF(Py_None); __pyx_pybuffernd_buf.rcbuffer->pybuffer.buf = NULL;
      __PYX_ERR(0, 989, __pyx_L1_error)
    } else {__pyx_pybuffernd_buf.diminfo[0].strides = __pyx_pybuffernd_buf.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_buf.diminfo[0].shape = __pyx_pybuffernd_buf.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_buf.diminfo[0].suboffsets = __pyx_pybuffernd_buf.rcbuffer->pybuffer.suboffsets[0];
    }
  }
  __pyx_v_buf = ((arrayobject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":991
 *             array[uint8] buf = array('B', [])
 *             uint16 sizeRead
 *         resize(buf, bytesToRead)             # <<<<<<<<<<<<<<
 * 
 *         status = FT4222_SPISlave_Read(self._handle, buf.data.as_uchars, bytesToRead, &sizeRead);
 */
  __pyx_t_3 = __Pyx_PyIndex_AsSsize_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_3 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 991, __pyx_L1_error)
  __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 991, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":993
 *         resize(buf, bytesToRead)
 * 
 *         status = FT4222_SPISlave_Read(self._handle, buf.data.as_uchars, bytesToRead, &sizeRead);             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             resize(buf, sizeRead)
 */
  __pyx_t_5 = __Pyx_PyInt_As_uint16_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_5 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 993, __pyx_L1_error)
  __pyx_v_status = FT4222_SPISlave_Read(__pyx_v_self->_handle, __pyx_v_buf->data.as_uchars, __pyx_t_5, (&__pyx_v_sizeRead));

  /* "ft4222/ft4222.pyx":994
 * 
 *         status = FT4222_SPISlave_Read(self._handle, buf.data.as_uchars, bytesToRead, &sizeRead);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             resize(buf, sizeRead)
 *             return bytes(buf)
 */
  __pyx_t_6 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_6) {

    /* "ft4222/ft4222.pyx":995
 *         status = FT4222_SPISlave_Read(self._handle, buf.data.as_uchars, bytesToRead, &sizeRead);
 *         if status == FT4222_OK:
 *             resize(buf, sizeRead)             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
    __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_v_sizeRead); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 995, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":996
 *         if status == FT4222_OK:
 *             resize(buf, sizeRead)
 *             return bytes(buf)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), ((PyObject *)__pyx_v_buf)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 996, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":994
 * 
 *         status = FT4222_SPISlave_Read(self._handle, buf.data.as_uchars, bytesToRead, &sizeRead);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             resize(buf, sizeRead)
 *             return bytes(buf)
 */
  }

  /* "ft4222/ft4222.pyx":997
 *             resize(buf, sizeRead)
 *             return bytes(buf)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiSlave_SetMode(self, cpol, cpha):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 997, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 997, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_Raise(__pyx_t_1, __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 997, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":975
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_Read(self, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the receive queue of the SPI slave device.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiSlave_Read", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_buf);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":999
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_SetMode(self, cpol, cpha):             # <<<<<<<<<<<<<<
 *         """Set SPI slave cpol and cpha. The Default value of cpol is (:obj:`ft4222.SPI.Cpol.CLK_IDLE_LOW`) , default value of cpha is (:obj:`ft4222.SPI.Cpol.CLK_LEADING`)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_83spiSlave_SetMode(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_82spiSlave_SetMode, "Set SPI slave cpol and cpha. The Default value of cpol is (:obj:`ft4222.SPI.Cpol.CLK_IDLE_LOW`) , default value of cpha is (:obj:`ft4222.SPI.Cpol.CLK_LEADING`)\n\n        Args:\n            cpol (:obj:`ft4222.SPI.Cpol`): Clock polarity\n            cpha (:obj:`ft4222.SPI.Cpha`): Clock phase\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_83spiSlave_SetMode = {"spiSlave_SetMode", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_83spiSlave_SetMode, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_82spiSlave_SetMode};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_83spiSlave_SetMode(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_cpol = 0;
  PyObject *__pyx_v_cpha = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiSlave_SetMode (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_cpol,&__pyx_n_s_cpha,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_cpol)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 999, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_cpha)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 999, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("spiSlave_SetMode", 1, 2, 2, 1); __PYX_ERR(0, 999, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "spiSlave_SetMode") < 0)) __PYX_ERR(0, 999, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_cpol = values[0];
    __pyx_v_cpha = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spiSlave_SetMode", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 999, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiSlave_SetMode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_82spiSlave_SetMode(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_cpol, __pyx_v_cpha);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_82spiSlave_SetMode(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_cpol, PyObject *__pyx_v_cpha) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  FT4222_SPICPOL __pyx_t_1;
  FT4222_SPICPHA __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiSlave_SetMode", 1);

  /* "ft4222/ft4222.pyx":1010
 * 
 *         """
 *         status = FT4222_SPISlave_SetMode(self._handle, cpol, cpha);             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((FT4222_SPICPOL)__Pyx_PyInt_As_FT4222_SPICPOL(__pyx_v_cpol)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1010, __pyx_L1_error)
  __pyx_t_2 = ((FT4222_SPICPHA)__Pyx_PyInt_As_FT4222_SPICPHA(__pyx_v_cpha)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1010, __pyx_L1_error)
  __pyx_v_status = FT4222_SPISlave_SetMode(__pyx_v_self->_handle, __pyx_t_1, __pyx_t_2);

  /* "ft4222/ft4222.pyx":1011
 *         """
 *         status = FT4222_SPISlave_SetMode(self._handle, cpol, cpha);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_3 = (__pyx_v_status != FT4222_OK);
  if (unlikely(__pyx_t_3)) {

    /* "ft4222/ft4222.pyx":1012
 *         status = FT4222_SPISlave_SetMode(self._handle, cpol, cpha);
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiSlave_GetRxStatus(self):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1012, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1012, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_4, __pyx_t_5, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 1012, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":1011
 *         """
 *         status = FT4222_SPISlave_SetMode(self._handle, cpol, cpha);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":999
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_SetMode(self, cpol, cpha):             # <<<<<<<<<<<<<<
 *         """Set SPI slave cpol and cpha. The Default value of cpol is (:obj:`ft4222.SPI.Cpol.CLK_IDLE_LOW`) , default value of cpha is (:obj:`ft4222.SPI.Cpol.CLK_LEADING`)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiSlave_SetMode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":1014
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_GetRxStatus(self):             # <<<<<<<<<<<<<<
 *         """Get number of bytes in the receive queue.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_85spiSlave_GetRxStatus(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_84spiSlave_GetRxStatus, "Get number of bytes in the receive queue.\n\n        Returns:\n            pRxSize (uint16): Number of bytes in the receive queue\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_85spiSlave_GetRxStatus = {"spiSlave_GetRxStatus", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_85spiSlave_GetRxStatus, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_84spiSlave_GetRxStatus};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_85spiSlave_GetRxStatus(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiSlave_GetRxStatus (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("spiSlave_GetRxStatus", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "spiSlave_GetRxStatus", 0))) return NULL;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_84spiSlave_GetRxStatus(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_84spiSlave_GetRxStatus(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  uint16 __pyx_v_pRxSize;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiSlave_GetRxStatus", 1);

  /* "ft4222/ft4222.pyx":1027
 *             uint16 pRxSize
 * 
 *         status = FT4222_SPISlave_GetRxStatus(self._handle, &pRxSize);             # <<<<<<<<<<<<<<
 * 
 *         if status == FT4222_OK:
 */
  __pyx_v_status = FT4222_SPISlave_GetRxStatus(__pyx_v_self->_handle, (&__pyx_v_pRxSize));

  /* "ft4222/ft4222.pyx":1029
 *         status = FT4222_SPISlave_GetRxStatus(self._handle, &pRxSize);
 * 
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return pRxSize
 *         raise FT4222DeviceError, status
 */
  __pyx_t_1 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":1030
 * 
 *         if status == FT4222_OK:
 *             return pRxSize             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_uint16_t(__pyx_v_pRxSize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1030, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":1029
 *         status = FT4222_SPISlave_GetRxStatus(self._handle, &pRxSize);
 * 
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return pRxSize
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":1031
 *         if status == FT4222_OK:
 *             return pRxSize
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiSlave_Write(self, data):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1031, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1031, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 1031, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":1014
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_GetRxStatus(self):             # <<<<<<<<<<<<<<
 *         """Get number of bytes in the receive queue.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiSlave_GetRxStatus", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":1033
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_Write(self, data):             # <<<<<<<<<<<<<<
 *         """Write data to the transmit queue of the SPI slave device.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_87spiSlave_Write(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6ft4222_6ft4222_6FT4222_86spiSlave_Write, "Write data to the transmit queue of the SPI slave device.\n\n        Args:\n            data (bytes, bytearray, int): Data to write to slave\n\n        Returns:\n            sizeTransferred (uint16): Number of bytes written to the device.\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ");
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_87spiSlave_Write = {"spiSlave_Write", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_87spiSlave_Write, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_86spiSlave_Write};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_87spiSlave_Write(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_data = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiSlave_Write (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_data)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1033, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "spiSlave_Write") < 0)) __PYX_ERR(0, 1033, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_data = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spiSlave_Write", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1033, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiSlave_Write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_86spiSlave_Write(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_data);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_86spiSlave_Write(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_data) {
  uint16 __pyx_v_sizeTransferred;
  uint8 *__pyx_v_cdata;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  uint8 *__pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiSlave_Write", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "ft4222/ft4222.pyx":1046
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_data); 
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":1047
 *         """
 *         if isinstance(data, int):
 *             data = bytes([data])             # <<<<<<<<<<<<<<
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1047, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_data);
    __Pyx_GIVEREF(__pyx_v_data);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_v_data)) __PYX_ERR(0, 1047, __pyx_L1_error);
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1047, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "ft4222/ft4222.pyx":1046
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":1048
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  __pyx_t_4 = PyBytes_Check(__pyx_v_data); 
  if (!__pyx_t_4) {
  } else {
    __pyx_t_1 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = PyByteArray_Check(__pyx_v_data); 
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  __pyx_t_4 = (!__pyx_t_1);
  if (unlikely(__pyx_t_4)) {

    /* "ft4222/ft4222.pyx":1049
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint16 sizeTransferred
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1049, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1049, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":1048
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":1052
 *         cdef:
 *             uint16 sizeTransferred
 *             uint8* cdata = data             # <<<<<<<<<<<<<<
 * 
 *         status = FT4222_SPISlave_Write(self._handle, cdata, len(data), &sizeTransferred);
 */
  __pyx_t_5 = __Pyx_PyObject_AsWritableUString(__pyx_v_data); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 1052, __pyx_L1_error)
  __pyx_v_cdata = __pyx_t_5;

  /* "ft4222/ft4222.pyx":1054
 *             uint8* cdata = data
 * 
 *         status = FT4222_SPISlave_Write(self._handle, cdata, len(data), &sizeTransferred);             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return sizeTransferred
 */
  __pyx_t_6 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1054, __pyx_L1_error)
  __pyx_v_status = FT4222_SPISlave_Write(__pyx_v_self->_handle, __pyx_v_cdata, __pyx_t_6, (&__pyx_v_sizeTransferred));

  /* "ft4222/ft4222.pyx":1055
 * 
 *         status = FT4222_SPISlave_Write(self._handle, cdata, len(data), &sizeTransferred);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return sizeTransferred
 *         raise FT4222DeviceError, status
 */
  __pyx_t_4 = (__pyx_v_status == FT4222_OK);
  if (__pyx_t_4) {

    /* "ft4222/ft4222.pyx":1056
 *         status = FT4222_SPISlave_Write(self._handle, cdata, len(data), &sizeTransferred);
 *         if status == FT4222_OK:
 *             return sizeTransferred             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_uint16_t(__pyx_v_sizeTransferred); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1056, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":1055
 * 
 *         status = FT4222_SPISlave_Write(self._handle, cdata, len(data), &sizeTransferred);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return sizeTransferred
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":1057
 *         if status == FT4222_OK:
 *             return sizeTransferred
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1057, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1057, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_Raise(__pyx_t_3, __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 1057, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":1033
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_Write(self, data):             # <<<<<<<<<<<<<<
 *         """Write data to the transmit queue of the SPI slave device.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiSlave_Write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_89__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_89__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_89__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_89__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__reduce_cython__", 0))) return NULL;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_88__reduce_cython__(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_88__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 1);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 */
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_kp_s_self__handle_cannot_be_converted, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_91__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_6FT4222_91__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_91__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_91__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_state,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_state)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 3, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__setstate_cython__") < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_90__setstate_cython__(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_90__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 1);

  /* "(tree fragment)":4
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 */
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_kp_s_self__handle_cannot_be_converted, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_6ft4222_6ft4222__closer __pyx_vtable_6ft4222_6ft4222__closer;

static PyObject *__pyx_tp_new_6ft4222_6ft4222__closer(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6ft4222_6ft4222__closer *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  if (likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  #endif
  p = ((struct __pyx_obj_6ft4222_6ft4222__closer *)o);
  p->__pyx_vtab = __pyx_vtabptr_6ft4222_6ft4222__closer;
  return o;
}

static void __pyx_tp_dealloc_6ft4222_6ft4222__closer(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && (!PyType_IS_GC(Py_TYPE(o)) || !__Pyx_PyObject_GC_IsFinalized(o))) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_6ft4222_6ft4222__closer) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  #if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
  (*Py_TYPE(o)->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
}

static PyMethodDef __pyx_methods_6ft4222_6ft4222__closer[] = {
  {"close", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_7_closer_1close, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_7_closer_3__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_7_closer_5__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_6ft4222_6ft4222__closer_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6ft4222_6ft4222__closer},
  {Py_tp_doc, (void *)PyDoc_STR("Wrap a hid_device *ptr and a provide a way to call hid_close() on it.\n    Used internally for weakref.finalize, which only accepts Python objects.\n    ")},
  {Py_tp_methods, (void *)__pyx_methods_6ft4222_6ft4222__closer},
  {Py_tp_new, (void *)__pyx_tp_new_6ft4222_6ft4222__closer},
  {0, 0},
};
static PyType_Spec __pyx_type_6ft4222_6ft4222__closer_spec = {
  "ft4222.ft4222._closer",
  sizeof(struct __pyx_obj_6ft4222_6ft4222__closer),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE,
  __pyx_type_6ft4222_6ft4222__closer_slots,
};
#else

static PyTypeObject __pyx_type_6ft4222_6ft4222__closer = {
  PyVarObject_HEAD_INIT(0, 0)
  "ft4222.ft4222.""_closer", /*tp_name*/
  sizeof(struct __pyx_obj_6ft4222_6ft4222__closer), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ft4222_6ft4222__closer, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  PyDoc_STR("Wrap a hid_device *ptr and a provide a way to call hid_close() on it.\n    Used internally for weakref.finalize, which only accepts Python objects.\n    "), /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ft4222_6ft4222__closer, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ft4222_6ft4222__closer, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_6ft4222_6ft4222_FT4222 __pyx_vtable_6ft4222_6ft4222_FT4222;

static PyObject *__pyx_tp_new_6ft4222_6ft4222_FT4222(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6ft4222_6ft4222_FT4222 *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  if (likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  #endif
  p = ((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)o);
  p->__pyx_vtab = __pyx_vtabptr_6ft4222_6ft4222_FT4222;
  p->_close = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ft4222_6ft4222_FT4222(PyObject *o) {
  struct __pyx_obj_6ft4222_6ft4222_FT4222 *p = (struct __pyx_obj_6ft4222_6ft4222_FT4222 *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_6ft4222_6ft4222_FT4222) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  if (p->__weakref__) PyObject_ClearWeakRefs(o);
  Py_CLEAR(p->_close);
  #if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
  (*Py_TYPE(o)->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
}

static int __pyx_tp_traverse_6ft4222_6ft4222_FT4222(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ft4222_6ft4222_FT4222 *p = (struct __pyx_obj_6ft4222_6ft4222_FT4222 *)o;
  if (p->_close) {
    e = (*v)(p->_close, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ft4222_6ft4222_FT4222(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ft4222_6ft4222_FT4222 *p = (struct __pyx_obj_6ft4222_6ft4222_FT4222 *)o;
  tmp = ((PyObject*)p->_close);
  p->_close = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6ft4222_6ft4222_6FT4222_chipVersion(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ft4222_6ft4222_6FT4222_11chipVersion_1__get__(o);
}

static PyObject *__pyx_getprop_6ft4222_6ft4222_6FT4222_libVersion(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ft4222_6ft4222_6FT4222_10libVersion_1__get__(o);
}

static PyObject *__pyx_getprop_6ft4222_6ft4222_6FT4222_chipRevision(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ft4222_6ft4222_6FT4222_12chipRevision_1__get__(o);
}

static PyObject *__pyx_specialmethod___pyx_pw_6ft4222_6ft4222_6FT4222_5__repr__(PyObject *self, CYTHON_UNUSED PyObject *arg) {
  return __pyx_pw_6ft4222_6ft4222_6FT4222_5__repr__(self);
}

static PyMethodDef __pyx_methods_6ft4222_6ft4222_FT4222[] = {
  {"close", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_3close, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_2close},
  {"__repr__", (PyCFunction)__pyx_specialmethod___pyx_pw_6ft4222_6ft4222_6FT4222_5__repr__, METH_NOARGS|METH_COEXIST, 0},
  {"setTimeouts", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_7setTimeouts, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_6setTimeouts},
  {"setClock", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_9setClock, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_8setClock},
  {"getClock", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_11getClock, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_10getClock},
  {"setSuspendOut", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_13setSuspendOut, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_12setSuspendOut},
  {"setWakeUpInterrupt", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_15setWakeUpInterrupt, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_14setWakeUpInterrupt},
  {"getMaxTransferSize", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_17getMaxTransferSize, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_16getMaxTransferSize},
  {"vendorCmdGet", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_19vendorCmdGet, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_18vendorCmdGet},
  {"vendorCmdSet", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_21vendorCmdSet, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_20vendorCmdSet},
  {"getLatencyTimer", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_23getLatencyTimer, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_22getLatencyTimer},
  {"setLatencyTimer", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_25setLatencyTimer, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_24setLatencyTimer},
  {"chipReset", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_27chipReset, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_26chipReset},
  {"gpio_Init", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_29gpio_Init, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_28gpio_Init},
  {"gpio_Read", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_31gpio_Read, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_30gpio_Read},
  {"gpio_Wait", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_33gpio_Wait, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_32gpio_Wait},
  {"gpio_Write", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_35gpio_Write, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_34gpio_Write},
  {"gpio_SetInputTrigger", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_37gpio_SetInputTrigger, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_36gpio_SetInputTrigger},
  {"gpio_GetTriggerStatus", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_39gpio_GetTriggerStatus, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_38gpio_GetTriggerStatus},
  {"gpio_ReadTriggerQueue", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_41gpio_ReadTriggerQueue, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_40gpio_ReadTriggerQueue},
  {"i2cMaster_Init", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_43i2cMaster_Init, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_42i2cMaster_Init},
  {"i2cMaster_Read", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_45i2cMaster_Read, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_44i2cMaster_Read},
  {"i2cMaster_Write", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_47i2cMaster_Write, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_46i2cMaster_Write},
  {"i2cMaster_ReadEx", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_49i2cMaster_ReadEx, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_48i2cMaster_ReadEx},
  {"i2cMaster_WriteEx", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_51i2cMaster_WriteEx, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_50i2cMaster_WriteEx},
  {"i2cMaster_Reset", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_53i2cMaster_Reset, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_52i2cMaster_Reset},
  {"i2cMaster_GetStatus", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_55i2cMaster_GetStatus, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_54i2cMaster_GetStatus},
  {"spi_Reset", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_57spi_Reset, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_56spi_Reset},
  {"spi_ResetTransaction", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_59spi_ResetTransaction, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_58spi_ResetTransaction},
  {"spi_SetDrivingStrength", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_61spi_SetDrivingStrength, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_60spi_SetDrivingStrength},
  {"spiMaster_Init", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_63spiMaster_Init, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_62spiMaster_Init},
  {"spiMaster_SetLines", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_65spiMaster_SetLines, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_64spiMaster_SetLines},
  {"spiMaster_SingleRead", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_67spiMaster_SingleRead, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_66spiMaster_SingleRead},
  {"spiMaster_SingleWrite", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_69spiMaster_SingleWrite, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_68spiMaster_SingleWrite},
  {"spiMaster_SingleReadWrite", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_71spiMaster_SingleReadWrite, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_70spiMaster_SingleReadWrite},
  {"spiMaster_MultiReadWrite", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_73spiMaster_MultiReadWrite, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_72spiMaster_MultiReadWrite},
  {"spiMaster_EndTransaction", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_75spiMaster_EndTransaction, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_74spiMaster_EndTransaction},
  {"spiSlave_Init", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_77spiSlave_Init, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_76spiSlave_Init},
  {"spiSlave_InitEx", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_79spiSlave_InitEx, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_78spiSlave_InitEx},
  {"spiSlave_Read", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_81spiSlave_Read, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_80spiSlave_Read},
  {"spiSlave_SetMode", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_83spiSlave_SetMode, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_82spiSlave_SetMode},
  {"spiSlave_GetRxStatus", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_85spiSlave_GetRxStatus, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_84spiSlave_GetRxStatus},
  {"spiSlave_Write", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_87spiSlave_Write, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_86spiSlave_Write},
  {"__reduce_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_89__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_91__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ft4222_6ft4222_FT4222[] = {
  {(char *)"chipVersion", __pyx_getprop_6ft4222_6ft4222_6FT4222_chipVersion, 0, (char *)PyDoc_STR("Chip version as number"), 0},
  {(char *)"libVersion", __pyx_getprop_6ft4222_6ft4222_6FT4222_libVersion, 0, (char *)PyDoc_STR("Library version as number"), 0},
  {(char *)"chipRevision", __pyx_getprop_6ft4222_6ft4222_6FT4222_chipRevision, 0, (char *)PyDoc_STR("The revision of the chip in human readable format"), 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_6ft4222_6ft4222_FT4222_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6ft4222_6ft4222_FT4222},
  {Py_tp_repr, (void *)__pyx_pw_6ft4222_6ft4222_6FT4222_5__repr__},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_6ft4222_6ft4222_FT4222},
  {Py_tp_clear, (void *)__pyx_tp_clear_6ft4222_6ft4222_FT4222},
  {Py_tp_methods, (void *)__pyx_methods_6ft4222_6ft4222_FT4222},
  {Py_tp_getset, (void *)__pyx_getsets_6ft4222_6ft4222_FT4222},
  {Py_tp_init, (void *)__pyx_pw_6ft4222_6ft4222_6FT4222_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_6ft4222_6ft4222_FT4222},
  {0, 0},
};
static PyType_Spec __pyx_type_6ft4222_6ft4222_FT4222_spec = {
  "ft4222.ft4222.FT4222",
  sizeof(struct __pyx_obj_6ft4222_6ft4222_FT4222),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_6ft4222_6ft4222_FT4222_slots,
};
#else

static PyTypeObject __pyx_type_6ft4222_6ft4222_FT4222 = {
  PyVarObject_HEAD_INIT(0, 0)
  "ft4222.ft4222.""FT4222", /*tp_name*/
  sizeof(struct __pyx_obj_6ft4222_6ft4222_FT4222), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ft4222_6ft4222_FT4222, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_6ft4222_6ft4222_6FT4222_5__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6ft4222_6ft4222_FT4222, /*tp_traverse*/
  __pyx_tp_clear_6ft4222_6ft4222_FT4222, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ft4222_6ft4222_FT4222, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ft4222_6ft4222_FT4222, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_6ft4222_6ft4222_6FT4222_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ft4222_6ft4222_FT4222, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif
/* #### Code section: pystring_table ### */

static int __Pyx_CreateStringTabAndInitStrings(void) {
  __Pyx_StringTabEntry __pyx_string_tab[] = {
    {&__pyx_n_u_B, __pyx_k_B, sizeof(__pyx_k_B), 0, 1, 0, 1},
    {&__pyx_n_s_CLK_24, __pyx_k_CLK_24, sizeof(__pyx_k_CLK_24), 0, 0, 1, 1},
    {&__pyx_n_s_CLK_48, __pyx_k_CLK_48, sizeof(__pyx_k_CLK_48), 0, 0, 1, 1},
    {&__pyx_n_s_CLK_60, __pyx_k_CLK_60, sizeof(__pyx_k_CLK_60), 0, 0, 1, 1},
    {&__pyx_n_s_CLK_80, __pyx_k_CLK_80, sizeof(__pyx_k_CLK_80), 0, 0, 1, 1},
    {&__pyx_n_u_CLK_NOT_SUPPORTED, __pyx_k_CLK_NOT_SUPPORTED, sizeof(__pyx_k_CLK_NOT_SUPPORTED), 0, 1, 0, 1},
    {&__pyx_kp_s_Chip_system_clock_Attributes_CLK, __pyx_k_Chip_system_clock_Attributes_CLK, sizeof(__pyx_k_Chip_system_clock_Attributes_CLK), 0, 0, 1, 0},
    {&__pyx_n_s_ControllerStatus, __pyx_k_ControllerStatus, sizeof(__pyx_k_ControllerStatus), 0, 0, 1, 1},
    {&__pyx_n_u_DEVICE_LIST_NOT_READY, __pyx_k_DEVICE_LIST_NOT_READY, sizeof(__pyx_k_DEVICE_LIST_NOT_READY), 0, 1, 0, 1},
    {&__pyx_n_u_DEVICE_NOT_FOUND, __pyx_k_DEVICE_NOT_FOUND, sizeof(__pyx_k_DEVICE_NOT_FOUND), 0, 1, 0, 1},
    {&__pyx_n_u_DEVICE_NOT_OPENED, __pyx_k_DEVICE_NOT_OPENED, sizeof(__pyx_k_DEVICE_NOT_OPENED), 0, 1, 0, 1},
    {&__pyx_n_u_DEVICE_NOT_OPENED_FOR_ERASE, __pyx_k_DEVICE_NOT_OPENED_FOR_ERASE, sizeof(__pyx_k_DEVICE_NOT_OPENED_FOR_ERASE), 0, 1, 0, 1},
    {&__pyx_n_u_DEVICE_NOT_OPENED_FOR_WRITE, __pyx_k_DEVICE_NOT_OPENED_FOR_WRITE, sizeof(__pyx_k_DEVICE_NOT_OPENED_FOR_WRITE), 0, 1, 0, 1},
    {&__pyx_n_u_DEVICE_NOT_SUPPORTED, __pyx_k_DEVICE_NOT_SUPPORTED, sizeof(__pyx_k_DEVICE_NOT_SUPPORTED), 0, 1, 0, 1},
    {&__pyx_n_s_Dir, __pyx_k_Dir, sizeof(__pyx_k_Dir), 0, 0, 1, 1},
    {&__pyx_n_u_EEPROM_ERASE_FAILED, __pyx_k_EEPROM_ERASE_FAILED, sizeof(__pyx_k_EEPROM_ERASE_FAILED), 0, 1, 0, 1},
    {&__pyx_n_u_EEPROM_NOT_PRESENT, __pyx_k_EEPROM_NOT_PRESENT, sizeof(__pyx_k_EEPROM_NOT_PRESENT), 0, 1, 0, 1},
    {&__pyx_n_u_EEPROM_NOT_PROGRAMMED, __pyx_k_EEPROM_NOT_PROGRAMMED, sizeof(__pyx_k_EEPROM_NOT_PROGRAMMED), 0, 1, 0, 1},
    {&__pyx_n_u_EEPROM_READ_FAILED, __pyx_k_EEPROM_READ_FAILED, sizeof(__pyx_k_EEPROM_READ_FAILED), 0, 1, 0, 1},
    {&__pyx_n_u_EEPROM_WRITE_FAILED, __pyx_k_EEPROM_WRITE_FAILED, sizeof(__pyx_k_EEPROM_WRITE_FAILED), 0, 1, 0, 1},
    {&__pyx_n_u_EVENT_NOT_SUPPORTED, __pyx_k_EVENT_NOT_SUPPORTED, sizeof(__pyx_k_EVENT_NOT_SUPPORTED), 0, 1, 0, 1},
    {&__pyx_n_u_EXCEEDED_MAX_TRANSFER_SIZE, __pyx_k_EXCEEDED_MAX_TRANSFER_SIZE, sizeof(__pyx_k_EXCEEDED_MAX_TRANSFER_SIZE), 0, 1, 0, 1},
    {&__pyx_kp_s_Exception_class_for_status_messa, __pyx_k_Exception_class_for_status_messa, sizeof(__pyx_k_Exception_class_for_status_messa), 0, 0, 1, 0},
    {&__pyx_n_u_FAILED_TO_READ_DEVICE, __pyx_k_FAILED_TO_READ_DEVICE, sizeof(__pyx_k_FAILED_TO_READ_DEVICE), 0, 1, 0, 1},
    {&__pyx_n_u_FAILED_TO_WRITE_DEVICE0, __pyx_k_FAILED_TO_WRITE_DEVICE0, sizeof(__pyx_k_FAILED_TO_WRITE_DEVICE0), 0, 1, 0, 1},
    {&__pyx_n_s_FT2XXDeviceError, __pyx_k_FT2XXDeviceError, sizeof(__pyx_k_FT2XXDeviceError), 0, 0, 1, 1},
    {&__pyx_n_s_FT2XXDeviceError___init, __pyx_k_FT2XXDeviceError___init, sizeof(__pyx_k_FT2XXDeviceError___init), 0, 0, 1, 1},
    {&__pyx_n_s_FT2XXDeviceError___str, __pyx_k_FT2XXDeviceError___str, sizeof(__pyx_k_FT2XXDeviceError___str), 0, 0, 1, 1},
    {&__pyx_n_s_FT2XXDeviceError__ftd2xx_msgs, __pyx_k_FT2XXDeviceError__ftd2xx_msgs, sizeof(__pyx_k_FT2XXDeviceError__ftd2xx_msgs), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222, __pyx_k_FT4222, sizeof(__pyx_k_FT4222), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222DeviceError, __pyx_k_FT4222DeviceError, sizeof(__pyx_k_FT4222DeviceError), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222DeviceError___init, __pyx_k_FT4222DeviceError___init, sizeof(__pyx_k_FT4222DeviceError___init), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222DeviceError___str, __pyx_k_FT4222DeviceError___str, sizeof(__pyx_k_FT4222DeviceError___str), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222DeviceError__ftd4222_msgs, __pyx_k_FT4222DeviceError__ftd4222_msgs, sizeof(__pyx_k_FT4222DeviceError__ftd4222_msgs), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222___reduce_cython, __pyx_k_FT4222___reduce_cython, sizeof(__pyx_k_FT4222___reduce_cython), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222___setstate_cython, __pyx_k_FT4222___setstate_cython, sizeof(__pyx_k_FT4222___setstate_cython), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222__chip_rev_map, __pyx_k_FT4222__chip_rev_map, sizeof(__pyx_k_FT4222__chip_rev_map), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222__chip_rev_min_lib, __pyx_k_FT4222__chip_rev_min_lib, sizeof(__pyx_k_FT4222__chip_rev_min_lib), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_chipReset, __pyx_k_FT4222_chipReset, sizeof(__pyx_k_FT4222_chipReset), 0, 0, 1, 1},
    {&__pyx_kp_u_FT4222_chipVersion_0x_x_s_libVer, __pyx_k_FT4222_chipVersion_0x_x_s_libVer, sizeof(__pyx_k_FT4222_chipVersion_0x_x_s_libVer), 0, 1, 0, 0},
    {&__pyx_n_s_FT4222_close, __pyx_k_FT4222_close, sizeof(__pyx_k_FT4222_close), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_getClock, __pyx_k_FT4222_getClock, sizeof(__pyx_k_FT4222_getClock), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_getLatencyTimer, __pyx_k_FT4222_getLatencyTimer, sizeof(__pyx_k_FT4222_getLatencyTimer), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_getMaxTransferSize, __pyx_k_FT4222_getMaxTransferSize, sizeof(__pyx_k_FT4222_getMaxTransferSize), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_gpio_GetTriggerStatus, __pyx_k_FT4222_gpio_GetTriggerStatus, sizeof(__pyx_k_FT4222_gpio_GetTriggerStatus), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_gpio_Init, __pyx_k_FT4222_gpio_Init, sizeof(__pyx_k_FT4222_gpio_Init), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_gpio_Read, __pyx_k_FT4222_gpio_Read, sizeof(__pyx_k_FT4222_gpio_Read), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_gpio_ReadTriggerQueue, __pyx_k_FT4222_gpio_ReadTriggerQueue, sizeof(__pyx_k_FT4222_gpio_ReadTriggerQueue), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_gpio_SetInputTrigger, __pyx_k_FT4222_gpio_SetInputTrigger, sizeof(__pyx_k_FT4222_gpio_SetInputTrigger), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_gpio_Wait, __pyx_k_FT4222_gpio_Wait, sizeof(__pyx_k_FT4222_gpio_Wait), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_gpio_Write, __pyx_k_FT4222_gpio_Write, sizeof(__pyx_k_FT4222_gpio_Write), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_i2cMaster_GetStatus, __pyx_k_FT4222_i2cMaster_GetStatus, sizeof(__pyx_k_FT4222_i2cMaster_GetStatus), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_i2cMaster_Init, __pyx_k_FT4222_i2cMaster_Init, sizeof(__pyx_k_FT4222_i2cMaster_Init), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_i2cMaster_Read, __pyx_k_FT4222_i2cMaster_Read, sizeof(__pyx_k_FT4222_i2cMaster_Read), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_i2cMaster_ReadEx, __pyx_k_FT4222_i2cMaster_ReadEx, sizeof(__pyx_k_FT4222_i2cMaster_ReadEx), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_i2cMaster_Reset, __pyx_k_FT4222_i2cMaster_Reset, sizeof(__pyx_k_FT4222_i2cMaster_Reset), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_i2cMaster_Write, __pyx_k_FT4222_i2cMaster_Write, sizeof(__pyx_k_FT4222_i2cMaster_Write), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_i2cMaster_WriteEx, __pyx_k_FT4222_i2cMaster_WriteEx, sizeof(__pyx_k_FT4222_i2cMaster_WriteEx), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_setClock, __pyx_k_FT4222_setClock, sizeof(__pyx_k_FT4222_setClock), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_setLatencyTimer, __pyx_k_FT4222_setLatencyTimer, sizeof(__pyx_k_FT4222_setLatencyTimer), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_setSuspendOut, __pyx_k_FT4222_setSuspendOut, sizeof(__pyx_k_FT4222_setSuspendOut), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_setTimeouts, __pyx_k_FT4222_setTimeouts, sizeof(__pyx_k_FT4222_setTimeouts), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_setWakeUpInterrupt, __pyx_k_FT4222_setWakeUpInterrupt, sizeof(__pyx_k_FT4222_setWakeUpInterrupt), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_spiMaster_EndTransaction, __pyx_k_FT4222_spiMaster_EndTransaction, sizeof(__pyx_k_FT4222_spiMaster_EndTransaction), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_spiMaster_Init, __pyx_k_FT4222_spiMaster_Init, sizeof(__pyx_k_FT4222_spiMaster_Init), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_spiMaster_MultiReadWrite, __pyx_k_FT4222_spiMaster_MultiReadWrite, sizeof(__pyx_k_FT4222_spiMaster_MultiReadWrite), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_spiMaster_SetLines, __pyx_k_FT4222_spiMaster_SetLines, sizeof(__pyx_k_FT4222_spiMaster_SetLines), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_spiMaster_SingleRead, __pyx_k_FT4222_spiMaster_SingleRead, sizeof(__pyx_k_FT4222_spiMaster_SingleRead), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_spiMaster_SingleReadWrite, __pyx_k_FT4222_spiMaster_SingleReadWrite, sizeof(__pyx_k_FT4222_spiMaster_SingleReadWrite), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_spiMaster_SingleWrite, __pyx_k_FT4222_spiMaster_SingleWrite, sizeof(__pyx_k_FT4222_spiMaster_SingleWrite), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_spiSlave_GetRxStatus, __pyx_k_FT4222_spiSlave_GetRxStatus, sizeof(__pyx_k_FT4222_spiSlave_GetRxStatus), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_spiSlave_Init, __pyx_k_FT4222_spiSlave_Init, sizeof(__pyx_k_FT4222_spiSlave_Init), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_spiSlave_InitEx, __pyx_k_FT4222_spiSlave_InitEx, sizeof(__pyx_k_FT4222_spiSlave_InitEx), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_spiSlave_Read, __pyx_k_FT4222_spiSlave_Read, sizeof(__pyx_k_FT4222_spiSlave_Read), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_spiSlave_SetMode, __pyx_k_FT4222_spiSlave_SetMode, sizeof(__pyx_k_FT4222_spiSlave_SetMode), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_spiSlave_Write, __pyx_k_FT4222_spiSlave_Write, sizeof(__pyx_k_FT4222_spiSlave_Write), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_spi_Reset, __pyx_k_FT4222_spi_Reset, sizeof(__pyx_k_FT4222_spi_Reset), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_spi_ResetTransaction, __pyx_k_FT4222_spi_ResetTransaction, sizeof(__pyx_k_FT4222_spi_ResetTransaction), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_spi_SetDrivingStrength, __pyx_k_FT4222_spi_SetDrivingStrength, sizeof(__pyx_k_FT4222_spi_SetDrivingStrength), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_vendorCmdGet, __pyx_k_FT4222_vendorCmdGet, sizeof(__pyx_k_FT4222_vendorCmdGet), 0, 0, 1, 1},
    {&__pyx_n_s_FT4222_vendorCmdSet, __pyx_k_FT4222_vendorCmdSet, sizeof(__pyx_k_FT4222_vendorCmdSet), 0, 0, 1, 1},
    {&__pyx_n_u_FUN_NOT_SUPPORT, __pyx_k_FUN_NOT_SUPPORT, sizeof(__pyx_k_FUN_NOT_SUPPORT), 0, 1, 0, 1},
    {&__pyx_n_s_GPIO, __pyx_k_GPIO, sizeof(__pyx_k_GPIO), 0, 0, 1, 1},
    {&__pyx_n_u_GPIO_EXCEEDED_MAX_PORTNUM, __pyx_k_GPIO_EXCEEDED_MAX_PORTNUM, sizeof(__pyx_k_GPIO_EXCEEDED_MAX_PORTNUM), 0, 1, 0, 1},
    {&__pyx_n_u_GPIO_INPUT_NOT_SUPPORTED, __pyx_k_GPIO_INPUT_NOT_SUPPORTED, sizeof(__pyx_k_GPIO_INPUT_NOT_SUPPORTED), 0, 1, 0, 1},
    {&__pyx_n_u_GPIO_NOT_SUPPORTED_IN_THIS_MODE, __pyx_k_GPIO_NOT_SUPPORTED_IN_THIS_MODE, sizeof(__pyx_k_GPIO_NOT_SUPPORTED_IN_THIS_MODE), 0, 1, 0, 1},
    {&__pyx_n_u_GPIO_OPENDRAIN_INVALID_IN_OUTPUT, __pyx_k_GPIO_OPENDRAIN_INVALID_IN_OUTPUT, sizeof(__pyx_k_GPIO_OPENDRAIN_INVALID_IN_OUTPUT), 0, 1, 0, 1},
    {&__pyx_n_u_GPIO_PULLDOWN_INVALID_IN_INPUTMO, __pyx_k_GPIO_PULLDOWN_INVALID_IN_INPUTMO, sizeof(__pyx_k_GPIO_PULLDOWN_INVALID_IN_INPUTMO), 0, 1, 0, 1},
    {&__pyx_n_u_GPIO_PULLUP_INVALID_IN_INPUTMODE, __pyx_k_GPIO_PULLUP_INVALID_IN_INPUTMODE, sizeof(__pyx_k_GPIO_PULLUP_INVALID_IN_INPUTMODE), 0, 1, 0, 1},
    {&__pyx_n_u_GPIO_WRITE_NOT_SUPPORTED, __pyx_k_GPIO_WRITE_NOT_SUPPORTED, sizeof(__pyx_k_GPIO_WRITE_NOT_SUPPORTED), 0, 1, 0, 1},
    {&__pyx_n_s_I2CMaster, __pyx_k_I2CMaster, sizeof(__pyx_k_I2CMaster), 0, 0, 1, 1},
    {&__pyx_n_u_I2C_NOT_SUPPORTED_IN_THIS_MODE, __pyx_k_I2C_NOT_SUPPORTED_IN_THIS_MODE, sizeof(__pyx_k_I2C_NOT_SUPPORTED_IN_THIS_MODE), 0, 1, 0, 1},
    {&__pyx_n_s_INPUT, __pyx_k_INPUT, sizeof(__pyx_k_INPUT), 0, 0, 1, 1},
    {&__pyx_n_u_INSUFFICIENT_RESOURCES, __pyx_k_INSUFFICIENT_RESOURCES, sizeof(__pyx_k_INSUFFICIENT_RESOURCES), 0, 1, 0, 1},
    {&__pyx_n_u_INTERRUPT_NOT_SUPPORTED, __pyx_k_INTERRUPT_NOT_SUPPORTED, sizeof(__pyx_k_INTERRUPT_NOT_SUPPORTED), 0, 1, 0, 1},
    {&__pyx_n_u_INVAILD_FUNCTION, __pyx_k_INVAILD_FUNCTION, sizeof(__pyx_k_INVAILD_FUNCTION), 0, 1, 0, 1},
    {&__pyx_n_u_INVALID_ARGS, __pyx_k_INVALID_ARGS, sizeof(__pyx_k_INVALID_ARGS), 0, 1, 0, 1},
    {&__pyx_n_u_INVALID_BAUD_RATE, __pyx_k_INVALID_BAUD_RATE, sizeof(__pyx_k_INVALID_BAUD_RATE), 0, 1, 0, 1},
    {&__pyx_n_u_INVALID_HANDLE, __pyx_k_INVALID_HANDLE, sizeof(__pyx_k_INVALID_HANDLE), 0, 1, 0, 1},
    {&__pyx_n_u_INVALID_PARAMETER, __pyx_k_INVALID_PARAMETER, sizeof(__pyx_k_INVALID_PARAMETER), 0, 1, 0, 1},
    {&__pyx_n_u_INVALID_POINTER, __pyx_k_INVALID_POINTER, sizeof(__pyx_k_INVALID_POINTER), 0, 1, 0, 1},
    {&__pyx_n_u_IO_ERROR, __pyx_k_IO_ERROR, sizeof(__pyx_k_IO_ERROR), 0, 1, 0, 1},
    {&__pyx_n_u_IS_NOT_I2C_MODE, __pyx_k_IS_NOT_I2C_MODE, sizeof(__pyx_k_IS_NOT_I2C_MODE), 0, 1, 0, 1},
    {&__pyx_n_u_IS_NOT_SPI_MODE, __pyx_k_IS_NOT_SPI_MODE, sizeof(__pyx_k_IS_NOT_SPI_MODE), 0, 1, 0, 1},
    {&__pyx_n_u_IS_NOT_SPI_MULTI_MODE, __pyx_k_IS_NOT_SPI_MULTI_MODE, sizeof(__pyx_k_IS_NOT_SPI_MULTI_MODE), 0, 1, 0, 1},
    {&__pyx_n_u_IS_NOT_SPI_SINGLE_MODE, __pyx_k_IS_NOT_SPI_SINGLE_MODE, sizeof(__pyx_k_IS_NOT_SPI_SINGLE_MODE), 0, 1, 0, 1},
    {&__pyx_n_s_IntEnum, __pyx_k_IntEnum, sizeof(__pyx_k_IntEnum), 0, 0, 1, 1},
    {&__pyx_n_s_KeyError, __pyx_k_KeyError, sizeof(__pyx_k_KeyError), 0, 0, 1, 1},
    {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
    {&__pyx_n_u_NOT_SUPPORTED, __pyx_k_NOT_SUPPORTED, sizeof(__pyx_k_NOT_SUPPORTED), 0, 1, 0, 1},
    {&__pyx_n_u_OK, __pyx_k_OK, sizeof(__pyx_k_OK), 0, 1, 0, 1},
    {&__pyx_n_u_OTHER_ERROR, __pyx_k_OTHER_ERROR, sizeof(__pyx_k_OTHER_ERROR), 0, 1, 0, 1},
    {&__pyx_kp_u_Rev_A, __pyx_k_Rev_A, sizeof(__pyx_k_Rev_A), 0, 1, 0, 0},
    {&__pyx_kp_u_Rev_B, __pyx_k_Rev_B, sizeof(__pyx_k_Rev_B), 0, 1, 0, 0},
    {&__pyx_kp_u_Rev_C, __pyx_k_Rev_C, sizeof(__pyx_k_Rev_C), 0, 1, 0, 0},
    {&__pyx_kp_u_Rev_D, __pyx_k_Rev_D, sizeof(__pyx_k_Rev_D), 0, 1, 0, 0},
    {&__pyx_kp_u_Rev_unknown, __pyx_k_Rev_unknown, sizeof(__pyx_k_Rev_unknown), 0, 1, 0, 0},
    {&__pyx_n_s_SysClock, __pyx_k_SysClock, sizeof(__pyx_k_SysClock), 0, 0, 1, 1},
    {&__pyx_n_s_TimeoutError, __pyx_k_TimeoutError, sizeof(__pyx_k_TimeoutError), 0, 0, 1, 1},
    {&__pyx_n_s_Trigger, __pyx_k_Trigger, sizeof(__pyx_k_Trigger), 0, 0, 1, 1},
    {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
    {&__pyx_n_u_VENDER_CMD_NOT_SUPPORTED, __pyx_k_VENDER_CMD_NOT_SUPPORTED, sizeof(__pyx_k_VENDER_CMD_NOT_SUPPORTED), 0, 1, 0, 1},
    {&__pyx_n_u_WRONG_I2C_ADDR, __pyx_k_WRONG_I2C_ADDR, sizeof(__pyx_k_WRONG_I2C_ADDR), 0, 1, 0, 1},
    {&__pyx_n_s__111, __pyx_k__111, sizeof(__pyx_k__111), 0, 0, 1, 1},
    {&__pyx_kp_u__8, __pyx_k__8, sizeof(__pyx_k__8), 0, 1, 0, 0},
    {&__pyx_n_s_addr, __pyx_k_addr, sizeof(__pyx_k_addr), 0, 0, 1, 1},
    {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
    {&__pyx_n_s_asyncio_coroutines, __pyx_k_asyncio_coroutines, sizeof(__pyx_k_asyncio_coroutines), 0, 0, 1, 1},
    {&__pyx_n_s_buf, __pyx_k_buf, sizeof(__pyx_k_buf), 0, 0, 1, 1},
    {&__pyx_n_s_bytesRead, __pyx_k_bytesRead, sizeof(__pyx_k_bytesRead), 0, 0, 1, 1},
    {&__pyx_n_s_bytesSent, __pyx_k_bytesSent, sizeof(__pyx_k_bytesSent), 0, 0, 1, 1},
    {&__pyx_n_s_bytesToRead, __pyx_k_bytesToRead, sizeof(__pyx_k_bytesToRead), 0, 0, 1, 1},
    {&__pyx_n_s_cdata, __pyx_k_cdata, sizeof(__pyx_k_cdata), 0, 0, 1, 1},
    {&__pyx_n_s_cdesc, __pyx_k_cdesc, sizeof(__pyx_k_cdesc), 0, 0, 1, 1},
    {&__pyx_n_s_chipReset, __pyx_k_chipReset, sizeof(__pyx_k_chipReset), 0, 0, 1, 1},
    {&__pyx_n_s_chipRevision, __pyx_k_chipRevision, sizeof(__pyx_k_chipRevision), 0, 0, 1, 1},
    {&__pyx_n_s_class_getitem, __pyx_k_class_getitem, sizeof(__pyx_k_class_getitem), 0, 0, 1, 1},
    {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
    {&__pyx_n_s_clk, __pyx_k_clk, sizeof(__pyx_k_clk), 0, 0, 1, 1},
    {&__pyx_n_s_clkStrength, __pyx_k_clkStrength, sizeof(__pyx_k_clkStrength), 0, 0, 1, 1},
    {&__pyx_n_s_clock, __pyx_k_clock, sizeof(__pyx_k_clock), 0, 0, 1, 1},
    {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
    {&__pyx_n_s_closer, __pyx_k_closer, sizeof(__pyx_k_closer), 0, 0, 1, 1},
    {&__pyx_n_s_closer___reduce_cython, __pyx_k_closer___reduce_cython, sizeof(__pyx_k_closer___reduce_cython), 0, 0, 1, 1},
    {&__pyx_n_s_closer___setstate_cython, __pyx_k_closer___setstate_cython, sizeof(__pyx_k_closer___setstate_cython), 0, 0, 1, 1},
    {&__pyx_n_s_closer_close, __pyx_k_closer_close, sizeof(__pyx_k_closer_close), 0, 0, 1, 1},
    {&__pyx_n_s_cpha, __pyx_k_cpha, sizeof(__pyx_k_cpha), 0, 0, 1, 1},
    {&__pyx_n_s_cpol, __pyx_k_cpol, sizeof(__pyx_k_cpol), 0, 0, 1, 1},
    {&__pyx_n_s_createDeviceInfoList, __pyx_k_createDeviceInfoList, sizeof(__pyx_k_createDeviceInfoList), 0, 0, 1, 1},
    {&__pyx_n_s_cs, __pyx_k_cs, sizeof(__pyx_k_cs), 0, 0, 1, 1},
    {&__pyx_n_s_cserial, __pyx_k_cserial, sizeof(__pyx_k_cserial), 0, 0, 1, 1},
    {&__pyx_n_s_d, __pyx_k_d, sizeof(__pyx_k_d), 0, 0, 1, 1},
    {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
    {&__pyx_n_s_desc, __pyx_k_desc, sizeof(__pyx_k_desc), 0, 0, 1, 1},
    {&__pyx_n_u_description, __pyx_k_description, sizeof(__pyx_k_description), 0, 1, 0, 1},
    {&__pyx_n_s_devnum, __pyx_k_devnum, sizeof(__pyx_k_devnum), 0, 0, 1, 1},
    {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
    {&__pyx_kp_u_disable, __pyx_k_disable, sizeof(__pyx_k_disable), 0, 1, 0, 0},
    {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
    {&__pyx_n_s_enable, __pyx_k_enable, sizeof(__pyx_k_enable), 0, 0, 1, 1},
    {&__pyx_kp_u_enable, __pyx_k_enable, sizeof(__pyx_k_enable), 0, 1, 0, 0},
    {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
    {&__pyx_n_s_enum, __pyx_k_enum, sizeof(__pyx_k_enum), 0, 0, 1, 1},
    {&__pyx_n_s_events, __pyx_k_events, sizeof(__pyx_k_events), 0, 0, 1, 1},
    {&__pyx_n_s_expectedValue, __pyx_k_expectedValue, sizeof(__pyx_k_expectedValue), 0, 0, 1, 1},
    {&__pyx_n_s_expectedValue_c, __pyx_k_expectedValue_c, sizeof(__pyx_k_expectedValue_c), 0, 0, 1, 1},
    {&__pyx_n_s_f, __pyx_k_f, sizeof(__pyx_k_f), 0, 0, 1, 1},
    {&__pyx_n_s_finalize, __pyx_k_finalize, sizeof(__pyx_k_finalize), 0, 0, 1, 1},
    {&__pyx_n_s_flag, __pyx_k_flag, sizeof(__pyx_k_flag), 0, 0, 1, 1},
    {&__pyx_n_u_flags, __pyx_k_flags, sizeof(__pyx_k_flags), 0, 1, 0, 1},
    {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
    {&__pyx_n_s_ft4222_ft4222, __pyx_k_ft4222_ft4222, sizeof(__pyx_k_ft4222_ft4222), 0, 0, 1, 1},
    {&__pyx_kp_s_ft4222_ft4222_pyx, __pyx_k_ft4222_ft4222_pyx, sizeof(__pyx_k_ft4222_ft4222_pyx), 0, 0, 1, 0},
    {&__pyx_kp_u_gc, __pyx_k_gc, sizeof(__pyx_k_gc), 0, 1, 0, 0},
    {&__pyx_n_s_getClock, __pyx_k_getClock, sizeof(__pyx_k_getClock), 0, 0, 1, 1},
    {&__pyx_n_s_getDeviceInfoDetail, __pyx_k_getDeviceInfoDetail, sizeof(__pyx_k_getDeviceInfoDetail), 0, 0, 1, 1},
    {&__pyx_n_s_getLatencyTimer, __pyx_k_getLatencyTimer, sizeof(__pyx_k_getLatencyTimer), 0, 0, 1, 1},
    {&__pyx_n_s_getMaxTransferSize, __pyx_k_getMaxTransferSize, sizeof(__pyx_k_getMaxTransferSize), 0, 0, 1, 1},
    {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
    {&__pyx_n_s_gpio0, __pyx_k_gpio0, sizeof(__pyx_k_gpio0), 0, 0, 1, 1},
    {&__pyx_n_s_gpio1, __pyx_k_gpio1, sizeof(__pyx_k_gpio1), 0, 0, 1, 1},
    {&__pyx_n_s_gpio2, __pyx_k_gpio2, sizeof(__pyx_k_gpio2), 0, 0, 1, 1},
    {&__pyx_n_s_gpio3, __pyx_k_gpio3, sizeof(__pyx_k_gpio3), 0, 0, 1, 1},
    {&__pyx_n_s_gpio_GetTriggerStatus, __pyx_k_gpio_GetTriggerStatus, sizeof(__pyx_k_gpio_GetTriggerStatus), 0, 0, 1, 1},
    {&__pyx_n_s_gpio_Init, __pyx_k_gpio_Init, sizeof(__pyx_k_gpio_Init), 0, 0, 1, 1},
    {&__pyx_n_s_gpio_Read, __pyx_k_gpio_Read, sizeof(__pyx_k_gpio_Read), 0, 0, 1, 1},
    {&__pyx_n_s_gpio_ReadTriggerQueue, __pyx_k_gpio_ReadTriggerQueue, sizeof(__pyx_k_gpio_ReadTriggerQueue), 0, 0, 1, 1},
    {&__pyx_n_s_gpio_SetInputTrigger, __pyx_k_gpio_SetInputTrigger, sizeof(__pyx_k_gpio_SetInputTrigger), 0, 0, 1, 1},
    {&__pyx_n_s_gpio_Wait, __pyx_k_gpio_Wait, sizeof(__pyx_k_gpio_Wait), 0, 0, 1, 1},
    {&__pyx_n_s_gpio_Write, __pyx_k_gpio_Write, sizeof(__pyx_k_gpio_Write), 0, 0, 1, 1},
    {&__pyx_n_s_h, __pyx_k_h, sizeof(__pyx_k_h), 0, 0, 1, 1},
    {&__pyx_n_s_handle, __pyx_k_handle, sizeof(__pyx_k_handle), 0, 0, 1, 1},
    {&__pyx_n_u_handle, __pyx_k_handle, sizeof(__pyx_k_handle), 0, 1, 0, 1},
    {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
    {&__pyx_n_s_i2cMaster_GetStatus, __pyx_k_i2cMaster_GetStatus, sizeof(__pyx_k_i2cMaster_GetStatus), 0, 0, 1, 1},
    {&__pyx_n_s_i2cMaster_Init, __pyx_k_i2cMaster_Init, sizeof(__pyx_k_i2cMaster_Init), 0, 0, 1, 1},
    {&__pyx_n_s_i2cMaster_Read, __pyx_k_i2cMaster_Read, sizeof(__pyx_k_i2cMaster_Read), 0, 0, 1, 1},
    {&__pyx_n_s_i2cMaster_ReadEx, __pyx_k_i2cMaster_ReadEx, sizeof(__pyx_k_i2cMaster_ReadEx), 0, 0, 1, 1},
    {&__pyx_n_s_i2cMaster_Reset, __pyx_k_i2cMaster_Reset, sizeof(__pyx_k_i2cMaster_Reset), 0, 0, 1, 1},
    {&__pyx_n_s_i2cMaster_Write, __pyx_k_i2cMaster_Write, sizeof(__pyx_k_i2cMaster_Write), 0, 0, 1, 1},
    {&__pyx_n_s_i2cMaster_WriteEx, __pyx_k_i2cMaster_WriteEx, sizeof(__pyx_k_i2cMaster_WriteEx), 0, 0, 1, 1},
    {&__pyx_n_u_id, __pyx_k_id, sizeof(__pyx_k_id), 0, 1, 0, 1},
    {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
    {&__pyx_n_u_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 1, 0, 1},
    {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
    {&__pyx_n_s_init_subclass, __pyx_k_init_subclass, sizeof(__pyx_k_init_subclass), 0, 0, 1, 1},
    {&__pyx_n_s_ioDir, __pyx_k_ioDir, sizeof(__pyx_k_ioDir), 0, 0, 1, 1},
    {&__pyx_n_s_ioStrength, __pyx_k_ioStrength, sizeof(__pyx_k_ioStrength), 0, 0, 1, 1},
    {&__pyx_n_s_isEndTransaction, __pyx_k_isEndTransaction, sizeof(__pyx_k_isEndTransaction), 0, 0, 1, 1},
    {&__pyx_n_s_is_coroutine, __pyx_k_is_coroutine, sizeof(__pyx_k_is_coroutine), 0, 0, 1, 1},
    {&__pyx_kp_u_isenabled, __pyx_k_isenabled, sizeof(__pyx_k_isenabled), 0, 1, 0, 0},
    {&__pyx_n_s_kbps, __pyx_k_kbps, sizeof(__pyx_k_kbps), 0, 0, 1, 1},
    {&__pyx_n_s_l, __pyx_k_l, sizeof(__pyx_k_l), 0, 0, 1, 1},
    {&__pyx_n_s_latency, __pyx_k_latency, sizeof(__pyx_k_latency), 0, 0, 1, 1},
    {&__pyx_n_s_locId, __pyx_k_locId, sizeof(__pyx_k_locId), 0, 0, 1, 1},
    {&__pyx_n_u_location, __pyx_k_location, sizeof(__pyx_k_location), 0, 1, 0, 1},
    {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
    {&__pyx_n_s_maxTransferSize, __pyx_k_maxTransferSize, sizeof(__pyx_k_maxTransferSize), 0, 0, 1, 1},
    {&__pyx_n_s_message, __pyx_k_message, sizeof(__pyx_k_message), 0, 0, 1, 1},
    {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
    {&__pyx_n_s_mode, __pyx_k_mode, sizeof(__pyx_k_mode), 0, 0, 1, 1},
    {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
    {&__pyx_n_s_mro_entries, __pyx_k_mro_entries, sizeof(__pyx_k_mro_entries), 0, 0, 1, 1},
    {&__pyx_n_s_msgnum, __pyx_k_msgnum, sizeof(__pyx_k_msgnum), 0, 0, 1, 1},
    {&__pyx_n_s_multiWrite, __pyx_k_multiWrite, sizeof(__pyx_k_multiWrite), 0, 0, 1, 1},
    {&__pyx_n_s_n, __pyx_k_n, sizeof(__pyx_k_n), 0, 0, 1, 1},
    {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
    {&__pyx_n_s_nb, __pyx_k_nb, sizeof(__pyx_k_nb), 0, 0, 1, 1},
    {&__pyx_n_s_openByDescription, __pyx_k_openByDescription, sizeof(__pyx_k_openByDescription), 0, 0, 1, 1},
    {&__pyx_n_s_openByLocation, __pyx_k_openByLocation, sizeof(__pyx_k_openByLocation), 0, 0, 1, 1},
    {&__pyx_n_s_openBySerial, __pyx_k_openBySerial, sizeof(__pyx_k_openBySerial), 0, 0, 1, 1},
    {&__pyx_n_s_pRxSize, __pyx_k_pRxSize, sizeof(__pyx_k_pRxSize), 0, 0, 1, 1},
    {&__pyx_n_s_portNum, __pyx_k_portNum, sizeof(__pyx_k_portNum), 0, 0, 1, 1},
    {&__pyx_n_s_portNum_c, __pyx_k_portNum_c, sizeof(__pyx_k_portNum_c), 0, 0, 1, 1},
    {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
    {&__pyx_n_s_proto, __pyx_k_proto, sizeof(__pyx_k_proto), 0, 0, 1, 1},
    {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
    {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
    {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
    {&__pyx_n_s_queueSize, __pyx_k_queueSize, sizeof(__pyx_k_queueSize), 0, 0, 1, 1},
    {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
    {&__pyx_n_s_readSize, __pyx_k_readSize, sizeof(__pyx_k_readSize), 0, 0, 1, 1},
    {&__pyx_n_s_read_timeout, __pyx_k_read_timeout, sizeof(__pyx_k_read_timeout), 0, 0, 1, 1},
    {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
    {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
    {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
    {&__pyx_n_s_req, __pyx_k_req, sizeof(__pyx_k_req), 0, 0, 1, 1},
    {&__pyx_n_s_res, __pyx_k_res, sizeof(__pyx_k_res), 0, 0, 1, 1},
    {&__pyx_n_s_round, __pyx_k_round, sizeof(__pyx_k_round), 0, 0, 1, 1},
    {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
    {&__pyx_kp_s_self__handle_cannot_be_converted, __pyx_k_self__handle_cannot_be_converted, sizeof(__pyx_k_self__handle_cannot_be_converted), 0, 0, 1, 0},
    {&__pyx_n_s_serial, __pyx_k_serial, sizeof(__pyx_k_serial), 0, 0, 1, 1},
    {&__pyx_n_u_serial, __pyx_k_serial, sizeof(__pyx_k_serial), 0, 1, 0, 1},
    {&__pyx_n_s_setClock, __pyx_k_setClock, sizeof(__pyx_k_setClock), 0, 0, 1, 1},
    {&__pyx_n_s_setLatencyTimer, __pyx_k_setLatencyTimer, sizeof(__pyx_k_setLatencyTimer), 0, 0, 1, 1},
    {&__pyx_n_s_setSuspendOut, __pyx_k_setSuspendOut, sizeof(__pyx_k_setSuspendOut), 0, 0, 1, 1},
    {&__pyx_n_s_setTimeouts, __pyx_k_setTimeouts, sizeof(__pyx_k_setTimeouts), 0, 0, 1, 1},
    {&__pyx_n_s_setWakeUpInterrupt, __pyx_k_setWakeUpInterrupt, sizeof(__pyx_k_setWakeUpInterrupt), 0, 0, 1, 1},
    {&__pyx_n_s_set_name, __pyx_k_set_name, sizeof(__pyx_k_set_name), 0, 0, 1, 1},
    {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
    {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
    {&__pyx_n_s_singleWrite, __pyx_k_singleWrite, sizeof(__pyx_k_singleWrite), 0, 0, 1, 1},
    {&__pyx_n_s_sizeRead, __pyx_k_sizeRead, sizeof(__pyx_k_sizeRead), 0, 0, 1, 1},
    {&__pyx_n_s_sizeTransferred, __pyx_k_sizeTransferred, sizeof(__pyx_k_sizeTransferred), 0, 0, 1, 1},
    {&__pyx_n_s_sleep, __pyx_k_sleep, sizeof(__pyx_k_sleep), 0, 0, 1, 1},
    {&__pyx_n_s_sleep_c, __pyx_k_sleep_c, sizeof(__pyx_k_sleep_c), 0, 0, 1, 1},
    {&__pyx_n_s_spiIdx, __pyx_k_spiIdx, sizeof(__pyx_k_spiIdx), 0, 0, 1, 1},
    {&__pyx_n_s_spiMaster_EndTransaction, __pyx_k_spiMaster_EndTransaction, sizeof(__pyx_k_spiMaster_EndTransaction), 0, 0, 1, 1},
    {&__pyx_n_s_spiMaster_Init, __pyx_k_spiMaster_Init, sizeof(__pyx_k_spiMaster_Init), 0, 0, 1, 1},
    {&__pyx_n_s_spiMaster_MultiReadWrite, __pyx_k_spiMaster_MultiReadWrite, sizeof(__pyx_k_spiMaster_MultiReadWrite), 0, 0, 1, 1},
    {&__pyx_n_s_spiMaster_SetLines, __pyx_k_spiMaster_SetLines, sizeof(__pyx_k_spiMaster_SetLines), 0, 0, 1, 1},
    {&__pyx_n_s_spiMaster_SingleRead, __pyx_k_spiMaster_SingleRead, sizeof(__pyx_k_spiMaster_SingleRead), 0, 0, 1, 1},
    {&__pyx_n_s_spiMaster_SingleReadWrite, __pyx_k_spiMaster_SingleReadWrite, sizeof(__pyx_k_spiMaster_SingleReadWrite), 0, 0, 1, 1},
    {&__pyx_n_s_spiMaster_SingleWrite, __pyx_k_spiMaster_SingleWrite, sizeof(__pyx_k_spiMaster_SingleWrite), 0, 0, 1, 1},
    {&__pyx_n_s_spiSlave_GetRxStatus, __pyx_k_spiSlave_GetRxStatus, sizeof(__pyx_k_spiSlave_GetRxStatus), 0, 0, 1, 1},
    {&__pyx_n_s_spiSlave_Init, __pyx_k_spiSlave_Init, sizeof(__pyx_k_spiSlave_Init), 0, 0, 1, 1},
    {&__pyx_n_s_spiSlave_InitEx, __pyx_k_spiSlave_InitEx, sizeof(__pyx_k_spiSlave_InitEx), 0, 0, 1, 1},
    {&__pyx_n_s_spiSlave_Read, __pyx_k_spiSlave_Read, sizeof(__pyx_k_spiSlave_Read), 0, 0, 1, 1},
    {&__pyx_n_s_spiSlave_SetMode, __pyx_k_spiSlave_SetMode, sizeof(__pyx_k_spiSlave_SetMode), 0, 0, 1, 1},
    {&__pyx_n_s_spiSlave_Write, __pyx_k_spiSlave_Write, sizeof(__pyx_k_spiSlave_Write), 0, 0, 1, 1},
    {&__pyx_n_s_spi_Reset, __pyx_k_spi_Reset, sizeof(__pyx_k_spi_Reset), 0, 0, 1, 1},
    {&__pyx_n_s_spi_ResetTransaction, __pyx_k_spi_ResetTransaction, sizeof(__pyx_k_spi_ResetTransaction), 0, 0, 1, 1},
    {&__pyx_n_s_spi_SetDrivingStrength, __pyx_k_spi_SetDrivingStrength, sizeof(__pyx_k_spi_SetDrivingStrength), 0, 0, 1, 1},
    {&__pyx_n_s_ssoMap, __pyx_k_ssoMap, sizeof(__pyx_k_ssoMap), 0, 0, 1, 1},
    {&__pyx_n_s_ssoStrength, __pyx_k_ssoStrength, sizeof(__pyx_k_ssoStrength), 0, 0, 1, 1},
    {&__pyx_n_s_start, __pyx_k_start, sizeof(__pyx_k_start), 0, 0, 1, 1},
    {&__pyx_n_s_status, __pyx_k_status, sizeof(__pyx_k_status), 0, 0, 1, 1},
    {&__pyx_n_s_str, __pyx_k_str, sizeof(__pyx_k_str), 0, 0, 1, 1},
    {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
    {&__pyx_n_s_super, __pyx_k_super, sizeof(__pyx_k_super), 0, 0, 1, 1},
    {&__pyx_n_s_t, __pyx_k_t, sizeof(__pyx_k_t), 0, 0, 1, 1},
    {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
    {&__pyx_kp_u_the_data_argument_must_be_of_typ, __pyx_k_the_data_argument_must_be_of_typ, sizeof(__pyx_k_the_data_argument_must_be_of_typ), 0, 1, 0, 0},
    {&__pyx_kp_u_the_multiWrite_argument_must_be, __pyx_k_the_multiWrite_argument_must_be, sizeof(__pyx_k_the_multiWrite_argument_must_be), 0, 1, 0, 0},
    {&__pyx_kp_u_the_singleWrite_argument_must_be, __pyx_k_the_singleWrite_argument_must_be, sizeof(__pyx_k_the_singleWrite_argument_must_be), 0, 1, 0, 0},
    {&__pyx_n_s_timeout, __pyx_k_timeout, sizeof(__pyx_k_timeout), 0, 0, 1, 1},
    {&__pyx_n_s_timeout_c, __pyx_k_timeout_c, sizeof(__pyx_k_timeout_c), 0, 0, 1, 1},
    {&__pyx_n_s_trigger, __pyx_k_trigger, sizeof(__pyx_k_trigger), 0, 0, 1, 1},
    {&__pyx_n_u_type, __pyx_k_type, sizeof(__pyx_k_type), 0, 1, 0, 1},
    {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
    {&__pyx_kp_u_utf_8, __pyx_k_utf_8, sizeof(__pyx_k_utf_8), 0, 1, 0, 0},
    {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
    {&__pyx_n_s_vendorCmdGet, __pyx_k_vendorCmdGet, sizeof(__pyx_k_vendorCmdGet), 0, 0, 1, 1},
    {&__pyx_n_s_vendorCmdSet, __pyx_k_vendorCmdSet, sizeof(__pyx_k_vendorCmdSet), 0, 0, 1, 1},
    {&__pyx_n_s_weakref, __pyx_k_weakref, sizeof(__pyx_k_weakref), 0, 0, 1, 1},
    {&__pyx_n_s_weakref_finalize, __pyx_k_weakref_finalize, sizeof(__pyx_k_weakref_finalize), 0, 0, 1, 1},
    {&__pyx_n_s_write, __pyx_k_write, sizeof(__pyx_k_write), 0, 0, 1, 1},
    {&__pyx_n_s_write_timeout, __pyx_k_write_timeout, sizeof(__pyx_k_write_timeout), 0, 0, 1, 1},
    {&__pyx_n_s_xrange, __pyx_k_xrange, sizeof(__pyx_k_xrange), 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0}
  };
  return __Pyx_InitStrings(__pyx_string_tab);
}
/* #### Code section: cached_builtins ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_n_s_super); if (!__pyx_builtin_super) __PYX_ERR(0, 73, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 2, __pyx_L1_error)
  __pyx_builtin_KeyError = __Pyx_GetBuiltinName(__pyx_n_s_KeyError); if (!__pyx_builtin_KeyError) __PYX_ERR(0, 256, __pyx_L1_error)
  #if PY_MAJOR_VERSION >= 3
  __pyx_builtin_xrange = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_xrange) __PYX_ERR(0, 431, __pyx_L1_error)
  #else
  __pyx_builtin_xrange = __Pyx_GetBuiltinName(__pyx_n_s_xrange); if (!__pyx_builtin_xrange) __PYX_ERR(0, 431, __pyx_L1_error)
  #endif
  __pyx_builtin_round = __Pyx_GetBuiltinName(__pyx_n_s_round); if (!__pyx_builtin_round) __PYX_ERR(0, 607, __pyx_L1_error)
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) __PYX_ERR(2, 120, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cached_constants ### */

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "ft4222/ft4222.pyx":365
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint16 bytesSent
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_u_the_data_argument_must_be_of_typ); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "ft4222/ft4222.pyx":919
 *             singleWrite = bytes([singleWrite])
 *         elif not isinstance(singleWrite, (bytes, bytearray)):
 *             raise TypeError("the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         if isinstance(multiWrite, int):
 *             multiWrite = bytes([multiWrite])
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_u_the_singleWrite_argument_must_be); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 919, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "ft4222/ft4222.pyx":923
 *             multiWrite = bytes([multiWrite])
 *         elif not isinstance(multiWrite, (bytes, bytearray)):
 *             raise TypeError("the multiWrite argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         write = singleWrite + multiWrite
 *         cdef:
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_u_the_multiWrite_argument_must_be); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 923, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "ft4222/ft4222.pyx":50
 *                  'NOT_SUPPORTED', 'OTHER_ERROR', 'DEVICE_LIST_NOT_READY']
 * 
 *     def __init__(self, msgnum):             # <<<<<<<<<<<<<<
 *         self.message = self.__ftd2xx_msgs[msgnum]
 * 
 */
  __pyx_tuple__9 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_msgnum); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);
  __pyx_codeobj__10 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__9, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_init, 50, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__10)) __PYX_ERR(0, 50, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":53
 *         self.message = self.__ftd2xx_msgs[msgnum]
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message
 * 
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);
  __pyx_codeobj__12 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__11, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_str, 53, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__12)) __PYX_ERR(0, 53, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":69
 *                   'GPIO_INPUT_NOT_SUPPORTED', 'EVENT_NOT_SUPPORTED', 'FUN_NOT_SUPPORT']
 * 
 *     def __init__(self, msgnum):             # <<<<<<<<<<<<<<
 *         if msgnum >= FT4222_DEVICE_NOT_SUPPORTED:
 *             self.message = self.__ftd4222_msgs[msgnum - FT4222_DEVICE_NOT_SUPPORTED]
 */
  __pyx_codeobj__13 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__9, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_init, 69, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__13)) __PYX_ERR(0, 69, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":75
 *             super(FT4222DeviceError, self).__init__(msgnum)
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message
 * 
 */
  __pyx_codeobj__14 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__11, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_str, 75, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__14)) __PYX_ERR(0, 75, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":93
 *     CLK_80 = 3
 * 
 * def createDeviceInfoList():             # <<<<<<<<<<<<<<
 *     """Create the internal device info list and return number of entries"""
 *     cdef DWORD nb
 */
  __pyx_tuple__15 = PyTuple_Pack(2, __pyx_n_s_nb, __pyx_n_s_status); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);
  __pyx_codeobj__16 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__15, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_createDeviceInfoList, 93, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__16)) __PYX_ERR(0, 93, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":101
 *     raise FT2XXDeviceError, status
 * 
 * def getDeviceInfoDetail(devnum=0, update=True):             # <<<<<<<<<<<<<<
 *     """Get an entry from the internal device info list. Set update to
 *     False to avoid a slow call to createDeviceInfoList."""
 */
  __pyx_tuple__17 = PyTuple_Pack(10, __pyx_n_s_devnum, __pyx_n_s_update, __pyx_n_s_f, __pyx_n_s_t, __pyx_n_s_i, __pyx_n_s_l, __pyx_n_s_h, __pyx_n_s_n, __pyx_n_s_d, __pyx_n_s_status); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);
  __pyx_codeobj__18 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 10, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__17, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_getDeviceInfoDetail, 101, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__18)) __PYX_ERR(0, 101, __pyx_L1_error)
  __pyx_tuple__19 = PyTuple_Pack(2, ((PyObject *)__pyx_int_0), ((PyObject *)Py_True)); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "ft4222/ft4222.pyx":118
 *     raise FT2XXDeviceError, status
 * 
 * def openBySerial(serial):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by serial number
 * 
 */
  __pyx_tuple__20 = PyTuple_Pack(4, __pyx_n_s_serial, __pyx_n_s_handle, __pyx_n_s_cserial, __pyx_n_s_status); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);
  __pyx_codeobj__21 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__20, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_openBySerial, 118, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__21)) __PYX_ERR(0, 118, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":140
 *     raise FT2XXDeviceError, status
 * 
 * def openByDescription(desc):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by description
 * 
 */
  __pyx_tuple__22 = PyTuple_Pack(4, __pyx_n_s_desc, __pyx_n_s_handle, __pyx_n_s_cdesc, __pyx_n_s_status); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);
  __pyx_codeobj__23 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__22, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_openByDescription, 140, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__23)) __PYX_ERR(0, 140, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":163
 *     raise FT2XXDeviceError, status
 * 
 * def openByLocation(locId):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by location
 * 
 */
  __pyx_tuple__24 = PyTuple_Pack(3, __pyx_n_s_locId, __pyx_n_s_handle, __pyx_n_s_status); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);
  __pyx_codeobj__25 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__24, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_openByLocation, 163, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__25)) __PYX_ERR(0, 163, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":196
 *         return closer
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         FT4222_UnInitialize(self._handle)
 *         FT_Close(self._handle)
 */
  __pyx_codeobj__26 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__11, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_close, 196, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__26)) __PYX_ERR(0, 196, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 */
  __pyx_codeobj__27 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__11, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_reduce_cython, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__27)) __PYX_ERR(1, 1, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 */
  __pyx_tuple__28 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_pyx_state); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);
  __pyx_codeobj__29 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__28, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_setstate_cython, 3, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__29)) __PYX_ERR(1, 3, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":226
 *         self._close = weakref_finalize(self, _closer.wrap(self._handle).close)
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """Closes the device."""
 *         FT4222_UnInitialize(self._handle)
 */
  __pyx_tuple__30 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_status); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);
  __pyx_codeobj__31 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__30, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_close, 226, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__31)) __PYX_ERR(0, 226, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":262
 *         return "FT4222: chipVersion: 0x{:x} ({:s}), libVersion: 0x{:x}".format(self._chip_version, self.chipRevision, self._dll_version)
 * 
 *     def setTimeouts(self, read_timeout, write_timeout):             # <<<<<<<<<<<<<<
 *         """Set the read and write timeouts
 * 
 */
  __pyx_tuple__32 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_read_timeout, __pyx_n_s_write_timeout, __pyx_n_s_status); if (unlikely(!__pyx_tuple__32)) __PYX_ERR(0, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);
  __pyx_codeobj__33 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__32, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_setTimeouts, 262, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__33)) __PYX_ERR(0, 262, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":277
 *             raise FT2XXDeviceError, status
 * 
 *     def setClock(self, clk):             # <<<<<<<<<<<<<<
 *         """Set the system clock
 * 
 */
  __pyx_tuple__34 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_clk, __pyx_n_s_status); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);
  __pyx_codeobj__35 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_setClock, 277, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__35)) __PYX_ERR(0, 277, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":291
 *             raise FT4222DeviceError, status
 * 
 *     def getClock(self):             # <<<<<<<<<<<<<<
 *         """Get the system clock
 * 
 */
  __pyx_codeobj__36 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_getClock, 291, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__36)) __PYX_ERR(0, 291, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":307
 *         raise FT4222DeviceError, status
 * 
 *     def setSuspendOut(self, enable):             # <<<<<<<<<<<<<<
 *         """Enable or disable, suspend out, which will emit a signal when FT4222H enters suspend mode.
 * 
 */
  __pyx_tuple__37 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_enable, __pyx_n_s_status); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);
  __pyx_codeobj__38 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__37, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_setSuspendOut, 307, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__38)) __PYX_ERR(0, 307, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":321
 *             raise FT4222DeviceError, status
 * 
 *     def setWakeUpInterrupt(self, enable):             # <<<<<<<<<<<<<<
 *         """Enable or disable the wakeup/interrupt
 * 
 */
  __pyx_codeobj__39 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__37, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_setWakeUpInterrupt, 321, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__39)) __PYX_ERR(0, 321, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":335
 *             raise FT4222DeviceError, status
 * 
 *     def getMaxTransferSize(self):             # <<<<<<<<<<<<<<
 *         """Get the maximum transfer size for the currently configured operating mode
 * 
 */
  __pyx_tuple__40 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_maxTransferSize, __pyx_n_s_status); if (unlikely(!__pyx_tuple__40)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__40);
  __Pyx_GIVEREF(__pyx_tuple__40);
  __pyx_codeobj__41 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__40, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_getMaxTransferSize, 335, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__41)) __PYX_ERR(0, 335, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":350
 *         raise FT4222DeviceError, status
 * 
 *     def vendorCmdGet(self, req, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Vendor get command"""
 *         cdef:
 */
  __pyx_tuple__42 = PyTuple_Pack(5, __pyx_n_s_self, __pyx_n_s_req, __pyx_n_s_bytesToRead, __pyx_n_s_buf, __pyx_n_s_status); if (unlikely(!__pyx_tuple__42)) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__42);
  __Pyx_GIVEREF(__pyx_tuple__42);
  __pyx_codeobj__43 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__42, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_vendorCmdGet, 350, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__43)) __PYX_ERR(0, 350, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":360
 *         raise FT4222DeviceError, status
 * 
 *     def vendorCmdSet(self, req, data):             # <<<<<<<<<<<<<<
 *         """Vendor set command"""
 *         if isinstance(data, int):
 */
  __pyx_tuple__44 = PyTuple_Pack(6, __pyx_n_s_self, __pyx_n_s_req, __pyx_n_s_data, __pyx_n_s_bytesSent, __pyx_n_s_cdata, __pyx_n_s_status); if (unlikely(!__pyx_tuple__44)) __PYX_ERR(0, 360, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__44);
  __Pyx_GIVEREF(__pyx_tuple__44);
  __pyx_codeobj__45 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__44, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_vendorCmdSet, 360, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__45)) __PYX_ERR(0, 360, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":373
 *             raise FT4222DeviceError, status
 * 
 *     def getLatencyTimer(self):             # <<<<<<<<<<<<<<
 *         """Get the current value of the latency timer.
 * 
 */
  __pyx_tuple__46 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_latency, __pyx_n_s_status); if (unlikely(!__pyx_tuple__46)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__46);
  __Pyx_GIVEREF(__pyx_tuple__46);
  __pyx_codeobj__47 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__46, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_getLatencyTimer, 373, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__47)) __PYX_ERR(0, 373, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":388
 *         raise FT2XXDeviceError, status
 * 
 *     def setLatencyTimer(self, latency):             # <<<<<<<<<<<<<<
 *         """Set the current value of the latency timer.
 * 
 */
  __pyx_codeobj__48 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__46, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_setLatencyTimer, 388, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__48)) __PYX_ERR(0, 388, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":401
 *             raise FT2XXDeviceError, status
 * 
 *     def chipReset(self):             # <<<<<<<<<<<<<<
 *         """Software reset for device.
 *         This function is used to attempt to recover system after a failure. It is a software reset for device.
 */
  __pyx_codeobj__49 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__30, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_chipReset, 401, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__49)) __PYX_ERR(0, 401, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":414
 *             raise FT2XXDeviceError, status
 * 
 *     def gpio_Init(self, *args, gpio0=Dir.INPUT, gpio1=Dir.INPUT, gpio2=Dir.INPUT, gpio3=Dir.INPUT):             # <<<<<<<<<<<<<<
 *         """Initialize the GPIO interface.
 * 
 */
  __pyx_tuple__50 = PyTuple_Pack(9, __pyx_n_s_self, __pyx_n_s_gpio0, __pyx_n_s_gpio1, __pyx_n_s_gpio2, __pyx_n_s_gpio3, __pyx_n_s_args, __pyx_n_s_ioDir, __pyx_n_s_i, __pyx_n_s_status); if (unlikely(!__pyx_tuple__50)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__50);
  __Pyx_GIVEREF(__pyx_tuple__50);
  __pyx_codeobj__51 = (PyObject*)__Pyx_PyCode_New(1, 0, 4, 9, 0, CO_OPTIMIZED|CO_NEWLOCALS|CO_VARARGS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__50, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_gpio_Init, 414, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__51)) __PYX_ERR(0, 414, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":444
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_Read(self, portNum):             # <<<<<<<<<<<<<<
 *         """Read value from selected GPIO
 * 
 */
  __pyx_tuple__52 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_portNum, __pyx_n_s_value, __pyx_n_s_status); if (unlikely(!__pyx_tuple__52)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__52);
  __Pyx_GIVEREF(__pyx_tuple__52);
  __pyx_codeobj__53 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__52, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_gpio_Read, 444, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__53)) __PYX_ERR(0, 444, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":464
 *         raise FT4222DeviceError, status
 * 
 *     def gpio_Wait(self, portNum, expectedValue, timeout=0, sleep=1):             # <<<<<<<<<<<<<<
 *         """Blocks until the selected GPIO reaches a certain value.
 * 
 */
  __pyx_tuple__54 = PyTuple_Pack(12, __pyx_n_s_self, __pyx_n_s_portNum, __pyx_n_s_expectedValue, __pyx_n_s_timeout, __pyx_n_s_sleep, __pyx_n_s_value, __pyx_n_s_status, __pyx_n_s_portNum_c, __pyx_n_s_expectedValue_c, __pyx_n_s_start, __pyx_n_s_timeout_c, __pyx_n_s_sleep_c); if (unlikely(!__pyx_tuple__54)) __PYX_ERR(0, 464, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__54);
  __Pyx_GIVEREF(__pyx_tuple__54);
  __pyx_codeobj__55 = (PyObject*)__Pyx_PyCode_New(5, 0, 0, 12, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__54, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_gpio_Wait, 464, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__55)) __PYX_ERR(0, 464, __pyx_L1_error)
  __pyx_tuple__56 = PyTuple_Pack(2, __pyx_int_0, __pyx_int_1); if (unlikely(!__pyx_tuple__56)) __PYX_ERR(0, 464, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__56);
  __Pyx_GIVEREF(__pyx_tuple__56);

  /* "ft4222/ft4222.pyx":504
 *                     msleep(sleep_c)
 * 
 *     def gpio_Write(self, portNum, value):             # <<<<<<<<<<<<<<
 *         """Write value to given GPIO
 * 
 */
  __pyx_codeobj__57 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__52, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_gpio_Write, 504, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__57)) __PYX_ERR(0, 504, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":519
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_SetInputTrigger(self, portNum, trigger):             # <<<<<<<<<<<<<<
 *         """Set software trigger conditions on the specified GPIO pin.
 * 
 */
  __pyx_tuple__58 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_portNum, __pyx_n_s_trigger, __pyx_n_s_status); if (unlikely(!__pyx_tuple__58)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__58);
  __Pyx_GIVEREF(__pyx_tuple__58);
  __pyx_codeobj__59 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__58, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_gpio_SetInputTrigger, 519, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__59)) __PYX_ERR(0, 519, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":534
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_GetTriggerStatus(self, portNum):             # <<<<<<<<<<<<<<
 *         """Get the size of trigger event queue.
 * 
 */
  __pyx_tuple__60 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_portNum, __pyx_n_s_queueSize, __pyx_n_s_status); if (unlikely(!__pyx_tuple__60)) __PYX_ERR(0, 534, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__60);
  __Pyx_GIVEREF(__pyx_tuple__60);
  __pyx_codeobj__61 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__60, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_gpio_GetTriggerStatus, 534, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__61)) __PYX_ERR(0, 534, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":554
 *         raise FT4222DeviceError, status
 * 
 *     def gpio_ReadTriggerQueue(self, portNum, readSize=None):             # <<<<<<<<<<<<<<
 *         """Get events recorded in the trigger event queue.
 * 
 */
  __pyx_tuple__62 = PyTuple_Pack(8, __pyx_n_s_self, __pyx_n_s_portNum, __pyx_n_s_readSize, __pyx_n_s_events, __pyx_n_s_sizeRead, __pyx_n_s_status, __pyx_n_s_res, __pyx_n_s_i); if (unlikely(!__pyx_tuple__62)) __PYX_ERR(0, 554, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__62);
  __Pyx_GIVEREF(__pyx_tuple__62);
  __pyx_codeobj__63 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 8, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__62, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_gpio_ReadTriggerQueue, 554, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__63)) __PYX_ERR(0, 554, __pyx_L1_error)
  __pyx_tuple__64 = PyTuple_Pack(1, Py_None); if (unlikely(!__pyx_tuple__64)) __PYX_ERR(0, 554, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__64);
  __Pyx_GIVEREF(__pyx_tuple__64);

  /* "ft4222/ft4222.pyx":582
 * 
 * 
 *     def i2cMaster_Init(self, kbps=100):             # <<<<<<<<<<<<<<
 *         """Initialize the FT4222H as an I2C master with the requested I2C speed.
 * 
 */
  __pyx_tuple__65 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_kbps, __pyx_n_s_status, __pyx_n_s_n); if (unlikely(!__pyx_tuple__65)) __PYX_ERR(0, 582, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__65);
  __Pyx_GIVEREF(__pyx_tuple__65);
  __pyx_codeobj__66 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__65, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_i2cMaster_Init, 582, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__66)) __PYX_ERR(0, 582, __pyx_L1_error)
  __pyx_tuple__67 = PyTuple_Pack(1, __pyx_int_100); if (unlikely(!__pyx_tuple__67)) __PYX_ERR(0, 582, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__67);
  __Pyx_GIVEREF(__pyx_tuple__67);

  /* "ft4222/ft4222.pyx":611
 *             self.vendorCmdSet(0x52, n)
 * 
 *     def i2cMaster_Read(self, addr, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the specified I2C slave device with START and STOP conditions.
 * 
 */
  __pyx_tuple__68 = PyTuple_Pack(6, __pyx_n_s_self, __pyx_n_s_addr, __pyx_n_s_bytesToRead, __pyx_n_s_buf, __pyx_n_s_bytesRead, __pyx_n_s_status); if (unlikely(!__pyx_tuple__68)) __PYX_ERR(0, 611, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__68);
  __Pyx_GIVEREF(__pyx_tuple__68);
  __pyx_codeobj__69 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__68, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_i2cMaster_Read, 611, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__69)) __PYX_ERR(0, 611, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":635
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_Write(self, addr, data):             # <<<<<<<<<<<<<<
 *         """Write data to the specified I2C slave device with START and STOP conditions.
 * 
 */
  __pyx_tuple__70 = PyTuple_Pack(6, __pyx_n_s_self, __pyx_n_s_addr, __pyx_n_s_data, __pyx_n_s_bytesSent, __pyx_n_s_cdata, __pyx_n_s_status); if (unlikely(!__pyx_tuple__70)) __PYX_ERR(0, 635, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__70);
  __Pyx_GIVEREF(__pyx_tuple__70);
  __pyx_codeobj__71 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__70, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_i2cMaster_Write, 635, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__71)) __PYX_ERR(0, 635, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":661
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_ReadEx(self, addr, flag, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the specified I2C slave device with the specified I2C condition.
 * 
 */
  __pyx_tuple__72 = PyTuple_Pack(7, __pyx_n_s_self, __pyx_n_s_addr, __pyx_n_s_flag, __pyx_n_s_bytesToRead, __pyx_n_s_buf, __pyx_n_s_bytesRead, __pyx_n_s_status); if (unlikely(!__pyx_tuple__72)) __PYX_ERR(0, 661, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__72);
  __Pyx_GIVEREF(__pyx_tuple__72);
  __pyx_codeobj__73 = (PyObject*)__Pyx_PyCode_New(4, 0, 0, 7, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__72, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_i2cMaster_ReadEx, 661, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__73)) __PYX_ERR(0, 661, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":686
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_WriteEx(self, addr, flag, data):             # <<<<<<<<<<<<<<
 *         """Write data to the specified I2C slave device with the specified I2C condition.
 * 
 */
  __pyx_tuple__74 = PyTuple_Pack(7, __pyx_n_s_self, __pyx_n_s_addr, __pyx_n_s_flag, __pyx_n_s_data, __pyx_n_s_bytesSent, __pyx_n_s_cdata, __pyx_n_s_status); if (unlikely(!__pyx_tuple__74)) __PYX_ERR(0, 686, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__74);
  __Pyx_GIVEREF(__pyx_tuple__74);
  __pyx_codeobj__75 = (PyObject*)__Pyx_PyCode_New(4, 0, 0, 7, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__74, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_i2cMaster_WriteEx, 686, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__75)) __PYX_ERR(0, 686, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":713
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_Reset(self):             # <<<<<<<<<<<<<<
 *         """Reset the I2C master device.
 * 
 */
  __pyx_codeobj__76 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__30, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_i2cMaster_Reset, 713, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__76)) __PYX_ERR(0, 713, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":727
 *             raise FT4222DeviceError, status
 * 
 *     def i2cMaster_GetStatus(self):             # <<<<<<<<<<<<<<
 *         """Read the status of the I2C master controller.
 * 
 */
  __pyx_tuple__77 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_cs, __pyx_n_s_status); if (unlikely(!__pyx_tuple__77)) __PYX_ERR(0, 727, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__77);
  __Pyx_GIVEREF(__pyx_tuple__77);
  __pyx_codeobj__78 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__77, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_i2cMaster_GetStatus, 727, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__78)) __PYX_ERR(0, 727, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":746
 * 
 * 
 *     def spi_Reset(self):             # <<<<<<<<<<<<<<
 *         """Reset the SPI master or slave device
 * 
 */
  __pyx_codeobj__79 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__30, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_spi_Reset, 746, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__79)) __PYX_ERR(0, 746, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":757
 *             raise FT4222DeviceError, status
 * 
 *     def spi_ResetTransaction(self, spiIdx):             # <<<<<<<<<<<<<<
 *         """Reset the SPI transaction
 * 
 */
  __pyx_tuple__80 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_spiIdx, __pyx_n_s_status); if (unlikely(!__pyx_tuple__80)) __PYX_ERR(0, 757, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__80);
  __Pyx_GIVEREF(__pyx_tuple__80);
  __pyx_codeobj__81 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__80, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_spi_ResetTransaction, 757, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__81)) __PYX_ERR(0, 757, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":771
 *             raise FT4222DeviceError, status
 * 
 *     def spi_SetDrivingStrength(self, clkStrength, ioStrength, ssoStrength):             # <<<<<<<<<<<<<<
 *         """Reset the SPI master or slave device.
 * 
 */
  __pyx_tuple__82 = PyTuple_Pack(5, __pyx_n_s_self, __pyx_n_s_clkStrength, __pyx_n_s_ioStrength, __pyx_n_s_ssoStrength, __pyx_n_s_status); if (unlikely(!__pyx_tuple__82)) __PYX_ERR(0, 771, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__82);
  __Pyx_GIVEREF(__pyx_tuple__82);
  __pyx_codeobj__83 = (PyObject*)__Pyx_PyCode_New(4, 0, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__82, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_spi_SetDrivingStrength, 771, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__83)) __PYX_ERR(0, 771, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":787
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_Init(self, mode, clock, cpol, cpha, ssoMap):             # <<<<<<<<<<<<<<
 *         """Initialize as an SPI master under all modes.
 * 
 */
  __pyx_tuple__84 = PyTuple_Pack(7, __pyx_n_s_self, __pyx_n_s_mode, __pyx_n_s_clock, __pyx_n_s_cpol, __pyx_n_s_cpha, __pyx_n_s_ssoMap, __pyx_n_s_status); if (unlikely(!__pyx_tuple__84)) __PYX_ERR(0, 787, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__84);
  __Pyx_GIVEREF(__pyx_tuple__84);
  __pyx_codeobj__85 = (PyObject*)__Pyx_PyCode_New(6, 0, 0, 7, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__84, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_spiMaster_Init, 787, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__85)) __PYX_ERR(0, 787, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":805
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_SetLines(self, mode):             # <<<<<<<<<<<<<<
 *         """Switch the FT4222H SPI master to single, dual, or quad mode.
 * 
 */
  __pyx_tuple__86 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_mode, __pyx_n_s_status); if (unlikely(!__pyx_tuple__86)) __PYX_ERR(0, 805, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__86);
  __Pyx_GIVEREF(__pyx_tuple__86);
  __pyx_codeobj__87 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__86, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_spiMaster_SetLines, 805, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__87)) __PYX_ERR(0, 805, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":822
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleRead(self, bytesToRead, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Read data from a SPI slave in single mode
 * 
 */
  __pyx_tuple__88 = PyTuple_Pack(6, __pyx_n_s_self, __pyx_n_s_bytesToRead, __pyx_n_s_isEndTransaction, __pyx_n_s_buf, __pyx_n_s_bytesRead, __pyx_n_s_status); if (unlikely(!__pyx_tuple__88)) __PYX_ERR(0, 822, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__88);
  __Pyx_GIVEREF(__pyx_tuple__88);
  __pyx_codeobj__89 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__88, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_spiMaster_SingleRead, 822, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__89)) __PYX_ERR(0, 822, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":846
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleWrite(self, data, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Write data to a SPI slave in single mode
 * 
 */
  __pyx_tuple__90 = PyTuple_Pack(6, __pyx_n_s_self, __pyx_n_s_data, __pyx_n_s_isEndTransaction, __pyx_n_s_bytesSent, __pyx_n_s_cdata, __pyx_n_s_status); if (unlikely(!__pyx_tuple__90)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__90);
  __Pyx_GIVEREF(__pyx_tuple__90);
  __pyx_codeobj__91 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__90, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_spiMaster_SingleWrite, 846, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__91)) __PYX_ERR(0, 846, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":872
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleReadWrite(self, data, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Write and read data to and from a SPI slave in single mode
 * 
 */
  __pyx_tuple__92 = PyTuple_Pack(7, __pyx_n_s_self, __pyx_n_s_data, __pyx_n_s_isEndTransaction, __pyx_n_s_sizeTransferred, __pyx_n_s_cdata, __pyx_n_s_buf, __pyx_n_s_status); if (unlikely(!__pyx_tuple__92)) __PYX_ERR(0, 872, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__92);
  __Pyx_GIVEREF(__pyx_tuple__92);
  __pyx_codeobj__93 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 7, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__92, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_spiMaster_SingleReadWrite, 872, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__93)) __PYX_ERR(0, 872, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":901
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_MultiReadWrite(self, singleWrite, multiWrite, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Write and read data to and from a SPI slave in dual- or quad-mode (multi-mode).
 * 
 */
  __pyx_tuple__94 = PyTuple_Pack(9, __pyx_n_s_self, __pyx_n_s_singleWrite, __pyx_n_s_multiWrite, __pyx_n_s_bytesToRead, __pyx_n_s_write, __pyx_n_s_cdata, __pyx_n_s_buf, __pyx_n_s_bytesRead, __pyx_n_s_status); if (unlikely(!__pyx_tuple__94)) __PYX_ERR(0, 901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__94);
  __Pyx_GIVEREF(__pyx_tuple__94);
  __pyx_codeobj__95 = (PyObject*)__Pyx_PyCode_New(4, 0, 0, 9, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__94, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_spiMaster_MultiReadWrite, 901, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__95)) __PYX_ERR(0, 901, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":936
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_EndTransaction(self):             # <<<<<<<<<<<<<<
 *         """End the current SPI transaction.
 * 
 */
  __pyx_tuple__96 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_bytesSent, __pyx_n_s_status); if (unlikely(!__pyx_tuple__96)) __PYX_ERR(0, 936, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__96);
  __Pyx_GIVEREF(__pyx_tuple__96);
  __pyx_codeobj__97 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__96, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_spiMaster_EndTransaction, 936, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__97)) __PYX_ERR(0, 936, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":950
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_Init(self):             # <<<<<<<<<<<<<<
 *         """Initialize the FT4222H as an SPI slave. Default SPI_SlaveProtocol is SPI_SLAVE_WITH_PROTOCOL.
 * 
 */
  __pyx_codeobj__98 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__30, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_spiSlave_Init, 950, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__98)) __PYX_ERR(0, 950, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":961
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_InitEx(self, proto):             # <<<<<<<<<<<<<<
 *         """Initialize as an SPI slave under all modes.
 * 
 */
  __pyx_tuple__99 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_proto, __pyx_n_s_status); if (unlikely(!__pyx_tuple__99)) __PYX_ERR(0, 961, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__99);
  __Pyx_GIVEREF(__pyx_tuple__99);
  __pyx_codeobj__100 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__99, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_spiSlave_InitEx, 961, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__100)) __PYX_ERR(0, 961, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":975
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_Read(self, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the receive queue of the SPI slave device.
 * 
 */
  __pyx_tuple__101 = PyTuple_Pack(5, __pyx_n_s_self, __pyx_n_s_bytesToRead, __pyx_n_s_buf, __pyx_n_s_sizeRead, __pyx_n_s_status); if (unlikely(!__pyx_tuple__101)) __PYX_ERR(0, 975, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__101);
  __Pyx_GIVEREF(__pyx_tuple__101);
  __pyx_codeobj__102 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__101, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_spiSlave_Read, 975, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__102)) __PYX_ERR(0, 975, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":999
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_SetMode(self, cpol, cpha):             # <<<<<<<<<<<<<<
 *         """Set SPI slave cpol and cpha. The Default value of cpol is (:obj:`ft4222.SPI.Cpol.CLK_IDLE_LOW`) , default value of cpha is (:obj:`ft4222.SPI.Cpol.CLK_LEADING`)
 * 
 */
  __pyx_tuple__103 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_cpol, __pyx_n_s_cpha, __pyx_n_s_status); if (unlikely(!__pyx_tuple__103)) __PYX_ERR(0, 999, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__103);
  __Pyx_GIVEREF(__pyx_tuple__103);
  __pyx_codeobj__104 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__103, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_spiSlave_SetMode, 999, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__104)) __PYX_ERR(0, 999, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":1014
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_GetRxStatus(self):             # <<<<<<<<<<<<<<
 *         """Get number of bytes in the receive queue.
 * 
 */
  __pyx_tuple__105 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_pRxSize, __pyx_n_s_status); if (unlikely(!__pyx_tuple__105)) __PYX_ERR(0, 1014, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__105);
  __Pyx_GIVEREF(__pyx_tuple__105);
  __pyx_codeobj__106 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__105, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_spiSlave_GetRxStatus, 1014, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__106)) __PYX_ERR(0, 1014, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":1033
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_Write(self, data):             # <<<<<<<<<<<<<<
 *         """Write data to the transmit queue of the SPI slave device.
 * 
 */
  __pyx_tuple__107 = PyTuple_Pack(5, __pyx_n_s_self, __pyx_n_s_data, __pyx_n_s_sizeTransferred, __pyx_n_s_cdata, __pyx_n_s_status); if (unlikely(!__pyx_tuple__107)) __PYX_ERR(0, 1033, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__107);
  __Pyx_GIVEREF(__pyx_tuple__107);
  __pyx_codeobj__108 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__107, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_spiSlave_Write, 1033, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__108)) __PYX_ERR(0, 1033, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 */
  __pyx_codeobj__109 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__11, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_reduce_cython, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__109)) __PYX_ERR(1, 1, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 */
  __pyx_codeobj__110 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__28, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_setstate_cython, 3, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__110)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_constants ### */

static CYTHON_SMALL_CODE int __Pyx_InitConstants(void) {
  if (__Pyx_CreateStringTabAndInitStrings() < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_24000000_0 = PyFloat_FromDouble(24000000.0); if (unlikely(!__pyx_float_24000000_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_60 = PyInt_FromLong(60); if (unlikely(!__pyx_int_60)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_82 = PyInt_FromLong(82); if (unlikely(!__pyx_int_82)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_100 = PyInt_FromLong(100); if (unlikely(!__pyx_int_100)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_8000 = PyInt_FromLong(8000); if (unlikely(!__pyx_int_8000)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_16908288 = PyInt_FromLong(16908288L); if (unlikely(!__pyx_int_16908288)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_16973824 = PyInt_FromLong(16973824L); if (unlikely(!__pyx_int_16973824)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_17039360 = PyInt_FromLong(17039360L); if (unlikely(!__pyx_int_17039360)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1109524736 = PyInt_FromLong(1109524736L); if (unlikely(!__pyx_int_1109524736)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1109524992 = PyInt_FromLong(1109524992L); if (unlikely(!__pyx_int_1109524992)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1109525248 = PyInt_FromLong(1109525248L); if (unlikely(!__pyx_int_1109525248)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1109525504 = PyInt_FromLong(1109525504L); if (unlikely(!__pyx_int_1109525504)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_globals ### */

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  return 0;
}
/* #### Code section: init_module ### */

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_6ft4222_6ft4222__closer = &__pyx_vtable_6ft4222_6ft4222__closer;
  __pyx_vtable_6ft4222_6ft4222__closer.wrap = (PyObject *(*)(FT_HANDLE))__pyx_f_6ft4222_6ft4222_7_closer_wrap;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_ptype_6ft4222_6ft4222__closer = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6ft4222_6ft4222__closer_spec, NULL); if (unlikely(!__pyx_ptype_6ft4222_6ft4222__closer)) __PYX_ERR(0, 183, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6ft4222_6ft4222__closer_spec, __pyx_ptype_6ft4222_6ft4222__closer) < 0) __PYX_ERR(0, 183, __pyx_L1_error)
  #else
  __pyx_ptype_6ft4222_6ft4222__closer = &__pyx_type_6ft4222_6ft4222__closer;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_ptype_6ft4222_6ft4222__closer) < 0) __PYX_ERR(0, 183, __pyx_L1_error)
  #endif
  #if PY_MAJOR_VERSION < 3
  __pyx_ptype_6ft4222_6ft4222__closer->tp_print = 0;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_ptype_6ft4222_6ft4222__closer->tp_dictoffset && __pyx_ptype_6ft4222_6ft4222__closer->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_ptype_6ft4222_6ft4222__closer->tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_ptype_6ft4222_6ft4222__closer, __pyx_vtabptr_6ft4222_6ft4222__closer) < 0) __PYX_ERR(0, 183, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_MergeVtables(__pyx_ptype_6ft4222_6ft4222__closer) < 0) __PYX_ERR(0, 183, __pyx_L1_error)
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_closer, (PyObject *) __pyx_ptype_6ft4222_6ft4222__closer) < 0) __PYX_ERR(0, 183, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_setup_reduce((PyObject *) __pyx_ptype_6ft4222_6ft4222__closer) < 0) __PYX_ERR(0, 183, __pyx_L1_error)
  #endif
  __pyx_vtabptr_6ft4222_6ft4222_FT4222 = &__pyx_vtable_6ft4222_6ft4222_FT4222;
  __pyx_vtable_6ft4222_6ft4222_FT4222._get_version = (PyObject *(*)(struct __pyx_obj_6ft4222_6ft4222_FT4222 *))__pyx_f_6ft4222_6ft4222_6FT4222__get_version;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_ptype_6ft4222_6ft4222_FT4222 = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6ft4222_6ft4222_FT4222_spec, NULL); if (unlikely(!__pyx_ptype_6ft4222_6ft4222_FT4222)) __PYX_ERR(0, 201, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6ft4222_6ft4222_FT4222_spec, __pyx_ptype_6ft4222_6ft4222_FT4222) < 0) __PYX_ERR(0, 201, __pyx_L1_error)
  #else
  __pyx_ptype_6ft4222_6ft4222_FT4222 = &__pyx_type_6ft4222_6ft4222_FT4222;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_ptype_6ft4222_6ft4222_FT4222) < 0) __PYX_ERR(0, 201, __pyx_L1_error)
  #endif
  #if PY_MAJOR_VERSION < 3
  __pyx_ptype_6ft4222_6ft4222_FT4222->tp_print = 0;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_ptype_6ft4222_6ft4222_FT4222->tp_dictoffset && __pyx_ptype_6ft4222_6ft4222_FT4222->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_ptype_6ft4222_6ft4222_FT4222->tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_vtabptr_6ft4222_6ft4222_FT4222) < 0) __PYX_ERR(0, 201, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_MergeVtables(__pyx_ptype_6ft4222_6ft4222_FT4222) < 0) __PYX_ERR(0, 201, __pyx_L1_error)
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_FT4222, (PyObject *) __pyx_ptype_6ft4222_6ft4222_FT4222) < 0) __PYX_ERR(0, 201, __pyx_L1_error)
  if (__pyx_ptype_6ft4222_6ft4222_FT4222->tp_weaklistoffset == 0) __pyx_ptype_6ft4222_6ft4222_FT4222->tp_weaklistoffset = offsetof(struct __pyx_obj_6ft4222_6ft4222_FT4222, __weakref__);
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_setup_reduce((PyObject *) __pyx_ptype_6ft4222_6ft4222_FT4222) < 0) __PYX_ERR(0, 201, __pyx_L1_error)
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType_3_0_5(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_5(PyTypeObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_5(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_5(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_0_5); if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("array"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_5array_array = __Pyx_ImportType_3_0_5(__pyx_t_1, "array", "array", sizeof(arrayobject), __PYX_GET_STRUCT_ALIGNMENT_3_0_5(arrayobject),__Pyx_ImportType_CheckSize_Warn_3_0_5); if (!__pyx_ptype_7cpython_5array_array) __PYX_ERR(2, 69, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_ft4222(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_ft4222},
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "ft4222",
      0, /* m_doc */
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      0, /* m_size */
    #elif CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstate), /* m_size */
    #else
      -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif
#endif

#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initft4222(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initft4222(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_ft4222(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_ft4222(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *module, const char* from_name, const char* to_name, int allow_none)
#else
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
#endif
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
#if CYTHON_COMPILING_IN_LIMITED_API
            result = PyModule_AddObject(module, to_name, value);
#else
            result = PyDict_SetItemString(moddict, to_name, value);
#endif
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    moddict = module;
#else
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
#endif
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_ft4222(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  int stringtab_initialized = 0;
  #if CYTHON_USE_MODULE_STATE
  int pystate_addmodule_run = 0;
  #endif
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'ft4222' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("ft4222", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #elif CYTHON_USE_MODULE_STATE
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  {
    int add_module_result = PyState_AddModule(__pyx_t_1, &__pyx_moduledef);
    __pyx_t_1 = 0; /* transfer ownership from __pyx_t_1 to ft4222 pseudovariable */
    if (unlikely((add_module_result < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    pystate_addmodule_run = 1;
  }
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #endif
  CYTHON_UNUSED_VAR(__pyx_t_1);
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = __Pyx_PyImport_AddModuleRef(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = __Pyx_PyImport_AddModuleRef((const char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_ft4222(void)", 0);
  if (__Pyx_check_binary_version(__PYX_LIMITED_VERSION_HEX, __Pyx_get_runtime_version(), CYTHON_COMPILING_IN_LIMITED_API) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  stringtab_initialized = 1;
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_ft4222__ft4222) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "ft4222.ft4222")) {
      if (unlikely((PyDict_SetItemString(modules, "ft4222.ft4222", __pyx_m) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely((__Pyx_modinit_type_init_code() < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_type_import_code() < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "ft4222/ft4222.pyx":16
 * from libc.stdio cimport printf
 * from libc.time cimport clock, clock_t, CLOCKS_PER_SEC
 * from enum import IntEnum             # <<<<<<<<<<<<<<
 * from .GPIO import Dir, Trigger
 * from .I2CMaster import ControllerStatus
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_IntEnum);
  __Pyx_GIVEREF(__pyx_n_s_IntEnum);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_IntEnum)) __PYX_ERR(0, 16, __pyx_L1_error);
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_enum, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_IntEnum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_IntEnum, __pyx_t_2) < 0) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ft4222/ft4222.pyx":17
 * from libc.time cimport clock, clock_t, CLOCKS_PER_SEC
 * from enum import IntEnum
 * from .GPIO import Dir, Trigger             # <<<<<<<<<<<<<<
 * from .I2CMaster import ControllerStatus
 * from weakref import finalize as weakref_finalize
 */
  __pyx_t_3 = PyList_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_s_Dir);
  __Pyx_GIVEREF(__pyx_n_s_Dir);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 0, __pyx_n_s_Dir)) __PYX_ERR(0, 17, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_s_Trigger);
  __Pyx_GIVEREF(__pyx_n_s_Trigger);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 1, __pyx_n_s_Trigger)) __PYX_ERR(0, 17, __pyx_L1_error);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_GPIO, __pyx_t_3, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Dir); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Dir, __pyx_t_3) < 0) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Trigger); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Trigger, __pyx_t_3) < 0) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":18
 * from enum import IntEnum
 * from .GPIO import Dir, Trigger
 * from .I2CMaster import ControllerStatus             # <<<<<<<<<<<<<<
 * from weakref import finalize as weakref_finalize
 * 
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_ControllerStatus);
  __Pyx_GIVEREF(__pyx_n_s_ControllerStatus);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_ControllerStatus)) __PYX_ERR(0, 18, __pyx_L1_error);
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_I2CMaster, __pyx_t_2, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_ControllerStatus); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ControllerStatus, __pyx_t_2) < 0) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ft4222/ft4222.pyx":19
 * from .GPIO import Dir, Trigger
 * from .I2CMaster import ControllerStatus
 * from weakref import finalize as weakref_finalize             # <<<<<<<<<<<<<<
 * 
 * cdef extern from *:
 */
  __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_s_finalize);
  __Pyx_GIVEREF(__pyx_n_s_finalize);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 0, __pyx_n_s_finalize)) __PYX_ERR(0, 19, __pyx_L1_error);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_weakref, __pyx_t_3, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_finalize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_weakref_finalize, __pyx_t_3) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":39
 * 
 * 
 * class FT2XXDeviceError(Exception):             # <<<<<<<<<<<<<<
 *     """Exception class for status messages"""
 * 
 */
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0]));
  __Pyx_GIVEREF((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0]));
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])))) __PYX_ERR(0, 39, __pyx_L1_error);
  __pyx_t_3 = __Pyx_PEP560_update_bases(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_CalculateMetaclass(NULL, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_Py3MetaclassPrepare(__pyx_t_4, __pyx_t_3, __pyx_n_s_FT2XXDeviceError, __pyx_n_s_FT2XXDeviceError, (PyObject *) NULL, __pyx_n_s_ft4222_ft4222, __pyx_kp_s_Exception_class_for_status_messa); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (__pyx_t_3 != __pyx_t_2) {
    if (unlikely((PyDict_SetItemString(__pyx_t_5, "__orig_bases__", __pyx_t_2) < 0))) __PYX_ERR(0, 39, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":42
 *     """Exception class for status messages"""
 * 
 *     __ftd2xx_msgs = ['OK', 'INVALID_HANDLE', 'DEVICE_NOT_FOUND', 'DEVICE_NOT_OPENED',             # <<<<<<<<<<<<<<
 *                  'IO_ERROR', 'INSUFFICIENT_RESOURCES', 'INVALID_PARAMETER',
 *                  'INVALID_BAUD_RATE', 'DEVICE_NOT_OPENED_FOR_ERASE',
 */
  __pyx_t_2 = PyList_New(20); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_OK);
  __Pyx_GIVEREF(__pyx_n_u_OK);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_OK)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_INVALID_HANDLE);
  __Pyx_GIVEREF(__pyx_n_u_INVALID_HANDLE);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_INVALID_HANDLE)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_DEVICE_NOT_FOUND);
  __Pyx_GIVEREF(__pyx_n_u_DEVICE_NOT_FOUND);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_DEVICE_NOT_FOUND)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_DEVICE_NOT_OPENED);
  __Pyx_GIVEREF(__pyx_n_u_DEVICE_NOT_OPENED);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_DEVICE_NOT_OPENED)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_IO_ERROR);
  __Pyx_GIVEREF(__pyx_n_u_IO_ERROR);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_u_IO_ERROR)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_INSUFFICIENT_RESOURCES);
  __Pyx_GIVEREF(__pyx_n_u_INSUFFICIENT_RESOURCES);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 5, __pyx_n_u_INSUFFICIENT_RESOURCES)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_INVALID_PARAMETER);
  __Pyx_GIVEREF(__pyx_n_u_INVALID_PARAMETER);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 6, __pyx_n_u_INVALID_PARAMETER)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_INVALID_BAUD_RATE);
  __Pyx_GIVEREF(__pyx_n_u_INVALID_BAUD_RATE);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 7, __pyx_n_u_INVALID_BAUD_RATE)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_DEVICE_NOT_OPENED_FOR_ERASE);
  __Pyx_GIVEREF(__pyx_n_u_DEVICE_NOT_OPENED_FOR_ERASE);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 8, __pyx_n_u_DEVICE_NOT_OPENED_FOR_ERASE)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_DEVICE_NOT_OPENED_FOR_WRITE);
  __Pyx_GIVEREF(__pyx_n_u_DEVICE_NOT_OPENED_FOR_WRITE);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 9, __pyx_n_u_DEVICE_NOT_OPENED_FOR_WRITE)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_FAILED_TO_WRITE_DEVICE0);
  __Pyx_GIVEREF(__pyx_n_u_FAILED_TO_WRITE_DEVICE0);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 10, __pyx_n_u_FAILED_TO_WRITE_DEVICE0)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_EEPROM_READ_FAILED);
  __Pyx_GIVEREF(__pyx_n_u_EEPROM_READ_FAILED);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 11, __pyx_n_u_EEPROM_READ_FAILED)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_EEPROM_WRITE_FAILED);
  __Pyx_GIVEREF(__pyx_n_u_EEPROM_WRITE_FAILED);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 12, __pyx_n_u_EEPROM_WRITE_FAILED)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_EEPROM_ERASE_FAILED);
  __Pyx_GIVEREF(__pyx_n_u_EEPROM_ERASE_FAILED);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 13, __pyx_n_u_EEPROM_ERASE_FAILED)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_EEPROM_NOT_PRESENT);
  __Pyx_GIVEREF(__pyx_n_u_EEPROM_NOT_PRESENT);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 14, __pyx_n_u_EEPROM_NOT_PRESENT)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_EEPROM_NOT_PROGRAMMED);
  __Pyx_GIVEREF(__pyx_n_u_EEPROM_NOT_PROGRAMMED);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 15, __pyx_n_u_EEPROM_NOT_PROGRAMMED)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_INVALID_ARGS);
  __Pyx_GIVEREF(__pyx_n_u_INVALID_ARGS);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 16, __pyx_n_u_INVALID_ARGS)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_NOT_SUPPORTED);
  __Pyx_GIVEREF(__pyx_n_u_NOT_SUPPORTED);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 17, __pyx_n_u_NOT_SUPPORTED)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_OTHER_ERROR);
  __Pyx_GIVEREF(__pyx_n_u_OTHER_ERROR);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 18, __pyx_n_u_OTHER_ERROR)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_DEVICE_LIST_NOT_READY);
  __Pyx_GIVEREF(__pyx_n_u_DEVICE_LIST_NOT_READY);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 19, __pyx_n_u_DEVICE_LIST_NOT_READY)) __PYX_ERR(0, 42, __pyx_L1_error);
  if (__Pyx_SetNameInClass(__pyx_t_5, __pyx_n_s_FT2XXDeviceError__ftd2xx_msgs, __pyx_t_2) < 0) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":50
 *                  'NOT_SUPPORTED', 'OTHER_ERROR', 'DEVICE_LIST_NOT_READY']
 * 
 *     def __init__(self, msgnum):             # <<<<<<<<<<<<<<
 *         self.message = self.__ftd2xx_msgs[msgnum]
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_16FT2XXDeviceError_1__init__, 0, __pyx_n_s_FT2XXDeviceError___init, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__10)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetNameInClass(__pyx_t_5, __pyx_n_s_init, __pyx_t_2) < 0) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":53
 *         self.message = self.__ftd2xx_msgs[msgnum]
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_16FT2XXDeviceError_3__str__, 0, __pyx_n_s_FT2XXDeviceError___str, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__12)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetNameInClass(__pyx_t_5, __pyx_n_s_str, __pyx_t_2) < 0) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":39
 * 
 * 
 * class FT2XXDeviceError(Exception):             # <<<<<<<<<<<<<<
 *     """Exception class for status messages"""
 * 
 */
  __pyx_t_2 = __Pyx_Py3ClassCreate(__pyx_t_4, __pyx_n_s_FT2XXDeviceError, __pyx_t_3, __pyx_t_5, NULL, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_FT2XXDeviceError, __pyx_t_2) < 0) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ft4222/ft4222.pyx":56
 *         return self.message
 * 
 * class FT4222DeviceError(FT2XXDeviceError):             # <<<<<<<<<<<<<<
 *     """Exception class for status messages"""
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT2XXDeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3)) __PYX_ERR(0, 56, __pyx_L1_error);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PEP560_update_bases(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_CalculateMetaclass(NULL, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_Py3MetaclassPrepare(__pyx_t_5, __pyx_t_3, __pyx_n_s_FT4222DeviceError, __pyx_n_s_FT4222DeviceError, (PyObject *) NULL, __pyx_n_s_ft4222_ft4222, __pyx_kp_s_Exception_class_for_status_messa); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__pyx_t_3 != __pyx_t_4) {
    if (unlikely((PyDict_SetItemString(__pyx_t_2, "__orig_bases__", __pyx_t_4) < 0))) __PYX_ERR(0, 56, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ft4222/ft4222.pyx":59
 *     """Exception class for status messages"""
 * 
 *     __ftd4222_msgs = ['DEVICE_NOT_SUPPORTED', 'CLK_NOT_SUPPORTED','VENDER_CMD_NOT_SUPPORTED',             # <<<<<<<<<<<<<<
 *                   'IS_NOT_SPI_MODE', 'IS_NOT_I2C_MODE', 'IS_NOT_SPI_SINGLE_MODE',
 *                   'IS_NOT_SPI_MULTI_MODE', 'WRONG_I2C_ADDR', 'INVAILD_FUNCTION',
 */
  __pyx_t_4 = PyList_New(23); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_n_u_DEVICE_NOT_SUPPORTED);
  __Pyx_GIVEREF(__pyx_n_u_DEVICE_NOT_SUPPORTED);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 0, __pyx_n_u_DEVICE_NOT_SUPPORTED)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_CLK_NOT_SUPPORTED);
  __Pyx_GIVEREF(__pyx_n_u_CLK_NOT_SUPPORTED);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 1, __pyx_n_u_CLK_NOT_SUPPORTED)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_VENDER_CMD_NOT_SUPPORTED);
  __Pyx_GIVEREF(__pyx_n_u_VENDER_CMD_NOT_SUPPORTED);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 2, __pyx_n_u_VENDER_CMD_NOT_SUPPORTED)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_IS_NOT_SPI_MODE);
  __Pyx_GIVEREF(__pyx_n_u_IS_NOT_SPI_MODE);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 3, __pyx_n_u_IS_NOT_SPI_MODE)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_IS_NOT_I2C_MODE);
  __Pyx_GIVEREF(__pyx_n_u_IS_NOT_I2C_MODE);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 4, __pyx_n_u_IS_NOT_I2C_MODE)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_IS_NOT_SPI_SINGLE_MODE);
  __Pyx_GIVEREF(__pyx_n_u_IS_NOT_SPI_SINGLE_MODE);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 5, __pyx_n_u_IS_NOT_SPI_SINGLE_MODE)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_IS_NOT_SPI_MULTI_MODE);
  __Pyx_GIVEREF(__pyx_n_u_IS_NOT_SPI_MULTI_MODE);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 6, __pyx_n_u_IS_NOT_SPI_MULTI_MODE)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_WRONG_I2C_ADDR);
  __Pyx_GIVEREF(__pyx_n_u_WRONG_I2C_ADDR);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 7, __pyx_n_u_WRONG_I2C_ADDR)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_INVAILD_FUNCTION);
  __Pyx_GIVEREF(__pyx_n_u_INVAILD_FUNCTION);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 8, __pyx_n_u_INVAILD_FUNCTION)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_INVALID_POINTER);
  __Pyx_GIVEREF(__pyx_n_u_INVALID_POINTER);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 9, __pyx_n_u_INVALID_POINTER)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_EXCEEDED_MAX_TRANSFER_SIZE);
  __Pyx_GIVEREF(__pyx_n_u_EXCEEDED_MAX_TRANSFER_SIZE);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 10, __pyx_n_u_EXCEEDED_MAX_TRANSFER_SIZE)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_FAILED_TO_READ_DEVICE);
  __Pyx_GIVEREF(__pyx_n_u_FAILED_TO_READ_DEVICE);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 11, __pyx_n_u_FAILED_TO_READ_DEVICE)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_I2C_NOT_SUPPORTED_IN_THIS_MODE);
  __Pyx_GIVEREF(__pyx_n_u_I2C_NOT_SUPPORTED_IN_THIS_MODE);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 12, __pyx_n_u_I2C_NOT_SUPPORTED_IN_THIS_MODE)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_GPIO_NOT_SUPPORTED_IN_THIS_MODE);
  __Pyx_GIVEREF(__pyx_n_u_GPIO_NOT_SUPPORTED_IN_THIS_MODE);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 13, __pyx_n_u_GPIO_NOT_SUPPORTED_IN_THIS_MODE)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_GPIO_EXCEEDED_MAX_PORTNUM);
  __Pyx_GIVEREF(__pyx_n_u_GPIO_EXCEEDED_MAX_PORTNUM);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 14, __pyx_n_u_GPIO_EXCEEDED_MAX_PORTNUM)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_GPIO_WRITE_NOT_SUPPORTED);
  __Pyx_GIVEREF(__pyx_n_u_GPIO_WRITE_NOT_SUPPORTED);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 15, __pyx_n_u_GPIO_WRITE_NOT_SUPPORTED)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_GPIO_PULLUP_INVALID_IN_INPUTMODE);
  __Pyx_GIVEREF(__pyx_n_u_GPIO_PULLUP_INVALID_IN_INPUTMODE);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 16, __pyx_n_u_GPIO_PULLUP_INVALID_IN_INPUTMODE)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_GPIO_PULLDOWN_INVALID_IN_INPUTMO);
  __Pyx_GIVEREF(__pyx_n_u_GPIO_PULLDOWN_INVALID_IN_INPUTMO);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 17, __pyx_n_u_GPIO_PULLDOWN_INVALID_IN_INPUTMO)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_GPIO_OPENDRAIN_INVALID_IN_OUTPUT);
  __Pyx_GIVEREF(__pyx_n_u_GPIO_OPENDRAIN_INVALID_IN_OUTPUT);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 18, __pyx_n_u_GPIO_OPENDRAIN_INVALID_IN_OUTPUT)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_INTERRUPT_NOT_SUPPORTED);
  __Pyx_GIVEREF(__pyx_n_u_INTERRUPT_NOT_SUPPORTED);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 19, __pyx_n_u_INTERRUPT_NOT_SUPPORTED)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_GPIO_INPUT_NOT_SUPPORTED);
  __Pyx_GIVEREF(__pyx_n_u_GPIO_INPUT_NOT_SUPPORTED);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 20, __pyx_n_u_GPIO_INPUT_NOT_SUPPORTED)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_EVENT_NOT_SUPPORTED);
  __Pyx_GIVEREF(__pyx_n_u_EVENT_NOT_SUPPORTED);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 21, __pyx_n_u_EVENT_NOT_SUPPORTED)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_u_FUN_NOT_SUPPORT);
  __Pyx_GIVEREF(__pyx_n_u_FUN_NOT_SUPPORT);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 22, __pyx_n_u_FUN_NOT_SUPPORT)) __PYX_ERR(0, 59, __pyx_L1_error);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_FT4222DeviceError__ftd4222_msgs, __pyx_t_4) < 0) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ft4222/ft4222.pyx":69
 *                   'GPIO_INPUT_NOT_SUPPORTED', 'EVENT_NOT_SUPPORTED', 'FUN_NOT_SUPPORT']
 * 
 *     def __init__(self, msgnum):             # <<<<<<<<<<<<<<
 *         if msgnum >= FT4222_DEVICE_NOT_SUPPORTED:
 *             self.message = self.__ftd4222_msgs[msgnum - FT4222_DEVICE_NOT_SUPPORTED]
 */
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_17FT4222DeviceError_1__init__, 0, __pyx_n_s_FT4222DeviceError___init, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__13)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_init, __pyx_t_4) < 0) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ft4222/ft4222.pyx":75
 *             super(FT4222DeviceError, self).__init__(msgnum)
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message
 * 
 */
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_17FT4222DeviceError_3__str__, 0, __pyx_n_s_FT4222DeviceError___str, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__14)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_str, __pyx_t_4) < 0) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ft4222/ft4222.pyx":56
 *         return self.message
 * 
 * class FT4222DeviceError(FT2XXDeviceError):             # <<<<<<<<<<<<<<
 *     """Exception class for status messages"""
 * 
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_5, __pyx_n_s_FT4222DeviceError, __pyx_t_3, __pyx_t_2, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_FT4222DeviceError, __pyx_t_4) < 0) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ft4222/ft4222.pyx":78
 *         return self.message
 * 
 * class SysClock(IntEnum):             # <<<<<<<<<<<<<<
 *     """Chip system clock
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_IntEnum); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3)) __PYX_ERR(0, 78, __pyx_L1_error);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PEP560_update_bases(__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_3, __pyx_n_s_SysClock, __pyx_n_s_SysClock, (PyObject *) NULL, __pyx_n_s_ft4222_ft4222, __pyx_kp_s_Chip_system_clock_Attributes_CLK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3 != __pyx_t_5) {
    if (unlikely((PyDict_SetItemString(__pyx_t_4, "__orig_bases__", __pyx_t_5) < 0))) __PYX_ERR(0, 78, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "ft4222/ft4222.pyx":88
 * 
 *     """
 *     CLK_60 = 0             # <<<<<<<<<<<<<<
 *     CLK_24 = 1
 *     CLK_48 = 2
 */
  if (__Pyx_SetNameInClass(__pyx_t_4, __pyx_n_s_CLK_60, __pyx_int_0) < 0) __PYX_ERR(0, 88, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":89
 *     """
 *     CLK_60 = 0
 *     CLK_24 = 1             # <<<<<<<<<<<<<<
 *     CLK_48 = 2
 *     CLK_80 = 3
 */
  if (__Pyx_SetNameInClass(__pyx_t_4, __pyx_n_s_CLK_24, __pyx_int_1) < 0) __PYX_ERR(0, 89, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":90
 *     CLK_60 = 0
 *     CLK_24 = 1
 *     CLK_48 = 2             # <<<<<<<<<<<<<<
 *     CLK_80 = 3
 * 
 */
  if (__Pyx_SetNameInClass(__pyx_t_4, __pyx_n_s_CLK_48, __pyx_int_2) < 0) __PYX_ERR(0, 90, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":91
 *     CLK_24 = 1
 *     CLK_48 = 2
 *     CLK_80 = 3             # <<<<<<<<<<<<<<
 * 
 * def createDeviceInfoList():
 */
  if (__Pyx_SetNameInClass(__pyx_t_4, __pyx_n_s_CLK_80, __pyx_int_3) < 0) __PYX_ERR(0, 91, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":78
 *         return self.message
 * 
 * class SysClock(IntEnum):             # <<<<<<<<<<<<<<
 *     """Chip system clock
 * 
 */
  __pyx_t_5 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SysClock, __pyx_t_3, __pyx_t_4, NULL, 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SysClock, __pyx_t_5) < 0) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ft4222/ft4222.pyx":93
 *     CLK_80 = 3
 * 
 * def createDeviceInfoList():             # <<<<<<<<<<<<<<
 *     """Create the internal device info list and return number of entries"""
 *     cdef DWORD nb
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_1createDeviceInfoList, 0, __pyx_n_s_createDeviceInfoList, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__16)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_createDeviceInfoList, __pyx_t_3) < 0) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ft4222/ft4222.pyx":101
 *     raise FT2XXDeviceError, status
 * 
 * def getDeviceInfoDetail(devnum=0, update=True):             # <<<<<<<<<<<<<<
 *     """Get an entry from the internal device info list. Set update to
 *     False to avoid a slow call to createDeviceInfoList."""
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_3getDeviceInfoDetail, 0, __pyx_n_s_getDeviceInfoDetail, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__18)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_3, __pyx_tuple__19);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_getDeviceInfoDetail, __pyx_t_3) < 0) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ft4222/ft4222.pyx":118
 *     raise FT2XXDeviceError, status
 * 
 * def openBySerial(serial):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by serial number
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_5openBySerial, 0, __pyx_n_s_openBySerial, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__21)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_openBySerial, __pyx_t_3) < 0) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ft4222/ft4222.pyx":140
 *     raise FT2XXDeviceError, status
 * 
 * def openByDescription(desc):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by description
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_7openByDescription, 0, __pyx_n_s_openByDescription, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__23)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_openByDescription, __pyx_t_3) < 0) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ft4222/ft4222.pyx":163
 *     raise FT2XXDeviceError, status
 * 
 * def openByLocation(locId):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by location
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_9openByLocation, 0, __pyx_n_s_openByLocation, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__25)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_openByLocation, __pyx_t_3) < 0) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ft4222/ft4222.pyx":196
 *         return closer
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         FT4222_UnInitialize(self._handle)
 *         FT_Close(self._handle)
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_7_closer_1close, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_closer_close, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__26)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222__closer, __pyx_n_s_close, __pyx_t_3) < 0) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222__closer);

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_7_closer_3__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_closer___reduce_cython, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__27)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_reduce_cython, __pyx_t_3) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_7_closer_5__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_closer___setstate_cython, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__29)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_setstate_cython, __pyx_t_3) < 0) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ft4222/ft4222.pyx":210
 *     # Revision D chips require version 1.4 or later of LibFT4222,
 *     # indicated by dllVersion being greater than 0x01040000.
 *     __chip_rev_map = { 0x42220100: "Rev. A", 0x42220200: "Rev. B", 0x42220300: "Rev. C", 0x42220400: "Rev. D" }             # <<<<<<<<<<<<<<
 *     __chip_rev_min_lib = { 0x42220100: 0, 0x42220200: 0x01020000, 0x42220300: 0x01030000, 0x42220400: 0x01040000 }
 * 
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_1109524736, __pyx_kp_u_Rev_A) < 0) __PYX_ERR(0, 210, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_1109524992, __pyx_kp_u_Rev_B) < 0) __PYX_ERR(0, 210, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_1109525248, __pyx_kp_u_Rev_C) < 0) __PYX_ERR(0, 210, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_1109525504, __pyx_kp_u_Rev_D) < 0) __PYX_ERR(0, 210, __pyx_L1_error)
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_FT4222__chip_rev_map, __pyx_t_3) < 0) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":211
 *     # indicated by dllVersion being greater than 0x01040000.
 *     __chip_rev_map = { 0x42220100: "Rev. A", 0x42220200: "Rev. B", 0x42220300: "Rev. C", 0x42220400: "Rev. D" }
 *     __chip_rev_min_lib = { 0x42220100: 0, 0x42220200: 0x01020000, 0x42220300: 0x01030000, 0x42220400: 0x01040000 }             # <<<<<<<<<<<<<<
 * 
 *     cdef FT_HANDLE _handle
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_1109524736, __pyx_int_0) < 0) __PYX_ERR(0, 211, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_1109524992, __pyx_int_16908288) < 0) __PYX_ERR(0, 211, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_1109525248, __pyx_int_16973824) < 0) __PYX_ERR(0, 211, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_1109525504, __pyx_int_17039360) < 0) __PYX_ERR(0, 211, __pyx_L1_error)
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_FT4222__chip_rev_min_lib, __pyx_t_3) < 0) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":226
 *         self._close = weakref_finalize(self, _closer.wrap(self._handle).close)
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """Closes the device."""
 *         FT4222_UnInitialize(self._handle)
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_3close, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_close, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__31)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_close, __pyx_t_3) < 0) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":262
 *         return "FT4222: chipVersion: 0x{:x} ({:s}), libVersion: 0x{:x}".format(self._chip_version, self.chipRevision, self._dll_version)
 * 
 *     def setTimeouts(self, read_timeout, write_timeout):             # <<<<<<<<<<<<<<
 *         """Set the read and write timeouts
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_7setTimeouts, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_setTimeouts, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__33)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_setTimeouts, __pyx_t_3) < 0) __PYX_ERR(0, 262, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":277
 *             raise FT2XXDeviceError, status
 * 
 *     def setClock(self, clk):             # <<<<<<<<<<<<<<
 *         """Set the system clock
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_9setClock, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_setClock, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__35)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_setClock, __pyx_t_3) < 0) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":291
 *             raise FT4222DeviceError, status
 * 
 *     def getClock(self):             # <<<<<<<<<<<<<<
 *         """Get the system clock
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_11getClock, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_getClock, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__36)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_getClock, __pyx_t_3) < 0) __PYX_ERR(0, 291, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":307
 *         raise FT4222DeviceError, status
 * 
 *     def setSuspendOut(self, enable):             # <<<<<<<<<<<<<<
 *         """Enable or disable, suspend out, which will emit a signal when FT4222H enters suspend mode.
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_13setSuspendOut, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_setSuspendOut, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__38)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_setSuspendOut, __pyx_t_3) < 0) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":321
 *             raise FT4222DeviceError, status
 * 
 *     def setWakeUpInterrupt(self, enable):             # <<<<<<<<<<<<<<
 *         """Enable or disable the wakeup/interrupt
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_15setWakeUpInterrupt, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_setWakeUpInterrupt, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__39)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_setWakeUpInterrupt, __pyx_t_3) < 0) __PYX_ERR(0, 321, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":335
 *             raise FT4222DeviceError, status
 * 
 *     def getMaxTransferSize(self):             # <<<<<<<<<<<<<<
 *         """Get the maximum transfer size for the currently configured operating mode
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_17getMaxTransferSize, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_getMaxTransferSize, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__41)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_getMaxTransferSize, __pyx_t_3) < 0) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":350
 *         raise FT4222DeviceError, status
 * 
 *     def vendorCmdGet(self, req, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Vendor get command"""
 *         cdef:
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_19vendorCmdGet, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_vendorCmdGet, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__43)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_vendorCmdGet, __pyx_t_3) < 0) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":360
 *         raise FT4222DeviceError, status
 * 
 *     def vendorCmdSet(self, req, data):             # <<<<<<<<<<<<<<
 *         """Vendor set command"""
 *         if isinstance(data, int):
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_21vendorCmdSet, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_vendorCmdSet, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__45)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 360, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_vendorCmdSet, __pyx_t_3) < 0) __PYX_ERR(0, 360, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":373
 *             raise FT4222DeviceError, status
 * 
 *     def getLatencyTimer(self):             # <<<<<<<<<<<<<<
 *         """Get the current value of the latency timer.
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_23getLatencyTimer, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_getLatencyTimer, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__47)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_getLatencyTimer, __pyx_t_3) < 0) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":388
 *         raise FT2XXDeviceError, status
 * 
 *     def setLatencyTimer(self, latency):             # <<<<<<<<<<<<<<
 *         """Set the current value of the latency timer.
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_25setLatencyTimer, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_setLatencyTimer, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__48)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 388, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_setLatencyTimer, __pyx_t_3) < 0) __PYX_ERR(0, 388, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":401
 *             raise FT2XXDeviceError, status
 * 
 *     def chipReset(self):             # <<<<<<<<<<<<<<
 *         """Software reset for device.
 *         This function is used to attempt to recover system after a failure. It is a software reset for device.
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_27chipReset, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_chipReset, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__49)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_chipReset, __pyx_t_3) < 0) __PYX_ERR(0, 401, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":414
 *             raise FT2XXDeviceError, status
 * 
 *     def gpio_Init(self, *args, gpio0=Dir.INPUT, gpio1=Dir.INPUT, gpio2=Dir.INPUT, gpio3=Dir.INPUT):             # <<<<<<<<<<<<<<
 *         """Initialize the GPIO interface.
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_Dir); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_INPUT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_k__2 = __pyx_t_2;
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_Dir); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_INPUT); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_k__3 = __pyx_t_3;
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_3 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_Dir); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_INPUT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_k__4 = __pyx_t_2;
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_Dir); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_INPUT); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_k__5 = __pyx_t_3;
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_Dir); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_INPUT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_gpio0, __pyx_t_4) < 0) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_Dir); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_INPUT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_gpio1, __pyx_t_2) < 0) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_Dir); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_INPUT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_gpio2, __pyx_t_4) < 0) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_Dir); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_INPUT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_gpio3, __pyx_t_2) < 0) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_29gpio_Init, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_gpio_Init, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__51)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_2, __pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_gpio_Init, __pyx_t_2) < 0) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":444
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_Read(self, portNum):             # <<<<<<<<<<<<<<
 *         """Read value from selected GPIO
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_31gpio_Read, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_gpio_Read, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__53)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_gpio_Read, __pyx_t_2) < 0) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":464
 *         raise FT4222DeviceError, status
 * 
 *     def gpio_Wait(self, portNum, expectedValue, timeout=0, sleep=1):             # <<<<<<<<<<<<<<
 *         """Blocks until the selected GPIO reaches a certain value.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_33gpio_Wait, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_gpio_Wait, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__55)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 464, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_tuple__56);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_gpio_Wait, __pyx_t_2) < 0) __PYX_ERR(0, 464, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":504
 *                     msleep(sleep_c)
 * 
 *     def gpio_Write(self, portNum, value):             # <<<<<<<<<<<<<<
 *         """Write value to given GPIO
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_35gpio_Write, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_gpio_Write, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__57)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 504, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_gpio_Write, __pyx_t_2) < 0) __PYX_ERR(0, 504, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":519
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_SetInputTrigger(self, portNum, trigger):             # <<<<<<<<<<<<<<
 *         """Set software trigger conditions on the specified GPIO pin.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_37gpio_SetInputTrigger, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_gpio_SetInputTrigger, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__59)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_gpio_SetInputTrigger, __pyx_t_2) < 0) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":534
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_GetTriggerStatus(self, portNum):             # <<<<<<<<<<<<<<
 *         """Get the size of trigger event queue.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_39gpio_GetTriggerStatus, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_gpio_GetTriggerStatus, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__61)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 534, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_gpio_GetTriggerStatus, __pyx_t_2) < 0) __PYX_ERR(0, 534, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":554
 *         raise FT4222DeviceError, status
 * 
 *     def gpio_ReadTriggerQueue(self, portNum, readSize=None):             # <<<<<<<<<<<<<<
 *         """Get events recorded in the trigger event queue.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_41gpio_ReadTriggerQueue, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_gpio_ReadTriggerQueue, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__63)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 554, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_tuple__64);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_gpio_ReadTriggerQueue, __pyx_t_2) < 0) __PYX_ERR(0, 554, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":582
 * 
 * 
 *     def i2cMaster_Init(self, kbps=100):             # <<<<<<<<<<<<<<
 *         """Initialize the FT4222H as an I2C master with the requested I2C speed.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_43i2cMaster_Init, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_i2cMaster_Init, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__66)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 582, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_tuple__67);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_i2cMaster_Init, __pyx_t_2) < 0) __PYX_ERR(0, 582, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":611
 *             self.vendorCmdSet(0x52, n)
 * 
 *     def i2cMaster_Read(self, addr, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the specified I2C slave device with START and STOP conditions.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_45i2cMaster_Read, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_i2cMaster_Read, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__69)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 611, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_i2cMaster_Read, __pyx_t_2) < 0) __PYX_ERR(0, 611, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":635
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_Write(self, addr, data):             # <<<<<<<<<<<<<<
 *         """Write data to the specified I2C slave device with START and STOP conditions.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_47i2cMaster_Write, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_i2cMaster_Write, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__71)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 635, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_i2cMaster_Write, __pyx_t_2) < 0) __PYX_ERR(0, 635, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":661
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_ReadEx(self, addr, flag, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the specified I2C slave device with the specified I2C condition.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_49i2cMaster_ReadEx, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_i2cMaster_ReadEx, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__73)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 661, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_i2cMaster_ReadEx, __pyx_t_2) < 0) __PYX_ERR(0, 661, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":686
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_WriteEx(self, addr, flag, data):             # <<<<<<<<<<<<<<
 *         """Write data to the specified I2C slave device with the specified I2C condition.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_51i2cMaster_WriteEx, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_i2cMaster_WriteEx, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__75)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 686, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_i2cMaster_WriteEx, __pyx_t_2) < 0) __PYX_ERR(0, 686, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":713
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_Reset(self):             # <<<<<<<<<<<<<<
 *         """Reset the I2C master device.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_53i2cMaster_Reset, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_i2cMaster_Reset, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__76)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 713, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_i2cMaster_Reset, __pyx_t_2) < 0) __PYX_ERR(0, 713, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":727
 *             raise FT4222DeviceError, status
 * 
 *     def i2cMaster_GetStatus(self):             # <<<<<<<<<<<<<<
 *         """Read the status of the I2C master controller.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_55i2cMaster_GetStatus, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_i2cMaster_GetStatus, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__78)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 727, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_i2cMaster_GetStatus, __pyx_t_2) < 0) __PYX_ERR(0, 727, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":746
 * 
 * 
 *     def spi_Reset(self):             # <<<<<<<<<<<<<<
 *         """Reset the SPI master or slave device
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_57spi_Reset, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_spi_Reset, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__79)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 746, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_spi_Reset, __pyx_t_2) < 0) __PYX_ERR(0, 746, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":757
 *             raise FT4222DeviceError, status
 * 
 *     def spi_ResetTransaction(self, spiIdx):             # <<<<<<<<<<<<<<
 *         """Reset the SPI transaction
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_59spi_ResetTransaction, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_spi_ResetTransaction, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__81)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 757, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_spi_ResetTransaction, __pyx_t_2) < 0) __PYX_ERR(0, 757, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":771
 *             raise FT4222DeviceError, status
 * 
 *     def spi_SetDrivingStrength(self, clkStrength, ioStrength, ssoStrength):             # <<<<<<<<<<<<<<
 *         """Reset the SPI master or slave device.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_61spi_SetDrivingStrength, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_spi_SetDrivingStrength, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__83)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 771, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_spi_SetDrivingStrength, __pyx_t_2) < 0) __PYX_ERR(0, 771, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":787
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_Init(self, mode, clock, cpol, cpha, ssoMap):             # <<<<<<<<<<<<<<
 *         """Initialize as an SPI master under all modes.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_63spiMaster_Init, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_spiMaster_Init, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__85)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 787, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_spiMaster_Init, __pyx_t_2) < 0) __PYX_ERR(0, 787, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":805
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_SetLines(self, mode):             # <<<<<<<<<<<<<<
 *         """Switch the FT4222H SPI master to single, dual, or quad mode.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_65spiMaster_SetLines, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_spiMaster_SetLines, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__87)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 805, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_spiMaster_SetLines, __pyx_t_2) < 0) __PYX_ERR(0, 805, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":822
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleRead(self, bytesToRead, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Read data from a SPI slave in single mode
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_67spiMaster_SingleRead, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_spiMaster_SingleRead, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__89)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 822, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_spiMaster_SingleRead, __pyx_t_2) < 0) __PYX_ERR(0, 822, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":846
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleWrite(self, data, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Write data to a SPI slave in single mode
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_69spiMaster_SingleWrite, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_spiMaster_SingleWrite, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__91)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_spiMaster_SingleWrite, __pyx_t_2) < 0) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":872
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleReadWrite(self, data, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Write and read data to and from a SPI slave in single mode
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_71spiMaster_SingleReadWrite, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_spiMaster_SingleReadWrite, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__93)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 872, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_spiMaster_SingleReadWrite, __pyx_t_2) < 0) __PYX_ERR(0, 872, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":901
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_MultiReadWrite(self, singleWrite, multiWrite, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Write and read data to and from a SPI slave in dual- or quad-mode (multi-mode).
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_73spiMaster_MultiReadWrite, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_spiMaster_MultiReadWrite, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__95)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_spiMaster_MultiReadWrite, __pyx_t_2) < 0) __PYX_ERR(0, 901, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":936
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_EndTransaction(self):             # <<<<<<<<<<<<<<
 *         """End the current SPI transaction.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_75spiMaster_EndTransaction, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_spiMaster_EndTransaction, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__97)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 936, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_spiMaster_EndTransaction, __pyx_t_2) < 0) __PYX_ERR(0, 936, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":950
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_Init(self):             # <<<<<<<<<<<<<<
 *         """Initialize the FT4222H as an SPI slave. Default SPI_SlaveProtocol is SPI_SLAVE_WITH_PROTOCOL.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_77spiSlave_Init, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_spiSlave_Init, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__98)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 950, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_spiSlave_Init, __pyx_t_2) < 0) __PYX_ERR(0, 950, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":961
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_InitEx(self, proto):             # <<<<<<<<<<<<<<
 *         """Initialize as an SPI slave under all modes.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_79spiSlave_InitEx, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_spiSlave_InitEx, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__100)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 961, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_spiSlave_InitEx, __pyx_t_2) < 0) __PYX_ERR(0, 961, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":975
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_Read(self, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the receive queue of the SPI slave device.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_81spiSlave_Read, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_spiSlave_Read, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__102)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 975, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_spiSlave_Read, __pyx_t_2) < 0) __PYX_ERR(0, 975, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":999
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_SetMode(self, cpol, cpha):             # <<<<<<<<<<<<<<
 *         """Set SPI slave cpol and cpha. The Default value of cpol is (:obj:`ft4222.SPI.Cpol.CLK_IDLE_LOW`) , default value of cpha is (:obj:`ft4222.SPI.Cpol.CLK_LEADING`)
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_83spiSlave_SetMode, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_spiSlave_SetMode, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__104)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 999, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_spiSlave_SetMode, __pyx_t_2) < 0) __PYX_ERR(0, 999, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":1014
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_GetRxStatus(self):             # <<<<<<<<<<<<<<
 *         """Get number of bytes in the receive queue.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_85spiSlave_GetRxStatus, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_spiSlave_GetRxStatus, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__106)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1014, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_spiSlave_GetRxStatus, __pyx_t_2) < 0) __PYX_ERR(0, 1014, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "ft4222/ft4222.pyx":1033
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_Write(self, data):             # <<<<<<<<<<<<<<
 *         """Write data to the transmit queue of the SPI slave device.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_87spiSlave_Write, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222_spiSlave_Write, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__108)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1033, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222, __pyx_n_s_spiSlave_Write, __pyx_t_2) < 0) __PYX_ERR(0, 1033, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_6ft4222_6ft4222_FT4222);

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_89__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222___reduce_cython, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__109)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_reduce_cython, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_6FT4222_91__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_FT4222___setstate_cython, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__110)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_setstate_cython, __pyx_t_2) < 0) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":1
 * #  _____ _____ _____             # <<<<<<<<<<<<<<
 * # |_    |   __| __  |
 * # |_| | |__   |    -|
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  if (__pyx_m) {
    if (__pyx_d && stringtab_initialized) {
      __Pyx_AddTraceback("init ft4222.ft4222", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_USE_MODULE_STATE
    Py_CLEAR(__pyx_m);
    #else
    Py_DECREF(__pyx_m);
    if (pystate_addmodule_run) {
      PyObject *tp, *value, *tb;
      PyErr_Fetch(&tp, &value, &tb);
      PyState_RemoveModule(&__pyx_moduledef);
      PyErr_Restore(tp, value, tb);
    }
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init ft4222.ft4222");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_pragmas ### */
#ifdef _MSC_VER
#pragma warning( push )
/* Warning 4127: conditional expression is constant
 * Cython uses constant conditional expressions to allow in inline functions to be optimized at
 * compile-time, so this warning is not useful
 */
#pragma warning( disable : 4127 )
#endif



/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    int result;
    PyObject *exc_type;
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *current_exception = tstate->current_exception;
    if (unlikely(!current_exception)) return 0;
    exc_type = (PyObject*) Py_TYPE(current_exception);
    if (exc_type == err) return 1;
#else
    exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
#endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(exc_type);
    #endif
    if (unlikely(PyTuple_Check(err))) {
        result = __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    } else {
        result = __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
    }
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(exc_type);
    #endif
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *tmp_value;
    assert(type == NULL || (value != NULL && type == (PyObject*) Py_TYPE(value)));
    if (value) {
        #if CYTHON_COMPILING_IN_CPYTHON
        if (unlikely(((PyBaseExceptionObject*) value)->traceback != tb))
        #endif
            PyException_SetTraceback(value, tb);
    }
    tmp_value = tstate->current_exception;
    tstate->current_exception = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
#else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject* exc_value;
    exc_value = tstate->current_exception;
    tstate->current_exception = 0;
    *value = exc_value;
    *type = NULL;
    *tb = NULL;
    if (exc_value) {
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        #if CYTHON_COMPILING_IN_CPYTHON
        *tb = ((PyBaseExceptionObject*) exc_value)->traceback;
        Py_XINCREF(*tb);
        #else
        *tb = PyException_GetTraceback(exc_value);
        #endif
    }
#else
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#endif
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError */
#if __PYX_LIMITED_VERSION_HEX < 0x030d00A1
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d00A1
    (void) PyObject_GetOptionalAttr(obj, attr_name, &result);
    return result;
#else
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
#endif
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* TupleAndListFromArray */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        Py_INCREF(__pyx_empty_tuple);
        return __pyx_empty_tuple;
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = PyTuple_GET_SIZE(kwnames);
    for (i = 0; i < n; i++)
    {
        if (s == PyTuple_GET_ITEM(kwnames, i)) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        int eq = __Pyx_PyUnicode_Equals(s, PyTuple_GET_ITEM(kwnames, i), Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;  // error
            return kwvalues[i];
        }
    }
    return NULL;  // not found (no exception set)
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
static CYTHON_UNUSED PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues) {
    Py_ssize_t i, nkwargs = PyTuple_GET_SIZE(kwnames);
    PyObject *dict;
    dict = PyDict_New();
    if (unlikely(!dict))
        return NULL;
    for (i=0; i<nkwargs; i++) {
        PyObject *key = PyTuple_GET_ITEM(kwnames, i);
        if (unlikely(PyDict_SetItem(dict, key, kwvalues[i]) < 0))
            goto bad;
    }
    return dict;
bad:
    Py_DECREF(dict);
    return NULL;
}
#endif
#endif

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject *const *kwvalues,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    int kwds_is_tuple = CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds));
    while (1) {
        Py_XDECREF(key); key = NULL;
        Py_XDECREF(value); value = NULL;
        if (kwds_is_tuple) {
            Py_ssize_t size;
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(kwds);
#else
            size = PyTuple_Size(kwds);
            if (size < 0) goto bad;
#endif
            if (pos >= size) break;
#if CYTHON_AVOID_BORROWED_REFS
            key = __Pyx_PySequence_ITEM(kwds, pos);
            if (!key) goto bad;
#elif CYTHON_ASSUME_SAFE_MACROS
            key = PyTuple_GET_ITEM(kwds, pos);
#else
            key = PyTuple_GetItem(kwds, pos);
            if (!key) goto bad;
#endif
            value = kwvalues[pos];
            pos++;
        }
        else
        {
            if (!PyDict_Next(kwds, &pos, &key, &value)) break;
#if CYTHON_AVOID_BORROWED_REFS
            Py_INCREF(key);
#endif
        }
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
            Py_INCREF(value); // transfer ownership of value to values
            Py_DECREF(key);
#endif
            key = NULL;
            value = NULL;
            continue;
        }
#if !CYTHON_AVOID_BORROWED_REFS
        Py_INCREF(key);
#endif
        Py_INCREF(value);
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
                    value = NULL;  // ownership transferred to values
#endif
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key)
                );
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
                    value = NULL; // ownership transferred to values
#endif
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    Py_XDECREF(key);
    Py_XDECREF(value);
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    #if PY_MAJOR_VERSION < 3
    PyErr_Format(PyExc_TypeError,
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    Py_XDECREF(key);
    Py_XDECREF(value);
    return -1;
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (unlikely(!j)) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (mm && mm->mp_subscript) {
            PyObject *r, *key = PyInt_FromSsize_t(i);
            if (unlikely(!key)) return NULL;
            r = mm->mp_subscript(o, key);
            Py_DECREF(key);
            return r;
        }
        if (likely(sm && sm->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return sm->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL && !CYTHON_VECTORCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object"))) {
        return NULL;
    }
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object"))) {
        return NULL;
    }
    #endif
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    #endif
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = __Pyx_CyOrPyCFunction_GET_FUNCTION(func);
    self = __Pyx_CyOrPyCFunction_GET_SELF(func);
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    #endif
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall */
#if PY_VERSION_HEX < 0x03090000 || CYTHON_COMPILING_IN_LIMITED_API
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject **args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result = 0;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        if (__Pyx_PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]) < 0) goto bad;
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
  bad:
    Py_DECREF(argstuple);
    return result;
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject **args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_NOARGS))
            return __Pyx_PyObject_CallMethO(func, NULL);
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_O))
            return __Pyx_PyObject_CallMethO(func, args[0]);
    }
#endif
    #if PY_VERSION_HEX < 0x030800B1
    #if CYTHON_FAST_PYCCALL
    if (PyCFunction_Check(func)) {
        if (kwargs) {
            return _PyCFunction_FastCallDict(func, args, nargs, kwargs);
        } else {
            return _PyCFunction_FastCallKeywords(func, args, nargs, NULL);
        }
    }
    #if PY_VERSION_HEX >= 0x030700A1
    if (!kwargs && __Pyx_IS_TYPE(func, &PyMethodDescr_Type)) {
        return _PyMethodDescr_FastCallKeywords(func, args, nargs, NULL);
    }
    #endif
    #endif
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs);
    }
    #endif
    #endif
    if (kwargs == NULL) {
        #if CYTHON_VECTORCALL
        #if PY_VERSION_HEX < 0x03090000
        vectorcallfunc f = _PyVectorcall_Function(func);
        #else
        vectorcallfunc f = PyVectorcall_Function(func);
        #endif
        if (f) {
            return f(func, args, (size_t)nargs, NULL);
        }
        #elif defined(__Pyx_CyFunction_USED) && CYTHON_BACKPORT_VECTORCALL
        if (__Pyx_CyFunction_CheckExact(func)) {
            __pyx_vectorcallfunc f = __Pyx_CyFunction_func_vectorcall(func);
            if (f) return f(func, args, (size_t)nargs, NULL);
        }
        #endif
    }
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_empty_tuple, kwargs);
    }
    #if PY_VERSION_HEX >= 0x03090000 && !CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_VectorcallDict(func, args, (size_t)nargs, kwargs);
    #else
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
    #endif
}

/* PyObjectCallOneArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *args[2] = {NULL, arg};
    return __Pyx_PyObject_FastCall(func, args+1, 1 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject *index) {
    PyObject *runerr = NULL;
    Py_ssize_t key_value;
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        __Pyx_TypeName index_type_name = __Pyx_PyType_GetName(Py_TYPE(index));
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError,
            "cannot fit '" __Pyx_FMT_TYPENAME "' into an index-sized integer", index_type_name);
        __Pyx_DECREF_TypeName(index_type_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem_Slow(PyObject *obj, PyObject *key) {
    __Pyx_TypeName obj_type_name;
    if (likely(PyType_Check(obj))) {
        PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(obj, __pyx_n_s_class_getitem);
        if (!meth) {
            PyErr_Clear();
        } else {
            PyObject *result = __Pyx_PyObject_CallOneArg(meth, key);
            Py_DECREF(meth);
            return result;
        }
    }
    obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is not subscriptable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key) {
    PyTypeObject *tp = Py_TYPE(obj);
    PyMappingMethods *mm = tp->tp_as_mapping;
    PySequenceMethods *sm = tp->tp_as_sequence;
    if (likely(mm && mm->mp_subscript)) {
        return mm->mp_subscript(obj, key);
    }
    if (likely(sm && sm->sq_item)) {
        return __Pyx_PyObject_GetIndex(obj, key);
    }
    return __Pyx_PyObject_GetItem_Slow(obj, key);
}
#endif

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && PY_VERSION_HEX < 0x030d0000
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#elif CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    __Pyx_PyThreadState_declare
    CYTHON_UNUSED_VAR(cause);
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
      #if PY_VERSION_HEX >= 0x030C00A6
        PyException_SetTraceback(value, tb);
      #elif CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* PyObjectCallNoArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
    PyObject *arg[2] = {NULL, NULL};
    return __Pyx_PyObject_FastCall(func, arg + 1, 0 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* KeywordStringCheck */
static int __Pyx_CheckKeywordStrings(
    PyObject *kw,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kw, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kw))) {
        Py_ssize_t kwsize;
#if CYTHON_ASSUME_SAFE_MACROS
        kwsize = PyTuple_GET_SIZE(kw);
#else
        kwsize = PyTuple_Size(kw);
        if (kwsize < 0) return 0;
#endif
        if (unlikely(kwsize == 0))
            return 1;
        if (!kw_allowed) {
#if CYTHON_ASSUME_SAFE_MACROS
            key = PyTuple_GET_ITEM(kw, 0);
#else
            key = PyTuple_GetItem(kw, pos);
            if (!key) return 0;
#endif
            goto invalid_keyword;
        }
#if PY_VERSION_HEX < 0x03090000
        for (pos = 0; pos < kwsize; pos++) {
#if CYTHON_ASSUME_SAFE_MACROS
            key = PyTuple_GET_ITEM(kw, pos);
#else
            key = PyTuple_GetItem(kw, pos);
            if (!key) return 0;
#endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
        }
#endif
        return 1;
    }
    while (PyDict_Next(kw, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if (!kw_allowed && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    #if PY_MAJOR_VERSION < 3
    PyErr_Format(PyExc_TypeError,
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_value == NULL || exc_info->exc_value == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    PyObject *exc_value = exc_info->exc_value;
    if (exc_value == NULL || exc_value == Py_None) {
        *value = NULL;
        *type = NULL;
        *tb = NULL;
    } else {
        *value = exc_value;
        Py_INCREF(*value);
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        *tb = PyException_GetTraceback(exc_value);
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #endif
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    PyObject *tmp_value = exc_info->exc_value;
    exc_info->exc_value = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
  #else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
  #endif
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type = NULL, *local_value, *local_tb = NULL;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if PY_VERSION_HEX >= 0x030C00A6
    local_value = tstate->current_exception;
    tstate->current_exception = 0;
    if (likely(local_value)) {
        local_type = (PyObject*) Py_TYPE(local_value);
        Py_INCREF(local_type);
        local_tb = PyException_GetTraceback(local_value);
    }
  #else
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
  #endif
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE && PY_VERSION_HEX >= 0x030C00A6
    if (unlikely(tstate->current_exception))
#elif CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
      #if PY_VERSION_HEX >= 0x030B00a4
        tmp_value = exc_info->exc_value;
        exc_info->exc_value = local_value;
        tmp_type = NULL;
        tmp_tb = NULL;
        Py_XDECREF(local_type);
        Py_XDECREF(local_tb);
      #else
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
      #endif
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* IsLittleEndian */
static CYTHON_INLINE int __Pyx_Is_Little_Endian(void)
{
  union {
    uint32_t u32;
    uint8_t u8[4];
  } S;
  S.u32 = 0x01020304;
  return S.u8[0] == 4;
}

/* BufferFormatCheck */
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type) {
  stack[0].field = &ctx->root;
  stack[0].parent_offset = 0;
  ctx->root.type = type;
  ctx->root.name = "buffer dtype";
  ctx->root.offset = 0;
  ctx->head = stack;
  ctx->head->field = &ctx->root;
  ctx->fmt_offset = 0;
  ctx->head->parent_offset = 0;
  ctx->new_packmode = '@';
  ctx->enc_packmode = '@';
  ctx->new_count = 1;
  ctx->enc_count = 0;
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  ctx->is_valid_array = 0;
  ctx->struct_alignment = 0;
  while (type->typegroup == 'S') {
    ++ctx->head;
    ctx->head->field = type->fields;
    ctx->head->parent_offset = 0;
    type = type->fields->type;
  }
}
static int __Pyx_BufFmt_ParseNumber(const char** ts) {
    int count;
    const char* t = *ts;
    if (*t < '0' || *t > '9') {
      return -1;
    } else {
        count = *t++ - '0';
        while (*t >= '0' && *t <= '9') {
            count *= 10;
            count += *t++ - '0';
        }
    }
    *ts = t;
    return count;
}
static int __Pyx_BufFmt_ExpectNumber(const char **ts) {
    int number = __Pyx_BufFmt_ParseNumber(ts);
    if (number == -1)
        PyErr_Format(PyExc_ValueError,\
                     "Does not understand character buffer dtype format string ('%c')", **ts);
    return number;
}
static void __Pyx_BufFmt_RaiseUnexpectedChar(char ch) {
  PyErr_Format(PyExc_ValueError,
               "Unexpected format string character: '%c'", ch);
}
static const char* __Pyx_BufFmt_DescribeTypeChar(char ch, int is_complex) {
  switch (ch) {
    case '?': return "'bool'";
    case 'c': return "'char'";
    case 'b': return "'signed char'";
    case 'B': return "'unsigned char'";
    case 'h': return "'short'";
    case 'H': return "'unsigned short'";
    case 'i': return "'int'";
    case 'I': return "'unsigned int'";
    case 'l': return "'long'";
    case 'L': return "'unsigned long'";
    case 'q': return "'long long'";
    case 'Q': return "'unsigned long long'";
    case 'f': return (is_complex ? "'complex float'" : "'float'");
    case 'd': return (is_complex ? "'complex double'" : "'double'");
    case 'g': return (is_complex ? "'complex long double'" : "'long double'");
    case 'T': return "a struct";
    case 'O': return "Python object";
    case 'P': return "a pointer";
    case 's': case 'p': return "a string";
    case 0: return "end";
    default: return "unparsable format string";
  }
}
static size_t __Pyx_BufFmt_TypeCharToStandardSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return 2;
    case 'i': case 'I': case 'l': case 'L': return 4;
    case 'q': case 'Q': return 8;
    case 'f': return (is_complex ? 8 : 4);
    case 'd': return (is_complex ? 16 : 8);
    case 'g': {
      PyErr_SetString(PyExc_ValueError, "Python does not define a standard format string size for long double ('g')..");
      return 0;
    }
    case 'O': case 'P': return sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static size_t __Pyx_BufFmt_TypeCharToNativeSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(short);
    case 'i': case 'I': return sizeof(int);
    case 'l': case 'L': return sizeof(long);
    #ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(PY_LONG_LONG);
    #endif
    case 'f': return sizeof(float) * (is_complex ? 2 : 1);
    case 'd': return sizeof(double) * (is_complex ? 2 : 1);
    case 'g': return sizeof(long double) * (is_complex ? 2 : 1);
    case 'O': case 'P': return sizeof(void*);
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
typedef struct { char c; short x; } __Pyx_st_short;
typedef struct { char c; int x; } __Pyx_st_int;
typedef struct { char c; long x; } __Pyx_st_long;
typedef struct { char c; float x; } __Pyx_st_float;
typedef struct { char c; double x; } __Pyx_st_double;
typedef struct { char c; long double x; } __Pyx_st_longdouble;
typedef struct { char c; void *x; } __Pyx_st_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { char c; PY_LONG_LONG x; } __Pyx_st_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToAlignment(char ch, int is_complex) {
  CYTHON_UNUSED_VAR(is_complex);
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_st_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_st_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_st_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_st_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_st_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_st_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_st_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_st_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
/* These are for computing the padding at the end of the struct to align
   on the first member of the struct. This will probably the same as above,
   but we don't have any guarantees.
 */
typedef struct { short x; char c; } __Pyx_pad_short;
typedef struct { int x; char c; } __Pyx_pad_int;
typedef struct { long x; char c; } __Pyx_pad_long;
typedef struct { float x; char c; } __Pyx_pad_float;
typedef struct { double x; char c; } __Pyx_pad_double;
typedef struct { long double x; char c; } __Pyx_pad_longdouble;
typedef struct { void *x; char c; } __Pyx_pad_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { PY_LONG_LONG x; char c; } __Pyx_pad_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToPadding(char ch, int is_complex) {
  CYTHON_UNUSED_VAR(is_complex);
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_pad_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_pad_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_pad_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_pad_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_pad_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_pad_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_pad_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_pad_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static char __Pyx_BufFmt_TypeCharToGroup(char ch, int is_complex) {
  switch (ch) {
    case 'c':
        return 'H';
    case 'b': case 'h': case 'i':
    case 'l': case 'q': case 's': case 'p':
        return 'I';
    case '?': case 'B': case 'H': case 'I': case 'L': case 'Q':
        return 'U';
    case 'f': case 'd': case 'g':
        return (is_complex ? 'C' : 'R');
    case 'O':
        return 'O';
    case 'P':
        return 'P';
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
static void __Pyx_BufFmt_RaiseExpected(__Pyx_BufFmt_Context* ctx) {
  if (ctx->head == NULL || ctx->head->field == &ctx->root) {
    const char* expected;
    const char* quote;
    if (ctx->head == NULL) {
      expected = "end";
      quote = "";
    } else {
      expected = ctx->head->field->type->name;
      quote = "'";
    }
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected %s%s%s but got %s",
                 quote, expected, quote,
                 __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex));
  } else {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_StructField* parent = (ctx->head - 1)->field;
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected '%s' but got %s in '%s.%s'",
                 field->type->name, __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex),
                 parent->type->name, field->name);
  }
}
static int __Pyx_BufFmt_ProcessTypeChunk(__Pyx_BufFmt_Context* ctx) {
  char group;
  size_t size, offset, arraysize = 1;
  if (ctx->enc_type == 0) return 0;
  if (ctx->head->field->type->arraysize[0]) {
    int i, ndim = 0;
    if (ctx->enc_type == 's' || ctx->enc_type == 'p') {
        ctx->is_valid_array = ctx->head->field->type->ndim == 1;
        ndim = 1;
        if (ctx->enc_count != ctx->head->field->type->arraysize[0]) {
            PyErr_Format(PyExc_ValueError,
                         "Expected a dimension of size %zu, got %zu",
                         ctx->head->field->type->arraysize[0], ctx->enc_count);
            return -1;
        }
    }
    if (!ctx->is_valid_array) {
      PyErr_Format(PyExc_ValueError, "Expected %d dimensions, got %d",
                   ctx->head->field->type->ndim, ndim);
      return -1;
    }
    for (i = 0; i < ctx->head->field->type->ndim; i++) {
      arraysize *= ctx->head->field->type->arraysize[i];
    }
    ctx->is_valid_array = 0;
    ctx->enc_count = 1;
  }
  group = __Pyx_BufFmt_TypeCharToGroup(ctx->enc_type, ctx->is_complex);
  do {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_TypeInfo* type = field->type;
    if (ctx->enc_packmode == '@' || ctx->enc_packmode == '^') {
      size = __Pyx_BufFmt_TypeCharToNativeSize(ctx->enc_type, ctx->is_complex);
    } else {
      size = __Pyx_BufFmt_TypeCharToStandardSize(ctx->enc_type, ctx->is_complex);
    }
    if (ctx->enc_packmode == '@') {
      size_t align_at = __Pyx_BufFmt_TypeCharToAlignment(ctx->enc_type, ctx->is_complex);
      size_t align_mod_offset;
      if (align_at == 0) return -1;
      align_mod_offset = ctx->fmt_offset % align_at;
      if (align_mod_offset > 0) ctx->fmt_offset += align_at - align_mod_offset;
      if (ctx->struct_alignment == 0)
          ctx->struct_alignment = __Pyx_BufFmt_TypeCharToPadding(ctx->enc_type,
                                                                 ctx->is_complex);
    }
    if (type->size != size || type->typegroup != group) {
      if (type->typegroup == 'C' && type->fields != NULL) {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        ++ctx->head;
        ctx->head->field = type->fields;
        ctx->head->parent_offset = parent_offset;
        continue;
      }
      if ((type->typegroup == 'H' || group == 'H') && type->size == size) {
      } else {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
      }
    }
    offset = ctx->head->parent_offset + field->offset;
    if (ctx->fmt_offset != offset) {
      PyErr_Format(PyExc_ValueError,
                   "Buffer dtype mismatch; next field is at offset %" CYTHON_FORMAT_SSIZE_T "d but %" CYTHON_FORMAT_SSIZE_T "d expected",
                   (Py_ssize_t)ctx->fmt_offset, (Py_ssize_t)offset);
      return -1;
    }
    ctx->fmt_offset += size;
    if (arraysize)
      ctx->fmt_offset += (arraysize - 1) * size;
    --ctx->enc_count;
    while (1) {
      if (field == &ctx->root) {
        ctx->head = NULL;
        if (ctx->enc_count != 0) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
        }
        break;
      }
      ctx->head->field = ++field;
      if (field->type == NULL) {
        --ctx->head;
        field = ctx->head->field;
        continue;
      } else if (field->type->typegroup == 'S') {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        if (field->type->fields->type == NULL) continue;
        field = field->type->fields;
        ++ctx->head;
        ctx->head->field = field;
        ctx->head->parent_offset = parent_offset;
        break;
      } else {
        break;
      }
    }
  } while (ctx->enc_count);
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  return 0;
}
static int
__pyx_buffmt_parse_array(__Pyx_BufFmt_Context* ctx, const char** tsp)
{
    const char *ts = *tsp;
    int i = 0, number, ndim;
    ++ts;
    if (ctx->new_count != 1) {
        PyErr_SetString(PyExc_ValueError,
                        "Cannot handle repeated arrays in format string");
        return -1;
    }
    if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return -1;
    ndim = ctx->head->field->type->ndim;
    while (*ts && *ts != ')') {
        switch (*ts) {
            case ' ': case '\f': case '\r': case '\n': case '\t': case '\v':  continue;
            default:  break;
        }
        number = __Pyx_BufFmt_ExpectNumber(&ts);
        if (number == -1) return -1;
        if (i < ndim && (size_t) number != ctx->head->field->type->arraysize[i]) {
            PyErr_Format(PyExc_ValueError,
                        "Expected a dimension of size %zu, got %d",
                        ctx->head->field->type->arraysize[i], number);
            return -1;
        }
        if (*ts != ',' && *ts != ')') {
            PyErr_Format(PyExc_ValueError,
                                "Expected a comma in format string, got '%c'", *ts);
            return -1;
        }
        if (*ts == ',') ts++;
        i++;
    }
    if (i != ndim) {
        PyErr_Format(PyExc_ValueError, "Expected %d dimension(s), got %d",
                            ctx->head->field->type->ndim, i);
        return -1;
    }
    if (!*ts) {
        PyErr_SetString(PyExc_ValueError,
                        "Unexpected end of format string, expected ')'");
        return -1;
    }
    ctx->is_valid_array = 1;
    ctx->new_count = 1;
    *tsp = ++ts;
    return 0;
}
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts) {
  int got_Z = 0;
  while (1) {
    switch(*ts) {
      case 0:
        if (ctx->enc_type != 0 && ctx->head == NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        if (ctx->head != NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        return ts;
      case ' ':
      case '\r':
      case '\n':
        ++ts;
        break;
      case '<':
        if (!__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Little-endian buffer not supported on big-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '>':
      case '!':
        if (__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Big-endian buffer not supported on little-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '=':
      case '@':
      case '^':
        ctx->new_packmode = *ts++;
        break;
      case 'T':
        {
          const char* ts_after_sub;
          size_t i, struct_count = ctx->new_count;
          size_t struct_alignment = ctx->struct_alignment;
          ctx->new_count = 1;
          ++ts;
          if (*ts != '{') {
            PyErr_SetString(PyExc_ValueError, "Buffer acquisition: Expected '{' after 'T'");
            return NULL;
          }
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          ctx->enc_count = 0;
          ctx->struct_alignment = 0;
          ++ts;
          ts_after_sub = ts;
          for (i = 0; i != struct_count; ++i) {
            ts_after_sub = __Pyx_BufFmt_CheckString(ctx, ts);
            if (!ts_after_sub) return NULL;
          }
          ts = ts_after_sub;
          if (struct_alignment) ctx->struct_alignment = struct_alignment;
        }
        break;
      case '}':
        {
          size_t alignment = ctx->struct_alignment;
          ++ts;
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          if (alignment && ctx->fmt_offset % alignment) {
            ctx->fmt_offset += alignment - (ctx->fmt_offset % alignment);
          }
        }
        return ts;
      case 'x':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->fmt_offset += ctx->new_count;
        ctx->new_count = 1;
        ctx->enc_count = 0;
        ctx->enc_type = 0;
        ctx->enc_packmode = ctx->new_packmode;
        ++ts;
        break;
      case 'Z':
        got_Z = 1;
        ++ts;
        if (*ts != 'f' && *ts != 'd' && *ts != 'g') {
          __Pyx_BufFmt_RaiseUnexpectedChar('Z');
          return NULL;
        }
        CYTHON_FALLTHROUGH;
      case '?': case 'c': case 'b': case 'B': case 'h': case 'H': case 'i': case 'I':
      case 'l': case 'L': case 'q': case 'Q':
      case 'f': case 'd': case 'g':
      case 'O': case 'p':
        if ((ctx->enc_type == *ts) && (got_Z == ctx->is_complex) &&
            (ctx->enc_packmode == ctx->new_packmode) && (!ctx->is_valid_array)) {
          ctx->enc_count += ctx->new_count;
          ctx->new_count = 1;
          got_Z = 0;
          ++ts;
          break;
        }
        CYTHON_FALLTHROUGH;
      case 's':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->enc_count = ctx->new_count;
        ctx->enc_packmode = ctx->new_packmode;
        ctx->enc_type = *ts;
        ctx->is_complex = got_Z;
        ++ts;
        ctx->new_count = 1;
        got_Z = 0;
        break;
      case ':':
        ++ts;
        while(*ts != ':') ++ts;
        ++ts;
        break;
      case '(':
        if (__pyx_buffmt_parse_array(ctx, &ts) < 0) return NULL;
        break;
      default:
        {
          int number = __Pyx_BufFmt_ExpectNumber(&ts);
          if (number == -1) return NULL;
          ctx->new_count = (size_t)number;
        }
    }
  }
}

/* BufferGetAndValidate */
  static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info) {
  if (unlikely(info->buf == NULL)) return;
  if (info->suboffsets == __Pyx_minusones) info->suboffsets = NULL;
  __Pyx_ReleaseBuffer(info);
}
static void __Pyx_ZeroBuffer(Py_buffer* buf) {
  buf->buf = NULL;
  buf->obj = NULL;
  buf->strides = __Pyx_zeros;
  buf->shape = __Pyx_zeros;
  buf->suboffsets = __Pyx_minusones;
}
static int __Pyx__GetBufferAndValidate(
        Py_buffer* buf, PyObject* obj,  __Pyx_TypeInfo* dtype, int flags,
        int nd, int cast, __Pyx_BufFmt_StackElem* stack)
{
  buf->buf = NULL;
  if (unlikely(__Pyx_GetBuffer(obj, buf, flags) == -1)) {
    __Pyx_ZeroBuffer(buf);
    return -1;
  }
  if (unlikely(buf->ndim != nd)) {
    PyErr_Format(PyExc_ValueError,
                 "Buffer has wrong number of dimensions (expected %d, got %d)",
                 nd, buf->ndim);
    goto fail;
  }
  if (!cast) {
    __Pyx_BufFmt_Context ctx;
    __Pyx_BufFmt_Init(&ctx, stack, dtype);
    if (!__Pyx_BufFmt_CheckString(&ctx, buf->format)) goto fail;
  }
  if (unlikely((size_t)buf->itemsize != dtype->size)) {
    PyErr_Format(PyExc_ValueError,
      "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "d byte%s) does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "d byte%s)",
      buf->itemsize, (buf->itemsize > 1) ? "s" : "",
      dtype->name, (Py_ssize_t)dtype->size, (dtype->size > 1) ? "s" : "");
    goto fail;
  }
  if (buf->suboffsets == NULL) buf->suboffsets = __Pyx_minusones;
  return 0;
fail:;
  __Pyx_SafeReleaseBuffer(buf);
  return -1;
}

/* PyIntBinop */
  #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_MAYBE_UNUSED_VAR(inplace);
    CYTHON_UNUSED_VAR(zerodivision_check);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
        
            x = (long)((unsigned long)a - (unsigned long)b);
            if (likely((x^a) >= 0 || (x^~b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        if (unlikely(__Pyx_PyLong_IsZero(op1))) {
            return PyLong_FromLong(-intval);
        }
        if (likely(__Pyx_PyLong_IsCompact(op1))) {
            a = __Pyx_PyLong_CompactValue(op1);
        } else {
            const digit* digits = __Pyx_PyLong_Digits(op1);
            const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(op1);
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
            }
        }
                x = a - b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla - llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
#if CYTHON_COMPILING_IN_LIMITED_API
        double a = __pyx_PyFloat_AsDouble(op1);
#else
        double a = PyFloat_AS_DOUBLE(op1);
#endif
            double result;
            
            PyFPE_START_PROTECT("subtract", return NULL)
            result = ((double)a) - (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceSubtract : PyNumber_Subtract)(op1, op2);
}
#endif

/* PyFloatBinop */
  #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_TrueDivideCObj(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check) {
    const double a = floatval;
    double b, result;
    CYTHON_UNUSED_VAR(inplace);
    CYTHON_UNUSED_VAR(zerodivision_check);
    if (likely(PyFloat_CheckExact(op2))) {
#if CYTHON_COMPILING_IN_LIMITED_API
        b = __pyx_PyFloat_AsDouble(op2);
#else
        b = PyFloat_AS_DOUBLE(op2);
#endif
        if (unlikely(zerodivision_check && ((b) == 0.0))) { PyErr_SetString(PyExc_ZeroDivisionError, "float division by zero"); return NULL;}
    } else
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op2))) {
        b = (double) PyInt_AS_LONG(op2);
        if (unlikely(zerodivision_check && ((b) == 0.0))) { PyErr_SetString(PyExc_ZeroDivisionError, "float division by zero"); return NULL;}
    } else
    #endif
    if (likely(PyLong_CheckExact(op2))) {
        #if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsZero(op2)) {
            b = 0.0;
            if (unlikely(zerodivision_check && ((b) == 0.0))) { PyErr_SetString(PyExc_ZeroDivisionError, "float division by zero"); return NULL;}
        } else if (__Pyx_PyLong_IsCompact(op2)) {
            b = (double) __Pyx_PyLong_CompactValue(op2);
        } else {
            const digit* digits = __Pyx_PyLong_Digits(op2);
            const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(op2);
            switch (size) {
                case -2:
                case 2:
                    if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (1 * PyLong_SHIFT < 53))) {
                        b = (double) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        if ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                            if (size == -2)
                                b = -b;
                            break;
                        }
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                case 3:
                    if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53))) {
                        b = (double) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        if ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                            if (size == -3)
                                b = -b;
                            break;
                        }
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                case 4:
                    if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53))) {
                        b = (double) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        if ((8 * sizeof(unsigned long) < 53) || (4 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                            if (size == -4)
                                b = -b;
                            break;
                        }
                    }
                    CYTHON_FALLTHROUGH;
                default:
        #endif
                    b = PyLong_AsDouble(op2);
                    if (unlikely(b == -1.0 && PyErr_Occurred())) return NULL;
                    #if !CYTHON_USE_PYLONG_INTERNALS
                    if (unlikely(zerodivision_check && ((b) == 0.0))) { PyErr_SetString(PyExc_ZeroDivisionError, "float division by zero"); return NULL;}
                    #endif
        #if CYTHON_USE_PYLONG_INTERNALS
            }
        }
        #endif
    } else {
        return (inplace ? PyNumber_InPlaceTrueDivide : PyNumber_TrueDivide)(op1, op2);
    }
        PyFPE_START_PROTECT("divide", return NULL)
        result = a / b;
        PyFPE_END_PROTECT(result)
        return PyFloat_FromDouble(result);
}
#endif

/* PyIntBinop */
  #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_MultiplyCObj(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_MAYBE_UNUSED_VAR(inplace);
    CYTHON_UNUSED_VAR(zerodivision_check);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op2))) {
        const long a = intval;
        long b = PyInt_AS_LONG(op2);
        
#ifdef HAVE_LONG_LONG
            if (sizeof(PY_LONG_LONG) > sizeof(long)) {
                PY_LONG_LONG result = (PY_LONG_LONG)a * (PY_LONG_LONG)b;
                return (result >= LONG_MIN && result <= LONG_MAX) ?
                    PyInt_FromLong((long)result) : PyLong_FromLongLong(result);
            }
#endif
#if CYTHON_USE_TYPE_SLOTS
            return PyInt_Type.tp_as_number->nb_multiply(op1, op2);
#else
            return PyNumber_Multiply(op1, op2);
#endif
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op2))) {
        const long a = intval;
        long b, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG lla = intval;
        PY_LONG_LONG llb, llx;
#endif
        if (unlikely(__Pyx_PyLong_IsZero(op2))) {
            return __Pyx_NewRef(op2);
        }
        if (likely(__Pyx_PyLong_IsCompact(op2))) {
            b = __Pyx_PyLong_CompactValue(op2);
        } else {
            const digit* digits = __Pyx_PyLong_Digits(op2);
            const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(op2);
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT+30) {
                        b = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT+30) {
                        llb = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT+30) {
                        b = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT+30) {
                        llb = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT+30) {
                        b = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT+30) {
                        llb = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT+30) {
                        b = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT+30) {
                        llb = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT+30) {
                        b = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT+30) {
                        llb = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT+30) {
                        b = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT+30) {
                        llb = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_multiply(op1, op2);
            }
        }
                CYTHON_UNUSED_VAR(a);
                CYTHON_UNUSED_VAR(b);
                #ifdef HAVE_LONG_LONG
                llb = b;
                goto long_long;
                #else
                return PyLong_Type.tp_as_number->nb_multiply(op1, op2);
                #endif
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla * llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op2)) {
        const long a = intval;
#if CYTHON_COMPILING_IN_LIMITED_API
        double b = __pyx_PyFloat_AsDouble(op2);
#else
        double b = PyFloat_AS_DOUBLE(op2);
#endif
            double result;
            
            PyFPE_START_PROTECT("multiply", return NULL)
            result = ((double)a) * (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceMultiply : PyNumber_Multiply)(op1, op2);
}
#endif

/* FixUpExtensionType */
  #if CYTHON_USE_TYPE_SPECS
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type) {
#if PY_VERSION_HEX > 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED_VAR(spec);
    CYTHON_UNUSED_VAR(type);
#else
    const PyType_Slot *slot = spec->slots;
    while (slot && slot->slot && slot->slot != Py_tp_members)
        slot++;
    if (slot && slot->slot == Py_tp_members) {
        int changed = 0;
#if !(PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON)
        const
#endif
            PyMemberDef *memb = (PyMemberDef*) slot->pfunc;
        while (memb && memb->name) {
            if (memb->name[0] == '_' && memb->name[1] == '_') {
#if PY_VERSION_HEX < 0x030900b1
                if (strcmp(memb->name, "__weaklistoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_weaklistoffset = memb->offset;
                    changed = 1;
                }
                else if (strcmp(memb->name, "__dictoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_dictoffset = memb->offset;
                    changed = 1;
                }
#if CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__vectorcalloffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
#if PY_VERSION_HEX >= 0x030800b4
                    type->tp_vectorcall_offset = memb->offset;
#else
                    type->tp_print = (printfunc) memb->offset;
#endif
                    changed = 1;
                }
#endif
#else
                if ((0));
#endif
#if PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON
                else if (strcmp(memb->name, "__module__") == 0) {
                    PyObject *descr;
                    assert(memb->type == T_OBJECT);
                    assert(memb->flags == 0 || memb->flags == READONLY);
                    descr = PyDescr_NewMember(type, memb);
                    if (unlikely(!descr))
                        return -1;
                    if (unlikely(PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr) < 0)) {
                        Py_DECREF(descr);
                        return -1;
                    }
                    Py_DECREF(descr);
                    changed = 1;
                }
#endif
            }
            memb++;
        }
        if (changed)
            PyType_Modified(type);
    }
#endif
    return 0;
}
#endif

/* PyObjectGetMethod */
  static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    __Pyx_TypeName type_name;
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if defined(Py_TPFLAGS_METHOD_DESCRIPTOR) && Py_TPFLAGS_METHOD_DESCRIPTOR
        if (__Pyx_PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_METHOD_DESCRIPTOR))
#elif PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (likely(descr != NULL)) {
        *method = descr;
        return 0;
    }
    type_name = __Pyx_PyType_GetName(tp);
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, name);
#else
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%.400s'",
                 type_name, PyString_AS_STRING(name));
#endif
    __Pyx_DECREF_TypeName(type_name);
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod0 */
  static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* ValidateBasesTuple */
  #if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases) {
    Py_ssize_t i, n;
#if CYTHON_ASSUME_SAFE_MACROS
    n = PyTuple_GET_SIZE(bases);
#else
    n = PyTuple_Size(bases);
    if (n < 0) return -1;
#endif
    for (i = 1; i < n; i++)
    {
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *b0 = PySequence_GetItem(bases, i);
        if (!b0) return -1;
#elif CYTHON_ASSUME_SAFE_MACROS
        PyObject *b0 = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *b0 = PyTuple_GetItem(bases, i);
        if (!b0) return -1;
#endif
        PyTypeObject *b;
#if PY_MAJOR_VERSION < 3
        if (PyClass_Check(b0))
        {
            PyErr_Format(PyExc_TypeError, "base class '%.200s' is an old-style class",
                         PyString_AS_STRING(((PyClassObject*)b0)->cl_name));
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
#endif
        b = (PyTypeObject*) b0;
        if (!__Pyx_PyType_HasFeature(b, Py_TPFLAGS_HEAPTYPE))
        {
            __Pyx_TypeName b_name = __Pyx_PyType_GetName(b);
            PyErr_Format(PyExc_TypeError,
                "base class '" __Pyx_FMT_TYPENAME "' is not a heap type", b_name);
            __Pyx_DECREF_TypeName(b_name);
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
#if !CYTHON_USE_TYPE_SLOTS
        if (dictoffset == 0) {
            PyErr_Format(PyExc_TypeError,
                "extension type '%s.200s': "
                "unable to validate whether bases have a __dict__ "
                "when CYTHON_USE_TYPE_SLOTS is off "
                "(likely because you are building in the limited API). "
                "Therefore, all extension types with multiple bases "
                "must add 'cdef dict __dict__' in this compilation mode",
                type_name);
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
#else
        if (dictoffset == 0 && b->tp_dictoffset)
        {
            __Pyx_TypeName b_name = __Pyx_PyType_GetName(b);
            PyErr_Format(PyExc_TypeError,
                "extension type '%.200s' has no __dict__ slot, "
                "but base type '" __Pyx_FMT_TYPENAME "' has: "
                "either add 'cdef dict __dict__' to the extension type "
                "or add '__slots__ = [...]' to the base type",
                type_name, b_name);
            __Pyx_DECREF_TypeName(b_name);
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
#endif
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(b0);
#endif
    }
    return 0;
}
#endif

/* PyType_Ready */
  static int __Pyx_PyType_Ready(PyTypeObject *t) {
#if CYTHON_USE_TYPE_SPECS || !(CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API) || defined(PYSTON_MAJOR_VERSION)
    (void)__Pyx_PyObject_CallMethod0;
#if CYTHON_USE_TYPE_SPECS
    (void)__Pyx_validate_bases_tuple;
#endif
    return PyType_Ready(t);
#else
    int r;
    PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
    if (bases && unlikely(__Pyx_validate_bases_tuple(t->tp_name, t->tp_dictoffset, bases) == -1))
        return -1;
#if PY_VERSION_HEX >= 0x03050000 && !defined(PYSTON_MAJOR_VERSION)
    {
        int gc_was_enabled;
    #if PY_VERSION_HEX >= 0x030A00b1
        gc_was_enabled = PyGC_Disable();
        (void)__Pyx_PyObject_CallMethod0;
    #else
        PyObject *ret, *py_status;
        PyObject *gc = NULL;
        #if PY_VERSION_HEX >= 0x030700a1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM+0 >= 0x07030400)
        gc = PyImport_GetModule(__pyx_kp_u_gc);
        #endif
        if (unlikely(!gc)) gc = PyImport_Import(__pyx_kp_u_gc);
        if (unlikely(!gc)) return -1;
        py_status = __Pyx_PyObject_CallMethod0(gc, __pyx_kp_u_isenabled);
        if (unlikely(!py_status)) {
            Py_DECREF(gc);
            return -1;
        }
        gc_was_enabled = __Pyx_PyObject_IsTrue(py_status);
        Py_DECREF(py_status);
        if (gc_was_enabled > 0) {
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_kp_u_disable);
            if (unlikely(!ret)) {
                Py_DECREF(gc);
                return -1;
            }
            Py_DECREF(ret);
        } else if (unlikely(gc_was_enabled == -1)) {
            Py_DECREF(gc);
            return -1;
        }
    #endif
        t->tp_flags |= Py_TPFLAGS_HEAPTYPE;
#if PY_VERSION_HEX >= 0x030A0000
        t->tp_flags |= Py_TPFLAGS_IMMUTABLETYPE;
#endif
#else
        (void)__Pyx_PyObject_CallMethod0;
#endif
    r = PyType_Ready(t);
#if PY_VERSION_HEX >= 0x03050000 && !defined(PYSTON_MAJOR_VERSION)
        t->tp_flags &= ~Py_TPFLAGS_HEAPTYPE;
    #if PY_VERSION_HEX >= 0x030A00b1
        if (gc_was_enabled)
            PyGC_Enable();
    #else
        if (gc_was_enabled) {
            PyObject *tp, *v, *tb;
            PyErr_Fetch(&tp, &v, &tb);
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_kp_u_enable);
            if (likely(ret || r == -1)) {
                Py_XDECREF(ret);
                PyErr_Restore(tp, v, tb);
            } else {
                Py_XDECREF(tp);
                Py_XDECREF(v);
                Py_XDECREF(tb);
                r = -1;
            }
        }
        Py_DECREF(gc);
    #endif
    }
#endif
    return r;
#endif
}

/* PyObject_GenericGetAttrNoDict */
  #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    __Pyx_TypeName type_name = __Pyx_PyType_GetName(tp);
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, attr_name);
#else
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%.400s'",
                 type_name, PyString_AS_STRING(attr_name));
#endif
    __Pyx_DECREF_TypeName(type_name);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
  #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
  static int __Pyx_SetVtable(PyTypeObject *type, void *vtable) {
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
    if (unlikely(!ob))
        goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(PyObject_SetAttr((PyObject *) type, __pyx_n_s_pyx_vtable, ob) < 0))
#else
    if (unlikely(PyDict_SetItem(type->tp_dict, __pyx_n_s_pyx_vtable, ob) < 0))
#endif
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* GetVTable */
  static void* __Pyx_GetVtable(PyTypeObject *type) {
    void* ptr;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *ob = PyObject_GetAttr((PyObject *)type, __pyx_n_s_pyx_vtable);
#else
    PyObject *ob = PyObject_GetItem(type->tp_dict, __pyx_n_s_pyx_vtable);
#endif
    if (!ob)
        goto bad;
    ptr = PyCapsule_GetPointer(ob, 0);
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* MergeVTables */
  #if !CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_MergeVtables(PyTypeObject *type) {
    int i;
    void** base_vtables;
    __Pyx_TypeName tp_base_name;
    __Pyx_TypeName base_name;
    void* unknown = (void*)-1;
    PyObject* bases = type->tp_bases;
    int base_depth = 0;
    {
        PyTypeObject* base = type->tp_base;
        while (base) {
            base_depth += 1;
            base = base->tp_base;
        }
    }
    base_vtables = (void**) malloc(sizeof(void*) * (size_t)(base_depth + 1));
    base_vtables[0] = unknown;
    for (i = 1; i < PyTuple_GET_SIZE(bases); i++) {
        void* base_vtable = __Pyx_GetVtable(((PyTypeObject*)PyTuple_GET_ITEM(bases, i)));
        if (base_vtable != NULL) {
            int j;
            PyTypeObject* base = type->tp_base;
            for (j = 0; j < base_depth; j++) {
                if (base_vtables[j] == unknown) {
                    base_vtables[j] = __Pyx_GetVtable(base);
                    base_vtables[j + 1] = unknown;
                }
                if (base_vtables[j] == base_vtable) {
                    break;
                } else if (base_vtables[j] == NULL) {
                    goto bad;
                }
                base = base->tp_base;
            }
        }
    }
    PyErr_Clear();
    free(base_vtables);
    return 0;
bad:
    tp_base_name = __Pyx_PyType_GetName(type->tp_base);
    base_name = __Pyx_PyType_GetName((PyTypeObject*)PyTuple_GET_ITEM(bases, i));
    PyErr_Format(PyExc_TypeError,
        "multiple bases have vtable conflict: '" __Pyx_FMT_TYPENAME "' and '" __Pyx_FMT_TYPENAME "'", tp_base_name, base_name);
    __Pyx_DECREF_TypeName(tp_base_name);
    __Pyx_DECREF_TypeName(base_name);
    free(base_vtables);
    return -1;
}
#endif

/* SetupReduce */
  #if !CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStrNoError(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_n_s_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred()) {
        __Pyx_TypeName type_obj_name =
            __Pyx_PyType_GetName((PyTypeObject*)type_obj);
        PyErr_Format(PyExc_RuntimeError,
            "Unable to initialize pickling for " __Pyx_FMT_TYPENAME, type_obj_name);
        __Pyx_DECREF_TypeName(type_obj_name);
    }
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}
#endif

/* TypeImport */
  #ifndef __PYX_HAVE_RT_ImportType_3_0_5
#define __PYX_HAVE_RT_ImportType_3_0_5
static PyTypeObject *__Pyx_ImportType_3_0_5(PyObject *module, const char *module_name, const char *class_name,
    size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_0_5 check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
    Py_ssize_t itemsize;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_basicsize;
    PyObject *py_itemsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#if !CYTHON_COMPILING_IN_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
    itemsize = ((PyTypeObject *)result)->tp_itemsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
    py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
    if (!py_itemsize)
        goto bad;
    itemsize = PyLong_AsSsize_t(py_itemsize);
    Py_DECREF(py_itemsize);
    py_itemsize = 0;
    if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (itemsize) {
        if (size % alignment) {
            alignment = size % alignment;
        }
        if (itemsize < (Py_ssize_t)alignment)
            itemsize = (Py_ssize_t)alignment;
    }
    if ((size_t)(basicsize + itemsize) < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize+itemsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error_3_0_5 &&
            ((size_t)basicsize > size || (size_t)(basicsize + itemsize) < size)) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd-%zd from PyObject",
            module_name, class_name, size, basicsize, basicsize+itemsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn_3_0_5 && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* Import */
  static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *module = 0;
    PyObject *empty_dict = 0;
    PyObject *empty_list = 0;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (unlikely(!py_import))
        goto bad;
    if (!from_list) {
        empty_list = PyList_New(0);
        if (unlikely(!empty_list))
            goto bad;
        from_list = empty_list;
    }
    #endif
    empty_dict = PyDict_New();
    if (unlikely(!empty_dict))
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.') != NULL) {
                module = PyImport_ImportModuleLevelObject(
                    name, __pyx_d, empty_dict, from_list, 1);
                if (unlikely(!module)) {
                    if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (unlikely(!py_level))
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, __pyx_d, empty_dict, from_list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, __pyx_d, empty_dict, from_list, level);
            #endif
        }
    }
bad:
    Py_XDECREF(empty_dict);
    Py_XDECREF(empty_list);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    return module;
}

/* ImportFrom */
  static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        const char* module_name_str = 0;
        PyObject* module_name = 0;
        PyObject* module_dot = 0;
        PyObject* full_name = 0;
        PyErr_Clear();
        module_name_str = PyModule_GetName(module);
        if (unlikely(!module_name_str)) { goto modbad; }
        module_name = PyUnicode_FromString(module_name_str);
        if (unlikely(!module_name)) { goto modbad; }
        module_dot = PyUnicode_Concat(module_name, __pyx_kp_u__8);
        if (unlikely(!module_dot)) { goto modbad; }
        full_name = PyUnicode_Concat(module_dot, name);
        if (unlikely(!full_name)) { goto modbad; }
        #if PY_VERSION_HEX < 0x030700A1 || (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400)
        {
            PyObject *modules = PyImport_GetModuleDict();
            if (unlikely(!modules))
                goto modbad;
            value = PyObject_GetItem(modules, full_name);
        }
        #else
        value = PyImport_GetModule(full_name);
        #endif
      modbad:
        Py_XDECREF(full_name);
        Py_XDECREF(module_dot);
        Py_XDECREF(module_name);
    }
    if (unlikely(!value)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* Py3UpdateBases */
  static PyObject*
__Pyx_PEP560_update_bases(PyObject *bases)
{
    Py_ssize_t i, j, size_bases;
    PyObject *base, *meth, *new_base, *result, *new_bases = NULL;
    size_bases = PyTuple_GET_SIZE(bases);
    for (i = 0; i < size_bases; i++) {
        base  = PyTuple_GET_ITEM(bases, i);
        if (PyType_Check(base)) {
            if (new_bases) {
                if (PyList_Append(new_bases, base) < 0) {
                    goto error;
                }
            }
            continue;
        }
        meth = __Pyx_PyObject_GetAttrStrNoError(base, __pyx_n_s_mro_entries);
        if (!meth && PyErr_Occurred()) {
            goto error;
        }
        if (!meth) {
            if (new_bases) {
                if (PyList_Append(new_bases, base) < 0) {
                    goto error;
                }
            }
            continue;
        }
        new_base = __Pyx_PyObject_CallOneArg(meth, bases);
        Py_DECREF(meth);
        if (!new_base) {
            goto error;
        }
        if (!PyTuple_Check(new_base)) {
            PyErr_SetString(PyExc_TypeError,
                            "__mro_entries__ must return a tuple");
            Py_DECREF(new_base);
            goto error;
        }
        if (!new_bases) {
            if (!(new_bases = PyList_New(i))) {
                goto error;
            }
            for (j = 0; j < i; j++) {
                base = PyTuple_GET_ITEM(bases, j);
                PyList_SET_ITEM(new_bases, j, base);
                Py_INCREF(base);
            }
        }
        j = PyList_GET_SIZE(new_bases);
        if (PyList_SetSlice(new_bases, j, j, new_base) < 0) {
            goto error;
        }
        Py_DECREF(new_base);
    }
    if (!new_bases) {
        Py_INCREF(bases);
        return bases;
    }
    result = PyList_AsTuple(new_bases);
    Py_DECREF(new_bases);
    return result;
error:
    Py_XDECREF(new_bases);
    return NULL;
}

/* CalculateMetaclass */
  static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases;
#if CYTHON_ASSUME_SAFE_MACROS
    nbases = PyTuple_GET_SIZE(bases);
#else
    nbases = PyTuple_Size(bases);
    if (nbases < 0) return NULL;
#endif
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
#if CYTHON_ASSUME_SAFE_MACROS
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *tmp = PyTuple_GetItem(bases, i);
        if (!tmp) return NULL;
#endif
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* FetchSharedCythonModule */
  static PyObject *__Pyx_FetchSharedCythonABIModule(void) {
    return __Pyx_PyImport_AddModuleRef((char*) __PYX_ABI_MODULE_NAME);
}

/* FetchCommonType */
  static int __Pyx_VerifyCachedType(PyObject *cached_type,
                               const char *name,
                               Py_ssize_t basicsize,
                               Py_ssize_t expected_basicsize) {
    if (!PyType_Check(cached_type)) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s is not a type object", name);
        return -1;
    }
    if (basicsize != expected_basicsize) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s has the wrong size, try recompiling",
            name);
        return -1;
    }
    return 0;
}
#if !CYTHON_USE_TYPE_SPECS
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* abi_module;
    const char* object_name;
    PyTypeObject *cached_type = NULL;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) return NULL;
    object_name = strrchr(type->tp_name, '.');
    object_name = object_name ? object_name+1 : type->tp_name;
    cached_type = (PyTypeObject*) PyObject_GetAttrString(abi_module, object_name);
    if (cached_type) {
        if (__Pyx_VerifyCachedType(
              (PyObject *)cached_type,
              object_name,
              cached_type->tp_basicsize,
              type->tp_basicsize) < 0) {
            goto bad;
        }
        goto done;
    }
    if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
    PyErr_Clear();
    if (PyType_Ready(type) < 0) goto bad;
    if (PyObject_SetAttrString(abi_module, object_name, (PyObject *)type) < 0)
        goto bad;
    Py_INCREF(type);
    cached_type = type;
done:
    Py_DECREF(abi_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}
#else
static PyTypeObject *__Pyx_FetchCommonTypeFromSpec(PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *abi_module, *cached_type = NULL;
    const char* object_name = strrchr(spec->name, '.');
    object_name = object_name ? object_name+1 : spec->name;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) return NULL;
    cached_type = PyObject_GetAttrString(abi_module, object_name);
    if (cached_type) {
        Py_ssize_t basicsize;
#if CYTHON_COMPILING_IN_LIMITED_API
        PyObject *py_basicsize;
        py_basicsize = PyObject_GetAttrString(cached_type, "__basicsize__");
        if (unlikely(!py_basicsize)) goto bad;
        basicsize = PyLong_AsSsize_t(py_basicsize);
        Py_DECREF(py_basicsize);
        py_basicsize = 0;
        if (unlikely(basicsize == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
#else
        basicsize = likely(PyType_Check(cached_type)) ? ((PyTypeObject*) cached_type)->tp_basicsize : -1;
#endif
        if (__Pyx_VerifyCachedType(
              cached_type,
              object_name,
              basicsize,
              spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    }
    if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
    PyErr_Clear();
    CYTHON_UNUSED_VAR(module);
    cached_type = __Pyx_PyType_FromModuleAndSpec(abi_module, spec, bases);
    if (unlikely(!cached_type)) goto bad;
    if (unlikely(__Pyx_fix_up_extension_type_from_spec(spec, (PyTypeObject *) cached_type) < 0)) goto bad;
    if (PyObject_SetAttrString(abi_module, object_name, cached_type) < 0) goto bad;
done:
    Py_DECREF(abi_module);
    assert(cached_type == NULL || PyType_Check(cached_type));
    return (PyTypeObject *) cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}
#endif

/* PyVectorcallFastCallDict */
  #if CYTHON_METH_FASTCALL
static PyObject *__Pyx_PyVectorcall_FastCallDict_kw(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    PyObject *res = NULL;
    PyObject *kwnames;
    PyObject **newargs;
    PyObject **kwvalues;
    Py_ssize_t i, pos;
    size_t j;
    PyObject *key, *value;
    unsigned long keys_are_strings;
    Py_ssize_t nkw = PyDict_GET_SIZE(kw);
    newargs = (PyObject **)PyMem_Malloc((nargs + (size_t)nkw) * sizeof(args[0]));
    if (unlikely(newargs == NULL)) {
        PyErr_NoMemory();
        return NULL;
    }
    for (j = 0; j < nargs; j++) newargs[j] = args[j];
    kwnames = PyTuple_New(nkw);
    if (unlikely(kwnames == NULL)) {
        PyMem_Free(newargs);
        return NULL;
    }
    kwvalues = newargs + nargs;
    pos = i = 0;
    keys_are_strings = Py_TPFLAGS_UNICODE_SUBCLASS;
    while (PyDict_Next(kw, &pos, &key, &value)) {
        keys_are_strings &= Py_TYPE(key)->tp_flags;
        Py_INCREF(key);
        Py_INCREF(value);
        PyTuple_SET_ITEM(kwnames, i, key);
        kwvalues[i] = value;
        i++;
    }
    if (unlikely(!keys_are_strings)) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        goto cleanup;
    }
    res = vc(func, newargs, nargs, kwnames);
cleanup:
    Py_DECREF(kwnames);
    for (i = 0; i < nkw; i++)
        Py_DECREF(kwvalues[i]);
    PyMem_Free(newargs);
    return res;
}
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    if (likely(kw == NULL) || PyDict_GET_SIZE(kw) == 0) {
        return vc(func, args, nargs, NULL);
    }
    return __Pyx_PyVectorcall_FastCallDict_kw(func, vc, args, nargs, kw);
}
#endif

/* CythonFunctionShared */
  #if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc) {
    if (__Pyx_CyFunction_Check(func)) {
        return PyCFunction_GetFunction(((__pyx_CyFunctionObject*)func)->func) == (PyCFunction) cfunc;
    } else if (PyCFunction_Check(func)) {
        return PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
    }
    return 0;
}
#else
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc) {
    return __Pyx_CyOrPyCFunction_Check(func) && __Pyx_CyOrPyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
}
#endif
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj) {
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    __Pyx_Py_XDECREF_SET(
        __Pyx_CyFunction_GetClassObj(f),
            ((classobj) ? __Pyx_NewRef(classobj) : NULL));
#else
    __Pyx_Py_XDECREF_SET(
        ((PyCMethodObject *) (f))->mm_class,
        (PyTypeObject*)((classobj) ? __Pyx_NewRef(classobj) : NULL));
#endif
}
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, void *closure)
{
    CYTHON_UNUSED_VAR(closure);
    if (unlikely(op->func_doc == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_doc = PyObject_GetAttrString(op->func, "__doc__");
        if (unlikely(!op->func_doc)) return NULL;
#else
        if (((PyCFunctionObject*)op)->m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
#endif
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_doc, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_name == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_name = PyObject_GetAttrString(op->func, "__name__");
#elif PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_name, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_qualname, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_dict, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(op);
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = __Pyx_PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = __Pyx_PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyTuple_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__defaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->defaults_tuple, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->defaults_tuple;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__kwdefaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->defaults_kwdict, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->defaults_kwdict;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value || value == Py_None) {
        value = NULL;
    } else if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_annotations, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->func_annotations;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine(__pyx_CyFunctionObject *op, void *context) {
    int is_coroutine;
    CYTHON_UNUSED_VAR(context);
    if (op->func_is_coroutine) {
        return __Pyx_NewRef(op->func_is_coroutine);
    }
    is_coroutine = op->flags & __Pyx_CYFUNCTION_COROUTINE;
#if PY_VERSION_HEX >= 0x03050000
    if (is_coroutine) {
        PyObject *module, *fromlist, *marker = __pyx_n_s_is_coroutine;
        fromlist = PyList_New(1);
        if (unlikely(!fromlist)) return NULL;
        Py_INCREF(marker);
#if CYTHON_ASSUME_SAFE_MACROS
        PyList_SET_ITEM(fromlist, 0, marker);
#else
        if (unlikely(PyList_SetItem(fromlist, 0, marker) < 0)) {
            Py_DECREF(marker);
            Py_DECREF(fromlist);
            return NULL;
        }
#endif
        module = PyImport_ImportModuleLevelObject(__pyx_n_s_asyncio_coroutines, NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
        if (unlikely(!module)) goto ignore;
        op->func_is_coroutine = __Pyx_PyObject_GetAttrStr(module, marker);
        Py_DECREF(module);
        if (likely(op->func_is_coroutine)) {
            return __Pyx_NewRef(op->func_is_coroutine);
        }
ignore:
        PyErr_Clear();
    }
#endif
    op->func_is_coroutine = __Pyx_PyBool_FromLong(is_coroutine);
    return __Pyx_NewRef(op->func_is_coroutine);
}
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *
__Pyx_CyFunction_get_module(__pyx_CyFunctionObject *op, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_GetAttrString(op->func, "__module__");
}
static int
__Pyx_CyFunction_set_module(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_SetAttrString(op->func, "__module__", value);
}
#endif
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {(char *) "_is_coroutine", (getter)__Pyx_CyFunction_get_is_coroutine, 0, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", (getter)__Pyx_CyFunction_get_module, (setter)__Pyx_CyFunction_set_module, 0, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
#if !CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), 0, 0},
#endif
#if CYTHON_USE_TYPE_SPECS
    {(char *) "__dictoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_dict), READONLY, 0},
#if CYTHON_METH_FASTCALL
#if CYTHON_BACKPORT_VECTORCALL
    {(char *) "__vectorcalloffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_vectorcall), READONLY, 0},
#else
#if !CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__vectorcalloffset__", T_PYSSIZET, offsetof(PyCFunctionObject, vectorcall), READONLY, 0},
#endif
#endif
#endif
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_weakreflist), READONLY, 0},
#else
    {(char *) "__weaklistoffset__", T_PYSSIZET, offsetof(PyCFunctionObject, m_weakreflist), READONLY, 0},
#endif
#endif
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, PyObject *args)
{
    CYTHON_UNUSED_VAR(args);
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(((PyCFunctionObject*)m)->m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) (((PyCFunctionObject*)cyfunc)->m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyCFunctionObject *cf = (PyCFunctionObject*) op;
#endif
    if (unlikely(op == NULL))
        return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    op->func = PyCFunction_NewEx(ml, (PyObject*)op, module);
    if (unlikely(!op->func)) return NULL;
#endif
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
#if !CYTHON_COMPILING_IN_LIMITED_API
    cf->m_ml = ml;
    cf->m_self = (PyObject *) op;
#endif
    Py_XINCREF(closure);
    op->func_closure = closure;
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_XINCREF(module);
    cf->m_module = module;
#endif
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_classobj = NULL;
#else
    ((PyCMethodObject*)op)->mm_class = NULL;
#endif
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    op->func_is_coroutine = NULL;
#if CYTHON_METH_FASTCALL
    switch (ml->ml_flags & (METH_VARARGS | METH_FASTCALL | METH_NOARGS | METH_O | METH_KEYWORDS | METH_METHOD)) {
    case METH_NOARGS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_NOARGS;
        break;
    case METH_O:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_O;
        break;
    case METH_METHOD | METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD;
        break;
    case METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS;
        break;
    case METH_VARARGS | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = NULL;
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        Py_DECREF(op);
        return NULL;
    }
#endif
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func);
#else
    Py_CLEAR(((PyCFunctionObject*)m)->m_module);
#endif
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
#if PY_VERSION_HEX < 0x030900B1
    Py_CLEAR(__Pyx_CyFunction_GetClassObj(m));
#else
    {
        PyObject *cls = (PyObject*) ((PyCMethodObject *) (m))->mm_class;
        ((PyCMethodObject *) (m))->mm_class = NULL;
        Py_XDECREF(cls);
    }
#endif
#endif
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    Py_CLEAR(m->func_is_coroutine);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    __Pyx_PyHeapTypeObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func);
#else
    Py_VISIT(((PyCFunctionObject*)m)->m_module);
#endif
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(__Pyx_CyFunction_GetClassObj(m));
#endif
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    Py_VISIT(m->func_is_coroutine);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *f = ((__pyx_CyFunctionObject*)func)->func;
    PyObject *py_name = NULL;
    PyCFunction meth;
    int flags;
    meth = PyCFunction_GetFunction(f);
    if (unlikely(!meth)) return NULL;
    flags = PyCFunction_GetFlags(f);
    if (unlikely(flags < 0)) return NULL;
#else
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    int flags = f->m_ml->ml_flags;
#endif
    Py_ssize_t size;
    switch (flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 0))
                return (*meth)(self, NULL);
#if CYTHON_COMPILING_IN_LIMITED_API
            py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
            if (!py_name) return NULL;
            PyErr_Format(PyExc_TypeError,
                "%.200S() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                py_name, size);
            Py_DECREF(py_name);
#else
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
#endif
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = __Pyx_PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
#if CYTHON_COMPILING_IN_LIMITED_API
            py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
            if (!py_name) return NULL;
            PyErr_Format(PyExc_TypeError,
                "%.200S() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                py_name, size);
            Py_DECREF(py_name);
#else
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
#endif
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        return NULL;
    }
#if CYTHON_COMPILING_IN_LIMITED_API
    py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
    if (!py_name) return NULL;
    PyErr_Format(PyExc_TypeError, "%.200S() takes no keyword arguments",
                 py_name);
    Py_DECREF(py_name);
#else
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
#endif
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *self, *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)func)->func);
    if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
    self = ((PyCFunctionObject*)func)->m_self;
#endif
    result = __Pyx_CyFunction_CallMethod(func, self, arg, kw);
    return result;
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
#if CYTHON_METH_FASTCALL
     __pyx_vectorcallfunc vc = __Pyx_CyFunction_func_vectorcall(cyfunc);
    if (vc) {
#if CYTHON_ASSUME_SAFE_MACROS
        return __Pyx_PyVectorcall_FastCallDict(func, vc, &PyTuple_GET_ITEM(args, 0), (size_t)PyTuple_GET_SIZE(args), kw);
#else
        (void) &__Pyx_PyVectorcall_FastCallDict;
        return PyVectorcall_Call(func, args, kw);
#endif
    }
#endif
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
#if CYTHON_ASSUME_SAFE_MACROS
        argc = PyTuple_GET_SIZE(args);
#else
        argc = PyTuple_Size(args);
        if (unlikely(!argc) < 0) return NULL;
#endif
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
#if PY_MAJOR_VERSION > 2
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
#else
            PyErr_SetString(PyExc_TypeError,
                            "unbound method needs an argument");
#endif
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE int __Pyx_CyFunction_Vectorcall_CheckArgs(__pyx_CyFunctionObject *cyfunc, Py_ssize_t nargs, PyObject *kwnames)
{
    int ret = 0;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        if (unlikely(nargs < 1)) {
            PyErr_Format(PyExc_TypeError, "%.200s() needs an argument",
                         ((PyCFunctionObject*)cyfunc)->m_ml->ml_name);
            return -1;
        }
        ret = 1;
    }
    if (unlikely(kwnames) && unlikely(PyTuple_GET_SIZE(kwnames))) {
        PyErr_Format(PyExc_TypeError,
                     "%.200s() takes no keyword arguments", ((PyCFunctionObject*)cyfunc)->m_ml->ml_name);
        return -1;
    }
    return ret;
}
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 0)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
            def->ml_name, nargs);
        return NULL;
    }
    return def->ml_meth(self, NULL);
}
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 1)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
            def->ml_name, nargs);
        return NULL;
    }
    return def->ml_meth(self, args[0]);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    return ((_PyCFunctionFastWithKeywords)(void(*)(void))def->ml_meth)(self, args, nargs, kwnames);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
    PyTypeObject *cls = (PyTypeObject *) __Pyx_CyFunction_GetClassObj(cyfunc);
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCMethod)(void(*)(void))def->ml_meth)(self, cls, args, (size_t)nargs, kwnames);
}
#endif
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_CyFunctionType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_CyFunction_dealloc},
    {Py_tp_repr, (void *)__Pyx_CyFunction_repr},
    {Py_tp_call, (void *)__Pyx_CyFunction_CallAsMethod},
    {Py_tp_traverse, (void *)__Pyx_CyFunction_traverse},
    {Py_tp_clear, (void *)__Pyx_CyFunction_clear},
    {Py_tp_methods, (void *)__pyx_CyFunction_methods},
    {Py_tp_members, (void *)__pyx_CyFunction_members},
    {Py_tp_getset, (void *)__pyx_CyFunction_getsets},
    {Py_tp_descr_get, (void *)__Pyx_PyMethod_New},
    {0, 0},
};
static PyType_Spec __pyx_CyFunctionType_spec = {
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if (defined(_Py_TPFLAGS_HAVE_VECTORCALL) && CYTHON_METH_FASTCALL)
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    __pyx_CyFunctionType_slots
};
#else
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
#if !CYTHON_METH_FASTCALL
    0,
#elif CYTHON_BACKPORT_VECTORCALL
    (printfunc)offsetof(__pyx_CyFunctionObject, func_vectorcall),
#else
    offsetof(PyCFunctionObject, vectorcall),
#endif
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if defined(_Py_TPFLAGS_HAVE_VECTORCALL) && CYTHON_METH_FASTCALL
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_PyMethod_New,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if __PYX_NEED_TP_PRINT_SLOT
    0,
#endif
#if PY_VERSION_HEX >= 0x030C0000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
    0,
#endif
};
#endif
static int __pyx_CyFunction_init(PyObject *module) {
#if CYTHON_USE_TYPE_SPECS
    __pyx_CyFunctionType = __Pyx_FetchCommonTypeFromSpec(module, &__pyx_CyFunctionType_spec, NULL);
#else
    CYTHON_UNUSED_VAR(module);
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
#endif
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
  static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* PyObjectCall2Args */
  static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args[3] = {NULL, arg1, arg2};
    return __Pyx_PyObject_FastCall(function, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectLookupSpecial */
  #if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error) {
    PyObject *res;
    PyTypeObject *tp = Py_TYPE(obj);
#if PY_MAJOR_VERSION < 3
    if (unlikely(PyInstance_Check(obj)))
        return with_error ? __Pyx_PyObject_GetAttrStr(obj, attr_name) : __Pyx_PyObject_GetAttrStrNoError(obj, attr_name);
#endif
    res = _PyType_Lookup(tp, attr_name);
    if (likely(res)) {
        descrgetfunc f = Py_TYPE(res)->tp_descr_get;
        if (!f) {
            Py_INCREF(res);
        } else {
            res = f(res, obj, (PyObject *)tp);
        }
    } else if (with_error) {
        PyErr_SetObject(PyExc_AttributeError, attr_name);
    }
    return res;
}
#endif

/* Py3ClassCreate */
  static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStrNoError(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs[3] = {NULL, name, bases};
            ns = __Pyx_PyObject_FastCallDict(prep, pargs+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, mkw);
            Py_DECREF(prep);
        } else {
            if (unlikely(PyErr_Occurred()))
                return NULL;
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
#if PY_VERSION_HEX >= 0x03030000
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
#else
    CYTHON_MAYBE_UNUSED_VAR(qualname);
#endif
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
#if PY_VERSION_HEX < 0x030600A4 && CYTHON_PEP487_INIT_SUBCLASS
static int __Pyx_SetNamesPEP487(PyObject *type_obj) {
    PyTypeObject *type = (PyTypeObject*) type_obj;
    PyObject *names_to_set, *key, *value, *set_name, *tmp;
    Py_ssize_t i = 0;
#if CYTHON_USE_TYPE_SLOTS
    names_to_set = PyDict_Copy(type->tp_dict);
#else
    {
        PyObject *d = PyObject_GetAttr(type_obj, __pyx_n_s_dict);
        names_to_set = NULL;
        if (likely(d)) {
            PyObject *names_to_set = PyDict_New();
            int ret = likely(names_to_set) ? PyDict_Update(names_to_set, d) : -1;
            Py_DECREF(d);
            if (unlikely(ret < 0))
                Py_CLEAR(names_to_set);
        }
    }
#endif
    if (unlikely(names_to_set == NULL))
        goto bad;
    while (PyDict_Next(names_to_set, &i, &key, &value)) {
        set_name = __Pyx_PyObject_LookupSpecialNoError(value, __pyx_n_s_set_name);
        if (unlikely(set_name != NULL)) {
            tmp = __Pyx_PyObject_Call2Args(set_name, type_obj, key);
            Py_DECREF(set_name);
            if (unlikely(tmp == NULL)) {
                __Pyx_TypeName value_type_name =
                    __Pyx_PyType_GetName(Py_TYPE(value));
                __Pyx_TypeName type_name = __Pyx_PyType_GetName(type);
                PyErr_Format(PyExc_RuntimeError,
#if PY_MAJOR_VERSION >= 3
                    "Error calling __set_name__ on '" __Pyx_FMT_TYPENAME "' instance %R " "in '" __Pyx_FMT_TYPENAME "'",
                    value_type_name, key, type_name);
#else
                    "Error calling __set_name__ on '" __Pyx_FMT_TYPENAME "' instance %.100s in '" __Pyx_FMT_TYPENAME "'",
                    value_type_name,
                    PyString_Check(key) ? PyString_AS_STRING(key) : "?",
                    type_name);
#endif
                goto bad;
            } else {
                Py_DECREF(tmp);
            }
        }
        else if (unlikely(PyErr_Occurred())) {
            goto bad;
        }
    }
    Py_DECREF(names_to_set);
    return 0;
bad:
    Py_XDECREF(names_to_set);
    return -1;
}
static PyObject *__Pyx_InitSubclassPEP487(PyObject *type_obj, PyObject *mkw) {
#if CYTHON_USE_TYPE_SLOTS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyTypeObject *type = (PyTypeObject*) type_obj;
    PyObject *mro = type->tp_mro;
    Py_ssize_t i, nbases;
    if (unlikely(!mro)) goto done;
    (void) &__Pyx_GetBuiltinName;
    Py_INCREF(mro);
    nbases = PyTuple_GET_SIZE(mro);
    assert(PyTuple_GET_ITEM(mro, 0) == type_obj);
    for (i = 1; i < nbases-1; i++) {
        PyObject *base, *dict, *meth;
        base = PyTuple_GET_ITEM(mro, i);
        dict = ((PyTypeObject *)base)->tp_dict;
        meth = __Pyx_PyDict_GetItemStrWithError(dict, __pyx_n_s_init_subclass);
        if (unlikely(meth)) {
            descrgetfunc f = Py_TYPE(meth)->tp_descr_get;
            PyObject *res;
            Py_INCREF(meth);
            if (likely(f)) {
                res = f(meth, NULL, type_obj);
                Py_DECREF(meth);
                if (unlikely(!res)) goto bad;
                meth = res;
            }
            res = __Pyx_PyObject_FastCallDict(meth, NULL, 0, mkw);
            Py_DECREF(meth);
            if (unlikely(!res)) goto bad;
            Py_DECREF(res);
            goto done;
        } else if (unlikely(PyErr_Occurred())) {
            goto bad;
        }
    }
done:
    Py_XDECREF(mro);
    return type_obj;
bad:
    Py_XDECREF(mro);
    Py_DECREF(type_obj);
    return NULL;
#else
    PyObject *super_type, *super, *func, *res;
#if CYTHON_COMPILING_IN_PYPY && !defined(PySuper_Type)
    super_type = __Pyx_GetBuiltinName(__pyx_n_s_super);
#else
    super_type = (PyObject*) &PySuper_Type;
    (void) &__Pyx_GetBuiltinName;
#endif
    super = likely(super_type) ? __Pyx_PyObject_Call2Args(super_type, type_obj, type_obj) : NULL;
#if CYTHON_COMPILING_IN_PYPY && !defined(PySuper_Type)
    Py_XDECREF(super_type);
#endif
    if (unlikely(!super)) {
        Py_CLEAR(type_obj);
        goto done;
    }
    func = __Pyx_PyObject_GetAttrStrNoError(super, __pyx_n_s_init_subclass);
    Py_DECREF(super);
    if (likely(!func)) {
        if (unlikely(PyErr_Occurred()))
            Py_CLEAR(type_obj);
        goto done;
    }
    res = __Pyx_PyObject_FastCallDict(func, NULL, 0, mkw);
    Py_DECREF(func);
    if (unlikely(!res))
        Py_CLEAR(type_obj);
    Py_XDECREF(res);
done:
    return type_obj;
#endif
}
#endif
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result;
    PyObject *owned_metaclass = NULL;
    PyObject *margs[4] = {NULL, name, bases, dict};
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    result = __Pyx_PyObject_FastCallDict(metaclass, margs+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET,
#if PY_VERSION_HEX < 0x030600A4
        (metaclass == (PyObject*)&PyType_Type) ? NULL : mkw
#else
        mkw
#endif
    );
    Py_XDECREF(owned_metaclass);
#if PY_VERSION_HEX < 0x030600A4 && CYTHON_PEP487_INIT_SUBCLASS
    if (likely(result) && likely(PyType_Check(result))) {
        if (unlikely(__Pyx_SetNamesPEP487(result) < 0)) {
            Py_CLEAR(result);
        } else {
            result = __Pyx_InitSubclassPEP487(result, mkw);
        }
    }
#else
    (void) &__Pyx_GetBuiltinName;
#endif
    return result;
}

/* CLineInTraceback */
  #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    CYTHON_MAYBE_UNUSED_VAR(tstate);
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStrNoError(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
  #if !CYTHON_COMPILING_IN_LIMITED_API
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}
#endif

/* AddTraceback */
  #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyCode_Replace_For_AddTraceback(PyObject *code, PyObject *scratch_dict,
                                                       PyObject *firstlineno, PyObject *name) {
    PyObject *replace = NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_firstlineno", firstlineno))) return NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_name", name))) return NULL;
    replace = PyObject_GetAttrString(code, "replace");
    if (likely(replace)) {
        PyObject *result;
        result = PyObject_Call(replace, __pyx_empty_tuple, scratch_dict);
        Py_DECREF(replace);
        return result;
    }
    PyErr_Clear();
    #if __PYX_LIMITED_VERSION_HEX < 0x030780000
    {
        PyObject *compiled = NULL, *result = NULL;
        if (unlikely(PyDict_SetItemString(scratch_dict, "code", code))) return NULL;
        if (unlikely(PyDict_SetItemString(scratch_dict, "type", (PyObject*)(&PyType_Type)))) return NULL;
        compiled = Py_CompileString(
            "out = type(code)(\n"
            "  code.co_argcount, code.co_kwonlyargcount, code.co_nlocals, code.co_stacksize,\n"
            "  code.co_flags, code.co_code, code.co_consts, code.co_names,\n"
            "  code.co_varnames, code.co_filename, co_name, co_firstlineno,\n"
            "  code.co_lnotab)\n", "<dummy>", Py_file_input);
        if (!compiled) return NULL;
        result = PyEval_EvalCode(compiled, scratch_dict, scratch_dict);
        Py_DECREF(compiled);
        if (!result) PyErr_Print();
        Py_DECREF(result);
        result = PyDict_GetItemString(scratch_dict, "out");
        if (result) Py_INCREF(result);
        return result;
    }
    #else
    return NULL;
    #endif
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyObject *code_object = NULL, *py_py_line = NULL, *py_funcname = NULL, *dict = NULL;
    PyObject *replace = NULL, *getframe = NULL, *frame = NULL;
    PyObject *exc_type, *exc_value, *exc_traceback;
    int success = 0;
    if (c_line) {
        (void) __pyx_cfilenm;
        (void) __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
    }
    PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
    code_object = Py_CompileString("_getframe()", filename, Py_eval_input);
    if (unlikely(!code_object)) goto bad;
    py_py_line = PyLong_FromLong(py_line);
    if (unlikely(!py_py_line)) goto bad;
    py_funcname = PyUnicode_FromString(funcname);
    if (unlikely(!py_funcname)) goto bad;
    dict = PyDict_New();
    if (unlikely(!dict)) goto bad;
    {
        PyObject *old_code_object = code_object;
        code_object = __Pyx_PyCode_Replace_For_AddTraceback(code_object, dict, py_py_line, py_funcname);
        Py_DECREF(old_code_object);
    }
    if (unlikely(!code_object)) goto bad;
    getframe = PySys_GetObject("_getframe");
    if (unlikely(!getframe)) goto bad;
    if (unlikely(PyDict_SetItemString(dict, "_getframe", getframe))) goto bad;
    frame = PyEval_EvalCode(code_object, dict, dict);
    if (unlikely(!frame) || frame == Py_None) goto bad;
    success = 1;
  bad:
    PyErr_Restore(exc_type, exc_value, exc_traceback);
    Py_XDECREF(code_object);
    Py_XDECREF(py_py_line);
    Py_XDECREF(py_funcname);
    Py_XDECREF(dict);
    Py_XDECREF(replace);
    if (success) {
        PyTraceBack_Here(
            (struct _frame*)frame);
    }
    Py_XDECREF(frame);
}
#else
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}
#endif

#if PY_MAJOR_VERSION < 3
static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags) {
    __Pyx_TypeName obj_type_name;
    if (PyObject_CheckBuffer(obj)) return PyObject_GetBuffer(obj, view, flags);
        if (__Pyx_TypeCheck(obj, __pyx_ptype_7cpython_5array_array)) return __pyx_pw_7cpython_5array_5array_1__getbuffer__(obj, view, flags);
    obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
                 "'" __Pyx_FMT_TYPENAME "' does not have the buffer interface",
                 obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return -1;
}
static void __Pyx_ReleaseBuffer(Py_buffer *view) {
    PyObject *obj = view->obj;
    if (!obj) return;
    if (PyObject_CheckBuffer(obj)) {
        PyBuffer_Release(view);
        return;
    }
    if ((0)) {}
        else if (__Pyx_TypeCheck(obj, __pyx_ptype_7cpython_5array_array)) __pyx_pw_7cpython_5array_5array_3__releasebuffer__(obj, view);
    view->obj = NULL;
    Py_DECREF(obj);
}
#endif


  /* CIntFromPyVerify */
  #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_FT4222_STATUS(FT4222_STATUS value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const FT4222_STATUS neg_one = (FT4222_STATUS) -1, const_zero = (FT4222_STATUS) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(FT4222_STATUS) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(FT4222_STATUS) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(FT4222_STATUS) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(FT4222_STATUS) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(FT4222_STATUS) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        return _PyLong_FromByteArray(bytes, sizeof(FT4222_STATUS),
                                     little, !is_unsigned);
#else
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(FT4222_STATUS));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_DWORD(DWORD value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const DWORD neg_one = (DWORD) -1, const_zero = (DWORD) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(DWORD) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(DWORD) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(DWORD) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(DWORD) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(DWORD) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        return _PyLong_FromByteArray(bytes, sizeof(DWORD),
                                     little, !is_unsigned);
#else
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(DWORD));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_ULONG(ULONG value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const ULONG neg_one = (ULONG) -1, const_zero = (ULONG) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(ULONG) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(ULONG) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(ULONG) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(ULONG) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(ULONG) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        return _PyLong_FromByteArray(bytes, sizeof(ULONG),
                                     little, !is_unsigned);
#else
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(ULONG));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
  static CYTHON_INLINE DWORD __Pyx_PyInt_As_DWORD(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const DWORD neg_one = (DWORD) -1, const_zero = (DWORD) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(DWORD) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(DWORD, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (DWORD) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(DWORD, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(DWORD) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(DWORD, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(DWORD) >= 2 * PyLong_SHIFT)) {
                                return (DWORD) (((((DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(DWORD) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(DWORD, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(DWORD) >= 3 * PyLong_SHIFT)) {
                                return (DWORD) (((((((DWORD)digits[2]) << PyLong_SHIFT) | (DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(DWORD) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(DWORD, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(DWORD) >= 4 * PyLong_SHIFT)) {
                                return (DWORD) (((((((((DWORD)digits[3]) << PyLong_SHIFT) | (DWORD)digits[2]) << PyLong_SHIFT) | (DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (DWORD) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(DWORD) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(DWORD, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(DWORD) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(DWORD, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(DWORD, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(DWORD) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(DWORD, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(DWORD) - 1 > 2 * PyLong_SHIFT)) {
                                return (DWORD) (((DWORD)-1)*(((((DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(DWORD) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(DWORD, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(DWORD) - 1 > 2 * PyLong_SHIFT)) {
                                return (DWORD) ((((((DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(DWORD) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(DWORD, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(DWORD) - 1 > 3 * PyLong_SHIFT)) {
                                return (DWORD) (((DWORD)-1)*(((((((DWORD)digits[2]) << PyLong_SHIFT) | (DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(DWORD) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(DWORD, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(DWORD) - 1 > 3 * PyLong_SHIFT)) {
                                return (DWORD) ((((((((DWORD)digits[2]) << PyLong_SHIFT) | (DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(DWORD) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(DWORD, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(DWORD) - 1 > 4 * PyLong_SHIFT)) {
                                return (DWORD) (((DWORD)-1)*(((((((((DWORD)digits[3]) << PyLong_SHIFT) | (DWORD)digits[2]) << PyLong_SHIFT) | (DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(DWORD) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(DWORD, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(DWORD) - 1 > 4 * PyLong_SHIFT)) {
                                return (DWORD) ((((((((((DWORD)digits[3]) << PyLong_SHIFT) | (DWORD)digits[2]) << PyLong_SHIFT) | (DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(DWORD) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(DWORD, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(DWORD) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(DWORD, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
            DWORD val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
#if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
#endif
            if (likely(v)) {
                int ret = -1;
#if PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                ret = _PyLong_AsByteArray((PyLongObject *)v,
                                           bytes, sizeof(val),
                                           is_little, !is_unsigned);
#else
                PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
                int bits, remaining_bits, is_negative = 0;
                long idigit;
                int chunk_size = (sizeof(long) < 8) ? 30 : 62;
                if (unlikely(!PyLong_CheckExact(v))) {
                    PyObject *tmp = v;
                    v = PyNumber_Long(v);
                    assert(PyLong_CheckExact(v));
                    Py_DECREF(tmp);
                    if (unlikely(!v)) return (DWORD) -1;
                }
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                if (Py_SIZE(x) == 0)
                    return (DWORD) 0;
                is_negative = Py_SIZE(x) < 0;
#else
                {
                    int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                    if (unlikely(result < 0))
                        return (DWORD) -1;
                    is_negative = result == 1;
                }
#endif
                if (is_unsigned && unlikely(is_negative)) {
                    goto raise_neg_overflow;
                } else if (is_negative) {
                    stepval = PyNumber_Invert(v);
                    if (unlikely(!stepval))
                        return (DWORD) -1;
                } else {
                    stepval = __Pyx_NewRef(v);
                }
                val = (DWORD) 0;
                mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
                shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
                for (bits = 0; bits < (int) sizeof(DWORD) * 8 - chunk_size; bits += chunk_size) {
                    PyObject *tmp, *digit;
                    digit = PyNumber_And(stepval, mask);
                    if (unlikely(!digit)) goto done;
                    idigit = PyLong_AsLong(digit);
                    Py_DECREF(digit);
                    if (unlikely(idigit < 0)) goto done;
                    tmp = PyNumber_Rshift(stepval, shift);
                    if (unlikely(!tmp)) goto done;
                    Py_DECREF(stepval); stepval = tmp;
                    val |= ((DWORD) idigit) << bits;
                    #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                    if (Py_SIZE(stepval) == 0)
                        goto unpacking_done;
                    #endif
                }
                idigit = PyLong_AsLong(stepval);
                if (unlikely(idigit < 0)) goto done;
                remaining_bits = ((int) sizeof(DWORD) * 8) - bits - (is_unsigned ? 0 : 1);
                if (unlikely(idigit >= (1L << remaining_bits)))
                    goto raise_overflow;
                val |= ((DWORD) idigit) << bits;
            #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
            unpacking_done:
            #endif
                if (!is_unsigned) {
                    if (unlikely(val & (((DWORD) 1) << (sizeof(DWORD) * 8 - 1))))
                        goto raise_overflow;
                    if (is_negative)
                        val = ~val;
                }
                ret = 0;
            done:
                Py_XDECREF(shift);
                Py_XDECREF(mask);
                Py_XDECREF(stepval);
#endif
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
            return (DWORD) -1;
        }
    } else {
        DWORD val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (DWORD) -1;
        val = __Pyx_PyInt_As_DWORD(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to DWORD");
    return (DWORD) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to DWORD");
    return (DWORD) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(size_t) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(size_t) >= 2 * PyLong_SHIFT)) {
                                return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(size_t) >= 3 * PyLong_SHIFT)) {
                                return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(size_t) >= 4 * PyLong_SHIFT)) {
                                return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(size_t) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                                return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                                return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                                return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                                return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                                return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                                return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(size_t) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(size_t) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
#if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
#endif
            if (likely(v)) {
                int ret = -1;
#if PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                ret = _PyLong_AsByteArray((PyLongObject *)v,
                                           bytes, sizeof(val),
                                           is_little, !is_unsigned);
#else
                PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
                int bits, remaining_bits, is_negative = 0;
                long idigit;
                int chunk_size = (sizeof(long) < 8) ? 30 : 62;
                if (unlikely(!PyLong_CheckExact(v))) {
                    PyObject *tmp = v;
                    v = PyNumber_Long(v);
                    assert(PyLong_CheckExact(v));
                    Py_DECREF(tmp);
                    if (unlikely(!v)) return (size_t) -1;
                }
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                if (Py_SIZE(x) == 0)
                    return (size_t) 0;
                is_negative = Py_SIZE(x) < 0;
#else
                {
                    int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                    if (unlikely(result < 0))
                        return (size_t) -1;
                    is_negative = result == 1;
                }
#endif
                if (is_unsigned && unlikely(is_negative)) {
                    goto raise_neg_overflow;
                } else if (is_negative) {
                    stepval = PyNumber_Invert(v);
                    if (unlikely(!stepval))
                        return (size_t) -1;
                } else {
                    stepval = __Pyx_NewRef(v);
                }
                val = (size_t) 0;
                mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
                shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
                for (bits = 0; bits < (int) sizeof(size_t) * 8 - chunk_size; bits += chunk_size) {
                    PyObject *tmp, *digit;
                    digit = PyNumber_And(stepval, mask);
                    if (unlikely(!digit)) goto done;
                    idigit = PyLong_AsLong(digit);
                    Py_DECREF(digit);
                    if (unlikely(idigit < 0)) goto done;
                    tmp = PyNumber_Rshift(stepval, shift);
                    if (unlikely(!tmp)) goto done;
                    Py_DECREF(stepval); stepval = tmp;
                    val |= ((size_t) idigit) << bits;
                    #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                    if (Py_SIZE(stepval) == 0)
                        goto unpacking_done;
                    #endif
                }
                idigit = PyLong_AsLong(stepval);
                if (unlikely(idigit < 0)) goto done;
                remaining_bits = ((int) sizeof(size_t) * 8) - bits - (is_unsigned ? 0 : 1);
                if (unlikely(idigit >= (1L << remaining_bits)))
                    goto raise_overflow;
                val |= ((size_t) idigit) << bits;
            #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
            unpacking_done:
            #endif
                if (!is_unsigned) {
                    if (unlikely(val & (((size_t) 1) << (sizeof(size_t) * 8 - 1))))
                        goto raise_overflow;
                    if (is_negative)
                        val = ~val;
                }
                ret = 0;
            done:
                Py_XDECREF(shift);
                Py_XDECREF(mask);
                Py_XDECREF(stepval);
#endif
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE ULONG __Pyx_PyInt_As_ULONG(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const ULONG neg_one = (ULONG) -1, const_zero = (ULONG) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(ULONG) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(ULONG, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (ULONG) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(ULONG, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(ULONG) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(ULONG, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(ULONG) >= 2 * PyLong_SHIFT)) {
                                return (ULONG) (((((ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(ULONG) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(ULONG, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(ULONG) >= 3 * PyLong_SHIFT)) {
                                return (ULONG) (((((((ULONG)digits[2]) << PyLong_SHIFT) | (ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(ULONG) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(ULONG, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(ULONG) >= 4 * PyLong_SHIFT)) {
                                return (ULONG) (((((((((ULONG)digits[3]) << PyLong_SHIFT) | (ULONG)digits[2]) << PyLong_SHIFT) | (ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (ULONG) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(ULONG) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(ULONG, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(ULONG) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(ULONG, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(ULONG, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(ULONG) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(ULONG, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(ULONG) - 1 > 2 * PyLong_SHIFT)) {
                                return (ULONG) (((ULONG)-1)*(((((ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(ULONG) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(ULONG, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(ULONG) - 1 > 2 * PyLong_SHIFT)) {
                                return (ULONG) ((((((ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(ULONG) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(ULONG, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(ULONG) - 1 > 3 * PyLong_SHIFT)) {
                                return (ULONG) (((ULONG)-1)*(((((((ULONG)digits[2]) << PyLong_SHIFT) | (ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(ULONG) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(ULONG, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(ULONG) - 1 > 3 * PyLong_SHIFT)) {
                                return (ULONG) ((((((((ULONG)digits[2]) << PyLong_SHIFT) | (ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(ULONG) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(ULONG, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(ULONG) - 1 > 4 * PyLong_SHIFT)) {
                                return (ULONG) (((ULONG)-1)*(((((((((ULONG)digits[3]) << PyLong_SHIFT) | (ULONG)digits[2]) << PyLong_SHIFT) | (ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(ULONG) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(ULONG, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(ULONG) - 1 > 4 * PyLong_SHIFT)) {
                                return (ULONG) ((((((((((ULONG)digits[3]) << PyLong_SHIFT) | (ULONG)digits[2]) << PyLong_SHIFT) | (ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(ULONG) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(ULONG, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(ULONG) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(ULONG, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
            ULONG val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
#if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
#endif
            if (likely(v)) {
                int ret = -1;
#if PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                ret = _PyLong_AsByteArray((PyLongObject *)v,
                                           bytes, sizeof(val),
                                           is_little, !is_unsigned);
#else
                PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
                int bits, remaining_bits, is_negative = 0;
                long idigit;
                int chunk_size = (sizeof(long) < 8) ? 30 : 62;
                if (unlikely(!PyLong_CheckExact(v))) {
                    PyObject *tmp = v;
                    v = PyNumber_Long(v);
                    assert(PyLong_CheckExact(v));
                    Py_DECREF(tmp);
                    if (unlikely(!v)) return (ULONG) -1;
                }
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                if (Py_SIZE(x) == 0)
                    return (ULONG) 0;
                is_negative = Py_SIZE(x) < 0;
#else
                {
                    int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                    if (unlikely(result < 0))
                        return (ULONG) -1;
                    is_negative = result == 1;
                }
#endif
                if (is_unsigned && unlikely(is_negative)) {
                    goto raise_neg_overflow;
                } else if (is_negative) {
                    stepval = PyNumber_Invert(v);
                    if (unlikely(!stepval))
                        return (ULONG) -1;
                } else {
                    stepval = __Pyx_NewRef(v);
                }
                val = (ULONG) 0;
                mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
                shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
                for (bits = 0; bits < (int) sizeof(ULONG) * 8 - chunk_size; bits += chunk_size) {
                    PyObject *tmp, *digit;
                    digit = PyNumber_And(stepval, mask);
                    if (unlikely(!digit)) goto done;
                    idigit = PyLong_AsLong(digit);
                    Py_DECREF(digit);
                    if (unlikely(idigit < 0)) goto done;
                    tmp = PyNumber_Rshift(stepval, shift);
                    if (unlikely(!tmp)) goto done;
                    Py_DECREF(stepval); stepval = tmp;
                    val |= ((ULONG) idigit) << bits;
                    #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                    if (Py_SIZE(stepval) == 0)
                        goto unpacking_done;
                    #endif
                }
                idigit = PyLong_AsLong(stepval);
                if (unlikely(idigit < 0)) goto done;
                remaining_bits = ((int) sizeof(ULONG) * 8) - bits - (is_unsigned ? 0 : 1);
                if (unlikely(idigit >= (1L << remaining_bits)))
                    goto raise_overflow;
                val |= ((ULONG) idigit) << bits;
            #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
            unpacking_done:
            #endif
                if (!is_unsigned) {
                    if (unlikely(val & (((ULONG) 1) << (sizeof(ULONG) * 8 - 1))))
                        goto raise_overflow;
                    if (is_negative)
                        val = ~val;
                }
                ret = 0;
            done:
                Py_XDECREF(shift);
                Py_XDECREF(mask);
                Py_XDECREF(stepval);
#endif
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
            return (ULONG) -1;
        }
    } else {
        ULONG val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (ULONG) -1;
        val = __Pyx_PyInt_As_ULONG(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to ULONG");
    return (ULONG) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to ULONG");
    return (ULONG) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE FT4222_ClockRate __Pyx_PyInt_As_FT4222_ClockRate(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const FT4222_ClockRate neg_one = (FT4222_ClockRate) -1, const_zero = (FT4222_ClockRate) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(FT4222_ClockRate) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (FT4222_ClockRate) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(FT4222_ClockRate) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_ClockRate) >= 2 * PyLong_SHIFT)) {
                                return (FT4222_ClockRate) (((((FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(FT4222_ClockRate) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_ClockRate) >= 3 * PyLong_SHIFT)) {
                                return (FT4222_ClockRate) (((((((FT4222_ClockRate)digits[2]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(FT4222_ClockRate) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_ClockRate) >= 4 * PyLong_SHIFT)) {
                                return (FT4222_ClockRate) (((((((((FT4222_ClockRate)digits[3]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[2]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (FT4222_ClockRate) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(FT4222_ClockRate) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_ClockRate, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(FT4222_ClockRate) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_ClockRate, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(FT4222_ClockRate) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_ClockRate) - 1 > 2 * PyLong_SHIFT)) {
                                return (FT4222_ClockRate) (((FT4222_ClockRate)-1)*(((((FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(FT4222_ClockRate) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_ClockRate) - 1 > 2 * PyLong_SHIFT)) {
                                return (FT4222_ClockRate) ((((((FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(FT4222_ClockRate) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_ClockRate) - 1 > 3 * PyLong_SHIFT)) {
                                return (FT4222_ClockRate) (((FT4222_ClockRate)-1)*(((((((FT4222_ClockRate)digits[2]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(FT4222_ClockRate) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_ClockRate) - 1 > 3 * PyLong_SHIFT)) {
                                return (FT4222_ClockRate) ((((((((FT4222_ClockRate)digits[2]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(FT4222_ClockRate) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_ClockRate) - 1 > 4 * PyLong_SHIFT)) {
                                return (FT4222_ClockRate) (((FT4222_ClockRate)-1)*(((((((((FT4222_ClockRate)digits[3]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[2]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(FT4222_ClockRate) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_ClockRate) - 1 > 4 * PyLong_SHIFT)) {
                                return (FT4222_ClockRate) ((((((((((FT4222_ClockRate)digits[3]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[2]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(FT4222_ClockRate) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_ClockRate, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(FT4222_ClockRate) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_ClockRate, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        PyErr_SetString(PyExc_RuntimeError,
                        "_PyLong_AsByteArray() not available, cannot convert large enums");
        return (FT4222_ClockRate) -1;
    } else {
        FT4222_ClockRate val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (FT4222_ClockRate) -1;
        val = __Pyx_PyInt_As_FT4222_ClockRate(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to FT4222_ClockRate");
    return (FT4222_ClockRate) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to FT4222_ClockRate");
    return (FT4222_ClockRate) -1;
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_FT4222_ClockRate(FT4222_ClockRate value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const FT4222_ClockRate neg_one = (FT4222_ClockRate) -1, const_zero = (FT4222_ClockRate) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(FT4222_ClockRate) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(FT4222_ClockRate) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(FT4222_ClockRate) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(FT4222_ClockRate) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(FT4222_ClockRate) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        return _PyLong_FromByteArray(bytes, sizeof(FT4222_ClockRate),
                                     little, !is_unsigned);
#else
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(FT4222_ClockRate));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
  static CYTHON_INLINE BOOL __Pyx_PyInt_As_BOOL(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const BOOL neg_one = (BOOL) -1, const_zero = (BOOL) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(BOOL) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(BOOL, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (BOOL) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(BOOL, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(BOOL) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(BOOL, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(BOOL) >= 2 * PyLong_SHIFT)) {
                                return (BOOL) (((((BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(BOOL) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(BOOL, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(BOOL) >= 3 * PyLong_SHIFT)) {
                                return (BOOL) (((((((BOOL)digits[2]) << PyLong_SHIFT) | (BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(BOOL) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(BOOL, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(BOOL) >= 4 * PyLong_SHIFT)) {
                                return (BOOL) (((((((((BOOL)digits[3]) << PyLong_SHIFT) | (BOOL)digits[2]) << PyLong_SHIFT) | (BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (BOOL) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(BOOL) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(BOOL, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(BOOL) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(BOOL, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(BOOL, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(BOOL) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(BOOL, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(BOOL) - 1 > 2 * PyLong_SHIFT)) {
                                return (BOOL) (((BOOL)-1)*(((((BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(BOOL) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(BOOL, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(BOOL) - 1 > 2 * PyLong_SHIFT)) {
                                return (BOOL) ((((((BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(BOOL) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(BOOL, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(BOOL) - 1 > 3 * PyLong_SHIFT)) {
                                return (BOOL) (((BOOL)-1)*(((((((BOOL)digits[2]) << PyLong_SHIFT) | (BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(BOOL) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(BOOL, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(BOOL) - 1 > 3 * PyLong_SHIFT)) {
                                return (BOOL) ((((((((BOOL)digits[2]) << PyLong_SHIFT) | (BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(BOOL) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(BOOL, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(BOOL) - 1 > 4 * PyLong_SHIFT)) {
                                return (BOOL) (((BOOL)-1)*(((((((((BOOL)digits[3]) << PyLong_SHIFT) | (BOOL)digits[2]) << PyLong_SHIFT) | (BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(BOOL) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(BOOL, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(BOOL) - 1 > 4 * PyLong_SHIFT)) {
                                return (BOOL) ((((((((((BOOL)digits[3]) << PyLong_SHIFT) | (BOOL)digits[2]) << PyLong_SHIFT) | (BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(BOOL) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(BOOL, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(BOOL) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(BOOL, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
            BOOL val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
#if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
#endif
            if (likely(v)) {
                int ret = -1;
#if PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                ret = _PyLong_AsByteArray((PyLongObject *)v,
                                           bytes, sizeof(val),
                                           is_little, !is_unsigned);
#else
                PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
                int bits, remaining_bits, is_negative = 0;
                long idigit;
                int chunk_size = (sizeof(long) < 8) ? 30 : 62;
                if (unlikely(!PyLong_CheckExact(v))) {
                    PyObject *tmp = v;
                    v = PyNumber_Long(v);
                    assert(PyLong_CheckExact(v));
                    Py_DECREF(tmp);
                    if (unlikely(!v)) return (BOOL) -1;
                }
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                if (Py_SIZE(x) == 0)
                    return (BOOL) 0;
                is_negative = Py_SIZE(x) < 0;
#else
                {
                    int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                    if (unlikely(result < 0))
                        return (BOOL) -1;
                    is_negative = result == 1;
                }
#endif
                if (is_unsigned && unlikely(is_negative)) {
                    goto raise_neg_overflow;
                } else if (is_negative) {
                    stepval = PyNumber_Invert(v);
                    if (unlikely(!stepval))
                        return (BOOL) -1;
                } else {
                    stepval = __Pyx_NewRef(v);
                }
                val = (BOOL) 0;
                mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
                shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
                for (bits = 0; bits < (int) sizeof(BOOL) * 8 - chunk_size; bits += chunk_size) {
                    PyObject *tmp, *digit;
                    digit = PyNumber_And(stepval, mask);
                    if (unlikely(!digit)) goto done;
                    idigit = PyLong_AsLong(digit);
                    Py_DECREF(digit);
                    if (unlikely(idigit < 0)) goto done;
                    tmp = PyNumber_Rshift(stepval, shift);
                    if (unlikely(!tmp)) goto done;
                    Py_DECREF(stepval); stepval = tmp;
                    val |= ((BOOL) idigit) << bits;
                    #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                    if (Py_SIZE(stepval) == 0)
                        goto unpacking_done;
                    #endif
                }
                idigit = PyLong_AsLong(stepval);
                if (unlikely(idigit < 0)) goto done;
                remaining_bits = ((int) sizeof(BOOL) * 8) - bits - (is_unsigned ? 0 : 1);
                if (unlikely(idigit >= (1L << remaining_bits)))
                    goto raise_overflow;
                val |= ((BOOL) idigit) << bits;
            #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
            unpacking_done:
            #endif
                if (!is_unsigned) {
                    if (unlikely(val & (((BOOL) 1) << (sizeof(BOOL) * 8 - 1))))
                        goto raise_overflow;
                    if (is_negative)
                        val = ~val;
                }
                ret = 0;
            done:
                Py_XDECREF(shift);
                Py_XDECREF(mask);
                Py_XDECREF(stepval);
#endif
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
            return (BOOL) -1;
        }
    } else {
        BOOL val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (BOOL) -1;
        val = __Pyx_PyInt_As_BOOL(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to BOOL");
    return (BOOL) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to BOOL");
    return (BOOL) -1;
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint16_t(uint16_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint16_t neg_one = (uint16_t) -1, const_zero = (uint16_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint16_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint16_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint16_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint16_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint16_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        return _PyLong_FromByteArray(bytes, sizeof(uint16_t),
                                     little, !is_unsigned);
#else
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(uint16_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
  static CYTHON_INLINE UCHAR __Pyx_PyInt_As_UCHAR(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const UCHAR neg_one = (UCHAR) -1, const_zero = (UCHAR) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(UCHAR) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(UCHAR, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (UCHAR) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(UCHAR, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(UCHAR) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(UCHAR, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(UCHAR) >= 2 * PyLong_SHIFT)) {
                                return (UCHAR) (((((UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(UCHAR) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(UCHAR, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(UCHAR) >= 3 * PyLong_SHIFT)) {
                                return (UCHAR) (((((((UCHAR)digits[2]) << PyLong_SHIFT) | (UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(UCHAR) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(UCHAR, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(UCHAR) >= 4 * PyLong_SHIFT)) {
                                return (UCHAR) (((((((((UCHAR)digits[3]) << PyLong_SHIFT) | (UCHAR)digits[2]) << PyLong_SHIFT) | (UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (UCHAR) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(UCHAR) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(UCHAR, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(UCHAR) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(UCHAR, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(UCHAR, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(UCHAR) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(UCHAR, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(UCHAR) - 1 > 2 * PyLong_SHIFT)) {
                                return (UCHAR) (((UCHAR)-1)*(((((UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(UCHAR) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(UCHAR, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(UCHAR) - 1 > 2 * PyLong_SHIFT)) {
                                return (UCHAR) ((((((UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(UCHAR) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(UCHAR, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(UCHAR) - 1 > 3 * PyLong_SHIFT)) {
                                return (UCHAR) (((UCHAR)-1)*(((((((UCHAR)digits[2]) << PyLong_SHIFT) | (UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(UCHAR) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(UCHAR, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(UCHAR) - 1 > 3 * PyLong_SHIFT)) {
                                return (UCHAR) ((((((((UCHAR)digits[2]) << PyLong_SHIFT) | (UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(UCHAR) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(UCHAR, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(UCHAR) - 1 > 4 * PyLong_SHIFT)) {
                                return (UCHAR) (((UCHAR)-1)*(((((((((UCHAR)digits[3]) << PyLong_SHIFT) | (UCHAR)digits[2]) << PyLong_SHIFT) | (UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(UCHAR) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(UCHAR, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(UCHAR) - 1 > 4 * PyLong_SHIFT)) {
                                return (UCHAR) ((((((((((UCHAR)digits[3]) << PyLong_SHIFT) | (UCHAR)digits[2]) << PyLong_SHIFT) | (UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(UCHAR) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(UCHAR, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(UCHAR) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(UCHAR, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
            UCHAR val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
#if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
#endif
            if (likely(v)) {
                int ret = -1;
#if PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                ret = _PyLong_AsByteArray((PyLongObject *)v,
                                           bytes, sizeof(val),
                                           is_little, !is_unsigned);
#else
                PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
                int bits, remaining_bits, is_negative = 0;
                long idigit;
                int chunk_size = (sizeof(long) < 8) ? 30 : 62;
                if (unlikely(!PyLong_CheckExact(v))) {
                    PyObject *tmp = v;
                    v = PyNumber_Long(v);
                    assert(PyLong_CheckExact(v));
                    Py_DECREF(tmp);
                    if (unlikely(!v)) return (UCHAR) -1;
                }
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                if (Py_SIZE(x) == 0)
                    return (UCHAR) 0;
                is_negative = Py_SIZE(x) < 0;
#else
                {
                    int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                    if (unlikely(result < 0))
                        return (UCHAR) -1;
                    is_negative = result == 1;
                }
#endif
                if (is_unsigned && unlikely(is_negative)) {
                    goto raise_neg_overflow;
                } else if (is_negative) {
                    stepval = PyNumber_Invert(v);
                    if (unlikely(!stepval))
                        return (UCHAR) -1;
                } else {
                    stepval = __Pyx_NewRef(v);
                }
                val = (UCHAR) 0;
                mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
                shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
                for (bits = 0; bits < (int) sizeof(UCHAR) * 8 - chunk_size; bits += chunk_size) {
                    PyObject *tmp, *digit;
                    digit = PyNumber_And(stepval, mask);
                    if (unlikely(!digit)) goto done;
                    idigit = PyLong_AsLong(digit);
                    Py_DECREF(digit);
                    if (unlikely(idigit < 0)) goto done;
                    tmp = PyNumber_Rshift(stepval, shift);
                    if (unlikely(!tmp)) goto done;
                    Py_DECREF(stepval); stepval = tmp;
                    val |= ((UCHAR) idigit) << bits;
                    #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                    if (Py_SIZE(stepval) == 0)
                        goto unpacking_done;
                    #endif
                }
                idigit = PyLong_AsLong(stepval);
                if (unlikely(idigit < 0)) goto done;
                remaining_bits = ((int) sizeof(UCHAR) * 8) - bits - (is_unsigned ? 0 : 1);
                if (unlikely(idigit >= (1L << remaining_bits)))
                    goto raise_overflow;
                val |= ((UCHAR) idigit) << bits;
            #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
            unpacking_done:
            #endif
                if (!is_unsigned) {
                    if (unlikely(val & (((UCHAR) 1) << (sizeof(UCHAR) * 8 - 1))))
                        goto raise_overflow;
                    if (is_negative)
                        val = ~val;
                }
                ret = 0;
            done:
                Py_XDECREF(shift);
                Py_XDECREF(mask);
                Py_XDECREF(stepval);
#endif
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
            return (UCHAR) -1;
        }
    } else {
        UCHAR val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (UCHAR) -1;
        val = __Pyx_PyInt_As_UCHAR(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to UCHAR");
    return (UCHAR) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to UCHAR");
    return (UCHAR) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE USHORT __Pyx_PyInt_As_USHORT(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const USHORT neg_one = (USHORT) -1, const_zero = (USHORT) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(USHORT) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(USHORT, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (USHORT) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(USHORT, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(USHORT) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(USHORT, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(USHORT) >= 2 * PyLong_SHIFT)) {
                                return (USHORT) (((((USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(USHORT) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(USHORT, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(USHORT) >= 3 * PyLong_SHIFT)) {
                                return (USHORT) (((((((USHORT)digits[2]) << PyLong_SHIFT) | (USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(USHORT) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(USHORT, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(USHORT) >= 4 * PyLong_SHIFT)) {
                                return (USHORT) (((((((((USHORT)digits[3]) << PyLong_SHIFT) | (USHORT)digits[2]) << PyLong_SHIFT) | (USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (USHORT) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(USHORT) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(USHORT, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(USHORT) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(USHORT, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(USHORT, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(USHORT) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(USHORT, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(USHORT) - 1 > 2 * PyLong_SHIFT)) {
                                return (USHORT) (((USHORT)-1)*(((((USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(USHORT) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(USHORT, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(USHORT) - 1 > 2 * PyLong_SHIFT)) {
                                return (USHORT) ((((((USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(USHORT) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(USHORT, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(USHORT) - 1 > 3 * PyLong_SHIFT)) {
                                return (USHORT) (((USHORT)-1)*(((((((USHORT)digits[2]) << PyLong_SHIFT) | (USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(USHORT) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(USHORT, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(USHORT) - 1 > 3 * PyLong_SHIFT)) {
                                return (USHORT) ((((((((USHORT)digits[2]) << PyLong_SHIFT) | (USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(USHORT) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(USHORT, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(USHORT) - 1 > 4 * PyLong_SHIFT)) {
                                return (USHORT) (((USHORT)-1)*(((((((((USHORT)digits[3]) << PyLong_SHIFT) | (USHORT)digits[2]) << PyLong_SHIFT) | (USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(USHORT) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(USHORT, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(USHORT) - 1 > 4 * PyLong_SHIFT)) {
                                return (USHORT) ((((((((((USHORT)digits[3]) << PyLong_SHIFT) | (USHORT)digits[2]) << PyLong_SHIFT) | (USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(USHORT) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(USHORT, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(USHORT) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(USHORT, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
            USHORT val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
#if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
#endif
            if (likely(v)) {
                int ret = -1;
#if PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                ret = _PyLong_AsByteArray((PyLongObject *)v,
                                           bytes, sizeof(val),
                                           is_little, !is_unsigned);
#else
                PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
                int bits, remaining_bits, is_negative = 0;
                long idigit;
                int chunk_size = (sizeof(long) < 8) ? 30 : 62;
                if (unlikely(!PyLong_CheckExact(v))) {
                    PyObject *tmp = v;
                    v = PyNumber_Long(v);
                    assert(PyLong_CheckExact(v));
                    Py_DECREF(tmp);
                    if (unlikely(!v)) return (USHORT) -1;
                }
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                if (Py_SIZE(x) == 0)
                    return (USHORT) 0;
                is_negative = Py_SIZE(x) < 0;
#else
                {
                    int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                    if (unlikely(result < 0))
                        return (USHORT) -1;
                    is_negative = result == 1;
                }
#endif
                if (is_unsigned && unlikely(is_negative)) {
                    goto raise_neg_overflow;
                } else if (is_negative) {
                    stepval = PyNumber_Invert(v);
                    if (unlikely(!stepval))
                        return (USHORT) -1;
                } else {
                    stepval = __Pyx_NewRef(v);
                }
                val = (USHORT) 0;
                mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
                shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
                for (bits = 0; bits < (int) sizeof(USHORT) * 8 - chunk_size; bits += chunk_size) {
                    PyObject *tmp, *digit;
                    digit = PyNumber_And(stepval, mask);
                    if (unlikely(!digit)) goto done;
                    idigit = PyLong_AsLong(digit);
                    Py_DECREF(digit);
                    if (unlikely(idigit < 0)) goto done;
                    tmp = PyNumber_Rshift(stepval, shift);
                    if (unlikely(!tmp)) goto done;
                    Py_DECREF(stepval); stepval = tmp;
                    val |= ((USHORT) idigit) << bits;
                    #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                    if (Py_SIZE(stepval) == 0)
                        goto unpacking_done;
                    #endif
                }
                idigit = PyLong_AsLong(stepval);
                if (unlikely(idigit < 0)) goto done;
                remaining_bits = ((int) sizeof(USHORT) * 8) - bits - (is_unsigned ? 0 : 1);
                if (unlikely(idigit >= (1L << remaining_bits)))
                    goto raise_overflow;
                val |= ((USHORT) idigit) << bits;
            #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
            unpacking_done:
            #endif
                if (!is_unsigned) {
                    if (unlikely(val & (((USHORT) 1) << (sizeof(USHORT) * 8 - 1))))
                        goto raise_overflow;
                    if (is_negative)
                        val = ~val;
                }
                ret = 0;
            done:
                Py_XDECREF(shift);
                Py_XDECREF(mask);
                Py_XDECREF(stepval);
#endif
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
            return (USHORT) -1;
        }
    } else {
        USHORT val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (USHORT) -1;
        val = __Pyx_PyInt_As_USHORT(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to USHORT");
    return (USHORT) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to USHORT");
    return (USHORT) -1;
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_UCHAR(UCHAR value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const UCHAR neg_one = (UCHAR) -1, const_zero = (UCHAR) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(UCHAR) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(UCHAR) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(UCHAR) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(UCHAR) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(UCHAR) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        return _PyLong_FromByteArray(bytes, sizeof(UCHAR),
                                     little, !is_unsigned);
#else
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(UCHAR));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
  static CYTHON_INLINE GPIO_Dir __Pyx_PyInt_As_GPIO_Dir(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const GPIO_Dir neg_one = (GPIO_Dir) -1, const_zero = (GPIO_Dir) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(GPIO_Dir) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(GPIO_Dir, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GPIO_Dir) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(GPIO_Dir, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(GPIO_Dir) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Dir, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Dir) >= 2 * PyLong_SHIFT)) {
                                return (GPIO_Dir) (((((GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(GPIO_Dir) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Dir, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Dir) >= 3 * PyLong_SHIFT)) {
                                return (GPIO_Dir) (((((((GPIO_Dir)digits[2]) << PyLong_SHIFT) | (GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(GPIO_Dir) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Dir, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Dir) >= 4 * PyLong_SHIFT)) {
                                return (GPIO_Dir) (((((((((GPIO_Dir)digits[3]) << PyLong_SHIFT) | (GPIO_Dir)digits[2]) << PyLong_SHIFT) | (GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GPIO_Dir) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(GPIO_Dir) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Dir, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(GPIO_Dir) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Dir, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(GPIO_Dir, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(GPIO_Dir) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Dir, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Dir) - 1 > 2 * PyLong_SHIFT)) {
                                return (GPIO_Dir) (((GPIO_Dir)-1)*(((((GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(GPIO_Dir) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Dir, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Dir) - 1 > 2 * PyLong_SHIFT)) {
                                return (GPIO_Dir) ((((((GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(GPIO_Dir) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Dir, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Dir) - 1 > 3 * PyLong_SHIFT)) {
                                return (GPIO_Dir) (((GPIO_Dir)-1)*(((((((GPIO_Dir)digits[2]) << PyLong_SHIFT) | (GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(GPIO_Dir) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Dir, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Dir) - 1 > 3 * PyLong_SHIFT)) {
                                return (GPIO_Dir) ((((((((GPIO_Dir)digits[2]) << PyLong_SHIFT) | (GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(GPIO_Dir) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Dir, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Dir) - 1 > 4 * PyLong_SHIFT)) {
                                return (GPIO_Dir) (((GPIO_Dir)-1)*(((((((((GPIO_Dir)digits[3]) << PyLong_SHIFT) | (GPIO_Dir)digits[2]) << PyLong_SHIFT) | (GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(GPIO_Dir) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Dir, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Dir) - 1 > 4 * PyLong_SHIFT)) {
                                return (GPIO_Dir) ((((((((((GPIO_Dir)digits[3]) << PyLong_SHIFT) | (GPIO_Dir)digits[2]) << PyLong_SHIFT) | (GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(GPIO_Dir) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Dir, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(GPIO_Dir) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Dir, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        PyErr_SetString(PyExc_RuntimeError,
                        "_PyLong_AsByteArray() not available, cannot convert large enums");
        return (GPIO_Dir) -1;
    } else {
        GPIO_Dir val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GPIO_Dir) -1;
        val = __Pyx_PyInt_As_GPIO_Dir(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GPIO_Dir");
    return (GPIO_Dir) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GPIO_Dir");
    return (GPIO_Dir) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE GPIO_Port __Pyx_PyInt_As_GPIO_Port(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const GPIO_Port neg_one = (GPIO_Port) -1, const_zero = (GPIO_Port) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(GPIO_Port) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(GPIO_Port, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GPIO_Port) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(GPIO_Port, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(GPIO_Port) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Port, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Port) >= 2 * PyLong_SHIFT)) {
                                return (GPIO_Port) (((((GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(GPIO_Port) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Port, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Port) >= 3 * PyLong_SHIFT)) {
                                return (GPIO_Port) (((((((GPIO_Port)digits[2]) << PyLong_SHIFT) | (GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(GPIO_Port) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Port, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Port) >= 4 * PyLong_SHIFT)) {
                                return (GPIO_Port) (((((((((GPIO_Port)digits[3]) << PyLong_SHIFT) | (GPIO_Port)digits[2]) << PyLong_SHIFT) | (GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GPIO_Port) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(GPIO_Port) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Port, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(GPIO_Port) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Port, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(GPIO_Port, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(GPIO_Port) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Port, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Port) - 1 > 2 * PyLong_SHIFT)) {
                                return (GPIO_Port) (((GPIO_Port)-1)*(((((GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(GPIO_Port) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Port, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Port) - 1 > 2 * PyLong_SHIFT)) {
                                return (GPIO_Port) ((((((GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(GPIO_Port) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Port, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Port) - 1 > 3 * PyLong_SHIFT)) {
                                return (GPIO_Port) (((GPIO_Port)-1)*(((((((GPIO_Port)digits[2]) << PyLong_SHIFT) | (GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(GPIO_Port) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Port, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Port) - 1 > 3 * PyLong_SHIFT)) {
                                return (GPIO_Port) ((((((((GPIO_Port)digits[2]) << PyLong_SHIFT) | (GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(GPIO_Port) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Port, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Port) - 1 > 4 * PyLong_SHIFT)) {
                                return (GPIO_Port) (((GPIO_Port)-1)*(((((((((GPIO_Port)digits[3]) << PyLong_SHIFT) | (GPIO_Port)digits[2]) << PyLong_SHIFT) | (GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(GPIO_Port) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Port, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Port) - 1 > 4 * PyLong_SHIFT)) {
                                return (GPIO_Port) ((((((((((GPIO_Port)digits[3]) << PyLong_SHIFT) | (GPIO_Port)digits[2]) << PyLong_SHIFT) | (GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(GPIO_Port) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Port, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(GPIO_Port) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Port, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        PyErr_SetString(PyExc_RuntimeError,
                        "_PyLong_AsByteArray() not available, cannot convert large enums");
        return (GPIO_Port) -1;
    } else {
        GPIO_Port val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GPIO_Port) -1;
        val = __Pyx_PyInt_As_GPIO_Port(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GPIO_Port");
    return (GPIO_Port) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GPIO_Port");
    return (GPIO_Port) -1;
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_BOOL(BOOL value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const BOOL neg_one = (BOOL) -1, const_zero = (BOOL) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(BOOL) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(BOOL) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(BOOL) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(BOOL) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(BOOL) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        return _PyLong_FromByteArray(bytes, sizeof(BOOL),
                                     little, !is_unsigned);
#else
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(BOOL));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
  static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(int) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                                return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                                return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                                return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(int) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                                return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                                return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                                return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                                return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                                return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                                return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(int) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
#if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
#endif
            if (likely(v)) {
                int ret = -1;
#if PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                ret = _PyLong_AsByteArray((PyLongObject *)v,
                                           bytes, sizeof(val),
                                           is_little, !is_unsigned);
#else
                PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
                int bits, remaining_bits, is_negative = 0;
                long idigit;
                int chunk_size = (sizeof(long) < 8) ? 30 : 62;
                if (unlikely(!PyLong_CheckExact(v))) {
                    PyObject *tmp = v;
                    v = PyNumber_Long(v);
                    assert(PyLong_CheckExact(v));
                    Py_DECREF(tmp);
                    if (unlikely(!v)) return (int) -1;
                }
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                if (Py_SIZE(x) == 0)
                    return (int) 0;
                is_negative = Py_SIZE(x) < 0;
#else
                {
                    int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                    if (unlikely(result < 0))
                        return (int) -1;
                    is_negative = result == 1;
                }
#endif
                if (is_unsigned && unlikely(is_negative)) {
                    goto raise_neg_overflow;
                } else if (is_negative) {
                    stepval = PyNumber_Invert(v);
                    if (unlikely(!stepval))
                        return (int) -1;
                } else {
                    stepval = __Pyx_NewRef(v);
                }
                val = (int) 0;
                mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
                shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
                for (bits = 0; bits < (int) sizeof(int) * 8 - chunk_size; bits += chunk_size) {
                    PyObject *tmp, *digit;
                    digit = PyNumber_And(stepval, mask);
                    if (unlikely(!digit)) goto done;
                    idigit = PyLong_AsLong(digit);
                    Py_DECREF(digit);
                    if (unlikely(idigit < 0)) goto done;
                    tmp = PyNumber_Rshift(stepval, shift);
                    if (unlikely(!tmp)) goto done;
                    Py_DECREF(stepval); stepval = tmp;
                    val |= ((int) idigit) << bits;
                    #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                    if (Py_SIZE(stepval) == 0)
                        goto unpacking_done;
                    #endif
                }
                idigit = PyLong_AsLong(stepval);
                if (unlikely(idigit < 0)) goto done;
                remaining_bits = ((int) sizeof(int) * 8) - bits - (is_unsigned ? 0 : 1);
                if (unlikely(idigit >= (1L << remaining_bits)))
                    goto raise_overflow;
                val |= ((int) idigit) << bits;
            #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
            unpacking_done:
            #endif
                if (!is_unsigned) {
                    if (unlikely(val & (((int) 1) << (sizeof(int) * 8 - 1))))
                        goto raise_overflow;
                    if (is_negative)
                        val = ~val;
                }
                ret = 0;
            done:
                Py_XDECREF(shift);
                Py_XDECREF(mask);
                Py_XDECREF(stepval);
#endif
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE GPIO_Trigger __Pyx_PyInt_As_GPIO_Trigger(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const GPIO_Trigger neg_one = (GPIO_Trigger) -1, const_zero = (GPIO_Trigger) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(GPIO_Trigger) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(GPIO_Trigger, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GPIO_Trigger) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(GPIO_Trigger, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(GPIO_Trigger) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Trigger, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Trigger) >= 2 * PyLong_SHIFT)) {
                                return (GPIO_Trigger) (((((GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(GPIO_Trigger) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Trigger, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Trigger) >= 3 * PyLong_SHIFT)) {
                                return (GPIO_Trigger) (((((((GPIO_Trigger)digits[2]) << PyLong_SHIFT) | (GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(GPIO_Trigger) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Trigger, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Trigger) >= 4 * PyLong_SHIFT)) {
                                return (GPIO_Trigger) (((((((((GPIO_Trigger)digits[3]) << PyLong_SHIFT) | (GPIO_Trigger)digits[2]) << PyLong_SHIFT) | (GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GPIO_Trigger) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(GPIO_Trigger) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Trigger, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(GPIO_Trigger) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Trigger, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(GPIO_Trigger, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(GPIO_Trigger) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Trigger, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Trigger) - 1 > 2 * PyLong_SHIFT)) {
                                return (GPIO_Trigger) (((GPIO_Trigger)-1)*(((((GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(GPIO_Trigger) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Trigger, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Trigger) - 1 > 2 * PyLong_SHIFT)) {
                                return (GPIO_Trigger) ((((((GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(GPIO_Trigger) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Trigger, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Trigger) - 1 > 3 * PyLong_SHIFT)) {
                                return (GPIO_Trigger) (((GPIO_Trigger)-1)*(((((((GPIO_Trigger)digits[2]) << PyLong_SHIFT) | (GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(GPIO_Trigger) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Trigger, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Trigger) - 1 > 3 * PyLong_SHIFT)) {
                                return (GPIO_Trigger) ((((((((GPIO_Trigger)digits[2]) << PyLong_SHIFT) | (GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(GPIO_Trigger) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Trigger, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Trigger) - 1 > 4 * PyLong_SHIFT)) {
                                return (GPIO_Trigger) (((GPIO_Trigger)-1)*(((((((((GPIO_Trigger)digits[3]) << PyLong_SHIFT) | (GPIO_Trigger)digits[2]) << PyLong_SHIFT) | (GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(GPIO_Trigger) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(GPIO_Trigger, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(GPIO_Trigger) - 1 > 4 * PyLong_SHIFT)) {
                                return (GPIO_Trigger) ((((((((((GPIO_Trigger)digits[3]) << PyLong_SHIFT) | (GPIO_Trigger)digits[2]) << PyLong_SHIFT) | (GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(GPIO_Trigger) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Trigger, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(GPIO_Trigger) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Trigger, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        PyErr_SetString(PyExc_RuntimeError,
                        "_PyLong_AsByteArray() not available, cannot convert large enums");
        return (GPIO_Trigger) -1;
    } else {
        GPIO_Trigger val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GPIO_Trigger) -1;
        val = __Pyx_PyInt_As_GPIO_Trigger(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GPIO_Trigger");
    return (GPIO_Trigger) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GPIO_Trigger");
    return (GPIO_Trigger) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE uint16_t __Pyx_PyInt_As_uint16_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint16_t neg_one = (uint16_t) -1, const_zero = (uint16_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(uint16_t) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(uint16_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint16_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(uint16_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(uint16_t) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint16_t) >= 2 * PyLong_SHIFT)) {
                                return (uint16_t) (((((uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(uint16_t) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint16_t) >= 3 * PyLong_SHIFT)) {
                                return (uint16_t) (((((((uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(uint16_t) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint16_t) >= 4 * PyLong_SHIFT)) {
                                return (uint16_t) (((((((((uint16_t)digits[3]) << PyLong_SHIFT) | (uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint16_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(uint16_t) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(uint16_t) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(uint16_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(uint16_t) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint16_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint16_t) - 1 > 2 * PyLong_SHIFT)) {
                                return (uint16_t) (((uint16_t)-1)*(((((uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(uint16_t) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint16_t) - 1 > 2 * PyLong_SHIFT)) {
                                return (uint16_t) ((((((uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(uint16_t) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint16_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint16_t) - 1 > 3 * PyLong_SHIFT)) {
                                return (uint16_t) (((uint16_t)-1)*(((((((uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(uint16_t) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint16_t) - 1 > 3 * PyLong_SHIFT)) {
                                return (uint16_t) ((((((((uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(uint16_t) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint16_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint16_t) - 1 > 4 * PyLong_SHIFT)) {
                                return (uint16_t) (((uint16_t)-1)*(((((((((uint16_t)digits[3]) << PyLong_SHIFT) | (uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(uint16_t) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint16_t) - 1 > 4 * PyLong_SHIFT)) {
                                return (uint16_t) ((((((((((uint16_t)digits[3]) << PyLong_SHIFT) | (uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(uint16_t) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(uint16_t) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
            uint16_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
#if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
#endif
            if (likely(v)) {
                int ret = -1;
#if PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                ret = _PyLong_AsByteArray((PyLongObject *)v,
                                           bytes, sizeof(val),
                                           is_little, !is_unsigned);
#else
                PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
                int bits, remaining_bits, is_negative = 0;
                long idigit;
                int chunk_size = (sizeof(long) < 8) ? 30 : 62;
                if (unlikely(!PyLong_CheckExact(v))) {
                    PyObject *tmp = v;
                    v = PyNumber_Long(v);
                    assert(PyLong_CheckExact(v));
                    Py_DECREF(tmp);
                    if (unlikely(!v)) return (uint16_t) -1;
                }
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                if (Py_SIZE(x) == 0)
                    return (uint16_t) 0;
                is_negative = Py_SIZE(x) < 0;
#else
                {
                    int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                    if (unlikely(result < 0))
                        return (uint16_t) -1;
                    is_negative = result == 1;
                }
#endif
                if (is_unsigned && unlikely(is_negative)) {
                    goto raise_neg_overflow;
                } else if (is_negative) {
                    stepval = PyNumber_Invert(v);
                    if (unlikely(!stepval))
                        return (uint16_t) -1;
                } else {
                    stepval = __Pyx_NewRef(v);
                }
                val = (uint16_t) 0;
                mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
                shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
                for (bits = 0; bits < (int) sizeof(uint16_t) * 8 - chunk_size; bits += chunk_size) {
                    PyObject *tmp, *digit;
                    digit = PyNumber_And(stepval, mask);
                    if (unlikely(!digit)) goto done;
                    idigit = PyLong_AsLong(digit);
                    Py_DECREF(digit);
                    if (unlikely(idigit < 0)) goto done;
                    tmp = PyNumber_Rshift(stepval, shift);
                    if (unlikely(!tmp)) goto done;
                    Py_DECREF(stepval); stepval = tmp;
                    val |= ((uint16_t) idigit) << bits;
                    #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                    if (Py_SIZE(stepval) == 0)
                        goto unpacking_done;
                    #endif
                }
                idigit = PyLong_AsLong(stepval);
                if (unlikely(idigit < 0)) goto done;
                remaining_bits = ((int) sizeof(uint16_t) * 8) - bits - (is_unsigned ? 0 : 1);
                if (unlikely(idigit >= (1L << remaining_bits)))
                    goto raise_overflow;
                val |= ((uint16_t) idigit) << bits;
            #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
            unpacking_done:
            #endif
                if (!is_unsigned) {
                    if (unlikely(val & (((uint16_t) 1) << (sizeof(uint16_t) * 8 - 1))))
                        goto raise_overflow;
                    if (is_negative)
                        val = ~val;
                }
                ret = 0;
            done:
                Py_XDECREF(shift);
                Py_XDECREF(mask);
                Py_XDECREF(stepval);
#endif
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
            return (uint16_t) -1;
        }
    } else {
        uint16_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint16_t) -1;
        val = __Pyx_PyInt_As_uint16_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint16_t");
    return (uint16_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint16_t");
    return (uint16_t) -1;
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GPIO_Trigger(GPIO_Trigger value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const GPIO_Trigger neg_one = (GPIO_Trigger) -1, const_zero = (GPIO_Trigger) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(GPIO_Trigger) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(GPIO_Trigger) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GPIO_Trigger) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(GPIO_Trigger) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GPIO_Trigger) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        return _PyLong_FromByteArray(bytes, sizeof(GPIO_Trigger),
                                     little, !is_unsigned);
#else
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(GPIO_Trigger));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
  static CYTHON_INLINE uint32_t __Pyx_PyInt_As_uint32_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint32_t neg_one = (uint32_t) -1, const_zero = (uint32_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(uint32_t) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(uint32_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint32_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(uint32_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(uint32_t) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint32_t) >= 2 * PyLong_SHIFT)) {
                                return (uint32_t) (((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(uint32_t) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint32_t) >= 3 * PyLong_SHIFT)) {
                                return (uint32_t) (((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(uint32_t) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint32_t) >= 4 * PyLong_SHIFT)) {
                                return (uint32_t) (((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint32_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(uint32_t) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(uint32_t) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(uint32_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(uint32_t) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT)) {
                                return (uint32_t) (((uint32_t)-1)*(((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(uint32_t) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT)) {
                                return (uint32_t) ((((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT)) {
                                return (uint32_t) (((uint32_t)-1)*(((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(uint32_t) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT)) {
                                return (uint32_t) ((((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT)) {
                                return (uint32_t) (((uint32_t)-1)*(((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(uint32_t) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT)) {
                                return (uint32_t) ((((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(uint32_t) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(uint32_t) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
            uint32_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
#if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
#endif
            if (likely(v)) {
                int ret = -1;
#if PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                ret = _PyLong_AsByteArray((PyLongObject *)v,
                                           bytes, sizeof(val),
                                           is_little, !is_unsigned);
#else
                PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
                int bits, remaining_bits, is_negative = 0;
                long idigit;
                int chunk_size = (sizeof(long) < 8) ? 30 : 62;
                if (unlikely(!PyLong_CheckExact(v))) {
                    PyObject *tmp = v;
                    v = PyNumber_Long(v);
                    assert(PyLong_CheckExact(v));
                    Py_DECREF(tmp);
                    if (unlikely(!v)) return (uint32_t) -1;
                }
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                if (Py_SIZE(x) == 0)
                    return (uint32_t) 0;
                is_negative = Py_SIZE(x) < 0;
#else
                {
                    int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                    if (unlikely(result < 0))
                        return (uint32_t) -1;
                    is_negative = result == 1;
                }
#endif
                if (is_unsigned && unlikely(is_negative)) {
                    goto raise_neg_overflow;
                } else if (is_negative) {
                    stepval = PyNumber_Invert(v);
                    if (unlikely(!stepval))
                        return (uint32_t) -1;
                } else {
                    stepval = __Pyx_NewRef(v);
                }
                val = (uint32_t) 0;
                mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
                shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
                for (bits = 0; bits < (int) sizeof(uint32_t) * 8 - chunk_size; bits += chunk_size) {
                    PyObject *tmp, *digit;
                    digit = PyNumber_And(stepval, mask);
                    if (unlikely(!digit)) goto done;
                    idigit = PyLong_AsLong(digit);
                    Py_DECREF(digit);
                    if (unlikely(idigit < 0)) goto done;
                    tmp = PyNumber_Rshift(stepval, shift);
                    if (unlikely(!tmp)) goto done;
                    Py_DECREF(stepval); stepval = tmp;
                    val |= ((uint32_t) idigit) << bits;
                    #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                    if (Py_SIZE(stepval) == 0)
                        goto unpacking_done;
                    #endif
                }
                idigit = PyLong_AsLong(stepval);
                if (unlikely(idigit < 0)) goto done;
                remaining_bits = ((int) sizeof(uint32_t) * 8) - bits - (is_unsigned ? 0 : 1);
                if (unlikely(idigit >= (1L << remaining_bits)))
                    goto raise_overflow;
                val |= ((uint32_t) idigit) << bits;
            #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
            unpacking_done:
            #endif
                if (!is_unsigned) {
                    if (unlikely(val & (((uint32_t) 1) << (sizeof(uint32_t) * 8 - 1))))
                        goto raise_overflow;
                    if (is_negative)
                        val = ~val;
                }
                ret = 0;
            done:
                Py_XDECREF(shift);
                Py_XDECREF(mask);
                Py_XDECREF(stepval);
#endif
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
            return (uint32_t) -1;
        }
    } else {
        uint32_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint32_t) -1;
        val = __Pyx_PyInt_As_uint32_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint32_t");
    return (uint32_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint32_t");
    return (uint32_t) -1;
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
#else
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(long));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
  static CYTHON_INLINE uint8_t __Pyx_PyInt_As_uint8_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint8_t neg_one = (uint8_t) -1, const_zero = (uint8_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(uint8_t) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(uint8_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint8_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(uint8_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(uint8_t) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint8_t) >= 2 * PyLong_SHIFT)) {
                                return (uint8_t) (((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(uint8_t) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint8_t) >= 3 * PyLong_SHIFT)) {
                                return (uint8_t) (((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(uint8_t) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint8_t) >= 4 * PyLong_SHIFT)) {
                                return (uint8_t) (((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint8_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(uint8_t) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(uint8_t) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(uint8_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(uint8_t) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT)) {
                                return (uint8_t) (((uint8_t)-1)*(((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(uint8_t) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT)) {
                                return (uint8_t) ((((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT)) {
                                return (uint8_t) (((uint8_t)-1)*(((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(uint8_t) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT)) {
                                return (uint8_t) ((((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint8_t) - 1 > 4 * PyLong_SHIFT)) {
                                return (uint8_t) (((uint8_t)-1)*(((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(uint8_t) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(uint8_t) - 1 > 4 * PyLong_SHIFT)) {
                                return (uint8_t) ((((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(uint8_t) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(uint8_t) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
            uint8_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
#if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
#endif
            if (likely(v)) {
                int ret = -1;
#if PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                ret = _PyLong_AsByteArray((PyLongObject *)v,
                                           bytes, sizeof(val),
                                           is_little, !is_unsigned);
#else
                PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
                int bits, remaining_bits, is_negative = 0;
                long idigit;
                int chunk_size = (sizeof(long) < 8) ? 30 : 62;
                if (unlikely(!PyLong_CheckExact(v))) {
                    PyObject *tmp = v;
                    v = PyNumber_Long(v);
                    assert(PyLong_CheckExact(v));
                    Py_DECREF(tmp);
                    if (unlikely(!v)) return (uint8_t) -1;
                }
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                if (Py_SIZE(x) == 0)
                    return (uint8_t) 0;
                is_negative = Py_SIZE(x) < 0;
#else
                {
                    int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                    if (unlikely(result < 0))
                        return (uint8_t) -1;
                    is_negative = result == 1;
                }
#endif
                if (is_unsigned && unlikely(is_negative)) {
                    goto raise_neg_overflow;
                } else if (is_negative) {
                    stepval = PyNumber_Invert(v);
                    if (unlikely(!stepval))
                        return (uint8_t) -1;
                } else {
                    stepval = __Pyx_NewRef(v);
                }
                val = (uint8_t) 0;
                mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
                shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
                for (bits = 0; bits < (int) sizeof(uint8_t) * 8 - chunk_size; bits += chunk_size) {
                    PyObject *tmp, *digit;
                    digit = PyNumber_And(stepval, mask);
                    if (unlikely(!digit)) goto done;
                    idigit = PyLong_AsLong(digit);
                    Py_DECREF(digit);
                    if (unlikely(idigit < 0)) goto done;
                    tmp = PyNumber_Rshift(stepval, shift);
                    if (unlikely(!tmp)) goto done;
                    Py_DECREF(stepval); stepval = tmp;
                    val |= ((uint8_t) idigit) << bits;
                    #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                    if (Py_SIZE(stepval) == 0)
                        goto unpacking_done;
                    #endif
                }
                idigit = PyLong_AsLong(stepval);
                if (unlikely(idigit < 0)) goto done;
                remaining_bits = ((int) sizeof(uint8_t) * 8) - bits - (is_unsigned ? 0 : 1);
                if (unlikely(idigit >= (1L << remaining_bits)))
                    goto raise_overflow;
                val |= ((uint8_t) idigit) << bits;
            #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
            unpacking_done:
            #endif
                if (!is_unsigned) {
                    if (unlikely(val & (((uint8_t) 1) << (sizeof(uint8_t) * 8 - 1))))
                        goto raise_overflow;
                    if (is_negative)
                        val = ~val;
                }
                ret = 0;
            done:
                Py_XDECREF(shift);
                Py_XDECREF(mask);
                Py_XDECREF(stepval);
#endif
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
            return (uint8_t) -1;
        }
    } else {
        uint8_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint8_t) -1;
        val = __Pyx_PyInt_As_uint8_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint8_t");
    return (uint8_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint8_t");
    return (uint8_t) -1;
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint8_t(uint8_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint8_t neg_one = (uint8_t) -1, const_zero = (uint8_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint8_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint8_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint8_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint8_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint8_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        return _PyLong_FromByteArray(bytes, sizeof(uint8_t),
                                     little, !is_unsigned);
#else
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(uint8_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
  static CYTHON_INLINE SPI_DrivingStrength __Pyx_PyInt_As_SPI_DrivingStrength(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const SPI_DrivingStrength neg_one = (SPI_DrivingStrength) -1, const_zero = (SPI_DrivingStrength) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(SPI_DrivingStrength) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (SPI_DrivingStrength) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(SPI_DrivingStrength) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_DrivingStrength) >= 2 * PyLong_SHIFT)) {
                                return (SPI_DrivingStrength) (((((SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(SPI_DrivingStrength) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_DrivingStrength) >= 3 * PyLong_SHIFT)) {
                                return (SPI_DrivingStrength) (((((((SPI_DrivingStrength)digits[2]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(SPI_DrivingStrength) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_DrivingStrength) >= 4 * PyLong_SHIFT)) {
                                return (SPI_DrivingStrength) (((((((((SPI_DrivingStrength)digits[3]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[2]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (SPI_DrivingStrength) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(SPI_DrivingStrength) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(SPI_DrivingStrength, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(SPI_DrivingStrength) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(SPI_DrivingStrength, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(SPI_DrivingStrength) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_DrivingStrength) - 1 > 2 * PyLong_SHIFT)) {
                                return (SPI_DrivingStrength) (((SPI_DrivingStrength)-1)*(((((SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(SPI_DrivingStrength) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_DrivingStrength) - 1 > 2 * PyLong_SHIFT)) {
                                return (SPI_DrivingStrength) ((((((SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(SPI_DrivingStrength) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_DrivingStrength) - 1 > 3 * PyLong_SHIFT)) {
                                return (SPI_DrivingStrength) (((SPI_DrivingStrength)-1)*(((((((SPI_DrivingStrength)digits[2]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(SPI_DrivingStrength) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_DrivingStrength) - 1 > 3 * PyLong_SHIFT)) {
                                return (SPI_DrivingStrength) ((((((((SPI_DrivingStrength)digits[2]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(SPI_DrivingStrength) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_DrivingStrength) - 1 > 4 * PyLong_SHIFT)) {
                                return (SPI_DrivingStrength) (((SPI_DrivingStrength)-1)*(((((((((SPI_DrivingStrength)digits[3]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[2]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(SPI_DrivingStrength) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_DrivingStrength) - 1 > 4 * PyLong_SHIFT)) {
                                return (SPI_DrivingStrength) ((((((((((SPI_DrivingStrength)digits[3]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[2]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(SPI_DrivingStrength) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(SPI_DrivingStrength, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(SPI_DrivingStrength) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(SPI_DrivingStrength, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        PyErr_SetString(PyExc_RuntimeError,
                        "_PyLong_AsByteArray() not available, cannot convert large enums");
        return (SPI_DrivingStrength) -1;
    } else {
        SPI_DrivingStrength val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (SPI_DrivingStrength) -1;
        val = __Pyx_PyInt_As_SPI_DrivingStrength(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to SPI_DrivingStrength");
    return (SPI_DrivingStrength) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to SPI_DrivingStrength");
    return (SPI_DrivingStrength) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE FT4222_SPIMode __Pyx_PyInt_As_FT4222_SPIMode(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const FT4222_SPIMode neg_one = (FT4222_SPIMode) -1, const_zero = (FT4222_SPIMode) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(FT4222_SPIMode) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (FT4222_SPIMode) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(FT4222_SPIMode) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIMode) >= 2 * PyLong_SHIFT)) {
                                return (FT4222_SPIMode) (((((FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(FT4222_SPIMode) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIMode) >= 3 * PyLong_SHIFT)) {
                                return (FT4222_SPIMode) (((((((FT4222_SPIMode)digits[2]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(FT4222_SPIMode) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIMode) >= 4 * PyLong_SHIFT)) {
                                return (FT4222_SPIMode) (((((((((FT4222_SPIMode)digits[3]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[2]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (FT4222_SPIMode) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(FT4222_SPIMode) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPIMode, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(FT4222_SPIMode) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPIMode, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(FT4222_SPIMode) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIMode) - 1 > 2 * PyLong_SHIFT)) {
                                return (FT4222_SPIMode) (((FT4222_SPIMode)-1)*(((((FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(FT4222_SPIMode) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIMode) - 1 > 2 * PyLong_SHIFT)) {
                                return (FT4222_SPIMode) ((((((FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(FT4222_SPIMode) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIMode) - 1 > 3 * PyLong_SHIFT)) {
                                return (FT4222_SPIMode) (((FT4222_SPIMode)-1)*(((((((FT4222_SPIMode)digits[2]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(FT4222_SPIMode) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIMode) - 1 > 3 * PyLong_SHIFT)) {
                                return (FT4222_SPIMode) ((((((((FT4222_SPIMode)digits[2]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(FT4222_SPIMode) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIMode) - 1 > 4 * PyLong_SHIFT)) {
                                return (FT4222_SPIMode) (((FT4222_SPIMode)-1)*(((((((((FT4222_SPIMode)digits[3]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[2]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(FT4222_SPIMode) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIMode) - 1 > 4 * PyLong_SHIFT)) {
                                return (FT4222_SPIMode) ((((((((((FT4222_SPIMode)digits[3]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[2]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(FT4222_SPIMode) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPIMode, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(FT4222_SPIMode) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPIMode, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        PyErr_SetString(PyExc_RuntimeError,
                        "_PyLong_AsByteArray() not available, cannot convert large enums");
        return (FT4222_SPIMode) -1;
    } else {
        FT4222_SPIMode val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (FT4222_SPIMode) -1;
        val = __Pyx_PyInt_As_FT4222_SPIMode(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to FT4222_SPIMode");
    return (FT4222_SPIMode) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to FT4222_SPIMode");
    return (FT4222_SPIMode) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE FT4222_SPIClock __Pyx_PyInt_As_FT4222_SPIClock(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const FT4222_SPIClock neg_one = (FT4222_SPIClock) -1, const_zero = (FT4222_SPIClock) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(FT4222_SPIClock) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (FT4222_SPIClock) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(FT4222_SPIClock) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIClock) >= 2 * PyLong_SHIFT)) {
                                return (FT4222_SPIClock) (((((FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(FT4222_SPIClock) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIClock) >= 3 * PyLong_SHIFT)) {
                                return (FT4222_SPIClock) (((((((FT4222_SPIClock)digits[2]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(FT4222_SPIClock) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIClock) >= 4 * PyLong_SHIFT)) {
                                return (FT4222_SPIClock) (((((((((FT4222_SPIClock)digits[3]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[2]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (FT4222_SPIClock) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(FT4222_SPIClock) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPIClock, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(FT4222_SPIClock) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPIClock, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(FT4222_SPIClock) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIClock) - 1 > 2 * PyLong_SHIFT)) {
                                return (FT4222_SPIClock) (((FT4222_SPIClock)-1)*(((((FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(FT4222_SPIClock) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIClock) - 1 > 2 * PyLong_SHIFT)) {
                                return (FT4222_SPIClock) ((((((FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(FT4222_SPIClock) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIClock) - 1 > 3 * PyLong_SHIFT)) {
                                return (FT4222_SPIClock) (((FT4222_SPIClock)-1)*(((((((FT4222_SPIClock)digits[2]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(FT4222_SPIClock) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIClock) - 1 > 3 * PyLong_SHIFT)) {
                                return (FT4222_SPIClock) ((((((((FT4222_SPIClock)digits[2]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(FT4222_SPIClock) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIClock) - 1 > 4 * PyLong_SHIFT)) {
                                return (FT4222_SPIClock) (((FT4222_SPIClock)-1)*(((((((((FT4222_SPIClock)digits[3]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[2]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(FT4222_SPIClock) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPIClock) - 1 > 4 * PyLong_SHIFT)) {
                                return (FT4222_SPIClock) ((((((((((FT4222_SPIClock)digits[3]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[2]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(FT4222_SPIClock) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPIClock, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(FT4222_SPIClock) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPIClock, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        PyErr_SetString(PyExc_RuntimeError,
                        "_PyLong_AsByteArray() not available, cannot convert large enums");
        return (FT4222_SPIClock) -1;
    } else {
        FT4222_SPIClock val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (FT4222_SPIClock) -1;
        val = __Pyx_PyInt_As_FT4222_SPIClock(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to FT4222_SPIClock");
    return (FT4222_SPIClock) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to FT4222_SPIClock");
    return (FT4222_SPIClock) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE FT4222_SPICPOL __Pyx_PyInt_As_FT4222_SPICPOL(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const FT4222_SPICPOL neg_one = (FT4222_SPICPOL) -1, const_zero = (FT4222_SPICPOL) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(FT4222_SPICPOL) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (FT4222_SPICPOL) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(FT4222_SPICPOL) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPOL) >= 2 * PyLong_SHIFT)) {
                                return (FT4222_SPICPOL) (((((FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(FT4222_SPICPOL) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPOL) >= 3 * PyLong_SHIFT)) {
                                return (FT4222_SPICPOL) (((((((FT4222_SPICPOL)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(FT4222_SPICPOL) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPOL) >= 4 * PyLong_SHIFT)) {
                                return (FT4222_SPICPOL) (((((((((FT4222_SPICPOL)digits[3]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (FT4222_SPICPOL) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(FT4222_SPICPOL) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPICPOL, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(FT4222_SPICPOL) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPICPOL, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(FT4222_SPICPOL) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPOL) - 1 > 2 * PyLong_SHIFT)) {
                                return (FT4222_SPICPOL) (((FT4222_SPICPOL)-1)*(((((FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(FT4222_SPICPOL) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPOL) - 1 > 2 * PyLong_SHIFT)) {
                                return (FT4222_SPICPOL) ((((((FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(FT4222_SPICPOL) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPOL) - 1 > 3 * PyLong_SHIFT)) {
                                return (FT4222_SPICPOL) (((FT4222_SPICPOL)-1)*(((((((FT4222_SPICPOL)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(FT4222_SPICPOL) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPOL) - 1 > 3 * PyLong_SHIFT)) {
                                return (FT4222_SPICPOL) ((((((((FT4222_SPICPOL)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(FT4222_SPICPOL) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPOL) - 1 > 4 * PyLong_SHIFT)) {
                                return (FT4222_SPICPOL) (((FT4222_SPICPOL)-1)*(((((((((FT4222_SPICPOL)digits[3]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(FT4222_SPICPOL) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPOL) - 1 > 4 * PyLong_SHIFT)) {
                                return (FT4222_SPICPOL) ((((((((((FT4222_SPICPOL)digits[3]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(FT4222_SPICPOL) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPICPOL, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(FT4222_SPICPOL) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPICPOL, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        PyErr_SetString(PyExc_RuntimeError,
                        "_PyLong_AsByteArray() not available, cannot convert large enums");
        return (FT4222_SPICPOL) -1;
    } else {
        FT4222_SPICPOL val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (FT4222_SPICPOL) -1;
        val = __Pyx_PyInt_As_FT4222_SPICPOL(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to FT4222_SPICPOL");
    return (FT4222_SPICPOL) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to FT4222_SPICPOL");
    return (FT4222_SPICPOL) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE FT4222_SPICPHA __Pyx_PyInt_As_FT4222_SPICPHA(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const FT4222_SPICPHA neg_one = (FT4222_SPICPHA) -1, const_zero = (FT4222_SPICPHA) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(FT4222_SPICPHA) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (FT4222_SPICPHA) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(FT4222_SPICPHA) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPHA) >= 2 * PyLong_SHIFT)) {
                                return (FT4222_SPICPHA) (((((FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(FT4222_SPICPHA) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPHA) >= 3 * PyLong_SHIFT)) {
                                return (FT4222_SPICPHA) (((((((FT4222_SPICPHA)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(FT4222_SPICPHA) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPHA) >= 4 * PyLong_SHIFT)) {
                                return (FT4222_SPICPHA) (((((((((FT4222_SPICPHA)digits[3]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (FT4222_SPICPHA) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(FT4222_SPICPHA) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPICPHA, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(FT4222_SPICPHA) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPICPHA, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(FT4222_SPICPHA) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPHA) - 1 > 2 * PyLong_SHIFT)) {
                                return (FT4222_SPICPHA) (((FT4222_SPICPHA)-1)*(((((FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(FT4222_SPICPHA) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPHA) - 1 > 2 * PyLong_SHIFT)) {
                                return (FT4222_SPICPHA) ((((((FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(FT4222_SPICPHA) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPHA) - 1 > 3 * PyLong_SHIFT)) {
                                return (FT4222_SPICPHA) (((FT4222_SPICPHA)-1)*(((((((FT4222_SPICPHA)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(FT4222_SPICPHA) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPHA) - 1 > 3 * PyLong_SHIFT)) {
                                return (FT4222_SPICPHA) ((((((((FT4222_SPICPHA)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(FT4222_SPICPHA) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPHA) - 1 > 4 * PyLong_SHIFT)) {
                                return (FT4222_SPICPHA) (((FT4222_SPICPHA)-1)*(((((((((FT4222_SPICPHA)digits[3]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(FT4222_SPICPHA) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(FT4222_SPICPHA) - 1 > 4 * PyLong_SHIFT)) {
                                return (FT4222_SPICPHA) ((((((((((FT4222_SPICPHA)digits[3]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(FT4222_SPICPHA) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPICPHA, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(FT4222_SPICPHA) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPICPHA, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        PyErr_SetString(PyExc_RuntimeError,
                        "_PyLong_AsByteArray() not available, cannot convert large enums");
        return (FT4222_SPICPHA) -1;
    } else {
        FT4222_SPICPHA val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (FT4222_SPICPHA) -1;
        val = __Pyx_PyInt_As_FT4222_SPICPHA(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to FT4222_SPICPHA");
    return (FT4222_SPICPHA) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to FT4222_SPICPHA");
    return (FT4222_SPICPHA) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE SPI_SlaveProtocol __Pyx_PyInt_As_SPI_SlaveProtocol(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const SPI_SlaveProtocol neg_one = (SPI_SlaveProtocol) -1, const_zero = (SPI_SlaveProtocol) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(SPI_SlaveProtocol) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (SPI_SlaveProtocol) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(SPI_SlaveProtocol) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_SlaveProtocol) >= 2 * PyLong_SHIFT)) {
                                return (SPI_SlaveProtocol) (((((SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(SPI_SlaveProtocol) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_SlaveProtocol) >= 3 * PyLong_SHIFT)) {
                                return (SPI_SlaveProtocol) (((((((SPI_SlaveProtocol)digits[2]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(SPI_SlaveProtocol) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_SlaveProtocol) >= 4 * PyLong_SHIFT)) {
                                return (SPI_SlaveProtocol) (((((((((SPI_SlaveProtocol)digits[3]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[2]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (SPI_SlaveProtocol) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(SPI_SlaveProtocol) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(SPI_SlaveProtocol, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(SPI_SlaveProtocol) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(SPI_SlaveProtocol, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(SPI_SlaveProtocol) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_SlaveProtocol) - 1 > 2 * PyLong_SHIFT)) {
                                return (SPI_SlaveProtocol) (((SPI_SlaveProtocol)-1)*(((((SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(SPI_SlaveProtocol) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_SlaveProtocol) - 1 > 2 * PyLong_SHIFT)) {
                                return (SPI_SlaveProtocol) ((((((SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(SPI_SlaveProtocol) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_SlaveProtocol) - 1 > 3 * PyLong_SHIFT)) {
                                return (SPI_SlaveProtocol) (((SPI_SlaveProtocol)-1)*(((((((SPI_SlaveProtocol)digits[2]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(SPI_SlaveProtocol) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_SlaveProtocol) - 1 > 3 * PyLong_SHIFT)) {
                                return (SPI_SlaveProtocol) ((((((((SPI_SlaveProtocol)digits[2]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(SPI_SlaveProtocol) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_SlaveProtocol) - 1 > 4 * PyLong_SHIFT)) {
                                return (SPI_SlaveProtocol) (((SPI_SlaveProtocol)-1)*(((((((((SPI_SlaveProtocol)digits[3]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[2]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(SPI_SlaveProtocol) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(SPI_SlaveProtocol) - 1 > 4 * PyLong_SHIFT)) {
                                return (SPI_SlaveProtocol) ((((((((((SPI_SlaveProtocol)digits[3]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[2]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(SPI_SlaveProtocol) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(SPI_SlaveProtocol, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(SPI_SlaveProtocol) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(SPI_SlaveProtocol, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        PyErr_SetString(PyExc_RuntimeError,
                        "_PyLong_AsByteArray() not available, cannot convert large enums");
        return (SPI_SlaveProtocol) -1;
    } else {
        SPI_SlaveProtocol val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (SPI_SlaveProtocol) -1;
        val = __Pyx_PyInt_As_SPI_SlaveProtocol(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to SPI_SlaveProtocol");
    return (SPI_SlaveProtocol) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to SPI_SlaveProtocol");
    return (SPI_SlaveProtocol) -1;
}

/* FormatTypeName */
  #if CYTHON_COMPILING_IN_LIMITED_API
static __Pyx_TypeName
__Pyx_PyType_GetName(PyTypeObject* tp)
{
    PyObject *name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_n_s_name);
    if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) {
        PyErr_Clear();
        Py_XDECREF(name);
        name = __Pyx_NewRef(__pyx_n_s__111);
    }
    return name;
}
#endif

/* CIntFromPy */
  static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(long) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                                return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                                return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                                return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(long) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                                return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                                return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                                return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                                return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                                return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                                return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(long) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
#if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
#endif
            if (likely(v)) {
                int ret = -1;
#if PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                ret = _PyLong_AsByteArray((PyLongObject *)v,
                                           bytes, sizeof(val),
                                           is_little, !is_unsigned);
#else
                PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
                int bits, remaining_bits, is_negative = 0;
                long idigit;
                int chunk_size = (sizeof(long) < 8) ? 30 : 62;
                if (unlikely(!PyLong_CheckExact(v))) {
                    PyObject *tmp = v;
                    v = PyNumber_Long(v);
                    assert(PyLong_CheckExact(v));
                    Py_DECREF(tmp);
                    if (unlikely(!v)) return (long) -1;
                }
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                if (Py_SIZE(x) == 0)
                    return (long) 0;
                is_negative = Py_SIZE(x) < 0;
#else
                {
                    int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                    if (unlikely(result < 0))
                        return (long) -1;
                    is_negative = result == 1;
                }
#endif
                if (is_unsigned && unlikely(is_negative)) {
                    goto raise_neg_overflow;
                } else if (is_negative) {
                    stepval = PyNumber_Invert(v);
                    if (unlikely(!stepval))
                        return (long) -1;
                } else {
                    stepval = __Pyx_NewRef(v);
                }
                val = (long) 0;
                mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
                shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
                for (bits = 0; bits < (int) sizeof(long) * 8 - chunk_size; bits += chunk_size) {
                    PyObject *tmp, *digit;
                    digit = PyNumber_And(stepval, mask);
                    if (unlikely(!digit)) goto done;
                    idigit = PyLong_AsLong(digit);
                    Py_DECREF(digit);
                    if (unlikely(idigit < 0)) goto done;
                    tmp = PyNumber_Rshift(stepval, shift);
                    if (unlikely(!tmp)) goto done;
                    Py_DECREF(stepval); stepval = tmp;
                    val |= ((long) idigit) << bits;
                    #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                    if (Py_SIZE(stepval) == 0)
                        goto unpacking_done;
                    #endif
                }
                idigit = PyLong_AsLong(stepval);
                if (unlikely(idigit < 0)) goto done;
                remaining_bits = ((int) sizeof(long) * 8) - bits - (is_unsigned ? 0 : 1);
                if (unlikely(idigit >= (1L << remaining_bits)))
                    goto raise_overflow;
                val |= ((long) idigit) << bits;
            #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
            unpacking_done:
            #endif
                if (!is_unsigned) {
                    if (unlikely(val & (((long) 1) << (sizeof(long) * 8 - 1))))
                        goto raise_overflow;
                    if (is_negative)
                        val = ~val;
                }
                ret = 0;
            done:
                Py_XDECREF(shift);
                Py_XDECREF(mask);
                Py_XDECREF(stepval);
#endif
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
  #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = __Pyx_PyType_GetSlot(a, tp_base, PyTypeObject*);
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (cls == a || cls == b) return 1;
    mro = cls->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            PyObject *base = PyTuple_GET_ITEM(mro, i);
            if (base == (PyObject *)a || base == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    if (exc_type1) {
        return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
    } else {
        return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
  static unsigned long __Pyx_get_runtime_version(void) {
#if __PYX_LIMITED_VERSION_HEX >= 0x030B00A4
    return Py_Version & ~0xFFUL;
#else
    const char* rt_version = Py_GetVersion();
    unsigned long version = 0;
    unsigned long factor = 0x01000000UL;
    unsigned int digit = 0;
    int i = 0;
    while (factor) {
        while ('0' <= rt_version[i] && rt_version[i] <= '9') {
            digit = digit * 10 + (unsigned int) (rt_version[i] - '0');
            ++i;
        }
        version += factor * digit;
        if (rt_version[i] != '.')
            break;
        digit = 0;
        factor >>= 8;
        ++i;
    }
    return version;
#endif
}
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer) {
    const unsigned long MAJOR_MINOR = 0xFFFF0000UL;
    if ((rt_version & MAJOR_MINOR) == (ct_version & MAJOR_MINOR))
        return 0;
    if (likely(allow_newer && (rt_version & MAJOR_MINOR) > (ct_version & MAJOR_MINOR)))
        return 1;
    {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compile time Python version %d.%d "
                      "of module '%.100s' "
                      "%s "
                      "runtime version %d.%d",
                       (int) (ct_version >> 24), (int) ((ct_version >> 16) & 0xFF),
                       __Pyx_MODULE_NAME,
                       (allow_newer) ? "was newer than" : "does not match",
                       (int) (rt_version >> 24), (int) ((rt_version >> 16) & 0xFF)
       );
        return PyErr_WarnEx(NULL, message, 1);
    }
}

/* InitStrings */
  #if PY_MAJOR_VERSION >= 3
static int __Pyx_InitString(__Pyx_StringTabEntry t, PyObject **str) {
    if (t.is_unicode | t.is_str) {
        if (t.intern) {
            *str = PyUnicode_InternFromString(t.s);
        } else if (t.encoding) {
            *str = PyUnicode_Decode(t.s, t.n - 1, t.encoding, NULL);
        } else {
            *str = PyUnicode_FromStringAndSize(t.s, t.n - 1);
        }
    } else {
        *str = PyBytes_FromStringAndSize(t.s, t.n - 1);
    }
    if (!*str)
        return -1;
    if (PyObject_Hash(*str) == -1)
        return -1;
    return 0;
}
#endif
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION >= 3
        __Pyx_InitString(*t, t->p);
        #else
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        #endif
        ++t;
    }
    return 0;
}

#include <string.h>
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s) {
    size_t len = strlen(s);
    if (unlikely(len > (size_t) PY_SSIZE_T_MAX)) {
        PyErr_SetString(PyExc_OverflowError, "byte string is too long");
        return -1;
    }
    return (Py_ssize_t) len;
}
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return __Pyx_PyUnicode_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return PyByteArray_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY && !CYTHON_COMPILING_IN_LIMITED_API) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetName(Py_TYPE(result));
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type " __Pyx_FMT_TYPENAME ")",
                 type_name, type_name, result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(__Pyx_PyLong_IsCompact(b))) {
        return __Pyx_PyLong_CompactValue(b);
    } else {
      const digit* digits = __Pyx_PyLong_Digits(b);
      const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(b);
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


/* #### Code section: utility_code_pragmas_end ### */
#ifdef _MSC_VER
#pragma warning( pop )
#endif



/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
